var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("debugsettings", ["require", "exports"], function (require, exports) {
    "use strict";
    var DebugSettings = (function () {
        function DebugSettings() {
            this.initialization = {
                stopOnLaunch: false,
                stopOnResume: false,
                metadata: false,
                imagePreloading: false
            };
            this.ios = {
                hasMultitasking: false
            };
            this.actionBlock = {
                traceBlocksVisbility: false
            };
            this.tableView = {
                trace: true,
                traceTables: [],
                reportApiError: false,
                traceGameEvents: false,
                betLevelChangeDelay: 4000,
                addonPeriodStartedDelay: 4000
            };
            this.lobby = {
                trace: false,
                useTournamentNameForTournamentCaption: true
            };
            this.home = {
                traceNews: false,
                traceBanners: false
            };
            this.device = {
                events: true,
                orientation: false,
                statusbar: false
            };
            this.connection = {
                signalR: true,
                slowInternet: false,
                dataEvents: false
            };
            this.application = {
                reloadManager: true,
                deactivateOnPause: false,
                goToLobbyAfterPause: false,
                reloadTablesDataOnResume: false,
                debugTimeouts: true,
                useUtcDates: false,
                debugTabbar: false
            };
            this.ui = {
                tracePages: false,
                tracePopups: false
            };
        }
        return DebugSettings;
    }());
    exports.debugSettings = new DebugSettings();
});
define("table/runtimesettings", ["require", "exports"], function (require, exports) {
    "use strict";
    var RuntimeSettings = (function () {
        function RuntimeSettings() {
            /**
            * Enables or disables all timer related content update on the pages
            */
            this.updateTimer = true;
            /**
            * Enables or disables all timer related content update on the pages
            */
            this.tableCardsAnimating = true;
            /**
            * Enables or disables showing news popup.
            */
            this.showNewsAfterLogin = true;
            /**
            * Sound settings
            */
            this.sounds = {
                /**
                * Value indicating whether play sound of deal cards
                */
                dealCardsEnabled: false
            };
        }
        RuntimeSettings.prototype.setShowNewsAfterLogin = function (value) {
            this.showNewsAfterLogin = value;
        };
        return RuntimeSettings;
    }());
    var runtimeSettings = new RuntimeSettings();
    return runtimeSettings;
});
define("timeService", ["require", "exports", "knockout", "moment", "debugsettings", "table/runtimesettings"], function (require, exports, ko, moment, debugsettings_1, runtimeSettings) {
    "use strict";
    var TimeService = (function () {
        function TimeService() {
            this.timeDiff = 0;
            this.timeouts = [];
            this.intervals = [];
            this.currentTime = ko.observable();
        }
        TimeService.prototype.start = function () {
            var _this = this;
            var api = new OnlinePoker.Commanding.API.Metadata(apiHost);
            api.GetDate(function (serverTime) {
                var currentDate = new Date();
                _this.timeDiff = serverTime - currentDate.valueOf();
            }).done(function () {
                // Wait until new minute starts.
                var currentTime = _this.getCurrentDateTime();
                var pauseBeforeStartLongInterval = TimeService.MillisecondsInMinutes - (currentTime.getSeconds() * 1000)
                    + currentTime.getMilliseconds();
                _this.updateCurrentTime();
                _this.setTimeout(function () {
                    _this.handle = _this.setInterval(function () {
                        _this.updateCurrentTime();
                    }, TimeService.MillisecondsInMinutes);
                }, pauseBeforeStartLongInterval);
            }).fail(function () { return _this.start(); });
        };
        TimeService.prototype.stop = function () {
            this.timeDiff = 0;
            this.clearInterval(this.handle);
        };
        TimeService.prototype.setTimeout = function (handler, timeout) {
            var _this = this;
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var handle = setTimeout(function () {
                _this.timeouts = _this.timeouts.filter(function (_) { return _ !== handle; });
                handler();
            }, timeout, args);
            this.timeouts.push(handle);
            return handle;
        };
        TimeService.prototype.clearTimeout = function (handle) {
            this.timeouts = this.timeouts.filter(function (_) { return _ !== handle; });
            clearTimeout(handle);
        };
        TimeService.prototype.setInterval = function (handler, timeout) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var handle = setInterval(handler, timeout, args);
            this.intervals.push(handle);
            return handle;
        };
        TimeService.prototype.clearInterval = function (handle) {
            this.intervals = this.intervals.filter(function (_) { return _ !== handle; });
            clearInterval(handle);
        };
        TimeService.prototype.printDebug = function () {
            if (debugsettings_1.debugSettings.application.debugTimeouts) {
                console.log("Timeouts running = " + this.timeouts.length + ". "
                    + "Intervals running = " + this.intervals.length);
            }
        };
        TimeService.prototype.updateCurrentTime = function () {
            var d = this.getCurrentDateTime();
            var date = moment(d.valueOf());
            if (debugsettings_1.debugSettings.application.useUtcDates) {
                date = date.utc();
            }
            var t = date.format("H:mm");
            if (runtimeSettings.updateTimer) {
                this.currentTime(t);
            }
        };
        TimeService.prototype.getCurrentDateTime = function () {
            var d = new Date();
            if (!this.timeDiff) {
                return d;
            }
            return new Date(d.valueOf() + this.timeDiff);
        };
        TimeService.MillisecondsInMinutes = 60 * 1000;
        return TimeService;
    }());
    var timeService = new TimeService();
    return timeService;
});
var beforeSendHandler = function (xhr) {
    xhr.withCredentials = true;
    if (authToken != null) {
        xhr.setRequestHeader("X-Auth-Token", authToken);
    }
};
var TournamentStatus;
(function (TournamentStatus) {
    TournamentStatus[TournamentStatus["Pending"] = 0] = "Pending";
    TournamentStatus[TournamentStatus["RegistrationStarted"] = 1] = "RegistrationStarted";
    TournamentStatus[TournamentStatus["RegistrationCancelled"] = 2] = "RegistrationCancelled";
    TournamentStatus[TournamentStatus["SettingUp"] = 3] = "SettingUp";
    TournamentStatus[TournamentStatus["WaitingTournamentStart"] = 4] = "WaitingTournamentStart";
    TournamentStatus[TournamentStatus["Started"] = 5] = "Started";
    TournamentStatus[TournamentStatus["Completed"] = 6] = "Completed";
    TournamentStatus[TournamentStatus["Cancelled"] = 7] = "Cancelled";
    TournamentStatus[TournamentStatus["LateRegistration"] = 8] = "LateRegistration";
})(TournamentStatus || (TournamentStatus = {}));
/**
* Represents player status in the tournament.
*/
var TournamentPlayerStatus;
(function (TournamentPlayerStatus) {
    /**
    * Player registered to play in the tournament
    */
    TournamentPlayerStatus[TournamentPlayerStatus["Registered"] = 0] = "Registered";
    /**
    * Player cancel his registration.
    */
    TournamentPlayerStatus[TournamentPlayerStatus["RegistrationCancelled"] = 1] = "RegistrationCancelled";
    /**
    * Player is currently playing in the tournament.
    */
    TournamentPlayerStatus[TournamentPlayerStatus["Playing"] = 2] = "Playing";
    /**
    * Player complete playing in the tournament.
    */
    TournamentPlayerStatus[TournamentPlayerStatus["Completed"] = 3] = "Completed";
})(TournamentPlayerStatus || (TournamentPlayerStatus = {}));
var TournamentOptionsEnum;
(function (TournamentOptionsEnum) {
    TournamentOptionsEnum[TournamentOptionsEnum["None"] = 0] = "None";
    TournamentOptionsEnum[TournamentOptionsEnum["HasBuyIn"] = 1] = "HasBuyIn";
    TournamentOptionsEnum[TournamentOptionsEnum["HasEntryFee"] = 2] = "HasEntryFee";
    TournamentOptionsEnum[TournamentOptionsEnum["HasRebuy"] = 4] = "HasRebuy";
    TournamentOptionsEnum[TournamentOptionsEnum["HasAddon"] = 8] = "HasAddon";
    TournamentOptionsEnum[TournamentOptionsEnum["RebuyGoesToPrizePool"] = 16] = "RebuyGoesToPrizePool";
    TournamentOptionsEnum[TournamentOptionsEnum["RebuyGoesToCasino"] = 32] = "RebuyGoesToCasino";
    TournamentOptionsEnum[TournamentOptionsEnum["AddonGoesToPrizePool"] = 64] = "AddonGoesToPrizePool";
    TournamentOptionsEnum[TournamentOptionsEnum["AddonGoesToCasino"] = 128] = "AddonGoesToCasino";
})(TournamentOptionsEnum || (TournamentOptionsEnum = {}));
var OnlinePoker;
(function (OnlinePoker) {
    var Commanding;
    (function (Commanding) {
        var API;
        (function (API) {
            (function (MoneyType) {
                MoneyType[MoneyType["Tenge"] = 1] = "Tenge";
                MoneyType[MoneyType["GameChips"] = 2] = "GameChips";
            })(API.MoneyType || (API.MoneyType = {}));
            var MoneyType = API.MoneyType;
            ;
            API.logging = false;
            API.version = 1;
            var WebApiProxy = (function () {
                function WebApiProxy(host, baseName) {
                    this.host = host;
                    this.baseName = baseName;
                    this.timeout = 5000;
                    this.baseUrl = host + '/' + baseName + '/';
                }
                WebApiProxy.prototype.Call = function (methodName, parameters, successCallback) {
                    var self = this;
                    var parametersString = "null";
                    if (parameters != null) {
                        parametersString = JSON.stringify(parameters);
                    }
                    this.Log(this.baseName, "Method " + methodName + " called. Parameters: " + parametersString);
                    var url = this.baseUrl + methodName;
                    return $.ajax({
                        type: 'POST',
                        url: url,
                        data: JSON.stringify(parameters),
                        success: function (data, textStatus, jqXHR) {
                            var dataString = data != null ? JSON.stringify(data) : "NULL";
                            textStatus = textStatus != null ? textStatus : "NULL";
                            var logMessage = "Method " + methodName + " finished. Status: " + textStatus + ". Results: " + dataString;
                            self.Log(self.baseName, logMessage);
                            if (successCallback != null) {
                                successCallback(data, textStatus, jqXHR);
                            }
                        },
                        timeout: this.timeout,
                        crossDomain: true,
                        async: true,
                        contentType: 'application/json',
                        beforeSend: beforeSendHandler,
                        dataType: 'json'
                    });
                };
                WebApiProxy.prototype.SimpleCall = function (methodName, successCallback) {
                    var data = {};
                    return this.Call(methodName, data, successCallback);
                };
                WebApiProxy.prototype.Log = function (api, msg) {
                    if (API.logging === false) {
                        return;
                    }
                    if (typeof (window.console) === "undefined") {
                        return;
                    }
                    var m = "[" + new Date().toTimeString() + "] OnlinePoker API(" + this.baseName + "): " + msg;
                    if (window.console.debug) {
                        window.console.debug(m);
                    }
                    else if (window.console.log) {
                        window.console.log(m);
                    }
                };
                return WebApiProxy;
            }());
            API.WebApiProxy = WebApiProxy;
            var Chat = (function (_super) {
                __extends(Chat, _super);
                function Chat(host) {
                    _super.call(this, host, 'Chat');
                }
                Chat.prototype.Send = function (tableId, message, callback) {
                    var data = { tableId: tableId, message: message };
                    return _super.prototype.Call.call(this, 'Send', data, callback);
                };
                return Chat;
            }(WebApiProxy));
            API.Chat = Chat;
            var Message = (function (_super) {
                __extends(Message, _super);
                function Message(host) {
                    _super.call(this, host, 'Message');
                }
                Message.prototype.Send = function (recepient, subject, body, callback) {
                    var data = { Recipient: recepient, Subject: subject, Body: body };
                    return _super.prototype.Call.call(this, 'Send', data, callback);
                };
                Message.prototype.GetInboxMessages = function (page, pageSize, filter, sortOrder, callback) {
                    var data = { Page: page, PageSize: pageSize, Filter: filter, SortOrder: sortOrder };
                    return _super.prototype.Call.call(this, 'GetInboxMessages', data, callback);
                };
                Message.prototype.GetSentMessages = function (page, pageSize, filter, sortOrder, callback) {
                    var data = { Page: page, PageSize: pageSize, Filter: filter, SortOrder: sortOrder };
                    return _super.prototype.Call.call(this, 'GetSentMessages', data, callback);
                };
                Message.prototype.GetMessage = function (id, callback) {
                    var data = { Id: id };
                    return _super.prototype.Call.call(this, 'GetMessage', data, callback);
                };
                return Message;
            }(WebApiProxy));
            API.Message = Message;
            var Game = (function (_super) {
                __extends(Game, _super);
                function Game(host) {
                    /// <signature>
                    ///   <summary>Wrapper for the Game Commanding API</summary>
                    ///   <param name="host">Host with Metadata Commanding API installed</param>
                    /// </signature>
                    _super.call(this, host, 'Game');
                }
                Game.prototype.GetTables = function (fullTables, privateTables, maxPlayers, betLevels, moneyType, limitType, callback) {
                    var data = {
                        FullTables: fullTables,
                        Private: privateTables,
                        MaxPlayers: maxPlayers,
                        BetLevels: betLevels,
                        MoneyType: moneyType,
                        LimitType: limitType
                    };
                    return _super.prototype.Call.call(this, 'GetTables', data, callback);
                };
                Game.prototype.GetTable = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'GetTable', data, callback);
                };
                Game.prototype.GetSitingTables = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetSitingTables', data, callback);
                };
                Game.prototype.Sit = function (tableId, seat, amount, callback) {
                    var data = { TableId: tableId, Seat: seat, Amount: amount };
                    return _super.prototype.Call.call(this, 'Sit', data, callback);
                };
                Game.prototype.SitAnywhere = function (tableId, amount, callback) {
                    var data = { TableId: tableId, Amount: amount };
                    return _super.prototype.Call.call(this, 'SitAnywhere', data, callback);
                };
                Game.prototype.Standup = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'Standup', data, callback);
                };
                Game.prototype.Fold = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'Fold', data, callback);
                };
                Game.prototype.CheckOrCall = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'CheckOrCall', data, callback);
                };
                Game.prototype.BetOrRaise = function (tableId, amount, callback) {
                    var data = { TableId: tableId, Amount: amount };
                    return _super.prototype.Call.call(this, 'BetOrRaise', data, callback);
                };
                Game.prototype.ForceJoinGame = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'ForceJoin', data, callback);
                };
                Game.prototype.WaitBigBlind = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'WaitBigBlind', data, callback);
                };
                Game.prototype.AddBalance = function (tableId, amount, callback) {
                    var data = { TableId: tableId, Amount: amount };
                    return _super.prototype.Call.call(this, 'AddBalance', data, callback);
                };
                Game.prototype.SitOut = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'SitOut', data, callback);
                };
                Game.prototype.ForceSitout = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'ForceSitout', data, callback);
                };
                Game.prototype.ComeBack = function (tableId, callback) {
                    var data = { TableId: tableId };
                    return _super.prototype.Call.call(this, 'ComeBack', data, callback);
                };
                Game.prototype.SetOpenCardsParameters = function (tableId, openCardsAutomatically, callback) {
                    var data = { TableId: tableId, OpenCardsAutomatically: openCardsAutomatically };
                    return _super.prototype.Call.call(this, 'SetOpenCardsParameters', data, callback);
                };
                return Game;
            }(WebApiProxy));
            API.Game = Game;
            var Tournament = (function (_super) {
                __extends(Tournament, _super);
                function Tournament(host) {
                    /// <signature>
                    ///   <summary>Wrapper for the Tournament Commanding API</summary>
                    ///   <param name="host">Host with Metadata Commanding API installed</param>
                    /// </signature>
                    _super.call(this, host, 'Tournament');
                }
                Tournament.prototype.GetTournaments = function (prizeCurrency, tournamentType, speed, buyin, maxPlayers, callback) {
                    var data = {
                        TournamentType: tournamentType,
                        PrizeCurrency: prizeCurrency,
                        Speed: speed,
                        BuyIn: buyin,
                        MaxPlayers: maxPlayers
                    };
                    return _super.prototype.Call.call(this, 'GetTournaments', data, callback);
                };
                Tournament.prototype.GetTournament = function (tournamentId, callback) {
                    var data = { Id: tournamentId };
                    return _super.prototype.Call.call(this, 'GetTournament', data, callback);
                };
                Tournament.prototype.Register = function (tournamentId, callback) {
                    var data = { Id: tournamentId };
                    return _super.prototype.Call.call(this, 'Register', data, callback);
                };
                Tournament.prototype.CancelRegistration = function (tournamentId, callback) {
                    var data = { Id: tournamentId };
                    return _super.prototype.Call.call(this, 'CancelRegistration', data, callback);
                };
                Tournament.prototype.Rebuy = function (tournamentId, double, callback) {
                    var data = { Id: tournamentId, Double: double };
                    return _super.prototype.Call.call(this, 'Rebuy', data, callback);
                };
                Tournament.prototype.Addon = function (tournamentId, callback) {
                    var data = { Id: tournamentId };
                    return _super.prototype.Call.call(this, 'Addon', data, callback);
                };
                Tournament.prototype.GetRegisteredTournamentsStatus = function (callback) {
                    var data = {};
                    // return super.SimpleCall('GetRegisteredTournamentsStatus', callback);
                    return _super.prototype.Call.call(this, 'GetRegisteredTournamentsStatus', data, callback);
                };
                return Tournament;
            }(WebApiProxy));
            API.Tournament = Tournament;
            var Account = (function (_super) {
                __extends(Account, _super);
                function Account(host) {
                    /// <signature>
                    ///   <summary>Wrapper for the Metadata Commanding API</summary>
                    ///   <param name="host">Host with Metadata Commanding API installed</param>
                    /// </signature>
                    _super.call(this, host, 'Account');
                }
                Account.prototype.ActivateAccount = function (login, token, callback) {
                    var data = { Login: login, Token: token };
                    return _super.prototype.Call.call(this, 'ActivateAccount', data, callback);
                };
                Account.prototype.Logout = function (callback) {
                    var data = {};
                    authToken = null;
                    return _super.prototype.Call.call(this, 'Logout', data, callback);
                };
                Account.prototype.Authenticate = function (login, password, rememberMe, callback) {
                    var data = { Login: login, Password: password, RememberMe: rememberMe };
                    var aquireTokenCallback = function (data, textStatus, xhr) {
                        authToken = xhr.getResponseHeader('X-Auth-Token');
                        console.log("Aquired auth token", authToken);
                        if (callback != null) {
                            callback(data, textStatus, xhr);
                        }
                    };
                    return _super.prototype.Call.call(this, 'Authenticate', data, aquireTokenCallback);
                };
                Account.prototype.CancelAccountActivation = function (login, token, callback) {
                    var data = { Login: login, Token: token };
                    return _super.prototype.Call.call(this, 'CancelAccountActivation', data, callback);
                };
                Account.prototype.ChangePassword = function (oldPassword, newPassword, callback) {
                    var data = { OldPassword: oldPassword, NewPassword: newPassword };
                    return _super.prototype.Call.call(this, 'ChangePassword', data, callback);
                };
                Account.prototype.GetPersonalAccount = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetPersonalAccount', data, callback);
                };
                Account.prototype.GetPlayerAccountHistory = function (fromDate, toDate, fromAmount, toAmount, operationType, callback) {
                    var data = {
                        FromDate: fromDate,
                        ToDate: toDate,
                        FromAmount: fromAmount,
                        ToAmount: toAmount,
                        OperationType: operationType
                    };
                    return _super.prototype.Call.call(this, 'GetPlayerAccountHistory', data, callback);
                };
                Account.prototype.GetPlayerDefinition = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetPlayerDefinition', data, callback);
                };
                Account.prototype.PutWithdrawalRequest = function (type, amount, parameters, callback) {
                    var data = { Type: type, Amount: amount, Parameters: parameters };
                    return _super.prototype.Call.call(this, 'PutWithdrawalRequest', data, callback);
                };
                Account.prototype.Register = function (login, email, password, firstName, lastName, patronymicName, country, city, additionalProperties, image, callback) {
                    var url = this.baseUrl + 'Register';
                    var data = {
                        Login: login,
                        Email: email,
                        Password: password,
                        FirstName: firstName,
                        LastName: lastName,
                        PatronymicName: patronymicName,
                        Country: country,
                        City: city,
                        Image: image,
                        AdditionalProperties: additionalProperties
                    };
                    return $.ajax({
                        type: 'POST',
                        url: url,
                        data: JSON.stringify(data),
                        success: callback,
                        crossDomain: true,
                        contentType: 'application/json',
                        beforeSend: beforeSendHandler,
                        dataType: 'json'
                    });
                };
                Account.prototype.RequestResetPassword = function (login, email, callback) {
                    var data = { Login: login, Email: email };
                    return _super.prototype.Call.call(this, 'RequestResetPassword', data, callback);
                };
                Account.prototype.ResetAvatar = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'ResetAvatar', data, callback);
                };
                Account.prototype.ResetPassword = function (token, newPassword, callback) {
                    var data = { Token: token, Password: newPassword };
                    return _super.prototype.Call.call(this, 'ResetPassword', data, callback);
                };
                Account.prototype.SetAvatarUrl = function (url, callback) {
                    var data = { Url: url };
                    return _super.prototype.Call.call(this, 'SetAvatarUrl', data, callback);
                };
                Account.prototype.UpdatePlayerProfile = function (firstName, lastName, patronymicName, email, country, city, image, callback) {
                    throw new Error('Calls with multipart data not supported');
                };
                Account.prototype.UploadAvatar = function (image, callback) {
                    throw new Error('Calls with multipart data not supported');
                };
                Account.prototype.GetBestPlayers = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetBestPlayers', data, callback);
                };
                Account.prototype.RegisterGuest = function (callback) {
                    var data = {};
                    return _super.prototype.Call.call(this, 'RegisterGuest', data, callback);
                };
                return Account;
            }(WebApiProxy));
            API.Account = Account;
            var Metadata = (function (_super) {
                __extends(Metadata, _super);
                function Metadata(host) {
                    /// <signature>
                    ///   <summary>Wrapper for the Metadata Commanding API</summary>
                    ///   <param name="host">Host with Metadata Commanding API installed</param>
                    /// </signature>
                    _super.call(this, host, 'Metadata');
                    this.timeout = 10000;
                }
                Metadata.prototype.GetServerLayout = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetServerLayout', data, callback);
                };
                Metadata.prototype.GetDefaultAvatars = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetDefaultAvatars', data, callback);
                };
                Metadata.prototype.GetWellKnownBetStructure = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetWellKnownBetStructure', data, callback);
                };
                Metadata.prototype.GetWellKnownPrizeStructure = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetWellKnownPrizeStructure', data, callback);
                };
                Metadata.prototype.GetNews = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetNews', data, callback);
                };
                Metadata.prototype.GetOnlinePlayers = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetOnlinePlayers', data, callback);
                };
                Metadata.prototype.GetBanners = function (format, callback) {
                    if (callback === void 0) { callback = null; }
                    var data = { Format: format };
                    return _super.prototype.Call.call(this, 'GetBanners', data, callback);
                };
                /**
                * Request server date.
                * @param callback A callback function that is executed if the request succeeds.
                */
                Metadata.prototype.GetDate = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'GetDate', data, callback);
                };
                /**
                * Perform version check.
                * @param callback A callback function that is executed if the request succeeds.
                */
                Metadata.prototype.VersionCheck = function (callback) {
                    if (callback === void 0) { callback = null; }
                    var data = {};
                    return _super.prototype.Call.call(this, 'VersionCheck', data, callback);
                };
                return Metadata;
            }(WebApiProxy));
            API.Metadata = Metadata;
            var Support = (function (_super) {
                __extends(Support, _super);
                function Support(host) {
                    _super.call(this, host, 'Support');
                }
                Support.prototype.ContactUs = function (fullName, email, subject, message, callback) {
                    var data = {
                        FullName: fullName,
                        Email: email,
                        Subject: subject,
                        Body: message
                    };
                    return _super.prototype.Call.call(this, 'ContactUs', data, callback);
                };
                return Support;
            }(WebApiProxy));
            API.Support = Support;
        })(API = Commanding.API || (Commanding.API = {}));
    })(Commanding = OnlinePoker.Commanding || (OnlinePoker.Commanding = {}));
})(OnlinePoker || (OnlinePoker = {}));
var PageBlock = (function () {
    function PageBlock(name, primaryName, primaryModel) {
        this.name = name;
        this.primaryName = primaryName;
        this.requireAuthentication = false;
        this.requireGuestAuthentication = false;
        this.loadPromises = [];
        this.secondaryViews = [];
        this.bindSubPage(primaryName, primaryModel);
    }
    PageBlock.prototype.addSecondary = function (name, viewModel, isDefault) {
        if (isDefault === void 0) { isDefault = false; }
        if (isDefault) {
            this.defaultSecondaryName = name;
        }
        this.secondaryViews.push(name);
        this.bindSubPage(name, viewModel);
    };
    PageBlock.prototype.showBlock = function () {
        if (typeof window !== "undefined") {
            $(this.getSelector()).show();
        }
        this.showPrimary();
        if (PageBlock.useDoubleView) {
            this.showSecondary(this.defaultSecondaryName);
        }
    };
    PageBlock.prototype.hideBlock = function () {
        if (typeof window !== "undefined") {
            $(this.getSelector()).hide();
        }
    };
    PageBlock.prototype.showPrimary = function () {
        this.showSubPage(this.primaryName);
    };
    PageBlock.prototype.showSecondary = function (name) {
        this.showSubPage(name);
    };
    PageBlock.prototype.isSecondary = function (pageName) {
        return this.secondaryViews.indexOf(pageName) !== -1;
    };
    PageBlock.prototype.isPrimary = function (pageName) {
        return pageName === this.primaryName;
    };
    PageBlock.prototype.isSameBlock = function (pageName) {
        return this.isPrimary(pageName) && this.isSecondary(pageName);
    };
    /**
    * Gets page by it's name
    * @param subPageName String Name of the sub page to get.
    */
    PageBlock.prototype.getSubPage = function (subPageName) {
        if (!this.hasOwnProperty(subPageName + "Page")) {
            return null;
        }
        return this[subPageName + "Page"];
    };
    PageBlock.prototype.showSubPage = function (pageName) {
        var nextPageObject = this.getSubPage(pageName);
        if (nextPageObject !== null) {
            if (nextPageObject.canActivate !== undefined) {
                if (!nextPageObject.canActivate()) {
                    return;
                }
            }
        }
        var needHidePrevView = this.currentPage != null;
        if (PageBlock.useDoubleView) {
            // Add additional logic when previous view should be hidden.
            // Never hide primary view.
            needHidePrevView = needHidePrevView && !this.isPrimary(this.currentPage);
        }
        if (needHidePrevView) {
            var pageObject = this.getSubPage(this.currentPage);
            if (pageObject !== null) {
                pageObject.deactivate(this.currentPage);
            }
            if (typeof window !== "undefined") {
                $(this.getPageSelector(this.currentPage)).css("display", "none");
            }
        }
        this.currentPage = pageName;
        if (nextPageObject !== null) {
            nextPageObject.activate(this.currentPage);
        }
        if (typeof window !== "undefined") {
            $(this.getPageSelector(pageName)).css("display", "block");
        }
    };
    PageBlock.prototype.getSelector = function () {
        return ".page-block." + this.name;
    };
    PageBlock.prototype.getPageSelector = function (pageName) {
        return this.getSelector() + " .sub-page." + pageName;
    };
    PageBlock.prototype.bindSubPage = function (pageName, viewModel) {
        this[pageName + "Page"] = viewModel;
        if (typeof window === "undefined") {
            return;
        }
        var pagejElement = $(this.getPageSelector(pageName));
        if (pagejElement.length === 0) {
            console.error("Could not bind sub page " + pageName + " since DOM element not found.");
            return;
        }
        if (pagejElement.length > 1) {
            console.warn("Page " + pageName + " has more then one element to bind.");
            return;
        }
        var pageElement = pagejElement[0];
        if (!pageElement.hasChildNodes()) {
            var templateSource = pagejElement.data("template");
            var pageLoadPromise = $.get(templateSource, "text/html").then(function (data) {
                pagejElement.html(data);
                try {
                    ko.applyBindings(viewModel, pageElement);
                }
                catch (e) {
                    console.log("Bind page " + pageName + " failed");
                }
            });
            this.loadPromises.push(pageLoadPromise);
            return;
        }
        ko.applyBindings(viewModel, pageElement);
    };
    PageBlock.useDoubleView = false;
    return PageBlock;
}());
/// <reference types="jquery" />
/// <reference path="../pageblock.ts" />
define("services/uimanager", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_2) {
    "use strict";
    var UIManager = (function () {
        function UIManager() {
            this.subPageHiding = new signals.Signal();
            this.subPageShowing = new signals.Signal();
            this.pageBlockHiding = new signals.Signal();
            this.pageBlockHidden = new signals.Signal();
            this.pageBlockShowing = new signals.Signal();
            this.pageBlockShown = new signals.Signal();
            this.popupClosed = new signals.Signal();
            this.currentPageContainer = null;
            this.effectivePageContainer = null;
            this.currentPage = null;
            this.currentPageBlock = null;
        }
        UIManager.prototype.showPageBlock = function (pageBlockName) {
            if (pageBlockName === this.currentPageBlock) {
                this.pageBlockHiding.dispatch(this.currentPageBlock);
                this.pageBlockHidden.dispatch(this.currentPageBlock);
                this.pageBlockShowing.dispatch(this.currentPageBlock);
                this.pageBlockShown.dispatch(this.currentPageBlock);
                return;
            }
            var nextPageBlockObject = this.getPageBlock(pageBlockName);
            if (this.currentPageBlock !== null) {
                this.pageBlockHiding.dispatch(this.currentPageBlock);
                var currentPageBlockObject = this.getPageBlock(this.currentPageBlock);
                if (currentPageBlockObject !== null) {
                    currentPageBlockObject.hideBlock();
                }
                else {
                    $(".page .page-block." + this.currentPageBlock).css("display", "none");
                }
            }
            this.pageBlockHidden.dispatch(this.currentPageBlock);
            this.currentPage = null;
            this.currentPageBlock = pageBlockName;
            this.pageBlockShowing.dispatch(this.currentPageBlock);
            if (nextPageBlockObject !== null) {
                nextPageBlockObject.showBlock();
            }
            else {
                $(".page .page-block." + pageBlockName).css("display", "block");
            }
            this.pageBlockShown.dispatch(this.currentPageBlock);
        };
        UIManager.prototype.showPage = function (pageName) {
            this.logPage("Showing page '" + pageName + "'");
            if (typeof window !== "undefined") {
                if (this.currentPageContainer !== null) {
                    $(".page." + this.currentPageContainer).css("display", "none");
                }
            }
            this.currentPageContainer = pageName;
            if (pageName !== "initialization") {
                this.effectivePageContainer = pageName;
            }
            if (typeof window !== "undefined") {
                $(".page." + pageName).css("display", "block");
                $(".page .sub-page.selector").css("display", "none");
            }
        };
        UIManager.prototype.showSubPage = function (pageName) {
            var nextPageObject = this.getSubPage(pageName);
            if (nextPageObject !== null) {
                if (nextPageObject.canActivate !== undefined) {
                    if (!nextPageObject.canActivate()) {
                        return;
                    }
                }
            }
            if (this.currentPage != null) {
                this.deactivateSubPage(this.currentPage);
            }
            this.currentPage = pageName;
            this.activateSubPage(pageName);
            if (typeof window !== "undefined") {
                $(".page .sub-page.selector").css("display", "none");
            }
        };
        UIManager.prototype.deactivateSubPage = function (pageName) {
            var pageObject = this.getSubPage(pageName);
            if (pageObject != null) {
                pageObject.deactivate();
            }
            this.subPageHiding.dispatch(pageName);
            $(".page .sub-page." + pageName).css("display", "none");
        };
        UIManager.prototype.activateSubPage = function (pageName) {
            var nextPageObject = this.getSubPage(pageName);
            if (nextPageObject != null) {
                nextPageObject.activate();
            }
            this.subPageShowing.dispatch(pageName);
            $(".page .sub-page." + pageName).css("display", "block");
        };
        /**
        * Gets page block by it's name
        * @param pageBlockName String Name of the page block to get.
        */
        UIManager.prototype.getPageBlock = function (pageBlockName) {
            if (!app.hasOwnProperty(pageBlockName + "PageBlock")) {
                return null;
            }
            return app[pageBlockName + "PageBlock"];
        };
        /**
        * Gets page by it's name
        * @param subPageName String Name of the sub page to get.
        */
        UIManager.prototype.getSubPage = function (subPageName) {
            if (!app.hasOwnProperty(subPageName + "Page")) {
                return null;
            }
            return app[subPageName + "Page"];
        };
        UIManager.getTabBarItemForPage = function (pageName) {
            if (UIManager.pageMappings[pageName]) {
                return UIManager.pageMappings[pageName];
            }
            return pageName;
        };
        UIManager.addTabBarItemMapping = function (tabBarItem, pageName) {
            UIManager.pageMappings[pageName] = tabBarItem;
        };
        UIManager.prototype.logPage = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_2.debugSettings.ui.tracePages) {
                console.log(message, params);
            }
        };
        UIManager.pageMappings = [];
        return UIManager;
    }());
    exports.UIManager = UIManager;
    exports.uiManager = new UIManager();
});
/// <reference types="jquery" />
/// <reference path="../typings/cordova.d.ts" />
/// <reference path="../app.ts" />
/* tslint:disable:no-string-literal */
define("services/keyboardactivationservice", ["require", "exports", "timeService", "services/uimanager"], function (require, exports, timeService, uimanager_1) {
    "use strict";
    var KeyboardActivationService = (function () {
        function KeyboardActivationService() {
            this.lastPage = null;
            this.lastPageBlock = null;
            this.lastPopup = null;
        }
        KeyboardActivationService.prototype.setup = function () {
            var _this = this;
            if (window["Keyboard"]) {
                var supported_1 = true;
                if (window["device"]) {
                    if (device.platform === "iOS") {
                        var iOS70 = device.version.indexOf("7.0") === 0;
                        var iOS6X = device.version.indexOf("6") === 0;
                        if (!iOS70 && !iOS6X) {
                            // apply only to the iOS 6.x and 7.0
                            supported_1 = false;
                        }
                        supported_1 = false;
                    }
                }
                if (!supported_1) {
                    console.log("Native keyboard adjust is available. Don't use our method for adjust page position");
                }
                else {
                    console.log("Native keyboard adjust is not available. Do our best to provide better UI.");
                }
                Keyboard.onshow = function () {
                    if (supported_1) {
                        _this.applyStyles();
                    }
                };
                Keyboard.onhiding = function () {
                    document.body.scrollLeft = 0;
                };
                Keyboard.onhide = function () {
                    timeService.setTimeout(function () {
                        _this.forceHideKeyboard();
                    }, 1);
                };
            }
            else {
                var enableKeyboardEmulationOnDesktop = false;
                if (enableKeyboardEmulationOnDesktop) {
                    var selector = "input[type='text'], input[type='password'], input[type='date'], input[type='number']";
                    $("body").on("focus", selector, function (event) {
                        _this.applyStyles();
                    });
                    $("body").on("blur", selector, function (event) {
                        _this.removeStyles();
                    });
                }
            }
        };
        KeyboardActivationService.prototype.applyStyles = function () {
            if (app.currentPopup === null) {
                console.log("Apply kbd styles to current page");
                $("." + uimanager_1.uiManager.currentPage + ".sub-page").addClass(KeyboardActivationService.keyboardShowModifier);
                $("." + uimanager_1.uiManager.currentPageBlock + ".page-block").addClass(KeyboardActivationService.keyboardShowModifier);
                this.lastPage = uimanager_1.uiManager.currentPage;
                this.lastPageBlock = uimanager_1.uiManager.currentPageBlock;
            }
            else {
                console.log("Apply kbd styles to popup " + app.currentPopup);
                $("." + app.currentPopup + ".popup").addClass(KeyboardActivationService.keyboardShowModifier);
                this.lastPopup = app.currentPopup;
            }
        };
        KeyboardActivationService.prototype.removeStyles = function () {
            if (this.lastPage !== null) {
                $("." + this.lastPage + ".sub-page").removeClass(KeyboardActivationService.keyboardShowModifier);
                this.lastPage = null;
            }
            if (this.lastPageBlock !== null) {
                $("." + this.lastPageBlock + ".page-block").removeClass(KeyboardActivationService.keyboardShowModifier);
            }
            if (this.lastPopup !== null) {
                $("." + this.lastPopup + ".popup").removeClass(KeyboardActivationService.keyboardShowModifier);
            }
        };
        KeyboardActivationService.prototype.forceHideKeyboard = function () {
            var activeElement = document.activeElement;
            if (activeElement !== null) {
                activeElement.blur();
            }
            this.removeStyles();
        };
        KeyboardActivationService.keyboardShowModifier = "keyboard-shown";
        return KeyboardActivationService;
    }());
    exports.KeyboardActivationService = KeyboardActivationService;
});
define("services/slowinternetservice", ["require", "exports", "services/index", "debugsettings"], function (require, exports, index_1, debugsettings_3) {
    "use strict";
    var SlowInternetService = (function () {
        function SlowInternetService() {
            this.fatalError = false;
            this.offline = false;
            this.manualDisconnect = false;
            this.suppressReconnected = false;
        }
        SlowInternetService.prototype.initialize = function () {
            var _this = this;
            index_1.connectionService.reconnecting.add(function () { return _this.onConnectionSlow(); });
            index_1.connectionService.reconnected.add(function () { return _this.onReconnected(); });
            index_1.connectionService.received.add(function () { return _this.onReceived(); });
            index_1.connectionService.disconnected.add(function () { return _this.onDisconnected(); });
            document.addEventListener("online", function () { return _this.onOnline(); }, false);
            document.addEventListener("offline", function () { return _this.onOffline(); }, false);
            app.slowConnectionPopup.onretry = function () {
                _this.log("Retry connection");
                _this.suppressReconnected = false;
                _this.executeRetryHandler();
                var connectionBuilder = index_1.connectionService.buildStartConnection();
                if (connectionBuilder === null) {
                    setTimeout(function () {
                        _this.onDisconnected();
                    }, 3000);
                }
                else {
                    connectionBuilder().pipe(function () {
                        _this.onReconnected();
                    }, function () {
                        _this.onDisconnected();
                    });
                }
            };
            this.offline = !this.hasInternet();
            if (this.offline) {
                this.onOffline();
            }
        };
        SlowInternetService.prototype.hasInternet = function () {
            if (navigator.connection === null || navigator.connection === undefined) {
                return true;
            }
            var expectedConnection = "none";
            if (window["Connection"]) {
                expectedConnection = Connection.NONE;
            }
            this.log("Connection type is " + navigator.connection.type + " testing against " + expectedConnection);
            return navigator.connection.type !== expectedConnection;
        };
        SlowInternetService.prototype.setRetryHandler = function (handler) {
            this.retyHandler = handler;
        };
        SlowInternetService.prototype.executeRetryHandler = function () {
            if (this.retyHandler !== null) {
                this.log("Executing retry handler");
                this.retyHandler();
                this.retyHandler = null;
            }
        };
        SlowInternetService.prototype.onConnectionSlow = function () {
            if (this.fatalError) {
                return;
            }
            this.log("Detected slow connection.");
            // Show popup only first time.
            // If we receive subsequent events then keep popup open and do nothing.
            if (app.currentPopup !== SlowInternetService.popupName) {
                app.showPopup(SlowInternetService.popupName);
            }
        };
        SlowInternetService.prototype.onReceived = function () {
            if (this.fatalError) {
                return;
            }
            if (app.currentPopup === SlowInternetService.popupName) {
                app.slowConnectionPopup.close();
            }
        };
        SlowInternetService.prototype.onReconnected = function () {
            if (this.fatalError) {
                return;
            }
            if (this.suppressReconnected) {
                return;
            }
            this.log("Close information popup, because connection is reestablished");
            this.log("Please add code which reconnects to the all opened tables here.");
            this.closePopup();
            this.executeRetryHandler();
        };
        SlowInternetService.prototype.closePopup = function () {
            if (app.currentPopup === SlowInternetService.popupName) {
                app.slowConnectionPopup.close();
            }
        };
        SlowInternetService.prototype.onDisconnected = function () {
            if (this.fatalError) {
                return;
            }
            this.log("disconnected");
            if (!this.offline && !this.manualDisconnect) {
                this.showReconnectFailedPopup();
            }
        };
        SlowInternetService.prototype.showReconnectFailedPopup = function () {
            if (app.currentPopup !== SlowInternetService.popupName) {
                app.showPopup(SlowInternetService.popupName);
            }
            app.slowConnectionPopup.reconnectFailed();
        };
        SlowInternetService.prototype.showDuplicatedConnectionPopup = function () {
            this.log("Duplicate connection detected");
            this.fatalError = true;
            if (app.currentPopup !== SlowInternetService.popupName) {
                app.showPopup(SlowInternetService.popupName);
            }
            this.setRetryHandler(function () {
                app.reloadApplication();
            });
            app.slowConnectionPopup.duplicatedConnection();
        };
        SlowInternetService.prototype.onOnline = function () {
            if (this.fatalError) {
                return;
            }
            this.offline = false;
            this.log("online");
            if (app.currentPopup === SlowInternetService.popupName) {
                app.slowConnectionPopup.connectionPresent();
            }
        };
        SlowInternetService.prototype.onOffline = function () {
            if (this.fatalError) {
                return;
            }
            this.offline = true;
            this.suppressReconnected = true;
            this.log("offline");
            if (app.currentPopup !== SlowInternetService.popupName) {
                app.showPopup(SlowInternetService.popupName);
            }
            app.slowConnectionPopup.noConnection();
        };
        SlowInternetService.prototype.log = function (message) {
            if (debugsettings_3.debugSettings.connection.slowInternet) {
                console.log(message);
            }
        };
        SlowInternetService.popupName = "slowConnection";
        return SlowInternetService;
    }());
    exports.SlowInternetService = SlowInternetService;
});
define("services/connectionwrapper", ["require", "exports", "timeService", "services/connectionservice", "services/index", "debugsettings"], function (require, exports, timeService, connectionservice_1, index_2, debugsettings_4) {
    "use strict";
    var ConnectionWrapper = (function () {
        function ConnectionWrapper(connection) {
            var _this = this;
            this.connection = connection;
            this.terminated = false;
            this.refreshHandle = null;
            connection.connectionSlow(function () {
                if (_this.terminated) {
                    return;
                }
                _this.logEvent("Connection slow");
                index_2.connectionService.connectionSlow.dispatch();
            });
            connection.reconnecting(function () {
                if (_this.terminated) {
                    return;
                }
                index_2.connectionService.reconnecting.dispatch();
            });
            connection.reconnected(function () {
                if (_this.terminated) {
                    return;
                }
                index_2.connectionService.reconnected.dispatch();
            });
            connection.received(function () {
                if (_this.terminated) {
                    return;
                }
                index_2.connectionService.received.dispatch();
            });
            connection.disconnected(function () {
                if (_this.terminated) {
                    return;
                }
                index_2.connectionService.disconnected.dispatch();
            });
            connection.stateChanged(function (state) {
                if (_this.terminated) {
                    return;
                }
                _this.onConnectionStateChanged(state);
            });
            connection.error(function (error) {
                if (_this.terminated) {
                    return;
                }
                var source = error.source;
                _this.logEvent("SignalR Error hapens", error);
                if (source === null) {
                    // We don't know that this is means, so just fail
                    // so issue will be easiely identifiable.
                    _this.logEvent("Unrecoverable SignalR error happens, please discover that this is means.");
                    return;
                }
                /* tslint:disable:no-string-literal */
                if (source["code"] === null || source["code"] === undefined) {
                    _this.logEvent("Unrecoverable SignalR error without code happens, please discover that this is means.", source);
                    return;
                }
                /* tslint:enable:no-string-literal */
                _this.logEvent("Error code is: ", source.code);
                if (source.code === 1006) {
                    // Attempt to reestablish connection.
                    _this.logEvent("Attempt to reestablish connection due to recoverable error.");
                    index_2.connectionService.recoverableError.dispatch();
                    return;
                }
                console.warn(error);
            });
        }
        ConnectionWrapper.prototype.terminateConnection = function (forceDisconnect) {
            if (forceDisconnect === void 0) { forceDisconnect = false; }
            var hubId = this.connection.id;
            var connectionInfo = "HID:" + hubId;
            this.logEvent("Terminating connection " + connectionInfo);
            index_2.slowInternetService.manualDisconnect = true;
            this.connection.stop(false, false);
            this.cancelRefereshConnection();
            this.terminated = true;
        };
        ConnectionWrapper.prototype.establishConnection = function (maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var attempts = index_2.connectionService.attempts++;
            index_2.connectionService.lastAttempt = attempts;
            var result = this.establishConnectionCore(maxAttempts);
            return result;
        };
        ConnectionWrapper.prototype.buildStartConnection = function () {
            var _this = this;
            var supportedTransports = null;
            var androidVersion = this.getAndroidVersion();
            if (androidVersion === false || androidVersion.indexOf("4.4") === 0) {
                supportedTransports = ["webSockets"];
            }
            var startConnection = function () {
                var buildMainPromise = function () {
                    var promise;
                    if (supportedTransports === null) {
                        promise = _this.connection.start();
                    }
                    else {
                        promise = _this.connection.start({ transport: supportedTransports });
                    }
                    return promise;
                };
                var fixup = $.Deferred();
                var x = function (attempts) {
                    if (attempts === 0) {
                        fixup.reject();
                    }
                    var promise = buildMainPromise();
                    promise.pipe(function () {
                        if (_this.terminated) {
                            fixup.reject();
                            return;
                        }
                        if (_this.connection.state === 1) {
                            fixup.resolve();
                            return;
                        }
                        if (_this.connection.state === 4) {
                            fixup.reject();
                            return;
                        }
                        timeService.setTimeout(function () {
                            x(attempts - 1);
                        }, 100);
                    }, function () {
                        fixup.reject();
                    });
                };
                x(30);
                return fixup;
            };
            return startConnection;
        };
        ConnectionWrapper.prototype.onConnectionStateChanged = function (state) {
            this.logEvent("SignalR state changed from: " + connectionservice_1.ConnectionService.stateConversion[state.oldState]
                + " to: " + connectionservice_1.ConnectionService.stateConversion[state.newState]);
            if (state.newState === 4) {
                index_2.connectionService.isDisconnected = true;
            }
            else {
                index_2.connectionService.isDisconnected = false;
            }
            if (state.newState === 1) {
                index_2.connectionService.isConnected = true;
            }
            else {
                index_2.connectionService.isConnected = false;
            }
        };
        ConnectionWrapper.prototype.establishConnectionCore = function (maxAttempts) {
            var _this = this;
            var result = $.Deferred();
            if (maxAttempts <= 0) {
                this.logEvent("Stop connection attempts");
                index_2.slowInternetService.onDisconnected();
                result.reject("maxAttemptsReached");
                return result;
            }
            var startConnection = this.buildStartConnection();
            this.logEvent("Establishing connection. Attempts left: " + (maxAttempts - 1));
            index_2.connectionService.isStopped = false;
            startConnection().done(function () {
                if (_this.terminated) {
                    result.reject();
                }
                var hubId = _this.connection.id;
                var connectionInfo = "HID:" + hubId;
                _this.logEvent("Connected to server! Connection " + connectionInfo);
                _this.refreshHandle = setTimeout(function () {
                    // this.terminateConnection();
                    // connectionService.recoverableError.dispatch();
                }, 1000 * 60 * 60);
                result.resolve(_this);
            }).fail(function (message) {
                _this.logEvent("Could not Connect!" + message);
                timeService.setTimeout(function () {
                    _this.establishConnection(maxAttempts - 1).then(function () {
                        result.resolve(_this);
                    }, function () {
                        result.reject();
                    });
                }, 100);
            });
            return result;
        };
        ConnectionWrapper.prototype.cancelRefereshConnection = function () {
            if (this.refreshHandle) {
                clearTimeout(this.refreshHandle);
            }
            this.refreshHandle = null;
        };
        ConnectionWrapper.prototype.getAndroidVersion = function (userAgent) {
            if (userAgent === void 0) { userAgent = null; }
            var ua = userAgent || navigator.userAgent;
            var match = ua.match(/Android\s([0-9\.]*)/);
            return match ? match[1] : false;
        };
        ConnectionWrapper.prototype.logEvent = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_4.debugSettings.connection.signalR) {
                console.log(message, params);
            }
        };
        return ConnectionWrapper;
    }());
    exports.ConnectionWrapper = ConnectionWrapper;
});
define("services/connectionservice", ["require", "exports", "services/connectionwrapper", "services/index", "debugsettings"], function (require, exports, connectionwrapper_1, index_3, debugsettings_5) {
    "use strict";
    var ConnectionService = (function () {
        function ConnectionService() {
            this.recoverableError = new signals.Signal();
            this.newConnection = new signals.Signal();
            this.terminatedConnection = new signals.Signal();
            this.attempts = 0;
            this.lastAttempt = 0;
            this.lastConnection = null;
            this.currentConnection = null;
            this.connectionSlow = new signals.Signal();
            this.reconnecting = new signals.Signal();
            this.reconnected = new signals.Signal();
            this.received = new signals.Signal();
            this.disconnected = new signals.Signal();
        }
        ConnectionService.prototype.initializeConnection = function () {
            if (this.currentConnection !== null) {
                return;
            }
            var connection = $.hubConnection(baseUrl);
            connection.logging = $.connection.hub.logging;
            if (authToken == null) {
                connection.qs = null;
            }
            else {
                connection.qs = { "token": authToken };
            }
            this.isDisconnected = true;
            this.currentConnection = new connectionwrapper_1.ConnectionWrapper(connection);
            $.extend(connection, connection.createHubProxies());
            this.newConnection.dispatch(this.currentConnection);
        };
        ConnectionService.prototype.terminateConnection = function (forceDisconnect) {
            if (forceDisconnect === void 0) { forceDisconnect = false; }
            this.isStopped = true;
            this.isDisconnected = true;
            index_3.slowInternetService.manualDisconnect = true;
            if (this.currentConnection == null) {
                this.logEvent("No active connection to terminate");
                return;
            }
            var hubId;
            if (this.currentConnection.connection == null) {
                hubId = this.currentConnection.connection.id;
            }
            else {
                hubId = "NULL";
            }
            var connectionInfo = "HID:" + hubId;
            this.logEvent("Terminating connection " + connectionInfo);
            var oldConnection = this.currentConnection;
            this.currentConnection.terminateConnection();
            this.currentConnection = null;
            this.terminatedConnection.dispatch(oldConnection);
        };
        ConnectionService.prototype.establishConnection = function (maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var attempts = this.attempts++;
            this.lastAttempt = attempts;
            this.cancelConnection();
            var result = this.currentConnection.establishConnection(maxAttempts);
            this.lastConnection = result;
            return result;
        };
        ConnectionService.prototype.cancelConnection = function () {
            if (this.lastConnection !== null && this.lastConnection.state() === "pending") {
                this.lastConnection.notify("cancel");
                this.lastConnection = null;
            }
        };
        ConnectionService.prototype.buildStartConnection = function () {
            if (this.currentConnection === null) {
                this.logEvent("No active connection to terminate");
                return null;
            }
            return this.currentConnection.buildStartConnection();
        };
        ConnectionService.prototype.logEvent = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_5.debugSettings.connection.signalR) {
                console.log(message, params);
            }
        };
        ConnectionService.stateConversion = {
            0: "connecting",
            1: "connected",
            2: "reconnecting",
            4: "disconnected"
        };
        return ConnectionService;
    }());
    exports.ConnectionService = ConnectionService;
});
define("settings", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var Settings = (function () {
        function Settings() {
            this.authToken = ko.observable();
            this.lastPage = ko.observable();
            this.lastTime = ko.observable(0);
            this.isGuest = ko.observable(false);
            this.login = ko.observable();
            this.password = ko.observable();
            this.autoSwitchTables = ko.observable(false);
            this.autoHideCards = ko.observable(true);
            this.soundEnabled = ko.observable(true);
            this.lastBannerId = ko.observable();
        }
        Settings.prototype.loadSettings = function () {
            this.login(this.getItemString("auth.login"));
            this.password(this.getItemString("auth.password"));
            this.isGuest(this.getItemBoolean("auth.isGuest", false));
            this.autoSwitchTables(this.getItemBoolean("table.autoSwitchTables"));
            this.autoHideCards(this.getItemBoolean("table.autoHideCards", true));
            this.soundEnabled(this.getItemBoolean("table.soundEnabled", true));
            this.lastBannerId(this.getItemNumber("banner.lastBannerId", 0));
            this.lastTime(this.getItemNumber("reload.lastTime", 0));
            this.lastPage(this.getItemString("reload.lastPage", "main"));
            this.authToken(this.getItemString("reload.authToken", null));
        };
        Settings.prototype.saveSettings = function () {
            this.setItemString("auth.login", this.login());
            this.setItemString("auth.password", this.password());
            this.setItemBoolean("auth.isGuest", this.isGuest(), false);
            this.setItemBoolean("table.autoSwitchTables", this.autoSwitchTables());
            this.setItemBoolean("table.autoHideCards", this.autoHideCards(), true);
            this.setItemBoolean("table.soundEnabled", this.soundEnabled(), true);
            this.setItemNumber("banner.lastBannerId", this.lastBannerId(), 0);
            this.setItemString("reload.lastPage", this.lastPage(), "main");
            this.setItemString("reload.authToken", this.authToken(), null);
            this.setItemNumber("reload.lastTime", this.lastTime(), 0);
        };
        Settings.prototype.getItemBoolean = function (name, defaultValue) {
            if (defaultValue === void 0) { defaultValue = false; }
            var item = localStorage.getItem(name);
            if (item === null) {
                return defaultValue;
            }
            return item.toLowerCase() === "true";
        };
        Settings.prototype.setItemBoolean = function (name, value, defaultValue) {
            if (defaultValue === void 0) { defaultValue = false; }
            if (value === defaultValue) {
                localStorage.removeItem(name);
            }
            else {
                localStorage.setItem(name, value ? "true" : "false");
            }
        };
        Settings.prototype.getItemString = function (name, defaultValue) {
            if (defaultValue === void 0) { defaultValue = null; }
            return localStorage.getItem(name) || defaultValue;
        };
        Settings.prototype.setItemString = function (name, value, defaultValue) {
            if (defaultValue === void 0) { defaultValue = null; }
            if (value === defaultValue) {
                localStorage.removeItem(name);
            }
            else {
                localStorage.setItem(name, value);
            }
        };
        Settings.prototype.getItemNumber = function (name, defaultValue) {
            if (defaultValue === void 0) { defaultValue = null; }
            return Number(localStorage.getItem(name)) || defaultValue;
        };
        Settings.prototype.setItemNumber = function (name, value, defaultValue) {
            if (defaultValue === void 0) { defaultValue = null; }
            if (value === defaultValue) {
                localStorage.removeItem(name);
            }
            else {
                localStorage.setItem(name, value === null ? null : value.toString());
            }
        };
        return Settings;
    }());
    exports.settings = new Settings();
});
/// <reference path="poker.commanding.api.ts" />
define("authmanager", ["require", "exports", "knockout", "settings"], function (require, exports, ko, settings_1) {
    "use strict";
    var AuthManager = (function () {
        function AuthManager() {
            this.authenticated = ko.observable(false);
            this.login = ko.observable();
            this.loginId = ko.observable();
        }
        AuthManager.prototype.authenticate = function (login, password, rememberMe) {
            if (rememberMe === void 0) { rememberMe = false; }
            var self = this;
            var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
            var result = $.Deferred();
            if (rememberMe) {
                settings_1.settings.login(login);
                settings_1.settings.password(password);
                settings_1.settings.saveSettings();
            }
            accountApi.Authenticate(login, password, false, function (data) {
                if (data.Status === "Ok") {
                    self.authenticated(true);
                    self.login(data.Login);
                    self.loginId(data.Id);
                    settings_1.settings.isGuest(data.IsGuest);
                    settings_1.settings.saveSettings();
                }
                else {
                    // Report authentication or authorization errors
                    self.authenticated(false);
                    self.login(null);
                    self.loginId(null);
                }
                result.resolve(data.Status);
            }).fail(function () {
                result.resolve(false);
            });
            return result;
        };
        AuthManager.prototype.logout = function () {
            settings_1.settings.login(null);
            settings_1.settings.password(null);
            settings_1.settings.saveSettings();
            authToken = null;
            this.authenticated(false);
            this.login(null);
        };
        AuthManager.prototype.loginAsGuest = function () {
            var _this = this;
            var result = $.Deferred();
            var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
            accountApi.RegisterGuest().then(function (value) {
                if (!value) {
                    result.resolve(false);
                }
                else {
                    settings_1.settings.login(value.Login);
                    settings_1.settings.password(value.Password);
                    settings_1.settings.saveSettings();
                    app.processing(false);
                    if (value.Status === "Ok") {
                        _this.authenticate(value.Login, value.Password, true).then(function (value) {
                            result.resolve(value);
                        });
                    }
                    else {
                        result.resolve(value.Status);
                    }
                }
            }, function (error) {
                result.resolve(false);
            });
            return result;
        };
        return AuthManager;
    }());
    var authManager = new AuthManager();
    return authManager;
});
/// <reference path="services.d.ts" />
/// <reference path="../poker.commanding.api.ts" />
define("services/accountservice", ["require", "exports", "authmanager"], function (require, exports, authManager) {
    "use strict";
    var AccountService = (function () {
        function AccountService() {
        }
        AccountService.prototype.getAccount = function () {
            var result = $.Deferred();
            var realMoneySupported = true;
            var gameMoneySupported = false;
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            api.GetPersonalAccount(null).done(function (apiResult) {
                var data = apiResult.Data;
                var accountsData = [];
                if (realMoneySupported) {
                    accountsData.push({
                        currencyName: "currency.realmoney",
                        available: data.RealMoney,
                        ingame: data.RealMoneyReserve == null ? 0 : data.RealMoneyReserve,
                        total: Number(data.RealMoney) + Number(data.RealMoneyReserve == null ? 0 : data.RealMoneyReserve)
                    });
                }
                if (gameMoneySupported) {
                    accountsData.push({
                        currencyName: "currency.gamemoney",
                        available: data.GameMoney,
                        ingame: data.GameMoneyReserve == null ? 0 : data.GameMoneyReserve,
                        total: Number(data.GameMoney) + Number(data.GameMoneyReserve == null ? 0 : data.GameMoneyReserve)
                    });
                }
                var transactionInfo = {
                    date: data.LastIncomeDate,
                    amount: data.LastIncomeAmount,
                    id: data.LastRequestNumber
                };
                result.resolve({
                    login: authManager.login(),
                    accounts: accountsData,
                    lastTransaction: transactionInfo
                });
            }).fail(function () {
                result.reject();
            });
            return result;
        };
        return AccountService;
    }());
    exports.AccountService = AccountService;
});
define("services/imagepreloadservice", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_6) {
    "use strict";
    var ImagePreloadService = (function () {
        function ImagePreloadService() {
        }
        ImagePreloadService.prototype.preload = function (url, useAbsolute) {
            var _this = this;
            if (useAbsolute === void 0) { useAbsolute = true; }
            var absoluteUrl = useAbsolute ? this.getAbsoluteUrl(url) : url;
            var result = $.Deferred();
            var image = new Image();
            image.onload = function () {
                _this.log("URL " + url + " preloaded");
                result.resolve(url);
            };
            image.onerror = function () {
                _this.log("URL " + url + " failed to load");
                result.reject(url);
            };
            image.src = absoluteUrl;
            return result;
        };
        /**
        * Preload resource based on the short name of the resource.
        * @param resourcePackName String Resource name to load.
        */
        ImagePreloadService.prototype.preloadResource = function (resourcePackName) {
            var dpr = window.devicePixelRatio;
            if (dpr === 2) {
                return this.preload(resourcePackName + "@2x.png", false);
            }
            if (dpr === 3) {
                return this.preload(resourcePackName + "@3x.png", false);
            }
            return this.preload(resourcePackName + ".png", false);
        };
        /**
        * Preload device specific resource based on naming convention in the project.
        * @param resourcePackName String Resource name to load.
        */
        ImagePreloadService.prototype.preloadDeviceSpecificResource = function (resourcePackName) {
            var dpr = window.devicePixelRatio;
            if (screen.width === 1024 || screen.height === 1024) {
                return this.preloadResource(resourcePackName + "-1024");
            }
            if (screen.width === 360 || screen.height === 360) {
                return this.preloadResource(resourcePackName + "-360");
            }
            return this.preloadResource(resourcePackName);
        };
        /**
        * Converts passed url to the absolute url.
        * @param url String URL which should be converted to absolute url.
        */
        ImagePreloadService.prototype.getAbsoluteUrl = function (url) {
            if (url.indexOf("http://") === 0
                || url.indexOf("https://") === 0) {
                return url;
            }
            var absoluteUrl = window.location.href.replace("index.html", url);
            this.log("Convert url " + url + " => " + absoluteUrl);
            return absoluteUrl;
        };
        /**
        * Trace execution messages inside this class.
        */
        ImagePreloadService.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_6.debugSettings.initialization.imagePreloading) {
                console.log(message, params);
            }
        };
        return ImagePreloadService;
    }());
    exports.ImagePreloadService = ImagePreloadService;
});
define("services/reloadmanager", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_7) {
    "use strict";
    var ReloadManager = (function () {
        function ReloadManager() {
        }
        ReloadManager.prototype.setReloadCallback = function (callback) {
            this.callback = callback;
        };
        ReloadManager.prototype.execute = function () {
            this.log("Executing reload callback");
            if (this.callback != null) {
                this.callback();
            }
        };
        ReloadManager.prototype.log = function (message) {
            if (debugsettings_7.debugSettings.application.reloadManager) {
                console.log(message);
            }
        };
        return ReloadManager;
    }());
    exports.ReloadManager = ReloadManager;
});
define("services/deviceeventservice", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_8) {
    "use strict";
    /**
    * Service which provides ability to subscribe on the
    * device events.
    */
    var DeviceEventService = (function () {
        function DeviceEventService() {
            this.ready = new signals.Signal();
            this.active = new signals.Signal();
            this.resignActive = new signals.Signal();
            this.pause = new signals.Signal();
            this.resume = new signals.Signal();
        }
        /**
        * Initializes the device event handler
        */
        DeviceEventService.prototype.initialize = function () {
            var _this = this;
            document.addEventListener("deviceready", function () { return _this.onDeviceReady(); }, false);
            if (debugsettings_8.debugSettings.ios.hasMultitasking) {
                document.addEventListener("resign", function () { return _this.onResign(); }, false);
                document.addEventListener("active", function () { return _this.onActive(); }, false);
            }
            document.addEventListener("pause", function () { return _this.onPause(); }, false);
            document.addEventListener("resume", function () { return _this.onResume(); }, false);
        };
        DeviceEventService.prototype.onDeviceReady = function () {
            this.ready.dispatch();
        };
        DeviceEventService.prototype.onResign = function () {
            this.resignActive.dispatch();
        };
        DeviceEventService.prototype.onActive = function () {
            this.active.dispatch();
        };
        DeviceEventService.prototype.onPause = function () {
            this.pause.dispatch();
        };
        DeviceEventService.prototype.onResume = function () {
            this.resume.dispatch();
        };
        return DeviceEventService;
    }());
    exports.DeviceEventService = DeviceEventService;
});
/// <reference path="../platform.d.ts" />
define("services/soundmanager", ["require", "exports", "knockout", "table/runtimesettings"], function (require, exports, ko, runtimeSettings) {
    "use strict";
    var SoundManager = (function () {
        function SoundManager() {
            this.enabled = ko.observable(false);
            this.tableSoundsEnabled = ko.observable(false);
        }
        SoundManager.prototype.playFold = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/fold.mp3");
        };
        SoundManager.prototype.playCheck = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/check.mp3");
        };
        SoundManager.prototype.playCall = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/call.mp3");
        };
        SoundManager.prototype.playBet = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/raise.mp3");
        };
        SoundManager.prototype.playRaise = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/raise.mp3");
        };
        SoundManager.prototype.playAllIn = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/raise.mp3");
        };
        SoundManager.prototype.playWinChips = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/winchips.mp3");
        };
        SoundManager.prototype.playTurnReminder = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/turnreminder.mp3");
        };
        SoundManager.prototype.playDealCards = function () {
            if (!this.enabled() || !this.tableSoundsEnabled() || !runtimeSettings.sounds.dealCardsEnabled) {
                return;
            }
            this.quickPlay("snd/shuffle.mp3");
        };
        SoundManager.prototype.playFlop = function () {
            if (!this.enabled() || !this.tableSoundsEnabled()) {
                return;
            }
            this.quickPlay("snd/flip_b.mp3");
        };
        SoundManager.prototype.quickPlay = function (fileName) {
            /* tslint:disable:no-string-literal */
            if (window["Media"] != null) {
                var platformPrefix = platformInfo.mediaRoot;
                var media_1 = new Media(platformPrefix + fileName, function () {
                    console.log("Playing audio completed");
                    media_1.release();
                }, function (err) {
                    console.log("Playing audio failed. Error ", err);
                    media_1.release();
                });
                media_1.play();
            }
            if (window["Audio"] != null) {
                var audio = new Audio(fileName);
                audio.play();
            }
            /* tslint:enable:no-string-literal */
        };
        return SoundManager;
    }());
    exports.SoundManager = SoundManager;
});
/// <reference path="../typings/cordova.d.ts" />
define("services/orientationservice", ["require", "exports"], function (require, exports) {
    "use strict";
    var OrientationService = (function () {
        function OrientationService() {
            this.lastOrientation = null;
            this.disableRotation = false;
        }
        OrientationService.prototype.setOrientation = function (orientation) {
            if (this.isScreenOrientationSupported() && !this.disableRotation) {
                ScreenOrientation.setOrientation(orientation);
            }
            this.lastOrientation = orientation;
        };
        OrientationService.prototype.suppressRotation = function () {
            this.disableRotation = true;
        };
        OrientationService.prototype.enableRotation = function () {
            this.disableRotation = false;
        };
        OrientationService.prototype.lock = function () {
            if (this.isScreenOrientationSupported() && !this.disableRotation) {
                ScreenOrientation.lock();
            }
        };
        OrientationService.prototype.unlock = function () {
            if (this.isScreenOrientationSupported() && !this.disableRotation) {
                ScreenOrientation.unlock();
            }
        };
        OrientationService.prototype.setLastOrientation = function () {
            this.setOrientation(this.lastOrientation);
        };
        OrientationService.prototype.isScreenOrientationSupported = function () {
            /* tslint:disable:no-string-literal */
            return window["ScreenOrientation"] && ScreenOrientation["setOrientation"];
            /* tslint:enable:no-string-literal */
        };
        return OrientationService;
    }());
    exports.OrientationService = OrientationService;
});
/// <reference path="services.d.ts" />
define("services/pushservice", ["require", "exports"], function (require, exports) {
    "use strict";
    var PushService = (function () {
        function PushService() {
        }
        PushService.prototype.register = function () {
            /* tslint:disable:no-string-literal */
            if (window["WindowsAzure"] == null) {
                return;
            }
            /* tslint:enable:no-string-literal */
            var hub = this.getHub();
            hub.registerApplicationAsync().then(function (result) {
                console.log("Registration successful: " + result.registrationId);
            }, function (error) {
                console.log("Registration failed: " + error);
            });
            hub.onPushNotificationReceived = function (msg) {
                console.log("onPushNotificationReceived: ", msg);
            };
        };
        PushService.prototype.getHub = function () {
            if (this.hub != null) {
                return this.hub;
            }
            var serviceBusNamespace = "azurebus-ns";
            var notificationHubPath = "azurebus";
            var hubKey = "azurebuskey=";
            var connectionString = "Endpoint=sb://" + serviceBusNamespace + ".servicebus.windows.net/;"
                + "SharedAccessKeyName=DefaultListenSharedAccessSignature;SharedAccessKey=" + hubKey;
            this.hub = new WindowsAzure.Messaging.NotificationHub(notificationHubPath, connectionString, "375486100658");
            return this.hub;
        };
        return PushService;
    }());
    exports.PushService = PushService;
});
define("languageManager", ["require", "exports"], function (require, exports) {
    "use strict";
    var LanguageManager = (function () {
        function LanguageManager() {
            this.currentLang = "ru_RU";
        }
        LanguageManager.prototype.setLang = function (lang) {
            this.currentLang = lang;
        };
        LanguageManager.prototype.getProvider = function () {
            return messages;
        };
        LanguageManager.prototype.setProvider = function (provider) {
            messages = provider;
        };
        LanguageManager.prototype.getMessage = function (message, parameters) {
            if (parameters === void 0) { parameters = null; }
            if (message == null) {
                return "";
            }
            var parts = message.split(".");
            var currentProvider = this.getProvider();
            for (var i = 0; i < parts.length; i++) {
                var propertyName = parts[i];
                currentProvider = currentProvider[propertyName];
                if (currentProvider == null) {
                    break;
                }
            }
            if (currentProvider == null) {
                return this.bind(message, parameters);
            }
            return this.bind(currentProvider, parameters);
        };
        LanguageManager.prototype.bind = function (template, parameters) {
            if (parameters == null) {
                return template;
            }
            var result = template;
            for (var propName in parameters) {
                if (parameters.hasOwnProperty(propName)) {
                    var value = parameters[propName];
                    if (value === null || value === undefined) {
                        value = "";
                    }
                    else {
                        value = value.toString();
                    }
                    result = result.replace("#" + propName + "#", value);
                    // Bind using old syntax.
                    result = result.replace("#" + propName, value);
                }
            }
            return result;
        };
        return LanguageManager;
    }());
    exports.LanguageManager = LanguageManager;
    exports.l = new LanguageManager();
    function _(message, parameters) {
        if (parameters === void 0) { parameters = null; }
        return exports.l.getMessage(message, parameters);
    }
    exports._ = _;
});
/// <reference path="../languageManager.ts" />
define("services/websiteservice", ["require", "exports", "languageManager"], function (require, exports, languagemanager_1) {
    "use strict";
    /**
    * Provides integration with the website.
    */
    var WebsiteService = (function () {
        function WebsiteService() {
            this.websiteRoot = "http://www.yourpoker.com/";
        }
        /**
        * Navigates to the forget password page
        */
        WebsiteService.prototype.forgetPassword = function () {
            this.open(this.getBase() + "reset");
        };
        /**
        * Navigates to the made deposit page
        */
        WebsiteService.prototype.madeDeposit = function () {
            this.open(this.getAuthUrl() + "area/kasa/deposit");
        };
        /**
        * Navigates to the made deposit page
        */
        WebsiteService.prototype.withdrawal = function () {
            this.open(this.getAuthUrl() + "area/kasa/pay");
        };
        /**
        * Navigates to the messages page
        */
        WebsiteService.prototype.messages = function () {
            this.open(this.getAuthUrl() + "area/message");
        };
        /**
        * Navigates to the informations page
        */
        WebsiteService.prototype.information = function () {
            this.open(this.getBase() + "info/main/");
        };
        /**
        * Navigates to the user agreement page
        */
        WebsiteService.prototype.userAgreement = function () {
            this.open(this.getBase() + "info/main/team.html");
        };
        /**
        * Navigates to the Contact Us page
        */
        WebsiteService.prototype.contactUs = function () {
            this.open(this.getBase() + "contacts");
        };
        /**
        * Navigates to the player profile
        */
        WebsiteService.prototype.profile = function () {
            this.open(this.getAuthUrl() + "area/");
        };
        /**
        * Navigates to the update APK url.
        */
        WebsiteService.prototype.navigateUpdateApk = function () {
            this.open(this.getBase() + "update/");
        };
        WebsiteService.prototype.open = function (url) {
            window.open(url, "_system", "location=yes");
        };
        WebsiteService.prototype.getBase = function () {
            return this.websiteRoot + languagemanager_1.l.currentLang + "/";
        };
        WebsiteService.prototype.getAuthUrl = function () {
            return this.websiteRoot + "redirect?auth=" + authToken + "&path=/" + languagemanager_1.l.currentLang + "/";
        };
        return WebsiteService;
    }());
    exports.WebsiteService = WebsiteService;
});
define("services/index", ["require", "exports", "services/keyboardactivationservice", "services/slowinternetservice", "services/connectionservice", "services/accountservice", "services/imagepreloadservice", "services/reloadmanager", "services/deviceeventservice", "services/soundmanager", "services/orientationservice", "services/pushservice", "services/websiteservice"], function (require, exports, keyboardactivationservice_1, slowinternetservice_1, connectionservice_2, accountservice_1, imagepreloadservice_1, reloadmanager_1, deviceeventservice_1, soundmanager_1, orientationservice_1, pushservice_1, websiteservice_1) {
    "use strict";
    exports.keyboardActivationService = new keyboardactivationservice_1.KeyboardActivationService();
    exports.slowInternetService = new slowinternetservice_1.SlowInternetService();
    exports.connectionService = new connectionservice_2.ConnectionService();
    exports.accountService = new accountservice_1.AccountService();
    exports.imagePreloadService = new imagepreloadservice_1.ImagePreloadService();
    exports.reloadManager = new reloadmanager_1.ReloadManager();
    exports.deviceEvents = new deviceeventservice_1.DeviceEventService();
    exports.soundManager = new soundmanager_1.SoundManager();
    exports.orientationService = new orientationservice_1.OrientationService();
    exports.pushService = new pushservice_1.PushService();
    exports.websiteService = new websiteservice_1.WebsiteService();
});
/// <reference path="poker.commanding.api.ts" />
/// <reference path="poker.commanding.api.d.ts" />
define("metadatamanager", ["require", "exports", "knockout", "services/index", "debugsettings"], function (require, exports, ko, services_1, debugsettings_9) {
    "use strict";
    var MetadataManager = (function () {
        function MetadataManager() {
            this.online = ko.observable("-");
            this.registered = ko.observable("-");
        }
        MetadataManager.prototype.setReady = function (value) {
            this.ready = value;
        };
        MetadataManager.prototype.setFailed = function (value) {
            this.failed = value;
        };
        MetadataManager.prototype.update = function () {
            var self = this;
            var metadataApi = new OnlinePoker.Commanding.API.Metadata(apiHost);
            var result = $.Deferred();
            var failHandler = function () {
                if (self.failed !== null) {
                    self.failed();
                }
                result.reject();
            };
            var successHander = function () {
                if (self.ready !== null) {
                    self.ready();
                }
                result.resolve();
            };
            var bannersRequest = metadataApi.GetBanners(self.getBannerFormat());
            $.when(bannersRequest).done(function (data, status) {
                if (data.Status !== "Ok") {
                    return;
                }
                if (data.Data.length === 0) {
                    return;
                }
                services_1.imagePreloadService.preload(data.Data[0].Url);
            });
            var smallBannersRequest = metadataApi.GetBanners(self.getSmallBannerFormat());
            $.when(smallBannersRequest).done(function (data, status) {
                if (data.Status !== "Ok") {
                    return;
                }
                for (var i = 0; i < data.Data.length; i++) {
                    services_1.imagePreloadService.preload(data.Data[i].Url);
                }
            });
            $.when(metadataApi.GetOnlinePlayers(), metadataApi.GetWellKnownPrizeStructure(), metadataApi.GetWellKnownBetStructure(), metadataApi.GetDefaultAvatars(), bannersRequest, smallBannersRequest)
                .done(function (onlinePlayersDataResult, prizeStructureDataResult, betStructureDataResult, avatarsDataResult, bannersDataResult, smallBannersDataResult) {
                var onlinePlayersData = onlinePlayersDataResult[0];
                var prizeStructureData = prizeStructureDataResult[0];
                var betStructureData = betStructureDataResult[0];
                var avatarsData = avatarsDataResult[0];
                var bannersData = bannersDataResult[0];
                var smallBannersData = smallBannersDataResult[0];
                if (onlinePlayersData.Status !== "Ok"
                    || prizeStructureData.Status !== "Ok"
                    || betStructureData.Status !== "Ok"
                    || bannersData.Status !== "Ok"
                    || avatarsData.Status !== "Ok"
                    || smallBannersData.Status !== "Ok") {
                    failHandler();
                    return;
                }
                self.log("Informaton about players online received: " + JSON.stringify(onlinePlayersData.Data));
                self.registered(onlinePlayersData.Data[1]);
                self.online(onlinePlayersData.Data[0]);
                self.log("Informaton about prize structure received: " + JSON.stringify(prizeStructureData.Data));
                self.prizes = prizeStructureData.Data;
                self.log("Informaton about bet structure received: " + JSON.stringify(betStructureData.Data));
                self.bets = betStructureData.Data;
                self.log("Informaton about avatars received: " + JSON.stringify(avatarsData.Data));
                self.avatars = avatarsData.Avatars;
                self.log("Informaton about banners received: " + JSON.stringify(bannersData.Data));
                self.banners = bannersData.Data;
                self.log("Informaton about small banners received: " + JSON.stringify(smallBannersData.Data));
                self.smallBanners = smallBannersData.Data;
                successHander();
            }).fail(failHandler);
            return result;
        };
        MetadataManager.prototype.versionCheck = function () {
            var self = this;
            var result = $.Deferred();
            var metadataApi = new OnlinePoker.Commanding.API.Metadata(apiHost);
            metadataApi.VersionCheck().then(function (serverInformation) {
                if (serverInformation.ServerApiVersion > OnlinePoker.Commanding.API.version) {
                    if (serverInformation.MinimumClientApiVersion <= OnlinePoker.Commanding.API.version) {
                        result.reject(true);
                    }
                    else {
                        result.reject(false);
                    }
                }
                result.resolve();
            });
            return result;
        };
        MetadataManager.prototype.updateOnline = function () {
            var self = this;
            var metadataApi = new OnlinePoker.Commanding.API.Metadata(apiHost);
            return metadataApi.GetOnlinePlayers().then(function (onlinePlayers) {
                self.registered(onlinePlayers.Data[1].toString());
                self.online(onlinePlayers.Data[0].toString());
                return onlinePlayers;
            });
        };
        MetadataManager.prototype.getBannerFormat = function () {
            if (PageBlock.useDoubleView) {
                return 7;
            }
            if (screen.width === 360 || screen.height === 360) {
                return 5;
            }
            if (screen.width === 375 || screen.height === 375) {
                return 7;
            }
            if (screen.width === 414 || screen.height === 414) {
                return 5;
            }
            return 3;
        };
        MetadataManager.prototype.getSmallBannerFormat = function () {
            if (PageBlock.useDoubleView) {
                return 8;
            }
            if (screen.width === 360 || screen.height === 360) {
                return 6;
            }
            if (screen.width === 375 || screen.height === 375) {
                return 8;
            }
            if (screen.width === 414 || screen.height === 414) {
                return 6;
            }
            return 4;
        };
        MetadataManager.prototype.log = function (message) {
            if (debugsettings_9.debugSettings.initialization.metadata) {
                console.log(message);
            }
        };
        return MetadataManager;
    }());
    var metadataManager = new MetadataManager();
    return metadataManager;
});
/* tslint:disable:no-string-literal */
define("ui/popupbase", ["require", "exports", "knockout", "debugsettings"], function (require, exports, ko, debugsettings_10) {
    "use strict";
    var PopupBase = (function () {
        function PopupBase() {
            this.visible = ko.observable(false);
        }
        PopupBase.prototype.shown = function (args) {
            if (args === void 0) { args = []; }
            var theConstructor = this["constructor"];
            this.trace("Popup shown " + theConstructor.name);
            this.visible(true);
        };
        PopupBase.prototype.close = function () {
            var theConstructor = this["constructor"];
            this.trace("Closing popup " + theConstructor.name);
            app.closePopup();
            this.visible(false);
        };
        PopupBase.prototype.trace = function (message) {
            if (debugsettings_10.debugSettings.ui.tracePopups) {
                console.log(message);
            }
        };
        return PopupBase;
    }());
    exports.PopupBase = PopupBase;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/simplepopup", ["require", "exports", "knockout", "ui/popupbase", "timeService"], function (require, exports, ko, popupbase_1, timeService) {
    "use strict";
    var SimplePopup = (function (_super) {
        __extends(SimplePopup, _super);
        function SimplePopup() {
            _super.call(this);
            this.title = ko.observable();
            this.messages = ko.observableArray();
        }
        SimplePopup.prototype.confirm = function () {
            _super.prototype.close.call(this);
        };
        SimplePopup.display = function (title, message) {
            app.simplePopup.title(title);
            if (typeof message === "string") {
                app.simplePopup.messages([message]);
            }
            else {
                // Assume that array is passed.
                app.simplePopup.messages(message);
            }
            return app.showPopup("simple").promise();
        };
        SimplePopup.displayWithTimeout = function (title, message, timeout) {
            var result = SimplePopup.display(title, message);
            var handle = timeService.setTimeout(function () {
                if (handle !== null) {
                    app.closePopup();
                    handle = null;
                }
            }, timeout);
            var deferred = $.Deferred();
            result.then(function (value) {
                timeService.clearTimeout(handle);
                deferred.resolve(value);
            }, function (reason) {
                deferred.reject(reason);
            });
            return deferred.promise();
        };
        return SimplePopup;
    }(popupbase_1.PopupBase));
    exports.SimplePopup = SimplePopup;
});
/// <reference path="../popups/simplepopup.ts" />
define("services/broadcastservice", ["require", "exports", "popups/simplepopup", "languageManager"], function (require, exports, simplepopup_1, languagemanager_2) {
    "use strict";
    /**
    * Manages broadcast services
    */
    var BroadcastService = (function () {
        function BroadcastService() {
        }
        /**
        * Displays broadcast messages
        * @message String Broadcast message to be displayed.
        */
        BroadcastService.prototype.displayMessage = function (message) {
            simplepopup_1.SimplePopup.display(languagemanager_2._("broadcast.caption"), message);
        };
        return BroadcastService;
    }());
    var broadcastService = new BroadcastService();
    return broadcastService;
});
define("commandManager", ["require", "exports"], function (require, exports) {
    "use strict";
    var CommandManager = (function () {
        function CommandManager() {
            this.commands = [];
        }
        CommandManager.prototype.registerCommand = function (commandName, handler) {
            this.commands[commandName] = handler;
        };
        CommandManager.prototype.executeCommand = function (commandName, parameters) {
            if (parameters === void 0) { parameters = []; }
            var handler = this.commands[commandName];
            if (handler == null) {
                console.log("Command call " + commandName + " not executed, since no handler registered");
                return null;
            }
            return handler(parameters);
        };
        return CommandManager;
    }());
    var commandManager = new CommandManager();
    return commandManager;
});
/* tslint:disable:no-bitwise */
define("table/tabpleplacemodel", ["require", "exports", "knockout", "timeService", "languageManager"], function (require, exports, ko, timeService, languagemanager_3) {
    "use strict";
    var TablePlaceModel = (function () {
        function TablePlaceModel(data) {
            /**
            * Last message in the chat which this player shows
            */
            this.LastChatMessage = ko.observable(null);
            this.cardsOverlayVisible = ko.observable(true);
            this.PlayerId = ko.observable(data.PlayerId);
            this.PlayerName = ko.observable(data.PlayerName);
            // Here should be added checks for the safe locations,
            // images from which could be loaded.
            if (data.PlayerUrl !== null) {
                var isSafeLocation = data.PlayerUrl !== "";
                this.PlayerUrl = ko.observable(isSafeLocation ? data.PlayerUrl : TablePlaceModel.DefaultAvatarUrl);
            }
            else {
                this.PlayerUrl = ko.observable(TablePlaceModel.DefaultAvatarUrl);
            }
            this.IsCurrent = ko.observable(data.IsCurrent);
            this.IsDealer = ko.observable(data.IsDealer);
            this.Money = ko.observable(data.Money);
            this.Seat = ko.observable(data.Seat);
            var cards = decodeCardsArray(data.Cards || null);
            var cardClasses = convertToCards(cards);
            this.RawCards = ko.observableArray(cards);
            this.Cards = ko.observableArray(cardClasses);
            this.HandCards = ko.observableArray(cardClasses);
            this.FoldedCards = ko.observableArray(null);
            this.Stars = ko.observable(data.Stars);
            this.Points = ko.observable(data.Points);
            this.Bet = ko.observable(data.Bet);
            this.IsBetAnimationLocked = ko.observable(false);
            this.IsMovingBetToPot = ko.observable(false);
            this.IsDealCards = ko.observable(false);
            this.IsFoldCards = ko.observable(false);
            this.Status = ko.observable(data.Status);
            this.WinAmount = ko.observable(data.WinAmount || null);
            this.TotalBet = ko.observable(data.TotalBet || null);
            var hasCards = cards == null ? null : true;
            this.WasInGame = ko.observable(data.WasInGame == null ? hasCards : data.WasInGame);
            this.IsCardsFolded = ko.observable(false);
            this.Card1Hightlighted = ko.observable(false);
            this.Card2Hightlighted = ko.observable(false);
            this.CardsHightlighted = ko.observable(false);
            this.CurrentAction = ko.observable();
            var self = this;
            this.IsCardsAnimating = ko.computed(function () {
                return self.IsDealCards() || self.IsFoldCards();
            }, this);
            this.IsBetPlaced = ko.computed(function () {
                return self.Bet() > 0 && !self.IsMovingBetToPot();
            }, this);
            this.IsSitoutStatus = ko.computed(function () {
                return (self.Status() & 1) !== 0;
            }, this);
            this.IsParticipatingStatus = ko.computed({
                read: function () {
                    return (self.Status() & 8) !== 0;
                },
                write: function (value) {
                    if (value) {
                        self.Status(self.Status() | 8);
                    }
                    else {
                        self.Status(self.Status() & ~8);
                    }
                },
                owner: this
            });
            this.IsInGameStatus = ko.computed({
                read: function () {
                    return (self.Status() & 16) !== 0;
                },
                write: function (value) {
                    if (value) {
                        self.Status(self.Status() | 16);
                    }
                    else {
                        self.Status(self.Status() & ~16);
                    }
                },
                owner: this
            });
            this.IsInGameStatus(this.IsParticipatingStatus() && this.Cards() !== null);
            this.IsBronse = ko.computed(function () {
                return self.Points() >= 100 * 1000
                    && self.Points() < 200 * 1000;
            });
            this.IsSilver = ko.computed(function () {
                return self.Points() >= 200 * 1000
                    && self.Points() < 500 * 1000;
            });
            this.IsGold = ko.computed(function () {
                return self.Points() >= 500 * 1000;
            });
            this.LastChatMessageTrimed = ko.computed(function () {
                if (self.LastChatMessage() === null) {
                    return null;
                }
                return self.LastChatMessage().substr(0, 17) + "\u2026";
            });
        }
        TablePlaceModel.prototype.prepareForNewGame = function () {
            this.TotalBet(null);
            this.Bet(null);
            this.IsCardsFolded(false);
            this.IsBetAnimationLocked(false);
            this.RawCards(null);
            this.Cards(null);
            this.HandCards(null);
            this.FoldedCards(null);
            this.WinAmount(null);
            if (this.Money() === 0) {
                this.IsInGameStatus(false);
            }
            this.Card1Hightlighted(false);
            this.Card2Hightlighted(false);
            this.CardsHightlighted(false);
        };
        TablePlaceModel.prototype.initializeForNewGame = function (money) {
            this.IsInGameStatus(true);
            this.Money(money);
            this.WasInGame(true);
            this.cardsOverlayVisible(true);
        };
        TablePlaceModel.prototype.startDealCards = function () {
            this.Cards(allBacksClasses);
            this.IsDealCards(true);
        };
        TablePlaceModel.prototype.setCards = function (cards) {
            if (cards === undefined) {
                console.warn("Passed undefined cards to the TablePlaceModel.setCards");
            }
            var cardsClasses = convertToCards(cards);
            this.RawCards(cards || null);
            this.Cards(cardsClasses);
            this.HandCards(cardsClasses);
        };
        TablePlaceModel.prototype.collectBet = function () {
            this.TotalBet((this.TotalBet() === null ? 0 : this.TotalBet()) + this.Bet());
            this.Bet(0);
            this.IsBetAnimationLocked(false);
            this.IsMovingBetToPot(false);
        };
        /**
        * Starts displaying action which player was perform during last turn.
        */
        TablePlaceModel.prototype.startAction = function (action) {
            var _this = this;
            this.CurrentAction(action);
            this.CurrentActionTimer = timeService.setTimeout(function () { return _this.clearAction(); }, 2000);
        };
        /**
        * Clears current action.
        */
        TablePlaceModel.prototype.clearAction = function () {
            timeService.clearTimeout(this.CurrentActionTimer);
            this.CurrentAction(null);
        };
        /**
        * Starts displaying chat message from player.
        */
        TablePlaceModel.prototype.displayChatMessage = function (message) {
            var _this = this;
            this.LastChatMessage(message);
            this.LastChatMessageTimer = timeService.setTimeout(function () { return _this.clearChatMessage(); }, 2000);
        };
        /**
        * Clears current chat message.
        */
        TablePlaceModel.prototype.clearChatMessage = function () {
            timeService.clearTimeout(this.LastChatMessageTimer);
            this.LastChatMessage(null);
        };
        TablePlaceModel.prototype.startFoldAnimation = function () {
            this.IsFoldCards(true);
            this.Cards(allBacksClasses);
        };
        TablePlaceModel.prototype.finishFoldAnimation = function () {
            this.Cards(null);
            this.IsFoldCards(false);
        };
        /**
         * Help message which indicate which action player should take.
         * @param tableView Table for which get help message.
         */
        TablePlaceModel.prototype.getHelpMessage = function (tableView) {
            if (tableView.actionBlock.sitoutBlockVisible()) {
                if (this.Money() === 0) {
                    return "Пожалуйста, пополните ваш счёт";
                }
                return "Нажмите кнопку Вернуться чтобы продолжить игру";
            }
            if (tableView.actionBlock.needBB()) {
                if (this.Money() === 0) {
                    return "Пожалуйста, пополните ваш счёт";
                }
                return "Подождите пожалуйста, вы начнете игру когда будете сидеть на большом блаинде";
            }
            if (tableView.actionBlock.mainButtonsBlockVisible()) {
                return "Пожалуйста сделайте Вашу ставку";
            }
            if (tableView.actionBlock.autoButtonsBlockVisible()) {
                return "Желаем удачи";
            }
            if (this.WinAmount() > 0) {
                return "Поздравляем, Ваш выигрыш: " + this.WinAmount();
            }
            if (this.Money() === 0) {
                return "Пожалуйста, пополните ваш счёт";
            }
            // When player fold cards.
            if (this.IsCardsFolded()) {
                return "Желаем удачи";
            }
            // This is fallback case for situations in the 
            // beginning of the game start, pause between move confirmation.
            return "Желаем удачи";
        };
        TablePlaceModel.prototype.getCombination = function (tableCards) {
            if (tableCards === null || tableCards === undefined || tableCards.length === 0) {
                return null;
            }
            var myCards = this.RawCards();
            if (myCards === null || myCards === undefined || myCards.length === 0) {
                return null;
            }
            var totalCards = [];
            if (tableCards !== null) {
                totalCards = totalCards.concat(tableCards);
            }
            totalCards = totalCards.concat(myCards);
            var handRepresentation = {
                Cards: [],
                Suits: []
            };
            totalCards.forEach(function (card) {
                handRepresentation.Cards.push((card % 13) + 2);
                handRepresentation.Suits.push(1 << (card / 13));
            });
            var rank = HoldemHand.getCardRank(handRepresentation);
            var winnerCards = [];
            rank.WinnerCardsSet.forEach(function (item) {
                winnerCards.push(totalCards[item]);
            });
            handRepresentation = {
                Cards: [],
                Suits: []
            };
            winnerCards.forEach(function (card) {
                handRepresentation.Cards.push((card % 13) + 2);
                handRepresentation.Suits.push(1 << (card / 13));
            });
            var type = HoldemHand.getHandTypeEx(handRepresentation);
            var typeIndex = HoldemHand.handTypeRanks[type.Type];
            var base = languagemanager_3._("table.combination.c" + typeIndex.toString());
            if (typeIndex === 1) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]))
                    .replace("##c3", HoldemHand.cardValue(type.Cards[2]))
                    .replace("##c4", HoldemHand.cardValue(type.Cards[3]))
                    .replace("##c5", HoldemHand.cardValue(type.Cards[4]));
            }
            if (typeIndex === 2) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]))
                    .replace("##c3", HoldemHand.cardValue(type.Cards[2]))
                    .replace("##c4", HoldemHand.cardValue(type.Cards[3]));
            }
            if (typeIndex === 3) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]))
                    .replace("##c3", HoldemHand.cardValue(type.Cards[2]));
            }
            if (typeIndex === 4) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]))
                    .replace("##c3", HoldemHand.cardValue(type.Cards[2]));
            }
            if (typeIndex === 5) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]));
            }
            if (typeIndex === 6) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]))
                    .replace("##c3", HoldemHand.cardValue(type.Cards[2]))
                    .replace("##c4", HoldemHand.cardValue(type.Cards[3]))
                    .replace("##c5", HoldemHand.cardValue(type.Cards[4]));
            }
            if (typeIndex === 7) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]));
            }
            if (typeIndex === 8) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]))
                    .replace("##c2", HoldemHand.cardValue(type.Cards[1]));
            }
            if (typeIndex === 9) {
                return base.replace("##c1", HoldemHand.cardValue(type.Cards[0]));
            }
            return base;
        };
        /**
        * Default avatar for unspecified images, and
        * images from unknown locations.
        */
        TablePlaceModel.DefaultAvatarUrl = "img/1px.png";
        return TablePlaceModel;
    }());
    exports.TablePlaceModel = TablePlaceModel;
});
define("appconfig", ["require", "exports"], function (require, exports) {
    "use strict";
    var GameHandHistory = (function () {
        function GameHandHistory() {
            /**
            * Show game history with cards.
            */
            this.showPictureHistory = true;
            /**
            * Show game history as text.
            */
            this.showTextHistory = true;
        }
        return GameHandHistory;
    }());
    var GameActionBlock = (function () {
        function GameActionBlock() {
            /**
            * Wherether action panel is present on the application
            */
            this.hasSecondaryPanel = true;
        }
        return GameActionBlock;
    }());
    var AppConfig = (function () {
        function AppConfig() {
            this.game = {
                handHistory: new GameHandHistory(),
                actionBlock: new GameActionBlock(),
                seatMode: document.body.classList.contains("poker-feature-single-seat")
            };
            this.tournament = {
                enabled: false,
                openTableAutomatically: true
            };
        }
        return AppConfig;
    }());
    exports.appConfig = new AppConfig();
});
define("table/tournamentView", ["require", "exports", "knockout", "timeService", "table/tablemanager", "authmanager", "metadatamanager", "services/index", "popups/simplepopup", "appconfig", "debugsettings", "languageManager"], function (require, exports, ko, timeService, tableManager, authManager, metadataManager, services_2, simplepopup_2, appconfig_1, debugsettings_11, languagemanager_4) {
    "use strict";
    var TournamentView = (function () {
        function TournamentView(tournamentId, data) {
            this.tournamentId = tournamentId;
            this.tournamentData = ko.observable(null);
            this.loading = ko.observable(false);
            /**
            * Indicates that rebuy is allowed in the tournament.
            */
            this.rebuyAllowed = ko.observable(false);
            /**
            * Indicates that add-on is allowed in the tournament.
            */
            this.addonAllowed = ko.observable(false);
            /**
            * Count of rebuys which player made in the tournament.
            */
            this.rebuyCount = ko.observable(0);
            /**
            * Count of add-ons which player made in the tournament.
            */
            this.addonCount = ko.observable(0);
            /**
            * Indicates that tournament is frozen
            */
            this.frozen = ko.observable(false);
            /**
            * Id of the current table where player sitting.
            */
            this.currentTableId = null;
            /**
            * Indicates that player is finished playing in the tournament.
            */
            this.finishedPlaying = ko.observable(false);
            /**
            * Indicates when player finish tournament.
            */
            this.finishTime = ko.observable(0);
            /**
            * Place which player take in the tournament.
            */
            this.finishedPlace = null;
            /**
            * Request which performs connecting to the table.
            */
            this.connectingRequest = null;
            /**
            * Status of the tournament.
            */
            this.status = ko.observable(TournamentStatus.Pending);
            var self = this;
            this.tournamentData(data);
            this.status(data.Status);
            this.rebuyAllowed(data.IsRebuyAllowed);
            this.addonAllowed(data.IsAddonAllowed);
            var currentPlayerCandidates = data.TournamentPlayers.filter(function (_) { return _.PlayerId === authManager.loginId(); });
            if (currentPlayerCandidates.length > 0) {
                var currentPlayer = currentPlayerCandidates[0];
                if (currentPlayer.Status === TournamentPlayerStatus.Playing) {
                    this.currentTableId = currentPlayer.TableId;
                }
            }
            this.totalPrize = ko.computed(function () {
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return null;
                }
                return tdata.PrizeAmount + (tdata.CollectedPrizeAmount || 0);
            }, this);
        }
        TournamentView.prototype.refreshTournament = function () {
            if (this.tournamentId === 0) {
                return $.Deferred().resolve({ Status: "Ok", Data: null });
            }
            var self = this;
            var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            this.loading(true);
            return tournamentApi.GetTournament(this.tournamentId, function (data) {
                if (data.Status === "Ok") {
                    var tournamentData = data.Data;
                    self.log("Informaton about tournament " + self.tournamentId + " received: ", data.Data);
                    self.log(tournamentData.TournamentName);
                    self.tournamentData(tournamentData);
                }
                self.loading(false);
            });
        };
        TournamentView.prototype.clearInformation = function () {
            // Do nothing.
        };
        TournamentView.prototype.updateTournamentInformation = function () {
            /// <signature>
            ///     <summary>Updates the information about the table from the server</summary>
            /// </signature>
            var self = this;
            if (this.connectingRequest !== null && this.connectingRequest.state() === "pending") {
                // Re-schedule updating information.
                this.connectingRequest.fail(function () {
                    self.log("Rescheduling the updating information.");
                    self.updateTournamentInformation();
                });
                self.log("Cancelling the connection request process");
                self.cancelUpdateTableInformation();
                return;
            }
            // this.connecting(true);
            var currentLoadingRequest = $.Deferred();
            var wrapper = services_2.connectionService.currentConnection;
            var hubId = wrapper.connection.id;
            var connectionInfo = "HID:" + hubId;
            this.log("Connecting to tournament " + this.tournamentId + " on connection " + connectionInfo);
            var startConnection = app.buildStartConnection();
            var api = new OnlinePoker.Commanding.API.Game(apiHost);
            startConnection().then(function () {
                if (wrapper.terminated) {
                    return;
                }
                hubId = wrapper.connection.id;
                self.log("Attempting to connect to table and chat over connection " + hubId);
                var joinTournamentRequest = self.joinTournament(wrapper);
                var joinRequest = $.when(joinTournamentRequest);
                currentLoadingRequest.progress(function (command) {
                    self.log("Receiving request to cancel all joining operations");
                    joinTournamentRequest.notify(command);
                });
                joinRequest.then(function () {
                    if (wrapper.terminated) {
                        currentLoadingRequest.reject("Cancelled");
                        return;
                    }
                    self.log("Joining to tournament finished");
                    currentLoadingRequest.resolve();
                }, function (result1) {
                    if (wrapper.terminated) {
                        return;
                    }
                    var message = "Rejecting request due to join tournament failure in the connection."
                        + "Failed request: " + result1[0];
                    self.log(message);
                    currentLoadingRequest.reject(message);
                });
            }, function (message) {
                self.log("Tournament connection failed. Error: " + message);
                currentLoadingRequest.reject("Table connection failed. Error: " + message);
            });
            this.connectingRequest = currentLoadingRequest;
        };
        TournamentView.prototype.cancelUpdateTableInformation = function () {
            if (this.connectingRequest != null) {
                this.connectingRequest.notify("cancel");
                this.connectingRequest = null;
            }
        };
        TournamentView.prototype.joinTournament = function (wrapper, maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var self = this;
            var result = $.Deferred();
            if (maxAttempts === 0 || wrapper.terminated) {
                this.log("Stop connecting to tournament");
                result.reject("Stop connecting to tournament", false);
                return result;
            }
            var hubId = services_2.connectionService.currentConnection.connection.id;
            var connectionInfo = "HID:" + hubId;
            this.log("Joining tournament on connection " + connectionInfo);
            var cancelled = false;
            var subsequentDeferred = null;
            var cancelOperation = function () {
                self.log("Cancelling join tournament request");
                result.reject("Cancelled", true);
            };
            wrapper.buildStartConnection()().pipe(function () {
                if (wrapper.terminated) {
                    cancelOperation();
                    return;
                }
                self.log("Executing Game.subscribeTournament on connection " + wrapper.connection.id + " in state " + wrapper.connection.state);
                var operation = wrapper.connection.Game.server.subscribeTournament(self.tournamentId)
                    .then(function () {
                    if (wrapper.terminated) {
                        cancelOperation();
                        return;
                    }
                    result.resolve();
                }, function (error) {
                    if (wrapper.terminated || cancelled || error === "Cancelled") {
                        cancelOperation();
                        return;
                    }
                    var message = "" + error;
                    self.log("Failed to join tournament " + self.tournamentId + ", " + connectionInfo + ". Reason: " + message);
                    if (message.indexOf("Connection was disconnected before invocation result was received.") >= 0) {
                        self.log("Stopped connecting to table since underlying connection is broken");
                        services_2.slowInternetService.showReconnectFailedPopup();
                        result.reject("Stopped connecting to table since underlying connection is broken", false);
                        return;
                    }
                    else {
                        subsequentDeferred = self.joinTournament(wrapper, maxAttempts - 1);
                        return subsequentDeferred.then(function () {
                            result.resolve();
                        }, function (error, cancelled) {
                            result.reject(error, cancelled);
                        });
                    }
                });
                result.progress(function (command) {
                    this.cancelled = true;
                    result.reject("Cancelled");
                    if (subsequentDeferred != null) {
                        subsequentDeferred.notify("cancel");
                        subsequentDeferred = null;
                    }
                });
            }, function () {
                cancelOperation();
            });
            return result;
        };
        TournamentView.prototype.onTournamentStatusChanged = function (status) {
            var self = this;
            var data = this.tournamentData();
            var oldStatus = this.status();
            this.status(status);
            var rebuyTime;
            var addonTime;
            if (status === TournamentStatus.LateRegistration) {
                rebuyTime = data.RebuyPeriodTime || 0;
                addonTime = data.AddonPeriodTime || 0;
                this.rebuyAllowed(rebuyTime > 0);
                this.addonAllowed(addonTime > 0 && rebuyTime === 0);
                if (this.finishedPlaying()) {
                    return;
                }
                self.openTournamentTableUI();
            }
            if (status === TournamentStatus.Started) {
                if (oldStatus === TournamentStatus.LateRegistration) {
                    return;
                }
                rebuyTime = data.RebuyPeriodTime || 0;
                addonTime = data.AddonPeriodTime || 0;
                this.rebuyAllowed(rebuyTime > 0);
                this.addonAllowed(addonTime > 0 && rebuyTime === 0);
                if (this.finishedPlaying()) {
                    return;
                }
                self.openTournamentTableUI();
            }
            if (status === TournamentStatus.Cancelled) {
                if (this.finishedPlaying()) {
                    return;
                }
                simplepopup_2.SimplePopup.display(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._("tournament.tournamentCancelled", { tournament: data.TournamentName }))
                    .always(function () {
                    self.log("Tournament " + self.tournamentId + " cancelled");
                    app.lobbyPageBlock.showLobby();
                    app.tablesPage.deactivate();
                });
            }
            if (status === TournamentStatus.Completed) {
                this.executeOnCurrentTable(function () {
                    self.displayTournamentFinished();
                });
            }
        };
        TournamentView.prototype.openTournamentTableUI = function () {
            var self = this;
            var data = this.tournamentData();
            var openTournamentPromise = null;
            if (appconfig_1.appConfig.tournament.openTableAutomatically) {
                openTournamentPromise = this.openTournamentTable(this.currentTableId);
            }
            var messageKey = appconfig_1.appConfig.tournament.openTableAutomatically
                ? "tournament.tournamentStarted"
                : "tournament.tournamentStartedNoOpen";
            simplepopup_2.SimplePopup.display(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._(messageKey, { tournament: data.TournamentName }))
                .always(function () {
                self.log("Tournament " + self.tournamentId + " started");
                if (appconfig_1.appConfig.tournament.openTableAutomatically) {
                    openTournamentPromise.then(function (value) {
                        self.log("Opeinin table for tournament " + self.tournamentId + "");
                        app.showSubPage("tables");
                    });
                }
            });
        };
        TournamentView.prototype.displayTournamentFinished = function () {
            var self = this;
            var data = this.tournamentData();
            var currentDate = new Date().valueOf();
            if (this.finishedPlaying() && (currentDate - this.finishTime()) > 2000) {
                return;
            }
            if (this.finishedPlace !== 1 && this.finishedPlace !== 2) {
                self.log("Tournament " + self.tournamentId + " completed");
                timeService.setTimeout(function () {
                    if (app.tablesPage.tablesShown() /* && app.tablesPage.currentTable().tournament() == self*/) {
                        simplepopup_2.SimplePopup.display(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._("tournament.tournamentCompleted", { tournament: data.TournamentName }))
                            .always(function () {
                            self.finalizeTournament();
                        });
                    }
                    else {
                        self.finalizeTournament();
                    }
                }, 2000);
            }
            else {
                app.closePopup();
                this.finalizeTournament();
            }
        };
        TournamentView.prototype.onTournamentTableChanged = function (tableId) {
            var self = this;
            if (this.finishedPlaying()) {
                return;
            }
            if (this.currentTableId === tableId) {
                return;
            }
            if (this.status() !== TournamentStatus.Started
                && this.status() !== TournamentStatus.LateRegistration) {
                this.currentTableId = tableId;
                return;
            }
            var data = this.tournamentData();
            if (this.currentTableId !== null) {
                this.log("Removing player from the tournament table " + this.currentTableId + " in tournament " + this.tournamentId);
                this.removeCurrentTable();
                simplepopup_2.SimplePopup.display(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._("tournament.tableChanged", { tournament: data.TournamentName }));
            }
            if (tableId != null) {
                this.log("Put player on the tournament table " + tableId + " in tournament " + this.tournamentId);
                this.currentTableId = tableId;
                this.openTournamentTable(tableId);
            }
        };
        TournamentView.prototype.onTournamentPlayerGameCompleted = function (placeTaken) {
            var self = this;
            this.finishedPlaying(true);
            this.finishTime(new Date().valueOf());
            this.finishedPlace = placeTaken;
            var data = this.tournamentData();
            var structure = this.getPrizeStructure(data.WellKnownPrizeStructure);
            var ascendingSort = function (a, b) {
                return a.MaxPlayer - b.MaxPlayer;
            };
            var prizes = structure.filter(function (_) {
                return _.MaxPlayer > data.JoinedPlayers;
            }).sort(ascendingSort);
            var prize;
            if (prizes.length > 0) {
                prize = prizes[0];
            }
            else {
                prize = structure.sort(ascendingSort)[0];
            }
            this.executeOnCurrentTable(function () {
                self.displayGameFinishedNotification(prize, placeTaken);
            });
        };
        TournamentView.prototype.onTournamentBetLevelChanged = function (level) {
            var data = this.tournamentData();
            var structure = this.getBetLevelStructure(data.WellKnownBetStructure).sort(function (a, b) { return a.Level - b.Level; });
            var betLevelCandidate = structure.filter(function (_) { return _.Level === level; });
            var betLevel = betLevelCandidate.length === 0
                ? structure[structure.length - 1]
                : betLevelCandidate[0];
            var currentTable = this.getTableForNotification();
            if (currentTable === null) {
                this.log("Table " + this.currentTableId + " is no longer valid for the tournament " + this.tournamentId);
                return;
            }
            currentTable.bigBlind(betLevel.BigBlind);
            currentTable.smallBlind(betLevel.SmallBlind);
            currentTable.ante(betLevel.Ante);
            if (this.finishedPlaying()) {
            }
            if (level > 1) {
                if (betLevel.Ante == null) {
                    var notificationParameters = {
                        tournament: data.TournamentName,
                        sb: betLevel.SmallBlind,
                        bb: betLevel.BigBlind
                    };
                    currentTable.showNotificationWithDelay(languagemanager_4._("tournament.betLevelChanged1", notificationParameters), debugsettings_11.debugSettings.tableView.betLevelChangeDelay);
                }
                else {
                    var notificationParameters = {
                        tournament: data.TournamentName,
                        sb: betLevel.SmallBlind,
                        bb: betLevel.BigBlind,
                        ante: betLevel.Ante
                    };
                    currentTable.showNotificationWithDelay(languagemanager_4._("tournament.betLevelChanged2", notificationParameters), debugsettings_11.debugSettings.tableView.betLevelChangeDelay);
                }
            }
        };
        TournamentView.prototype.onTournamentRoundChanged = function (round) {
            // Do nothing.
        };
        TournamentView.prototype.onTournamentRebuyStatusChanged = function (rebuyAllowed, addonAllowed) {
            if (this.finishedPlaying()) {
            }
            this.rebuyAllowed(rebuyAllowed);
            this.addonAllowed(addonAllowed);
            if (addonAllowed) {
                var currentTable = this.getTableForNotification();
                if (currentTable != null) {
                    if (this.isInTournament()) {
                        currentTable.showNotificationWithDelay(languagemanager_4._("table.addonAllowed"), debugsettings_11.debugSettings.tableView.addonPeriodStartedDelay);
                    }
                    else {
                        currentTable.showNotificationWithDelay(languagemanager_4._("table.addonAllowedNotPlaying"), debugsettings_11.debugSettings.tableView.addonPeriodStartedDelay);
                    }
                }
            }
        };
        /**
        * Notifies that in tournament count of made rebuy or addons changed.
        * @param rebuyCount Count of rebuys which player done.
        * @param addonCount Count of add-ons which player done.
        */
        TournamentView.prototype.onTournamentRebuyCountChanged = function (rebuyCount, addonCount) {
            this.rebuyCount(rebuyCount);
            this.addonCount(addonCount);
        };
        TournamentView.prototype.onTournamentFrozen = function () {
            this.frozen(true);
        };
        TournamentView.prototype.onTournamentUnfrozen = function () {
            this.frozen(false);
        };
        TournamentView.prototype.onTournamentRegistrationCancelled = function () {
            // Do nothing.
        };
        TournamentView.prototype.getBetLevelStructure = function (level) {
            return metadataManager.bets[level];
        };
        TournamentView.prototype.getPrizeStructure = function (structure) {
            return metadataManager.prizes[structure];
        };
        TournamentView.prototype.removeCurrentTable = function () {
            tableManager.removeTableById(this.currentTableId);
            tableManager.adjustTablePosition();
            this.currentTableId = null;
        };
        TournamentView.prototype.isInTournament = function () {
            if (this.currentTableId == null) {
                return false;
            }
            return !this.finishedPlaying();
        };
        TournamentView.prototype.getTableForNotification = function () {
            var self = this;
            var table = tableManager.getTableById(this.currentTableId);
            if (table !== null) {
                return table;
            }
            var tournamentTables = tableManager.tables().filter(function (_) { return _.tournament() !== null
                && _.tournament().tournamentId === self.tournamentId; });
            if (tournamentTables.length !== 0) {
                return tournamentTables[0];
            }
            return null;
        };
        TournamentView.prototype.displayGameFinishedNotification = function (prize, placeTaken) {
            var _this = this;
            var self = this;
            var data = this.tournamentData();
            if (prize.PrizeLevel.length < placeTaken) {
                simplepopup_2.SimplePopup.displayWithTimeout(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._("tournament.playerGameCompleted", { tournament: data.TournamentName, place: placeTaken }), 10 * 1000);
            }
            else {
                var winAmount = (self.totalPrize() * prize.PrizeLevel[placeTaken - 1] / 100).toFixed();
                simplepopup_2.SimplePopup.displayWithTimeout(languagemanager_4._("tournament.caption", { tournament: data.TournamentName }), languagemanager_4._("tournament.playerGameCompletedAndWin", { tournament: data.TournamentName, place: placeTaken, win: winAmount }), 10 * 1000)
                    .always(function () {
                    if (placeTaken === 1 || placeTaken === 2) {
                        self.finalizeTournament();
                    }
                    else {
                        var currentTable = tableManager.getTableById(_this.currentTableId);
                        if (!currentTable.opened()) {
                            self.finalizeTournament();
                        }
                    }
                });
            }
        };
        TournamentView.prototype.executeOnCurrentTable = function (callback) {
            var currentTable = tableManager.getTableById(this.currentTableId);
            if (app.tablesPage.visible() && currentTable != null) {
                currentTable.pushCallback(function () {
                    callback();
                });
            }
            else {
                callback();
            }
        };
        TournamentView.prototype.openTournamentTable = function (tableId) {
            var self = this;
            var api = new OnlinePoker.Commanding.API.Game(apiHost);
            return api.GetTable(tableId).then(function (data) {
                tableManager.selectTable(data.Data, true);
                var currentTable = tableManager.getTableById(tableId);
                currentTable.tournament(self);
            });
        };
        TournamentView.prototype.finalizeTournament = function () {
            if (tableManager.tables().length <= 1) {
                app.lobbyPageBlock.showLobby();
                app.tablesPage.deactivate();
            }
            this.removeCurrentTable();
        };
        TournamentView.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            console.log(message);
        };
        return TournamentView;
    }());
    exports.TournamentView = TournamentView;
});
define("table/tableplaces", ["require", "exports", "knockout", "table/tabpleplacemodel"], function (require, exports, ko, tabpleplacemodel_1) {
    "use strict";
    var TablePlaces = (function () {
        function TablePlaces(maxPlayers) {
            var _this = this;
            this.maxPlayers = maxPlayers;
            this.placesRefreshTrigger = ko.observable();
            this.place1 = ko.observable(null);
            this.place2 = ko.observable(null);
            this.place3 = ko.observable(null);
            this.place4 = ko.observable(null);
            this.place5 = ko.observable(null);
            this.place6 = ko.observable(null);
            this.place7 = ko.observable(null);
            this.place8 = ko.observable(null);
            this.place9 = ko.observable(null);
            this.place10 = ko.observable(null);
            this.virtualOffset = ko.observable(0);
            this.offsetPlace1 = ko.computed(function () {
                var seat = _this.getVirtualOffset(1);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace2 = ko.computed(function () {
                var seat = _this.getVirtualOffset(2);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace3 = ko.computed(function () {
                var seat = _this.getVirtualOffset(3);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace4 = ko.computed(function () {
                var seat = _this.getVirtualOffset(4);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace5 = ko.computed(function () {
                var seat = _this.getVirtualOffset(5);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace6 = ko.computed(function () {
                var seat = _this.getVirtualOffset(6);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace7 = ko.computed(function () {
                var seat = _this.getVirtualOffset(7);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace8 = ko.computed(function () {
                var seat = _this.getVirtualOffset(8);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace9 = ko.computed(function () {
                var seat = _this.getVirtualOffset(9);
                return _this.getPlaceBySeat(seat);
            });
            this.offsetPlace10 = ko.computed(function () {
                var seat = _this.getVirtualOffset(10);
                return _this.getPlaceBySeat(seat);
            });
            this.places = ko.computed(function () {
                _this.placesRefreshTrigger();
                var result = [];
                if (_this.place1() !== null) {
                    result[_this.getRealOffset(1)] = _this.place1();
                }
                if (_this.place2() !== null) {
                    result[_this.getRealOffset(2)] = _this.place2();
                }
                if (_this.place3() !== null) {
                    result[_this.getRealOffset(3)] = _this.place3();
                }
                if (_this.place4() !== null) {
                    result[_this.getRealOffset(4)] = _this.place4();
                }
                if (_this.place5() !== null) {
                    result[_this.getRealOffset(5)] = _this.place5();
                }
                if (_this.place6() !== null) {
                    result[_this.getRealOffset(6)] = _this.place6();
                }
                if (_this.place7() !== null) {
                    result[_this.getRealOffset(7)] = _this.place7();
                }
                if (_this.place8() !== null) {
                    result[_this.getRealOffset(8)] = _this.place8();
                }
                if (_this.place9() !== null) {
                    result[_this.getRealOffset(9)] = _this.place9();
                }
                if (_this.place10() !== null) {
                    result[_this.getRealOffset(10)] = _this.place10();
                }
                return result;
            }, this);
        }
        TablePlaces.prototype.getRealOffset = function (seat) {
            if (this.maxPlayers === 0) {
                return 1;
            }
            return ((seat - 1 + this.virtualOffset()) % this.maxPlayers) + 1;
        };
        TablePlaces.prototype.getVirtualOffset = function (seat) {
            if (this.maxPlayers === 0) {
                return 1;
            }
            return ((seat - 1 + this.maxPlayers - this.virtualOffset()) % this.maxPlayers) + 1;
        };
        TablePlaces.prototype.rotate = function (offset) {
            var newOffset = (this.maxPlayers + offset + this.virtualOffset()) % this.maxPlayers;
            this.virtualOffset(newOffset);
        };
        TablePlaces.prototype.sit = function (seat, player) {
            if (seat < 1 || seat > this.maxPlayers) {
                throw new Error("Invalid seat number: " + seat);
            }
            var seatVar = this["place" + seat];
            seatVar(player);
        };
        TablePlaces.prototype.standup = function (seat) {
            if (seat < 1 || seat > this.maxPlayers) {
                throw new Error("Invalid seat number: " + seat);
            }
            var seatVar = this["place" + seat];
            seatVar(null);
        };
        TablePlaces.prototype.getPlaceBySeat = function (seat) {
            var seatVar = this["place" + seat];
            return seatVar();
        };
        TablePlaces.prototype.getPlaceByPlayerId = function (playerId) {
            var playerList = this.places().filter(function (_) { return _.PlayerId() === playerId; });
            if (playerList.length === 0) {
                return null;
            }
            return playerList[0];
        };
        TablePlaces.prototype.setPlayers = function (players) {
            /// <signature>
            ///     <summary>Sets players on the table.</summary>
            ///     <param name="players" type="Array">Array of players</param>
            /// </signature>
            this.clear();
            var places = this.places();
            var _loop_1 = function(i) {
                var playersOnSeat = players.filter(function (player) { return player.Seat === i; });
                if (playersOnSeat.length !== 0) {
                    var model = new tabpleplacemodel_1.TablePlaceModel(playersOnSeat[0]);
                    model.IsBetAnimationLocked(true);
                    this_1.sit(i, model);
                }
            };
            var this_1 = this;
            for (var i = 1; i <= this.maxPlayers; i++) {
                _loop_1(i);
            }
            this.refreshPlaces();
        };
        TablePlaces.prototype.clear = function () {
            this.place1(null);
            this.place2(null);
            this.place3(null);
            this.place4(null);
            this.place5(null);
            this.place6(null);
            this.place7(null);
            this.place8(null);
            this.place9(null);
            this.place10(null);
        };
        TablePlaces.prototype.refreshPlaces = function () {
            this.placesRefreshTrigger.notifySubscribers();
            this.placesRefreshTrigger.valueHasMutated();
        };
        return TablePlaces;
    }());
    exports.TablePlaces = TablePlaces;
});
/* tslint:disable:no-empty */
define("table/tablemonitor", ["require", "exports"], function (require, exports) {
    "use strict";
    /**
    * Base class for the table monitoring.
    */
    var TableMonitor = (function () {
        /**
        * Initializes a new instance of the table monitor
        */
        function TableMonitor(tableView) {
            this.tableView = tableView;
        }
        TableMonitor.prototype.onGameStarted = function (gameId, players, actions, dealerSeat) {
        };
        TableMonitor.prototype.onGameFinished = function (gameId, winners, rake) {
        };
        TableMonitor.prototype.onBet = function (playerId, type, amount, nextPlayerId) {
        };
        TableMonitor.prototype.onPlayerCards = function (playerId, cards) {
        };
        TableMonitor.prototype.onOpenCards = function (cards) {
        };
        TableMonitor.prototype.onMoveMoneyToPot = function (amount) {
        };
        TableMonitor.prototype.onPlayerStatus = function (playerId, status) {
        };
        return TableMonitor;
    }());
    exports.TableMonitor = TableMonitor;
});
define("table/handhistory", ["require", "exports", "table/tablemonitor", "languageManager"], function (require, exports, tablemonitor_1, languagemanager_5) {
    "use strict";
    /**
    * Class which performs collection of hand history
    */
    var HandHistory = (function (_super) {
        __extends(HandHistory, _super);
        function HandHistory(tableView) {
            _super.call(this, tableView);
            this.valid = true;
            this.detailedOperations = ko.observableArray([]);
            this.shortOperations = ko.observableArray([]);
            this.cards = ko.observableArray([]);
            this.playersData = ko.observableArray([]);
            this.players = [];
        }
        HandHistory.prototype.onGameStarted = function (gameId, players, actions, dealerSeat) {
            this.detailedOperations([]);
            this.playersData([]);
            this.id = gameId;
            this.detailedOperations.push(languagemanager_5._("handhistory.gamesstarted", { gameId: gameId }));
            this.players = [];
            var _loop_2 = function(i) {
                var playerId = players[i].PlayerId;
                var tablePlayerInfo = this_2.tableView.places().filter(function (_) { return _.PlayerId() === playerId; });
                var playerName = void 0;
                if (tablePlayerInfo.length === 0) {
                    playerName = playerId.toString();
                }
                else {
                    playerName = tablePlayerInfo[0].PlayerName();
                }
                this_2.players[playerId] = playerName;
            };
            var this_2 = this;
            for (var i = 0; i < players.length; i++) {
                _loop_2(i);
            }
        };
        HandHistory.prototype.onGameFinished = function (gameId, winners, rake) {
            var _loop_3 = function(potNumber) {
                var potWinners = winners.filter(function (_) { return _.Pot === potNumber; });
                if (potWinners.length === 0) {
                    return "break";
                }
                this_3.detailedOperations.push(languagemanager_5._("handhistory.potInfo", { pot: potNumber }));
                for (var i = 0; i < potWinners.length; i++) {
                    var pw = potWinners[i];
                    this_3.detailedOperations.push(languagemanager_5._("handhistory.playerWin", { player: this_3.getPlayer(pw.PlayerId), amount: pw.Amount }));
                }
            };
            var this_3 = this;
            for (var potNumber = 1; potNumber <= winners.length; potNumber++) {
                var state_3 = _loop_3(potNumber);
                if (state_3 === "break") break;
            }
            if (rake > 0) {
                this.detailedOperations.push(languagemanager_5._("handhistory.rake", { amount: rake }));
            }
            this.detailedOperations.push(languagemanager_5._("handhistory.gameFinished", { gameId: gameId }));
            var winnersList = [];
            var _loop_4 = function(pid) {
                if (!this_4.players.hasOwnProperty(pid)) {
                    return "continue";
                }
                var playerId = parseInt(pid, 10);
                var playerName = this_4.players[pid];
                var winAmount = winners.reduce(function (prev, item) {
                    if (item.PlayerId !== playerId) {
                        return prev;
                    }
                    return prev + item.Amount;
                }, 0);
                var combination = languagemanager_5._("handhistory.cardsWasFolded");
                var currentPlayer = this_4.tableView.places().filter(function (item) {
                    return item.PlayerId() === playerId;
                });
                var cards = void 0;
                if (currentPlayer.length === 0) {
                    cards = ["cards back", "cards back"];
                }
                else {
                    cards = currentPlayer[0].Cards();
                    if (cards === null) {
                        cards = currentPlayer[0].HandCards();
                        if (cards == null || cards.length === 0) {
                            cards = ["cards back", "cards back"];
                        }
                    }
                    else {
                        var rawCards = currentPlayer[0].RawCards();
                        if (rawCards !== null && rawCards !== undefined && rawCards[0] < 100) {
                            var combinationText = currentPlayer[0].getCombination(this_4.rawCards);
                            if (combinationText != null) {
                                combination = languagemanager_5._("handhistory.hasFollowingCombination", { combination: combinationText });
                            }
                            else {
                                combination = languagemanager_5._("handhistory.dontShowCards");
                            }
                        }
                        else {
                            combination = languagemanager_5._("handhistory.dontShowCards");
                        }
                    }
                }
                var description = void 0;
                if (winAmount > 0) {
                    description = "Игрок " + playerName + " выиграл " + winAmount + " " + combination;
                }
                else {
                    description = "Игрок " + playerName + " проиграл " + " " + combination;
                }
                winnersList.push({
                    id: playerId,
                    login: playerName,
                    cards: cards,
                    winAmount: winAmount,
                    combination: combination,
                    description: description
                });
            };
            var this_4 = this;
            for (var pid in this.players) {
                _loop_4(pid);
            }
            // Sort players by total winning.
            winnersList = winnersList.sort(function (a, b) {
                if (a.winAmount === null) {
                    return 1;
                }
                if (b.winAmount === null) {
                    return 1;
                }
                return b.winAmount - a.winAmount;
            });
            this.playersData(winnersList);
        };
        HandHistory.prototype.onBet = function (playerId, type, amount, nextPlayerId) {
            var operation;
            if (type === 0) {
                operation = languagemanager_5._("handhistory.blind", { player: this.getPlayer(playerId), amount: amount });
                this.detailedOperations.push(operation);
                this.shortOperations.push(operation);
            }
            if (type === 1) {
                operation = languagemanager_5._("handhistory.ante", { player: this.getPlayer(playerId), amount: amount });
                this.detailedOperations.push(operation);
                this.shortOperations.push(operation);
            }
            if (type === 5) {
                operation = languagemanager_5._("handhistory.deadblind", { player: this.getPlayer(playerId), amount: amount });
                this.detailedOperations.push(operation);
                this.shortOperations.push(operation);
            }
        };
        HandHistory.prototype.onPlayerCards = function (playerId, cards) {
            var c1 = this.getCard(cards[0]);
            var c2 = this.getCard(cards[1]);
            var operation = languagemanager_5._("handhistory.playerCardsOpened", { player: this.getPlayer(playerId), card1: c1, card2: c2 });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onPlayerHoleCards = function (playerId, cards) {
            var c1 = this.getCard(cards[0]);
            var c2 = this.getCard(cards[1]);
            var operation = languagemanager_5._("handhistory.playerHoleOpened", { player: this.getPlayer(playerId), card1: c1, card2: c2 });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onOpenCards = function (cards) {
            this.rawCards = cards || null;
            this.cards(cards.map(function (item) { return cardValue(item); }));
        };
        HandHistory.prototype.onFlop = function (card1, card2, card3) {
            var c1 = this.getCard(card1);
            var c2 = this.getCard(card2);
            var c3 = this.getCard(card3);
            var operation = languagemanager_5._("handhistory.flop");
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
            operation = languagemanager_5._("handhistory.flopOpenCards", { card1: c1, card2: c2, card3: c3 });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onTurn = function (card4) {
            var c4 = this.getCard(card4);
            var operation = languagemanager_5._("handhistory.turn");
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
            operation = languagemanager_5._("handhistory.turnOpenCards", { card4: c4 });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onRiver = function (card5) {
            var c5 = this.getCard(card5);
            var operation = languagemanager_5._("handhistory.river");
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
            operation = languagemanager_5._("handhistory.riverOpenCards", { card5: c5 });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onMoveMoneyToPot = function (amount) {
            this.detailedOperations.push(languagemanager_5._("handhistory.potscollection"));
        };
        HandHistory.prototype.onPlayerStatus = function (playerId, status) {
            // Do nothing.
        };
        HandHistory.prototype.onReturnMoney = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.returnMoney", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onFold = function (playerId) {
            var operation = languagemanager_5._("handhistory.fold", { player: this.getPlayer(playerId) });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onAllIn = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.allin", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onCheck = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.check", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onCall = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.call", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onBet2 = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.bet", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onRaise = function (playerId, amount) {
            var operation = languagemanager_5._("handhistory.raise", { player: this.getPlayer(playerId), amount: amount });
            this.detailedOperations.push(operation);
            this.shortOperations.push(operation);
        };
        HandHistory.prototype.onPotCreated = function (potNumber, amount) {
            this.detailedOperations.push(languagemanager_5._("handhistory.potcreated", { pot: potNumber, amount: amount }));
        };
        HandHistory.prototype.onPotUpdated = function (potNumber, amount) {
            this.detailedOperations.push(languagemanager_5._("handhistory.potupdated", { pot: potNumber, amount: amount }));
        };
        HandHistory.prototype.getPlayer = function (playerId) {
            return this.players[playerId];
        };
        HandHistory.prototype.getCard = function (card) {
            var cardValueCode = (card % 13) + 2;
            var cardSuiteCode = Math.floor(card / 13);
            var cardValue = "";
            var cardSuite = "";
            switch (cardSuiteCode) {
                case 0:
                    cardSuite = languagemanager_5._("handhistory.suitClubs");
                    break;
                case 1:
                    cardSuite = languagemanager_5._("handhistory.suitDiamonds");
                    break;
                case 2:
                    cardSuite = languagemanager_5._("handhistory.suitHearts");
                    break;
                case 3:
                    cardSuite = languagemanager_5._("handhistory.suitSpades");
                    break;
            }
            if (cardValueCode === 14) {
                cardValue = "A";
            }
            else if (cardValueCode === 13) {
                cardValue = "K";
            }
            else if (cardValueCode === 12) {
                cardValue = "Q";
            }
            else if (cardValueCode === 11) {
                cardValue = "J";
            }
            else {
                cardValue = cardValueCode.toString();
            }
            return cardValue + " " + cardSuite;
        };
        return HandHistory;
    }(tablemonitor_1.TableMonitor));
    exports.HandHistory = HandHistory;
});
define("table/playerMessage", ["require", "exports", "knockout", "authmanager"], function (require, exports, ko, authManager) {
    "use strict";
    var PlayerMessage = (function () {
        function PlayerMessage(messageId, sender, message) {
            var _this = this;
            this.messageId = messageId;
            this.isAdmin = false;
            if (sender.toLowerCase() === "admin") {
                this.isAdmin = true;
            }
            this.isMy = ko.observable(authManager.login() === sender);
            authManager.authenticated.subscribe(function () {
                _this.isMy(authManager.login() === sender);
            });
            this.sender = sender;
            this.message = ko.observable(message);
            var d = new Date();
            this.date = d.getHours() + ":" + d.getMinutes();
            this.fullMessage = ko.computed(function () {
                return "[" + _this.date + "]" + _this.sender + " - " + _this.message();
            });
        }
        return PlayerMessage;
    }());
    exports.PlayerMessage = PlayerMessage;
});
define("table/SystemMessage", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var SystemMessage = (function () {
        function SystemMessage(messageId, message) {
            this.messageId = messageId;
            this.message = ko.observable(message);
        }
        return SystemMessage;
    }());
    exports.SystemMessage = SystemMessage;
});
define("table/tableSlider", ["require", "exports", "knockout", "timeService"], function (require, exports, ko, timeService) {
    "use strict";
    var TableSlider = (function () {
        function TableSlider() {
            var self = this;
            this.current = ko.observable(null);
            this.currentValue = ko.computed({
                read: function () {
                    var ivalue = self.current();
                    if (ivalue === null) {
                        return null;
                    }
                    return ivalue.toString();
                },
                write: function (value) {
                    var ivalue = parseInt(value, 10);
                    if (isNaN(ivalue) || !isFinite(ivalue)) {
                        return;
                    }
                    if (ivalue >= self.maximum()) {
                        ivalue = self.maximum();
                    }
                    if (ivalue <= self.minimum()) {
                        ivalue = self.minimum();
                    }
                    self.current(ivalue);
                }
            });
            this.minimum = ko.observable();
            this.maximum = ko.observable();
            this.position = ko.computed({
                read: function () {
                    var pixelDistance = self.maxRelative - self.minRelative;
                    if (pixelDistance === 0) {
                        return self.maxRelative;
                    }
                    var delta = self.maximum() - self.minimum();
                    if (delta === 0) {
                        return self.maxRelative;
                    }
                    var ratio = pixelDistance / delta;
                    var currentRelative = Math.floor(ratio * (self.current() - self.minimum()));
                    return self.minRelative + currentRelative;
                },
                write: function (value) {
                    if (value >= self.maxRelative) {
                        self.current(self.maximum());
                        return;
                    }
                    if (value <= self.minRelative) {
                        self.current(self.minimum());
                        return;
                    }
                    var pixelDistance = self.maxRelative - self.minRelative;
                    if (pixelDistance === 0) {
                        self.current(self.minimum());
                        return;
                    }
                    var delta = self.maximum() - self.minimum();
                    if (delta === 0) {
                        self.current(self.minimum());
                        return;
                    }
                    var ratio = delta / pixelDistance;
                    var currentAbsolute = Math.floor(ratio * (value - self.minRelative));
                    // Round currentAbsolute to step
                    var currentAbsoluteAligned = Math.floor(currentAbsolute / self.step) * self.step;
                    if ((currentAbsolute - currentAbsoluteAligned) < (currentAbsoluteAligned + self.step - currentAbsolute)) {
                        currentAbsolute = currentAbsoluteAligned;
                    }
                    else {
                        currentAbsolute = currentAbsoluteAligned + self.step;
                    }
                    if (currentAbsolute + self.minimum() > self.maximum()) {
                        self.current(self.maximum());
                        return;
                    }
                    if (currentAbsolute + self.minimum() < self.minimum()) {
                        self.current(self.minimum());
                        return;
                    }
                    self.current(currentAbsolute + self.minimum());
                }
            });
        }
        /**
        * Set bounds for the slider in the relative coordinates.
        * @param minRelative Number Minimum coordinate in the relative coordinates.
        * @param maxRelative Number Maximum coordinate in the relative coordinates.
        * @param translator Function Function which translate the page coordinates to the relative coordinates.
        */
        TableSlider.prototype.setBounds = function (minRelative, maxRelative, translator) {
            this.minRelative = minRelative;
            this.maxRelative = maxRelative;
            this.translator = translator;
        };
        TableSlider.prototype.setParameters = function (value, step, min, max) {
            this.current(value);
            this.minimum(min);
            this.maximum(max);
            this.position.notifySubscribers();
            this.step = step;
        };
        TableSlider.prototype.setStep = function (step) {
            this.step = step;
        };
        TableSlider.prototype.increase = function () {
            var current = this.current();
            current = Math.min(current + this.step, this.maximum());
            this.current(current);
            this.position.notifySubscribers();
        };
        TableSlider.prototype.decrease = function () {
            var current = this.current();
            current = Math.max(current - this.step, this.minimum());
            this.current(current);
            this.position.notifySubscribers();
        };
        TableSlider.prototype.withinRange = function (value) {
            value = Math.max(value, this.minimum());
            value = Math.min(value, this.maximum());
            return value;
        };
        TableSlider.prototype.isWithinRange = function (value) {
            if (value >= this.minimum() && value <= this.maximum()) {
                return true;
            }
            return false;
        };
        TableSlider.prototype.fixupValue = function () {
            var self = this;
            this.currentValue(this.current().toFixed());
            timeService.setTimeout(function () {
                var c = self.current();
                self.current(0);
                self.current(c);
            }, 10);
        };
        TableSlider.prototype.selectManually = function (event) {
            var relativePosition = this.translator(event.pageX);
            this.setPosition(relativePosition);
        };
        TableSlider.prototype.setPosition = function (relativePosition) {
            relativePosition = Math.max(relativePosition, this.minRelative);
            relativePosition = Math.min(relativePosition, this.maxRelative);
            this.position(relativePosition);
        };
        return TableSlider;
    }());
    exports.TableSlider = TableSlider;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/morepopup", ["require", "exports", "knockout", "authmanager", "settings", "services/index"], function (require, exports, ko, authManager, settings_2, services_3) {
    "use strict";
    var MorePopup = (function () {
        function MorePopup() {
            this.points = ko.observable(0);
            this.hasMessages = ko.observable(false);
            this.visible = ko.observable(false);
            var self = this;
            this.authenticated = ko.computed(function () {
                return authManager.authenticated();
            }, this);
            this.login = ko.computed(function () {
                return authManager.login();
            }, this);
            this.loading = ko.observable(false);
            authManager.login.subscribe(function (newValue) {
                if (newValue === null) {
                    self.amount(0);
                }
                else {
                    self.update();
                }
            });
            this.amount = ko.observable(0);
        }
        MorePopup.prototype.update = function () {
            if (!authManager.authenticated()) {
                return;
            }
            var self = this;
            this.loading(true);
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            $.when(this.updateAccountData(), this.updateMessagesStatus()).then(function () {
                self.loading(false);
            }, function () {
                self.update();
            });
        };
        MorePopup.prototype.showAccount = function () {
            app.executeCommand("pageblock.cashier");
        };
        MorePopup.prototype.showRating = function () {
            app.executeCommand("pageblock.other");
            app.otherPageBlock.showSecondary("rating");
        };
        MorePopup.prototype.showChat = function () {
            app.executeCommand("pageblock.other");
            app.otherPageBlock.showSecondary("chat");
        };
        MorePopup.prototype.showMessages = function () {
            services_3.websiteService.messages();
        };
        MorePopup.prototype.showInformation = function () {
            app.executeCommand("pageblock.info");
            app.infoPageBlock.showPrimary();
        };
        MorePopup.prototype.showProfile = function () {
            services_3.websiteService.profile();
        };
        MorePopup.prototype.showContactUs = function () {
            app.executeCommand("pageblock.info");
            app.infoPageBlock.showContactUs();
        };
        MorePopup.prototype.showRegistration = function () {
            app.showPopup("registration");
        };
        /**
        * Starts request for the account data.
        */
        MorePopup.prototype.updateAccountData = function () {
            var self = this;
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            return api.GetPlayerDefinition(null).then(function (data) {
                if (data.Status === "Ok") {
                    var personalAccountData = data.Data;
                    var total = settings_2.settings.isGuest() ? personalAccountData.GameMoney : personalAccountData.RealMoney;
                    self.amount(total);
                    self.points(personalAccountData.Points);
                }
                else {
                    console.error("Error during making call to Account.GetPlayerDefinition in MorePopup");
                }
                return data;
            });
        };
        /**
        * Starts requesting message status
        */
        MorePopup.prototype.updateMessagesStatus = function () {
            var self = this;
            var mapi = new OnlinePoker.Commanding.API.Message(apiHost);
            return mapi.GetInboxMessages(0, 20, 1 /* Unread */, false, null).then(function (data) {
                if (data.Status === "Ok") {
                    if (data.Data.Messages.length > 0) {
                        self.hasMessages(true);
                    }
                    else {
                        self.hasMessages(false);
                    }
                }
                else {
                    console.error("Error during making call to Message.GetInboxMessages in MorePopup");
                }
                return data;
            });
        };
        return MorePopup;
    }());
    exports.MorePopup = MorePopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/accountstatuspopup", ["require", "exports", "knockout", "ui/popupbase", "authmanager", "services/index", "languageManager"], function (require, exports, ko, popupbase_2, authManager, services_4, languagemanager_6) {
    "use strict";
    var AccountStatusPopup = (function (_super) {
        __extends(AccountStatusPopup, _super);
        function AccountStatusPopup() {
            var _this = this;
            _super.call(this);
            this.addMoneyAvailable = ko.observable(false);
            this.addMoneyAllowed = ko.observable(false);
            this.displayCurrencyName = false;
            this.loading = ko.observable(true);
            this.information = ko.observable();
            this.loginName = ko.observable();
            this.tableName = ko.observable();
            this.smallBlind = ko.observable();
            this.bigBlind = ko.observable();
            this.tableInformation = ko.computed(function () {
                return languagemanager_6._("accountStatus.tableInfo", { name: _this.tableName() });
            }, this);
            this.betInformation = ko.computed(function () {
                return languagemanager_6._("accountStatus.betInfo", { sb: _this.smallBlind(), bb: _this.bigBlind() });
            }, this);
        }
        AccountStatusPopup.prototype.shown = function () {
            _super.prototype.shown.call(this);
            this.requestData();
            this.loginName(authManager.login());
            var tableView = app.tablesPage.currentTable();
            this.tableName(tableView.model.TableName);
            this.smallBlind(tableView.model.SmallBlind);
            this.bigBlind(tableView.model.BigBlind);
            this.addMoneyAvailable(tableView.tournament() == null && tableView.opened());
            this.addMoneyAllowed(tableView.couldAddChips());
        };
        AccountStatusPopup.prototype.addMoney = function () {
            if (!this.addMoneyAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            app.addMoneyPopup.tableView(currentTable);
            _super.prototype.close.call(this);
            app.showPopup("addMoney").done(function (results) {
                if (results.result === "cancel") {
                    app.showPopup("accountStatus");
                }
            });
        };
        AccountStatusPopup.prototype.requestData = function () {
            var _this = this;
            this.loading(true);
            services_4.accountService.getAccount().done(function (result) {
                _this.loading(false);
                _this.information(result);
            }).fail(function () {
                app.closePopup();
            });
        };
        return AccountStatusPopup;
    }(popupbase_2.PopupBase));
    exports.AccountStatusPopup = AccountStatusPopup;
});
/// <reference path="../poker.commanding.api.ts" />
/* tslint:disable:no-bitwise */
define("popups/tablemenupopup", ["require", "exports", "authmanager", "settings"], function (require, exports, authManager, settings_3) {
    "use strict";
    var TableMenuPopup = (function () {
        function TableMenuPopup() {
            this.addMoneyAvailable = ko.observable(false);
            /**
            * Tournament has rebuys.
            */
            this.tournamentHasRebuy = ko.observable(false);
            /**
            * Tournament has addons.
            */
            this.tournamentHasAddon = ko.observable(false);
            this.soundEnabled = ko.computed({
                read: function () {
                    return settings_3.settings.soundEnabled();
                },
                write: function (value) {
                    settings_3.settings.soundEnabled(value);
                },
                owner: this
            });
            this.autoSwitchTables = ko.computed({
                read: function () {
                    return settings_3.settings.autoSwitchTables();
                },
                write: function (value) {
                    settings_3.settings.autoSwitchTables(value);
                },
                owner: this
            });
            this.autoHideCards = ko.computed({
                read: function () {
                    return settings_3.settings.autoHideCards();
                },
                write: function (value) {
                    settings_3.settings.autoHideCards(value);
                },
                owner: this
            });
            this.showInRating = ko.observable(false);
            this.addMoneyAllowed = ko.observable(false);
            this.handHistoryAllowed = ko.observable(false);
            this.accountStatusAllowed = ko.observable(false);
            this.tournamentInformationAllowed = ko.observable(false);
            this.rebuyAllowed = ko.observable(false);
            this.doublerebuyAllowed = ko.observable(false);
            this.addonAllowed = ko.observable(false);
            this.isTournamentTable = ko.observable(false);
        }
        TableMenuPopup.prototype.shown = function () {
            // Load settings
            var self = this;
            var currentTable = app.tablesPage.currentTable();
            var playerIsInGame = currentTable.myPlayer() != null;
            this.addMoneyAvailable(currentTable.tournament() == null && currentTable.opened());
            this.addMoneyAllowed(currentTable.couldAddChips());
            this.handHistoryAllowed(playerIsInGame && currentTable.lastHandHistory() != null);
            this.accountStatusAllowed(authManager.authenticated());
            var tournamentView = currentTable.tournament();
            this.isTournamentTable(tournamentView != null);
            this.tournamentInformationAllowed(tournamentView != null);
            if (tournamentView !== null && currentTable.opened()) {
                // Set rebuy/addon status temporary to current tournament status
                // until data is loaded.
                // The rebuy and addons could be not supported by the 
                // actual tournament so buttons should be hidden.
                var tdata_1 = tournamentView.tournamentData();
                this.tournamentHasRebuy((tdata_1.Options & TournamentOptionsEnum.HasRebuy) > 0);
                this.tournamentHasAddon((tdata_1.Options & TournamentOptionsEnum.HasAddon) > 0);
                if ((this.tournamentHasRebuy() || this.tournamentHasAddon())
                    && playerIsInGame) {
                    var moneyInGame = currentTable.myPlayer().TotalBet() + currentTable.myPlayer().Money();
                    this.addonAllowed(tournamentView.addonAllowed() && tournamentView.addonCount() === 0);
                    this.rebuyAllowed(tournamentView.rebuyAllowed()
                        && (moneyInGame + tdata_1.ChipsAddedAtReBuy) <= tdata_1.MaximumAmountForRebuy
                        && !currentTable.hasPendingMoney());
                    this.doublerebuyAllowed(tournamentView.rebuyAllowed()
                        && (moneyInGame + tdata_1.ChipsAddedAtDoubleReBuy) <= tdata_1.MaximumAmountForRebuy);
                    if (tournamentView.addonAllowed() || tournamentView.rebuyAllowed()) {
                        var api = new OnlinePoker.Commanding.API.Account(apiHost);
                        api.GetPersonalAccount().then(function (data) {
                            var personalAccount = data.Data;
                            var currentMoney = self.getCurrentMoney(tournamentView, personalAccount);
                            var addonPrice = tdata_1.AddonPrice + tdata_1.AddonFee;
                            var rebuyPrice = tdata_1.RebuyFee + tdata_1.RebuyPrice;
                            self.addonAllowed(self.addonAllowed() && addonPrice < currentMoney);
                            self.rebuyAllowed(self.rebuyAllowed() && (rebuyPrice < currentMoney));
                            self.doublerebuyAllowed(self.doublerebuyAllowed() && ((2 * rebuyPrice) > currentMoney));
                        });
                    }
                }
            }
            else {
                this.tournamentHasRebuy(false);
                this.tournamentHasAddon(false);
                this.addonAllowed(false);
                this.rebuyAllowed(false);
                this.doublerebuyAllowed(false);
            }
        };
        TableMenuPopup.prototype.confirm = function () {
            app.closePopup();
        };
        TableMenuPopup.prototype.cancel = function () {
            app.closePopup();
        };
        TableMenuPopup.prototype.accountStatus = function () {
            if (!this.accountStatusAllowed()) {
                return;
            }
            app.showPopup("accountStatus");
        };
        TableMenuPopup.prototype.handHistory = function () {
            if (!this.handHistoryAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            app.handHistoryPopup.tableView(currentTable);
            app.showPopup("handHistory");
        };
        TableMenuPopup.prototype.addMoney = function () {
            if (!this.addMoneyAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            app.addMoneyPopup.tableView(currentTable);
            app.closePopup();
            app.showPopup("addMoney").done(function (results) {
                if (results.result === "cancel") {
                    app.executeCommand("popup.tableMenu");
                }
            });
        };
        TableMenuPopup.prototype.showTournamentInformation = function () {
            if (!this.tournamentInformationAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            var tournamentView = currentTable.tournament();
            app.lobbyPageBlock.showLobby();
            app.lobbyPageBlock.selectTournament({ TournamentId: tournamentView.tournamentId });
            app.tablesPage.deactivate();
            this.confirm();
        };
        TableMenuPopup.prototype.rebuy = function () {
            if (!this.rebuyAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            currentTable.showRebuyPrompt();
        };
        TableMenuPopup.prototype.doubleRebuy = function () {
            if (!this.doublerebuyAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            currentTable.showDoubleRebuyPrompt();
        };
        TableMenuPopup.prototype.addon = function () {
            if (!this.addonAllowed()) {
                return;
            }
            var currentTable = app.tablesPage.currentTable();
            currentTable.showAddonPrompt();
        };
        TableMenuPopup.prototype.getCurrentMoney = function (tournament, personalAccount) {
            return personalAccount.RealMoney;
        };
        return TableMenuPopup;
    }());
    exports.TableMenuPopup = TableMenuPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/selectavatarpopup", ["require", "exports", "knockout", "ui/popupbase", "metadatamanager"], function (require, exports, ko, popupbase_3, metadataManager) {
    "use strict";
    var SelectAvatarPopup = (function (_super) {
        __extends(SelectAvatarPopup, _super);
        function SelectAvatarPopup() {
            _super.apply(this, arguments);
            this.avatars = ko.observableArray();
            this.selectedAvatar = ko.observable();
            this.selected = new signals.Signal();
        }
        SelectAvatarPopup.prototype.shown = function () {
            _super.prototype.shown.call(this);
            this.avatars(metadataManager.avatars);
        };
        SelectAvatarPopup.prototype.confirm = function () {
            this.selected.dispatch(this.selectedAvatar());
        };
        SelectAvatarPopup.prototype.selectAvatar = function (item) {
            this.selectedAvatar(item);
            this.close();
        };
        SelectAvatarPopup.prototype.close = function () {
            _super.prototype.close.call(this);
            this.confirm();
            app.registrationPopup.visible(true);
        };
        return SelectAvatarPopup;
    }(popupbase_3.PopupBase));
    exports.SelectAvatarPopup = SelectAvatarPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/slowconnectionpopup", ["require", "exports", "knockout", "timeService", "ui/popupbase", "table/runtimesettings", "languageManager"], function (require, exports, ko, timeService, popupbase_4, runtimeSettings, languagemanager_7) {
    "use strict";
    var SlowConnectionPopup = (function (_super) {
        __extends(SlowConnectionPopup, _super);
        function SlowConnectionPopup() {
            var _this = this;
            _super.call(this);
            this.handle = null;
            this.index = ko.observable(0);
            this.message = ko.observable(languagemanager_7._("connection.slow"));
            this.retryCaption = ko.observable(languagemanager_7._("connection.retry"));
            this.onretry = null;
            this.caption = ko.computed(function () {
                return languagemanager_7._("connection.caption" + _this.index().toString());
            });
            this.allowRetry = ko.observable(false);
        }
        SlowConnectionPopup.prototype.shown = function () {
            _super.prototype.shown.call(this);
            this.startUpdatingCaption();
            this.reset();
        };
        SlowConnectionPopup.prototype.close = function () {
            this.stopUpdatingCaption();
            _super.prototype.close.call(this);
        };
        SlowConnectionPopup.prototype.startUpdatingCaption = function () {
            var _this = this;
            if (this.handle !== null) {
                this.stopUpdatingCaption();
            }
            this.handle = timeService.setInterval(function () {
                if (runtimeSettings.updateTimer) {
                    _this.index((_this.index() + 1) % 4);
                }
            }, 300);
        };
        SlowConnectionPopup.prototype.stopUpdatingCaption = function () {
            timeService.clearInterval(this.handle);
            this.handle = null;
        };
        SlowConnectionPopup.prototype.reset = function () {
            this.index();
            this.allowRetry(false);
            this.message(languagemanager_7._("connection.slow"));
            // this.retryCaption(_("connection.retry"));
        };
        SlowConnectionPopup.prototype.reconnectFailed = function () {
            this.allowRetry(true);
            this.message(languagemanager_7._("connection.reconnectfailed"));
            this.retryCaption(languagemanager_7._("connection.retry"));
        };
        SlowConnectionPopup.prototype.duplicatedConnection = function () {
            this.allowRetry(true);
            this.message(languagemanager_7._("connection.duplicateconnectiondetected"));
            this.retryCaption(languagemanager_7._("connection.reestablishConnection"));
        };
        SlowConnectionPopup.prototype.noConnection = function () {
            this.allowRetry(false);
            this.message(languagemanager_7._("connection.noconnection"));
        };
        SlowConnectionPopup.prototype.connectionPresent = function () {
            this.allowRetry(true);
            this.message(languagemanager_7._("connection.connectionpresent"));
            this.retryCaption(languagemanager_7._("connection.reconnect"));
        };
        SlowConnectionPopup.prototype.retry = function () {
            this.reset();
            if (this.onretry !== null) {
                this.onretry();
            }
        };
        return SlowConnectionPopup;
    }(popupbase_4.PopupBase));
    exports.SlowConnectionPopup = SlowConnectionPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/registrationpopup", ["require", "exports", "knockout", "ui/popupbase", "metadatamanager", "authmanager", "popups/simplepopup", "languageManager"], function (require, exports, ko, popupbase_5, metadataManager, authManager, simplepopup_3, languagemanager_8) {
    "use strict";
    var RegistrationPopup = (function (_super) {
        __extends(RegistrationPopup, _super);
        function RegistrationPopup() {
            _super.call(this);
            this.imageFile = ko.observable();
            this.login = ko.observable().extend({ required: true, minLength: 5, maxLength: 12, validatable: true });
            this.email = ko.observable().extend({ required: true, email: true, validatable: true });
            this.firstName = ko.observable().extend({ required: true });
            this.lastName = ko.observable().extend({ required: true });
            this.patronymicName = ko.observable();
            this.password = ko.observable().extend({ required: true, maxLength: 16 });
            this.confirmPassword = ko.observable().extend({ required: true, equal: this.password });
            this.city = ko.observable();
            this.country = ko.observable(1);
            this.ageValid = ko.observable(true).extend({ required: true });
            this.agreeEula = ko.observable(true).extend({ required: true });
            this.imageUrl = ko.observable("");
            this.loading = ko.observable(false);
            this.errors = ko.validation.group(this);
        }
        RegistrationPopup.prototype.shown = function (args) {
            if (args === void 0) { args = []; }
            if (args.length === 1 && args[0] === true) {
                _super.prototype.shown.call(this, args);
                return;
            }
            this.login(null);
            this.email(null);
            this.firstName(null);
            this.lastName(null);
            this.patronymicName(null);
            this.password(null);
            this.confirmPassword(null);
            this.city(null);
            this.country(1);
            this.ageValid(false);
            this.agreeEula(false);
            this.errors.showAllMessages(false);
            _super.prototype.shown.call(this, args);
        };
        /**
        * Selects avatar from list of predefined avatars
        */
        RegistrationPopup.prototype.selectPicture = function () {
            var _this = this;
            app.showPopup("selectAvatar", metadataManager.avatars);
            app.selectAvatarPopup.selected.addOnce(function (avatarUrl) {
                if (avatarUrl != null) {
                    _this.imageUrl(avatarUrl);
                }
                app.showPopup("registration", true);
            }, this, 0);
        };
        /**
        * Selects avatar from the gallery.
        */
        RegistrationPopup.prototype.uploadPicture = function () {
            var _this = this;
            var captureSuccess = function (imageData) {
                app.suppressResume = true;
                _this.imageFile(imageData);
            };
            // capture error callback
            var captureError = function (message) {
                app.suppressResume = true;
                console.log("Error code: " + message, null, "Capture Error");
            };
            navigator.camera.getPicture(captureSuccess, captureError, {
                quality: 75,
                sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
                destinationType: Camera.DestinationType.DATA_URL
            });
        };
        RegistrationPopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                this.moveToError();
                return;
            }
            this.loading(true);
            var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
            var additionalProperties = {
                ImageUrl: this.imageUrl(),
                ImageDataBase64: this.imageFile()
            };
            accountApi.Register(this.login(), this.email(), this.password(), this.firstName(), this.lastName(), this.patronymicName(), this.country(), this.city(), additionalProperties, [], function (data) {
                if (data.Status === "Ok") {
                    self.close();
                    simplepopup_3.SimplePopup.display(languagemanager_8._("auth.registration"), languagemanager_8._("auth.registrationsuccess"));
                }
                else {
                    // Report registration errors;
                    if (data.Status === "LoginAlreadyUsed") {
                        self.login.setError(languagemanager_8._("errors." + data.Status));
                        self.moveToError();
                    }
                    else if (data.Status === "EmailAlreadyUsed") {
                        self.email.setError(languagemanager_8._("errors." + data.Status));
                        self.moveToError();
                    }
                    else {
                        simplepopup_3.SimplePopup.display(languagemanager_8._("auth.registration"), languagemanager_8._("errors." + data.Status));
                    }
                    authManager.authenticated(false);
                }
            }).always(function () {
                self.loading(false);
            });
        };
        RegistrationPopup.prototype.showAgreement = function () {
            app.executeCommand("pageblock.info");
            app.infoPageBlock.showLicenseAgreement();
        };
        /**
        * Move screen to show error field.
        */
        RegistrationPopup.prototype.moveToError = function () {
            $(".popup." + app.currentPopup + " .popup-container")[0].scrollTop = 0;
        };
        return RegistrationPopup;
    }(popupbase_5.PopupBase));
    exports.RegistrationPopup = RegistrationPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/okcancelpopup", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var OkCancelPopup = (function () {
        function OkCancelPopup() {
            this.buttons = ko.observableArray([]);
            this.customStyle = ko.observable("");
            this.title = ko.observable();
            this.messages = ko.observableArray();
        }
        OkCancelPopup.prototype.shown = function () {
            this.deferred = $.Deferred();
        };
        OkCancelPopup.prototype.confirm = function () {
            this.deferred.resolve();
            app.closePopup();
        };
        OkCancelPopup.prototype.cancel = function () {
            this.deferred.reject();
            app.closePopup();
        };
        return OkCancelPopup;
    }());
    exports.OkCancelPopup = OkCancelPopup;
});
define("popups/newspopup", ["require", "exports", "knockout", "ui/popupbase"], function (require, exports, ko, popupbase_6) {
    "use strict";
    var NewsPopup = (function (_super) {
        __extends(NewsPopup, _super);
        function NewsPopup() {
            _super.apply(this, arguments);
            this.link = ko.observable();
            this.url = ko.observable();
            this.title = ko.observable();
            this.loading = ko.observable(false);
        }
        NewsPopup.prototype.setData = function (data) {
            this.link(data.Link);
            this.url(data.Url);
            this.title(data.Title);
        };
        NewsPopup.prototype.openBanner = function () {
            window.open(this.link(), "_system", "location=yes");
        };
        return NewsPopup;
    }(popupbase_6.PopupBase));
    exports.NewsPopup = NewsPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/jointablepopup", ["require", "exports", "knockout", "popups/simplepopup", "languageManager", "appconfig"], function (require, exports, ko, simplepopup_4, languagemanager_9, appconfig_2) {
    "use strict";
    var JoinTablePopup = (function () {
        function JoinTablePopup() {
            this.buyin = ko.observable().extend({ required: true, validatable: true });
            this.tableView = ko.observable();
            this.seatNumber = ko.observable(0);
            this.accountTotal = ko.observable(0);
            this.loading = ko.observable(false);
            this.tableName = ko.observable();
            this.minBuyin = ko.observable(0);
            this.maxBuyin = ko.observable(0);
            this.minBet = ko.observable(0);
            this.maxBet = ko.observable(0);
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
        }
        JoinTablePopup.prototype.shown = function () {
            var self = this;
            var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
            self.loading(true);
            accountApi.GetPersonalAccount(function (data) {
                self.loading(false);
                if (data.Status === "Ok") {
                    var personalAccountData = data.Data;
                    var tableData = self.tableView().model;
                    var balance = 0;
                    var currencyId = tableData.CurrencyId;
                    if (currencyId === 1) {
                        balance = personalAccountData.RealMoney;
                    }
                    else {
                        balance = personalAccountData.GameMoney;
                    }
                    var tableView = self.tableView();
                    self.accountTotal(balance);
                    self.tableName(tableData.TableName);
                    self.minBet(tableData.SmallBlind);
                    self.maxBet(tableData.BigBlind);
                    self.updateEntries();
                    if (appconfig_2.appConfig.game.seatMode) {
                        app.executeCommand("page.seats");
                    }
                }
                else {
                    simplepopup_4.SimplePopup.display(languagemanager_9._("joinTable.caption"), languagemanager_9._("errors." + data.Status));
                }
            }).fail(function () {
                self.loading(false);
                simplepopup_4.SimplePopup.display(languagemanager_9._("joinTable.caption"), languagemanager_9._("joinTable.joinError"));
            });
        };
        JoinTablePopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (this.buyin() < this.minBuyin()) {
                this.buyin.setError(languagemanager_9._("joinTable.putMoreMoney"));
                return;
            }
            if (this.buyin() > this.maxBuyin()) {
                this.buyin.setError(languagemanager_9._("joinTable.putLessMoney"));
                return;
            }
            if (this.accountTotal() < this.minBuyin()) {
                this.buyin.setError(languagemanager_9._("errors.NotSufficiendFunds"));
                return;
            }
            if (this.accountTotal() < this.buyin()) {
                this.buyin.setError(languagemanager_9._("errors.NotSufficiendFunds"));
                return;
            }
            var seat = this.seatNumber();
            var amount = this.buyin();
            this.loading(true);
            this.tableView().sit(seat, amount).then(function () {
                self.loading(false);
                app.closePopup();
            }, function (status, minimalAmount) {
                self.loading(false);
                if (status === "AmountTooLow") {
                    self.tableView().minimalPlayerBuyIn(minimalAmount);
                    self.updateEntries();
                    self.buyin.setError(languagemanager_9._("errors." + status));
                }
                else {
                    simplepopup_4.SimplePopup.display(languagemanager_9._("joinTable.caption"), languagemanager_9._("errors." + status));
                }
            });
        };
        JoinTablePopup.prototype.updateEntries = function () {
            var self = this;
            var tableView = self.tableView();
            var tableData = self.tableView().model;
            var baseMinimalBuyIn = tableView.minimalBuyIn() * tableData.BigBlind;
            self.minBuyin(Math.max(baseMinimalBuyIn, tableView.minimalPlayerBuyIn()));
            self.maxBuyin(20 * baseMinimalBuyIn);
            self.buyin(Math.max(2 * baseMinimalBuyIn, tableView.minimalPlayerBuyIn()));
            if (self.maxBuyin() < tableView.minimalPlayerBuyIn()) {
                self.maxBuyin(tableView.minimalPlayerBuyIn());
                self.buyin(tableView.minimalPlayerBuyIn());
            }
        };
        return JoinTablePopup;
    }());
    exports.JoinTablePopup = JoinTablePopup;
});
define("services/timedeferred", ["require", "exports", "timeService"], function (require, exports, timeService) {
    "use strict";
    function wait(time) {
        return $.Deferred(function (dfd) {
            timeService.setTimeout(dfd.resolve, time);
        });
    }
    exports.wait = wait;
});
define("table/gameactionsqueue", ["require", "exports", "services/timedeferred"], function (require, exports, timedeferred_1) {
    "use strict";
    var GameActionsQueue = (function () {
        /**
        * Initializes a new instance of the GameActionsQueue class.
        */
        function GameActionsQueue() {
            this.tasks = [];
            this.counter = 0;
        }
        /**
        * Inject worker to the beginning of tasks stack
        * @param worker QueueWorker The function which generated deferred.
        */
        GameActionsQueue.prototype.inject = function (worker) {
            this.tasks.unshift(worker);
            this.execute();
        };
        /**
        * Push worker to the end of tasks stack
        * @param worker QueueWorker The function which generated deferred.
        */
        GameActionsQueue.prototype.push = function (worker) {
            this.tasks.push(worker);
            this.execute();
        };
        /**
        * Push waiting task in the queue
        * @param timeout Number The function which generated deferred.
        */
        GameActionsQueue.prototype.wait = function (timeout) {
            if (!GameActionsQueue.waitDisabled && timeout > 0) {
                this.push(function () {
                    return timedeferred_1.wait(timeout);
                });
            }
        };
        /**
        * Push waiting task in the queue
        * @param timeout Number The function which generated deferred.
        */
        GameActionsQueue.prototype.injectWait = function (timeout) {
            if (!GameActionsQueue.waitDisabled && timeout > 0) {
                this.inject(function () {
                    return timedeferred_1.wait(timeout);
                });
            }
        };
        /**
        * Push worker to the stack
        * @param callback Function The function which generated deferred.
        */
        GameActionsQueue.prototype.injectCallback = function (callback) {
            this.inject(function () {
                return $.Deferred().done(function () {
                    callback();
                }).resolve();
            });
        };
        /**
        * Push worker to the stack
        * @param callback Function The function which generated deferred.
        */
        GameActionsQueue.prototype.pushCallback = function (callback) {
            this.push(function () {
                return $.Deferred().done(function () {
                    callback();
                }).resolve();
            });
        };
        /**
        * Clears execution chain.
        */
        GameActionsQueue.prototype.clear = function () {
            this.tasks = [];
        };
        /**
        * Starts execution chain.
        */
        GameActionsQueue.prototype.execute = function () {
            if (this.isExecuting) {
                this.log("Currently task is executing, task will be executed later.");
                return;
            }
            if (this.tasks.length === 0) {
                this.log("No task to execute.");
                return;
            }
            var self = this;
            var worker = this.tasks.shift();
            if (worker === null) {
                this.error("Worker is null");
            }
            this.counter++;
            this.isExecuting = true;
            var task = worker();
            if (task === null) {
                this.error("Task is null");
            }
            this.log("Executing task with id: " + this.counter.toString());
            $.when(task).done(function () {
                self.isExecuting = false;
                self.log("Finished task with id: " + self.counter.toString());
                setTimeout(function () {
                    self.execute();
                }, 100);
            });
        };
        GameActionsQueue.prototype.log = function (message) {
            var optionalParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                optionalParams[_i - 1] = arguments[_i];
            }
            if (GameActionsQueue.logging) {
                console.log(message, optionalParams);
            }
        };
        GameActionsQueue.prototype.error = function (message) {
            var optionalParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                optionalParams[_i - 1] = arguments[_i];
            }
            if (GameActionsQueue.logging) {
                console.error(message, optionalParams);
            }
        };
        GameActionsQueue.logging = false;
        GameActionsQueue.waitDisabled = false;
        return GameActionsQueue;
    }());
    exports.GameActionsQueue = GameActionsQueue;
});
define("popups/handhistorypopup", ["require", "exports", "knockout", "table/gameactionsqueue", "ui/popupbase", "appconfig"], function (require, exports, ko, gameactionsqueue_1, popupbase_7, appconfig_3) {
    "use strict";
    var HandHistoryDisplayMode;
    (function (HandHistoryDisplayMode) {
        HandHistoryDisplayMode[HandHistoryDisplayMode["Short"] = 1] = "Short";
        HandHistoryDisplayMode[HandHistoryDisplayMode["Detailed"] = 2] = "Detailed";
    })(HandHistoryDisplayMode || (HandHistoryDisplayMode = {}));
    var HandHistoryPopup = (function (_super) {
        __extends(HandHistoryPopup, _super);
        function HandHistoryPopup() {
            _super.call(this);
            this.detailedOperations = ko.observableArray([]);
            this.shortOperations = ko.observableArray([]);
            this.tableView = ko.observable();
            if (appconfig_3.appConfig.game.handHistory.showPictureHistory) {
                this.mode = ko.observable(HandHistoryDisplayMode.Short);
            }
            else {
                this.mode = ko.observable(HandHistoryDisplayMode.Detailed);
            }
            this.cards = ko.observableArray([]);
            this.playersData = ko.observableArray([]);
            this.displayLoginSeparately = false;
        }
        HandHistoryPopup.prototype.shown = function () {
            _super.prototype.shown.call(this);
            var view = this.tableView();
            var lastHand = view.lastHandHistory();
            this.detailedOperations(lastHand.detailedOperations());
            this.shortOperations(lastHand.shortOperations());
            this.cards(lastHand.cards());
            this.playersData(lastHand.playersData());
            gameactionsqueue_1.GameActionsQueue.waitDisabled = true;
            app.tablesPage.tablesShown(false);
            app.popupClosed.addOnce(function (popupName) {
                if (popupName === "handHistory") {
                    gameactionsqueue_1.GameActionsQueue.waitDisabled = false;
                    app.tablesPage.tablesShown(true);
                }
            }, this, 0);
        };
        HandHistoryPopup.prototype.selectMode = function (mode) {
            this.mode(mode);
        };
        return HandHistoryPopup;
    }(popupbase_7.PopupBase));
    exports.HandHistoryPopup = HandHistoryPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/forgetpasswordpopup", ["require", "exports", "knockout", "popups/simplepopup", "ui/popupbase", "languageManager"], function (require, exports, ko, simplepopup_5, popupbase_8, languagemanager_10) {
    "use strict";
    var ForgetPasswordPopup = (function (_super) {
        __extends(ForgetPasswordPopup, _super);
        function ForgetPasswordPopup() {
            _super.call(this);
            this.login = ko.observable();
            this.email = ko.observable();
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
            this.loading = ko.observable(false);
        }
        ForgetPasswordPopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (!this.loading()) {
                self.loading(true);
                var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
                accountApi.RequestResetPassword(this.login(), this.email(), function (data) {
                    self.loading(false);
                    if (data.Status === "Ok") {
                        self.errorMessage(null);
                        self.login(null);
                        self.email(null);
                        self.close();
                        simplepopup_5.SimplePopup.display(languagemanager_10._("auth.forgetPassword"), languagemanager_10._("auth.forgetPasswordSuccess"));
                    }
                    else {
                        // Report authentication or authorization errors
                        self.errorMessage(languagemanager_10._("errors." + data.Status));
                    }
                }).fail(function (jqXHR, textStatus, errorThrown) {
                    self.loading(false);
                });
            }
        };
        return ForgetPasswordPopup;
    }(popupbase_8.PopupBase));
    exports.ForgetPasswordPopup = ForgetPasswordPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/custompopup", ["require", "exports", "knockout", "ui/popupbase"], function (require, exports, ko, popupbase_9) {
    "use strict";
    var CustomPopup = (function (_super) {
        __extends(CustomPopup, _super);
        function CustomPopup() {
            _super.call(this);
            this.buttons = ko.observableArray([]);
            this.actions = ko.observableArray([]);
            this.title = ko.observable();
            this.messages = ko.observableArray();
        }
        CustomPopup.prototype.shown = function () {
            _super.prototype.shown.call(this);
            this.deferred = $.Deferred();
        };
        CustomPopup.prototype.close = function () {
            _super.prototype.close.call(this);
            this.deferred.reject();
        };
        CustomPopup.prototype.execute = function (index) {
            var action = this.actions()[index];
            var result = action();
            _super.prototype.close.call(this);
            if (result === undefined) {
                this.deferred.reject();
            }
            else {
                this.deferred.resolve(result);
            }
        };
        return CustomPopup;
    }(popupbase_9.PopupBase));
    exports.CustomPopup = CustomPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/addmoneypopup", ["require", "exports", "knockout", "popups/simplepopup", "languageManager"], function (require, exports, ko, simplepopup_6, languagemanager_11) {
    "use strict";
    var AddMoneyPopup = (function () {
        function AddMoneyPopup() {
            this.buyin = ko.observable().extend({ required: true, validatable: true });
            this.tableView = ko.observable();
            this.accountTotal = ko.observable(0);
            this.loading = ko.observable(false);
            this.tableName = ko.observable();
            this.minBuyin = ko.observable(0);
            this.maxBuyin = ko.observable(0);
            this.minBet = ko.observable(0);
            this.maxBet = ko.observable(0);
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
            this.processing = ko.observable(false);
        }
        AddMoneyPopup.prototype.shown = function () {
            var self = this;
            var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
            self.loading(true);
            self.processing(false);
            accountApi.GetPersonalAccount(function (data) {
                self.loading(false);
                if (data.Status === "Ok") {
                    var personalAccountData = data.Data;
                    var tableData = self.tableView().model;
                    var balance = 0;
                    var currencyId = tableData.CurrencyId;
                    if (currencyId === 1) {
                        balance = personalAccountData.RealMoney;
                    }
                    else {
                        balance = personalAccountData.GameMoney;
                    }
                    var tableView = self.tableView();
                    var myPlayer = tableView.myPlayer();
                    var totalBet = (myPlayer.TotalBet() === null ? 0 : myPlayer.TotalBet()) + myPlayer.Bet();
                    var tableTotal = totalBet + myPlayer.Money();
                    self.accountTotal(balance);
                    self.tableName(tableData.TableName);
                    self.minBet(tableData.SmallBlind);
                    self.maxBet(tableData.BigBlind);
                    var baseMinimalBuyIn = tableView.minimalBuyIn() * tableData.BigBlind;
                    var maxBuyIn = (20 * baseMinimalBuyIn) - tableTotal;
                    self.minBuyin(1);
                    self.maxBuyin(maxBuyIn);
                    self.buyin(Math.min(2 * baseMinimalBuyIn, maxBuyIn));
                }
                else {
                    simplepopup_6.SimplePopup.display(languagemanager_11._("addMoney.caption"), languagemanager_11._("errors." + data.Status));
                }
            }).fail(function () {
                self.loading(false);
                simplepopup_6.SimplePopup.display(languagemanager_11._("addMoney.caption"), languagemanager_11._("addMoney.joinError"));
            });
        };
        AddMoneyPopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (self.processing()) {
                return;
            }
            if (this.buyin() < this.minBuyin()) {
                this.buyin.setError(languagemanager_11._("addMoney.putMoreMoney"));
                return;
            }
            if (this.buyin() > this.maxBuyin()) {
                this.buyin.setError(languagemanager_11._("addMoney.putLessMoney"));
                return;
            }
            if (this.accountTotal() < this.minBuyin()) {
                this.buyin.setError(languagemanager_11._("errors.NotSufficiendFunds"));
                return;
            }
            if (this.accountTotal() < this.buyin()) {
                this.buyin.setError(languagemanager_11._("errors.NotSufficiendFunds"));
                return;
            }
            if (this.buyin() + this.tableView().myPlayer().Money() + this.tableView().myPlayer().TotalBet() > this.maxBuyin()) {
                this.buyin.setError(languagemanager_11._("addMoney.couldnotAddMoreThenMax"));
                return;
            }
            var amount = this.buyin();
            self.processing(true);
            this.tableView().addBalance(amount).then(function () {
                self.processing(false);
                app.closePopup("ok");
                simplepopup_6.SimplePopup.display(languagemanager_11._("addMoney.caption"), languagemanager_11._("addMoney.success"));
            }, function (status) {
                self.processing(false);
                self.buyin.setError(languagemanager_11._("errors." + status));
            });
        };
        return AddMoneyPopup;
    }());
    exports.AddMoneyPopup = AddMoneyPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/changepasswordpopup", ["require", "exports", "knockout", "popups/simplepopup", "ui/popupbase", "languageManager"], function (require, exports, ko, simplepopup_7, popupbase_10, languagemanager_12) {
    "use strict";
    var ChangePasswordPopup = (function (_super) {
        __extends(ChangePasswordPopup, _super);
        function ChangePasswordPopup() {
            _super.call(this);
            this.oldPassword = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.confirmPassword = ko.observable().extend({ required: true });
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
            this.loading = ko.observable(false);
        }
        ChangePasswordPopup.prototype.shown = function (args) {
            if (args === void 0) { args = []; }
            this.oldPassword(null);
            this.password(null);
            this.confirmPassword(null);
            this.errors.showAllMessages(false);
            _super.prototype.shown.call(this, args);
        };
        ChangePasswordPopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (!this.loading()) {
                self.loading(true);
                self.errorMessage(null);
                var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
                accountApi.ChangePassword(this.oldPassword(), this.password(), function (data) {
                    if (data.Status === "Ok") {
                        self.loading(false);
                        self.oldPassword(null);
                        self.password(null);
                        self.confirmPassword(null);
                        app.closePopup();
                        simplepopup_7.SimplePopup.display(languagemanager_12._("auth.changePassword"), languagemanager_12._("auth.passwordChangedSuccess"));
                    }
                    else {
                        // Report authentication or authorization errors
                        self.errorMessage(languagemanager_12._("errors." + data.Status));
                    }
                });
            }
        };
        return ChangePasswordPopup;
    }(popupbase_10.PopupBase));
    exports.ChangePasswordPopup = ChangePasswordPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/continueforgetpasswordpopup", ["require", "exports", "knockout", "popups/simplepopup", "ui/popupbase", "languageManager"], function (require, exports, ko, simplepopup_8, popupbase_11, languagemanager_13) {
    "use strict";
    var ContinueForgetPasswordPopup = (function (_super) {
        __extends(ContinueForgetPasswordPopup, _super);
        function ContinueForgetPasswordPopup() {
            _super.call(this);
            this.token = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.confirmPassword = ko.observable().extend({ required: true });
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
            this.loading = ko.observable(false);
        }
        ContinueForgetPasswordPopup.prototype.shown = function (args) {
            if (args === void 0) { args = []; }
            this.token(null);
            this.password(null);
            this.confirmPassword(null);
            this.errors.showAllMessages(false);
            _super.prototype.shown.call(this, args);
        };
        ContinueForgetPasswordPopup.prototype.confirm = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (!this.loading()) {
                self.loading(true);
                self.errorMessage(null);
                var accountApi = new OnlinePoker.Commanding.API.Account(apiHost);
                accountApi.ResetPassword(this.token(), this.password(), function (data) {
                    if (data.Status === "Ok") {
                        self.token(null);
                        self.password(null);
                        self.confirmPassword(null);
                        app.closePopup();
                        self.loading(false);
                        simplepopup_8.SimplePopup.display(languagemanager_13._("auth.passwordRecovery"), languagemanager_13._("auth.passwordRecoveredSuccess"));
                    }
                    else {
                        // Report authentication or authorization errors
                        self.errorMessage(languagemanager_13._("errors." + data.Status));
                    }
                });
            }
        };
        return ContinueForgetPasswordPopup;
    }(popupbase_11.PopupBase));
    exports.ContinueForgetPasswordPopup = ContinueForgetPasswordPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("ui/chatcontrol", ["require", "exports", "knockout", "timeService", "services/index"], function (require, exports, ko, timeService, services_5) {
    "use strict";
    var ChatControl = (function () {
        function ChatControl() {
            this.currentMessage = ko.observable();
            this.messages = ko.observableArray([]);
            this.loading = ko.observable(false);
            this.tableId = ko.observable(0);
        }
        ChatControl.prototype.initialize = function () {
            var _this = this;
            services_5.connectionService.newConnection.add(function () {
                var chatHub = services_5.connectionService.currentConnection.connection.createHubProxy("chat");
                var handler = function () {
                    var msg = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        msg[_i - 0] = arguments[_i];
                    }
                    var messageId = msg[0];
                    var tableId = msg[1];
                    var type = msg[2];
                    var sender = msg[3];
                    var message = msg[4];
                    if (tableId !== _this.tableId()) {
                        return;
                    }
                    _this.append(messageId, tableId, type, sender, message);
                };
                chatHub.on("Message", handler);
                services_5.connectionService.terminatedConnection.addOnce(function () {
                    chatHub.off("Message", handler);
                }, self, 0);
            });
        };
        ChatControl.prototype.append = function (messageId, tableId, type, sender, message) {
            this.loading(false);
            var m = this.messages();
            this.messages(["[" + sender + "] " + message].concat(m));
        };
        ChatControl.prototype.attachToHub = function () {
            var _this = this;
            this.loading(true);
            var wrapper = services_5.connectionService.currentConnection;
            wrapper.buildStartConnection()().done(function () {
                if (wrapper.terminated) {
                    return;
                }
                wrapper.connection.Chat.server.join(_this.tableId());
            });
            this.timeoutHandler = timeService.setTimeout(function () {
                _this.loading(false);
                _this.timeoutHandler = 0;
            }, 2000);
        };
        ChatControl.prototype.detachFromHub = function () {
            var _this = this;
            var wrapper = services_5.connectionService.currentConnection;
            wrapper.buildStartConnection()().done(function () {
                if (wrapper.terminated) {
                    return;
                }
                wrapper.connection.Chat.server.leave(_this.tableId());
            });
            this.messages([]);
            if (this.timeoutHandler !== 0) {
                timeService.clearTimeout(this.timeoutHandler);
            }
        };
        return ChatControl;
    }());
    exports.ChatControl = ChatControl;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/chatpopup", ["require", "exports", "ui/chatcontrol", "timeService", "languageManager"], function (require, exports, chatcontrol_1, timeService, languagemanager_14) {
    "use strict";
    var ChatPopup = (function () {
        function ChatPopup() {
            this.subscription = null;
            this.control = new chatcontrol_1.ChatControl();
            this.currentMessage = ko.observable("");
            this.loading = ko.observable(false);
            this.caption = ko.observable(languagemanager_14._("chat.tableCaption"));
            this.control.initialize();
            this.messages = ko.observableArray([]);
            this.systemMessages = ko.observableArray([]);
        }
        ChatPopup.prototype.attach = function (view) {
            var _this = this;
            if (this.subscription !== null) {
                this.subscription.dispose();
                this.subscription = null;
            }
            this.tableView = view;
            this.messages(this.tableView.messages());
            this.subscription = this.tableView.messages.subscribe(function (value) {
                _this.messages(value);
            });
            this.systemMessages(this.tableView.systemMessages());
            this.subscription = this.tableView.systemMessages.subscribe(function (value) {
                _this.systemMessages(value);
            });
        };
        ChatPopup.prototype.send = function () {
            if (this.currentMessage() === "") {
                return;
            }
            this.tableView.chatMessage(this.currentMessage());
            this.tableView.sendMessage();
            this.currentMessage("");
        };
        ChatPopup.prototype.close = function () {
            if (this.subscription !== null) {
                this.subscription.dispose();
                this.subscription = null;
            }
            app.closePopup();
            /* Workaround for iOS 7.1 issue which break layout. */
            timeService.setTimeout(function () {
                $(window).scrollLeft(0);
            }, 10);
        };
        ChatPopup.prototype.shown = function () {
            this.control.attachToHub();
        };
        return ChatPopup;
    }());
    exports.ChatPopup = ChatPopup;
});
/// <reference path="../poker.commanding.api.ts" />
define("popups/authpopup", ["require", "exports", "knockout", "ui/popupbase", "authmanager", "services/index", "settings", "languageManager"], function (require, exports, ko, popupbase_12, authManager, services_6, settings_4, languagemanager_15) {
    "use strict";
    var AuthPopup = (function (_super) {
        __extends(AuthPopup, _super);
        function AuthPopup() {
            _super.call(this);
            this.validationLogin = ko.observable().extend({ required: true, maxLength: 12 });
            this.validationPassword = ko.observable().extend({ required: true, maxLength: 16 });
            this.login = ko.observable();
            this.password = ko.observable();
            this.rememberMe = ko.observable(false);
            this.errors = ko.validation.group(this);
            this.errorMessage = ko.observable();
            this.loading = ko.observable(false);
        }
        AuthPopup.prototype.shown = function (args) {
            if (args === void 0) { args = []; }
            this.login(settings_4.settings.login());
            this.password(settings_4.settings.password());
            this.rememberMe(settings_4.settings.login() != null);
            this.errors.showAllMessages(false);
            _super.prototype.shown.call(this, args);
        };
        AuthPopup.prototype.logon = function () {
            var self = this;
            if (authManager.authenticated()) {
                return;
            }
            this.validationLogin(this.login());
            this.validationPassword(this.password());
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (!this.loading()) {
                this.loading(true);
                var login = this.login();
                var password = this.password();
                var rememberMe = this.rememberMe();
                self.errorMessage(null);
                authManager.authenticate(login, password, rememberMe).done(function (result) {
                    if (result === "Ok") {
                        self.login(null);
                        self.password(null);
                        services_6.keyboardActivationService.forceHideKeyboard();
                        self.close();
                    }
                    else {
                        // Report authentication or authorization errors
                        if (result) {
                            self.errorMessage(languagemanager_15._("errors." + result));
                        }
                        else {
                            self.errorMessage(languagemanager_15._("auth.unspecifiedError"));
                        }
                    }
                }).always(function () {
                    self.loading(false);
                });
            }
        };
        AuthPopup.prototype.registration = function () {
            // Do nothing.
        };
        AuthPopup.prototype.forgetPassword = function () {
            app.showPopup("forgetPassword");
        };
        return AuthPopup;
    }(popupbase_12.PopupBase));
    exports.AuthPopup = AuthPopup;
});
define("popups/index", ["require", "exports", "popups/morepopup", "popups/accountstatuspopup", "popups/tablemenupopup", "popups/selectavatarpopup", "popups/slowconnectionpopup", "popups/registrationpopup", "popups/okcancelpopup", "popups/newspopup", "popups/jointablepopup", "popups/handhistorypopup", "popups/forgetpasswordpopup", "popups/custompopup", "popups/addmoneypopup", "popups/changepasswordpopup", "popups/continueforgetpasswordpopup", "popups/chatpopup", "popups/authpopup", "popups/simplepopup"], function (require, exports, morepopup_1, accountstatuspopup_1, tablemenupopup_1, selectavatarpopup_1, slowconnectionpopup_1, registrationpopup_1, okcancelpopup_1, newspopup_1, jointablepopup_1, handhistorypopup_1, forgetpasswordpopup_1, custompopup_1, addmoneypopup_1, changepasswordpopup_1, continueforgetpasswordpopup_1, chatpopup_1, authpopup_1, simplepopup_9) {
    "use strict";
    exports.MorePopup = morepopup_1.MorePopup;
    exports.AccountStatusPopup = accountstatuspopup_1.AccountStatusPopup;
    exports.TableMenuPopup = tablemenupopup_1.TableMenuPopup;
    exports.SelectAvatarPopup = selectavatarpopup_1.SelectAvatarPopup;
    exports.SlowConnectionPopup = slowconnectionpopup_1.SlowConnectionPopup;
    exports.RegistrationPopup = registrationpopup_1.RegistrationPopup;
    exports.OkCancelPopup = okcancelpopup_1.OkCancelPopup;
    exports.NewsPopup = newspopup_1.NewsPopup;
    exports.JoinTablePopup = jointablepopup_1.JoinTablePopup;
    exports.HandHistoryPopup = handhistorypopup_1.HandHistoryPopup;
    exports.ForgetPasswordPopup = forgetpasswordpopup_1.ForgetPasswordPopup;
    exports.CustomPopup = custompopup_1.CustomPopup;
    exports.AddMoneyPopup = addmoneypopup_1.AddMoneyPopup;
    exports.ChangePasswordPopup = changepasswordpopup_1.ChangePasswordPopup;
    exports.ContinueForgetPasswordPopup = continueforgetpasswordpopup_1.ContinueForgetPasswordPopup;
    exports.ChatPopup = chatpopup_1.ChatPopup;
    exports.AuthPopup = authpopup_1.AuthPopup;
    exports.SimplePopup = simplepopup_9.SimplePopup;
});
/// <reference path="../typings/signalr.d.ts" />
define("table/actionblock", ["require", "exports", "knockout", "table/tableSlider", "authmanager", "popups/index", "appconfig", "debugsettings", "languageManager"], function (require, exports, ko, tableSlider_1, authManager, popups_1, appconfig_4, debugsettings_12, languagemanager_16) {
    "use strict";
    var ActionBlock = (function () {
        function ActionBlock() {
            var _this = this;
            this.suppressWaitBBNotifications = false;
            this.button1Visible = ko.observable(false);
            this.button2Visible = ko.observable(false);
            this.button3Visible = ko.observable(false);
            /**
            * Value indicating whether Wait for BB checkbox is visible
            */
            this.waitBigBlindBlockVisible = ko.observable(true);
            /**
            * Value indicating where game is closed.
            */
            this.gameClosed = ko.observable(false);
            /**
            * Value indicating where component operating in test mode and some checks are disabled.
            */
            this.testMode = ko.observable(false);
            this.cardsOverlayVisible = ko.observable(true);
            var self = this;
            this.tableSlider = new tableSlider_1.TableSlider();
            this.dealsAllowed = ko.observable(false);
            this.buttonsEnabled = ko.observable(false);
            this.processing = ko.observable(false);
            this.isSitOut = ko.observable(false);
            this.turnEnabled = ko.observable(false);
            this.inGame = ko.observable(false);
            this.isCheck = ko.observable(false);
            this.isRaise = ko.observable(true);
            this.callAmount = ko.observable(0);
            this.playerMoney = ko.observable(0);
            this.supportDirectAmount = ko.observable(false);
            this.supportAny = ko.observable(false);
            this.foldOnRaise = ko.observable(false);
            this.supportDirectAmountCaption = ko.observable("Чек");
            this.supportAnyCaption = ko.observable("Колл любую");
            this.foldOnRaiseCaption = ko.observable("Чек/Фолд");
            this.amountSupported = ko.observable(-1);
            this.maxAmountOfMoneyForOtherActivePlayers = ko.observable();
            this.myPlayer = ko.observable();
            this.notMyTurn = ko.observable(false);
            this.isInGame = ko.observable(false);
            this.myPlayerInGame = ko.observable(false);
            this.myPlayerWasInGame = ko.observable(false);
            this.messages = ko.observableArray();
            this.systemMessages = ko.observableArray();
            this.needBB = ko.observable(false);
            this.gameFinished = ko.observable(true);
            this.prizesDistributed = ko.observable(true);
            this.expanded = ko.observable(false);
            this.waitbb = ko.observable(true);
            this.skipDeals = ko.observable(false);
            this.autoFoldOrCheck = ko.observable(false);
            this.allInCaption = ko.observable("");
            this.halfPotCaption = ko.observable("");
            this.potCaption = ko.observable("");
            this.checkOrCallAmount = ko.observable();
            this.button1Caption = ko.observable(languagemanager_16._("table.halfpot"));
            this.button2Caption = ko.observable(languagemanager_16._("table.pot"));
            this.button3Caption = ko.observable(languagemanager_16._("table.allin"));
            this.button1Amount = ko.observable(0);
            this.button2Amount = ko.observable(0);
            this.button3Amount = ko.observable(0);
            this.foldExecuted = new signals.Signal();
            this.checkOrCallExecuted = new signals.Signal();
            this.betOrRaiseExecuted = new signals.Signal();
            this.suppressSetSitoutStatus = false;
            this.suppressActions = false;
            this.couldRaise = ko.computed(function () {
                var amountLess = _this.checkOrCallAmount() < _this.playerMoney();
                if (!amountLess) {
                    return false;
                }
                var maxAmountOfMoneyForOtherActivePlayers = _this.maxAmountOfMoneyForOtherActivePlayers();
                var otherPlayersHasMoneyToSupport = _this.maxAmountOfMoneyForOtherActivePlayers() > _this.callAmount();
                return otherPlayersHasMoneyToSupport;
            });
            this.checkCallButtonCaption = ko.computed(function () {
                var currentAmount = self.checkOrCallAmount();
                currentAmount = currentAmount == null ? 0 : currentAmount;
                var playerMoney = self.playerMoney();
                playerMoney = playerMoney == null ? 0 : playerMoney;
                if (playerMoney <= currentAmount) {
                    var myself = self.myPlayer();
                    if (myself != null) {
                        return languagemanager_16._("table.allin").replace("#amount", (myself.Bet() + playerMoney).toString());
                    }
                    return "";
                }
                else {
                    if (self.isCheck()) {
                        return languagemanager_16._("table.check");
                    }
                    else {
                        return languagemanager_16._("table.call").replace("#amount", currentAmount.toString());
                    }
                }
            });
            this.raiseBetButtonCaption = ko.computed(function () {
                var currentAmount = self.tableSlider.current();
                currentAmount = currentAmount == null ? 0 : currentAmount;
                var player = self.myPlayer();
                var playerMoney = self.playerMoney();
                if (player != null) {
                    playerMoney += player.Bet();
                }
                playerMoney = playerMoney == null ? 0 : playerMoney;
                if (playerMoney <= currentAmount) {
                    var myself = self.myPlayer();
                    if (myself != null) {
                        return languagemanager_16._("table.allin").replace("#amount", (playerMoney).toString());
                    }
                    return "";
                }
                else {
                    if (self.isRaise()) {
                        return languagemanager_16._("table.raise").replace("#amount", currentAmount.toString());
                    }
                    else {
                        return languagemanager_16._("table.bet").replace("#amount", currentAmount.toString());
                    }
                }
            });
            this.foldOnRaise.subscribe(function (value) {
                if (value) {
                    self.supportAny(false);
                    self.supportDirectAmount(false);
                }
            });
            this.supportDirectAmount.subscribe(function (value) {
                if (value) {
                    self.supportAny(false);
                    self.foldOnRaise(false);
                    self.amountSupported(self.tableView.maximumBet() - self.tableView.myBet());
                }
            });
            this.supportAny.subscribe(function (value) {
                if (value) {
                    self.supportDirectAmount(false);
                    self.foldOnRaise(false);
                }
            });
            this.sitoutBlockVisible = ko.computed(function () {
                return self.isSitOut() && !self.gameClosed();
            });
            this.mainButtonsBlockVisible = ko.computed(function () {
                return self.turnEnabled() && !self.isSitOut() && self.buttonsEnabled()
                    && self.dealsAllowed() && self.myPlayerInGame()
                    && !self.gameClosed();
            });
            this.autoButtonsBlockVisible = ko.computed(function () {
                if (!self.isInGame()) {
                    return false;
                }
                return !self.turnEnabled() && !self.isSitOut()
                    && self.notMyTurn()
                    && self.dealsAllowed() && self.myPlayerInGame()
                    && !self.gameClosed();
            });
            this.raiseBlockVisible = ko.computed(function () {
                if (self.gameFinished()) {
                    return false;
                }
                return self.mainButtonsBlockVisible()
                    && self.couldRaise() && self.myPlayerInGame()
                    && !self.gameClosed();
            });
            this.observerModeBlockVisible = ko.computed(function () {
                return (!authManager.authenticated() && !self.testMode()) || self.myPlayer() == null;
            });
            this.startBlockVisible = ko.computed(function () {
                if (self.sitoutBlockVisible()) {
                    return false;
                }
                if (self.autoButtonsBlockVisible()) {
                    return false;
                }
                if (self.mainButtonsBlockVisible()) {
                    return false;
                }
                return !self.gameClosed() && !self.observerModeBlockVisible();
            });
            this.chatVisible = ko.computed(function () {
                return !self.raiseBlockVisible()
                    && !self.gameClosed();
            });
            if (debugsettings_12.debugSettings.actionBlock.traceBlocksVisbility) {
                this.waitBigBlindBlockVisible.subscribe(function (visbile) {
                    self.log("WaitBB block " + (visbile ? "visible" : "hidden"));
                });
                this.mainButtonsBlockVisible.subscribe(function (visbile) {
                    self.log("Main Buttons block " + (visbile ? "visible" : "hidden"));
                });
                this.sitoutBlockVisible.subscribe(function (visbile) {
                    self.log("Sit out block " + (visbile ? "visible" : "hidden"));
                });
                this.autoButtonsBlockVisible.subscribe(function (visbile) {
                    _this.log("Auto Buttons block " + (visbile ? "visible" : "hidden"));
                });
            }
        }
        ActionBlock.prototype.attach = function (tableView) {
            var _this = this;
            if (this.tableView != null) {
                throw new Error("Table view already attached to the action block");
            }
            this.tableView = tableView;
            if (tableView.model != null) {
                this.tableSlider.setStep(tableView.model.BigBlind);
                tableView.bigBlind.subscribe(function (bigBlind) {
                    _this.tableSlider.setStep(bigBlind);
                });
            }
            this.tableView.turnEnabled.subscribe(function (value) {
                _this.turnEnabled(value);
            });
            this.tableView.isSitOut.subscribe(function (value) {
                if (value !== _this.isSitOut()) {
                    _this.suppressSetSitoutStatus = true;
                    _this.isSitOut(value);
                    _this.skipDeals(value);
                    _this.suppressSetSitoutStatus = false;
                }
            });
            this.tableView.myPlayer.subscribe(function (value) {
                _this.myPlayer(value);
                if (value == null) {
                    _this.inGame(false);
                }
                else {
                    _this.inGame(true);
                }
            });
            this.tableView.minimumRaiseAmount.subscribe(function (value) {
                _this.tableSlider.minimum(value);
                var myself = _this.tableView.myPlayer();
                if (myself != null) {
                    _this.playerMoney(myself.Money());
                }
            });
            this.tableView.messages.subscribe(function (value) {
                _this.messages(value);
            });
            this.tableView.systemMessages.subscribe(function (value) {
                _this.systemMessages(value);
            });
            this.tableView.myPlayerInGame.subscribe(function (value) {
                _this.myPlayerInGame(value);
            });
            this.tableView.myPlayerWasInGame.subscribe(function (value) {
                _this.myPlayerWasInGame(value);
            });
            this.tableView.gameFinished.subscribe(function (value) {
                _this.gameFinished(value);
            });
            this.tableView.prizesDistributed.subscribe(function (value) {
                _this.prizesDistributed(value);
            });
            this.tableView.notMyTurn.subscribe(function (value) {
                _this.notMyTurn(value);
            });
            this.tableView.isInGame.subscribe(function (value) {
                _this.isInGame(value);
            });
            this.tableView.maximumRaiseAmount.subscribe(function (value) {
                var myself = _this.tableView.myPlayer();
                if (myself != null) {
                    _this.tableSlider.maximum(value + myself.Bet());
                }
            });
            this.tableView.checkOrCallAmount.subscribe(function (value) {
                _this.isCheck(value === 0);
                _this.checkOrCallAmount(value);
            });
            this.tableView.maximumBet.subscribe(function (value) {
                _this.callAmount(value);
            });
            this.tableView.maxAmountOfMoneyForOtherActivePlayers.subscribe(function (value) {
                _this.maxAmountOfMoneyForOtherActivePlayers(value);
            });
            this.waitbb.subscribe(function (value) {
                if (_this.suppressWaitBBNotifications) {
                    return;
                }
                var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
                if (value) {
                    gameApi.WaitBigBlind(_this.tableView.tableId);
                }
                else {
                    gameApi.ForceJoinGame(_this.tableView.tableId);
                }
            });
            this.autoFoldOrCheck.subscribe(function (value) {
                _this.expanded(false);
            });
            this.skipDeals.subscribe(function (value) {
                if (_this.suppressSetSitoutStatus) {
                    return;
                }
                var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
                if (value) {
                    gameApi.SitOut(_this.tableView.tableId);
                }
                else {
                    gameApi.ComeBack(_this.tableView.tableId);
                }
                _this.expanded(false);
            });
        };
        ActionBlock.prototype.resetWaitBB = function () {
            this.suppressWaitBBNotifications = true;
            this.waitbb(true);
            this.suppressWaitBBNotifications = false;
        };
        ActionBlock.prototype.fold = function () {
            if (!this.suppressActions) {
                this.tableView.fold();
            }
            this.foldExecuted.dispatch();
        };
        ActionBlock.prototype.checkOrCall = function () {
            if (!this.suppressActions) {
                this.tableView.checkOrCall();
            }
            this.checkOrCallExecuted.dispatch();
        };
        ActionBlock.prototype.betOrRaise = function () {
            if (!this.suppressActions) {
                this.tableView.betOrRaise();
            }
            this.betOrRaiseExecuted.dispatch();
        };
        ActionBlock.prototype.showChatPopup = function () {
            app.tableChatPopup.attach(this.tableView);
            app.showPopup("tableChat");
        };
        ActionBlock.prototype.comeBack = function () {
            var _this = this;
            if (this.tableView.myPlayer().Money() === 0) {
                if (this.tableView.tournament() != null) {
                    this.tableView.proposeRebuyOrAddon();
                    return;
                }
                if (appconfig_4.appConfig.game.seatMode) {
                    popups_1.SimplePopup.display(languagemanager_16._("table.comeback"), languagemanager_16._("table.askAdministratorToAddMoney"));
                }
                else {
                    app.addMoneyPopup.tableView(this.tableView);
                    app.showPopup("addMoney").done(function (results) {
                        if (results.result === "ok") {
                            _this.comeBackCore();
                        }
                    });
                }
            }
            else {
                this.comeBackCore();
            }
        };
        ActionBlock.prototype.comeBackCore = function () {
            var _this = this;
            if (this.processing()) {
                return;
            }
            this.processing(true);
            this.tableView.comeBack().fail(function () {
                _this.processing(false);
            });
        };
        ActionBlock.prototype.updateBounds = function () {
            if ($(".slider-line").length > 0) {
                var lineWidth = $(".slider-line").width();
                var handleWidth = $(".slider-handle").width();
                var adj_1 = -5;
                var translator = function (pageX) {
                    var startOffset = $(".slider-line").offset().left;
                    return pageX - startOffset + adj_1;
                };
                // -5 is base adjustment from one size; width - 5(base adj.) - 10(?)
                this.tableSlider.setBounds(adj_1, lineWidth - handleWidth + (-adj_1), translator);
            }
            else {
                this.tableSlider.setBounds(1, 100, function (x) { return x; });
            }
        };
        ActionBlock.prototype.resetAutomaticAction = function () {
            this.supportAny(false);
            this.supportDirectAmount(false);
            this.foldOnRaise(false);
            this.amountSupported(-1);
        };
        ActionBlock.prototype.performAutomaticActions = function () {
            if (this.automaticallyFold()) {
                return true;
            }
            if (this.supportAnyBet()) {
                return true;
            }
            if (this.supportDirectAmountBet()) {
                return true;
            }
            if (this.checkOrFold()) {
                return true;
            }
            return false;
        };
        /**
        * Automatically fold cards if possible.
        *
        * @returns true if cards was folded; false otherwise.
        */
        ActionBlock.prototype.automaticallyFold = function () {
            if (this.autoFoldOrCheck()) {
                this.fold();
                return true;
            }
            return false;
        };
        /**
        * Automatically support any bet placed on the table.
        * @returns true if automatic action was performed; false otherwise.
        */
        ActionBlock.prototype.supportAnyBet = function () {
            if (this.supportAny()) {
                this.checkOrCall();
                return true;
            }
            return false;
        };
        /**
        * Automatically support any bet placed on the table.
        * @returns true if automatic action was performed; false otherwise.
        */
        ActionBlock.prototype.supportDirectAmountBet = function () {
            if (this.supportDirectAmount() && (this.amountSupported() >= 0)) {
                this.checkOrCall();
                this.supportDirectAmount(false);
                this.amountSupported(-1);
                return true;
            }
            this.amountSupported(-1);
            return false;
        };
        /**
        * Automatically check current bet or fold in case of bet raise.
        * @returns if automatic action was performed; false otherwise.
        */
        ActionBlock.prototype.checkOrFold = function () {
            if (this.foldOnRaise()) {
                var requiredBet = this.tableView.maximumBet() - this.tableView.myBet();
                if (requiredBet === 0) {
                    this.checkOrCall();
                }
                else {
                    this.fold();
                }
                return true;
            }
            return false;
        };
        ActionBlock.prototype.updateAutomaticActionsText = function (playerMoney, requiredBet) {
            if (requiredBet === 0) {
                this.supportDirectAmountCaption("Чек");
            }
            else {
                if (playerMoney <= requiredBet) {
                    this.supportDirectAmountCaption("Олл-ин #amount".replace("#amount", playerMoney.toFixed()));
                }
                else {
                    var callAmount = (this.tableView.maximumBet() - this.tableView.myBet()).toFixed();
                    this.supportDirectAmountCaption("Колл #amount".replace("#amount", callAmount));
                }
            }
            if (this.supportDirectAmount()) {
                if (requiredBet > this.amountSupported()) {
                    this.amountSupported(-1);
                    this.supportDirectAmount(false);
                }
            }
            if (requiredBet > 0) {
                this.foldOnRaiseCaption("Фолд");
            }
            else {
                this.foldOnRaiseCaption("Чек/Фолд");
            }
            if (requiredBet > 0) {
                this.supportAnyCaption("Колл любую ставку");
            }
            else {
                this.supportAnyCaption("Колл любую ставку/Чек");
            }
        };
        ActionBlock.prototype.updateSupportDirectAmountStatus = function (requiredBet) {
            var isDirectAmountSupported = (requiredBet === this.amountSupported());
            this.supportDirectAmount(isDirectAmountSupported);
            if (!isDirectAmountSupported) {
                this.amountSupported(-1);
            }
        };
        ActionBlock.prototype.updateAdditionalButtons = function () {
            var threebbAmountOriginal = this.callAmount() + (3 * this.tableView.model.BigBlind);
            var threebbAmount = this.tableSlider.withinRange(threebbAmountOriginal);
            var potAmountOriginal = this.tableView.pots().reduce(function (pv, v) {
                return pv + v;
            }, 0) + this.checkOrCallAmount();
            potAmountOriginal += this.tableView.places().reduce(function (pv, v) {
                if (v.Bet() == null) {
                    return pv;
                }
                return pv + v.Bet();
            }, 0);
            potAmountOriginal = 2 * potAmountOriginal;
            var potAmount = this.tableSlider.withinRange(potAmountOriginal);
            var maxMoneyAmount = this.tableSlider.maximum();
            this.button1Amount(threebbAmount);
            this.button2Amount(potAmount);
            this.button3Amount(maxMoneyAmount);
            this.button1Caption(languagemanager_16._("table.halfpot").replace("#amount", this.button1Amount().toFixed()));
            this.button2Caption(languagemanager_16._("table.pot").replace("#amount", this.button2Amount().toFixed()));
            var player = this.myPlayer();
            var playerMoney = this.playerMoney();
            if (player != null) {
                playerMoney += player.Bet();
            }
            if (playerMoney <= this.maxAmountOfMoneyForOtherActivePlayers()) {
                this.button3Caption(languagemanager_16._("table.allin").replace("#amount", playerMoney.toFixed()));
            }
            else {
                this.button3Caption(languagemanager_16._("table.raise").replace("#amount", this.button3Amount().toFixed()));
            }
            this.button1Visible(this.tableSlider.isWithinRange(threebbAmountOriginal));
            this.button2Visible(this.tableSlider.isWithinRange(potAmountOriginal));
            this.button3Visible(true);
        };
        /**
        * Updates block visibility.
        */
        ActionBlock.prototype.updateBlocks = function () {
            var waitBBBlockVisible = this.getWaitBBBlockVisible();
            this.waitBigBlindBlockVisible(waitBBBlockVisible);
        };
        /**
        * Toggles automatic panel.
        */
        ActionBlock.prototype.toggle = function () {
            if (!appconfig_4.appConfig.game.actionBlock.hasSecondaryPanel) {
                return;
            }
            this.expanded(!this.expanded());
        };
        /**
        * Expand automatic panel.
        */
        ActionBlock.prototype.expand = function () {
            if (!appconfig_4.appConfig.game.actionBlock.hasSecondaryPanel) {
                return;
            }
            this.expanded(true);
        };
        /**
        * Collapse automatic panel.
        */
        ActionBlock.prototype.collapse = function () {
            if (!appconfig_4.appConfig.game.actionBlock.hasSecondaryPanel) {
                return;
            }
            this.expanded(false);
        };
        ActionBlock.prototype.allin = function () {
            this.tableSlider.current(this.button3Amount());
            this.betOrRaise();
            this.expanded(false);
        };
        ActionBlock.prototype.halfPot = function () {
            this.tableSlider.current(this.button1Amount());
            this.betOrRaise();
            this.expanded(false);
        };
        ActionBlock.prototype.pot = function () {
            this.tableSlider.current(this.button2Amount());
            this.betOrRaise();
            this.expanded(false);
        };
        ActionBlock.prototype.updateNeedBB = function () {
            var currentPlayer = this.tableView.myPlayer();
            var hasPlayer = currentPlayer != null;
            if (!hasPlayer) {
                this.needBB(false);
                return;
            }
            var tableIsBig = this.tableView.places().filter(function (item) { return item != null; }).length > 3;
            this.needBB(tableIsBig && !currentPlayer.IsParticipatingStatus());
        };
        ActionBlock.prototype.getWaitBBBlockVisible = function () {
            if (this.sitoutBlockVisible()) {
                return false;
            }
            if (!this.needBB()) {
                return false;
            }
            if (this.myPlayer() == null) {
                return false;
            }
            var waitBBHidden = this.isInGame() || this.myPlayerWasInGame();
            return !waitBBHidden;
        };
        ActionBlock.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_12.debugSettings.actionBlock.traceBlocksVisbility) {
                console.log(message, params);
            }
        };
        return ActionBlock;
    }());
    exports.ActionBlock = ActionBlock;
});
define("table/tableCardsPlace", ["require", "exports", "knockout", "table/runtimesettings"], function (require, exports, ko, runtimeSettings) {
    "use strict";
    var TableCardsPlace = (function () {
        function TableCardsPlace() {
            var _this = this;
            this.tableCards = ko.observableArray([]);
            this.tableCardsData = ko.observableArray([]);
            this.isFlop = ko.observable(false);
            this.isFlop5 = ko.observable(false);
            this.isTurn = ko.observable(false);
            this.isTurn5 = ko.observable(false);
            this.isRiver = ko.observable(false);
            this.CardsHightlighted = ko.observable(false);
            this.Card1Hightlighted = ko.observable(false);
            this.Card2Hightlighted = ko.observable(false);
            this.Card3Hightlighted = ko.observable(false);
            this.Card4Hightlighted = ko.observable(false);
            this.Card5Hightlighted = ko.observable(false);
            this.isAnimating = ko.computed(function () {
                return _this.isFlop() || _this.isFlop5()
                    || _this.isTurn() || _this.isTurn5()
                    || _this.isRiver();
            }, this);
        }
        /**
        * Clear cards on the table
        */
        TableCardsPlace.prototype.clear = function () {
            this.CardsHightlighted(false);
            this.Card1Hightlighted(false);
            this.Card2Hightlighted(false);
            this.Card3Hightlighted(false);
            this.Card4Hightlighted(false);
            this.Card5Hightlighted(false);
            this.tableCards([]);
            this.tableCardsData([]);
        };
        /**
        * Clear all currently running animation
        */
        TableCardsPlace.prototype.clearAnimation = function () {
            this.isFlop(false);
            this.isFlop5(false);
            this.isTurn(false);
            this.isTurn5(false);
            this.isRiver(false);
        };
        /**
        * Set cards on the table.
        * @cardsData Array of table cards to display
        */
        TableCardsPlace.prototype.setCards = function (cardsData) {
            var cards = convertToCards(cardsData);
            this.tableCards(cards);
            this.tableCardsData(cardsData);
        };
        TableCardsPlace.prototype.openCards = function (cardsData) {
            var cards = convertToCards(cardsData);
            var currentCards = this.tableCards();
            if (runtimeSettings.tableCardsAnimating) {
                if (currentCards == null || currentCards.length === 0) {
                    if (cards.length === 3) {
                        this.isFlop(true);
                    }
                    else {
                        this.isFlop5(true);
                    }
                }
                else if (currentCards.length === 3) {
                    if (cards.length === 4) {
                        this.isTurn(true);
                    }
                    else {
                        this.isTurn5(true);
                    }
                }
                else {
                    if (currentCards.length !== 5) {
                        this.isRiver(true);
                    }
                }
            }
            currentCards = cards;
            this.tableCards(currentCards);
            this.tableCardsData(cardsData);
        };
        return TableCardsPlace;
    }());
    exports.TableCardsPlace = TableCardsPlace;
});
/* tslint:disable:no-bitwise */
define("table/tableview", ["require", "exports", "knockout", "moment", "timeService", "authmanager", "table/tabpleplacemodel", "table/tableplaces", "table/handhistory", "table/playerMessage", "table/SystemMessage", "services/index", "services/slowinternetservice", "popups/simplepopup", "table/actionblock", "table/gameactionsqueue", "debugsettings", "settings", "table/runtimesettings", "table/tableCardsPlace", "languageManager"], function (require, exports, ko, moment, timeService, authManager, tabpleplacemodel_2, tableplaces_1, handhistory_1, playerMessage_1, SystemMessage_1, services_7, slowinternetservice_2, simplepopup_10, actionblock_1, gameactionsqueue_2, debugsettings_13, settings_5, runtimeSettings, tableCardsPlace_1, languagemanager_17) {
    "use strict";
    var TableView = (function () {
        function TableView(tableId, model) {
            this.tableId = tableId;
            this.model = model;
            /**
            * Request which performs connecting to the table.
            */
            this.connectingRequest = null;
            this.sitting = false;
            this.currentCombination = ko.observable("");
            /**
            * Indicates that animation is suppressed from playing.
            */
            this.animationSuppressed = ko.observable(false);
            this.animationSettings = AnimationSettings.getSettings();
            /**
            * Tournament to which belongs given table.
            */
            this.tournament = ko.observable(null);
            /**
            * Indicates that table is frozen.
            */
            this.frozen = ko.observable(false);
            /**
            * Indicates that table is opened
            */
            this.opened = ko.observable(true);
            /**
            * Indicates that table is paused
            */
            this.paused = ko.observable(false);
            this.pauseDate = ko.observable(null);
            this.pauseDescription = ko.observable("");
            this.pauseDescriptionHandle = null;
            this.soundEnabled = false;
            this.hasPendingMoney = ko.observable(false);
            this.hasNotification = ko.observable(false);
            this.notification = ko.observable("");
            this.notificationHandleTimeout = null;
            this.notificationHandleInterval = null;
            this.displayingRebuyAddonNotification = false;
            /* If of the last message Id starting from which messages could be displayed */
            this.lastMessageId = 0;
            /// <signature>
            ///     <summary>Updates the information about the table from the server</summary>
            ///     <param name="tableId" type="Number">Id of the table for which view has to be created.</param>
            /// </signature>
            var self = this;
            this.tableId = tableId;
            this.tableName = ko.observable(model === null ? "0" : model.TableName);
            this.connecting = ko.observable(true);
            this.gamePlayers = ko.observableArray([]);
            this.pots = ko.observableArray([]);
            this.tableCards = new tableCardsPlace_1.TableCardsPlace();
            this.messages = ko.observableArray([]);
            this.systemMessages = ko.observableArray([]);
            this.chatMessage = ko.observable("");
            this.playerActions = ko.observable();
            this.mainButtonsEnabled = ko.observable(false);
            this.gameId = ko.observable();
            this.actionsCount = ko.observable(0);
            this.bigBlind = ko.observable(model === null ? 0 : model.BigBlind);
            this.smallBlind = ko.observable(model === null ? 0 : model.SmallBlind);
            this.ante = ko.observable(null);
            this.minimalPlayerBuyIn = ko.observable();
            this.minimalBuyIn = ko.observable();
            this.lastRaise = ko.observable();
            this.timePass = ko.observable();
            this.gameFinished = ko.observable(true);
            this.prizesDistributed = ko.observable(true);
            this.gameStarted = ko.observable(false);
            this.combinations = ko.observableArray([]);
            this.actionBlock = new actionblock_1.ActionBlock();
            this.tablePlaces = new tableplaces_1.TablePlaces(model == null ? 0 : model.MaxPlayers);
            this.chipWidth = 30;
            this.cardsReceived = false;
            this.onMyTurn = new signals.Signal();
            this.queue = new gameactionsqueue_2.GameActionsQueue();
            this.places = ko.computed(function () {
                return self.tablePlaces.places();
            }, this);
            if (typeof window !== "undefined") {
                if (window.innerWidth >= 1024 || window.innerHeight >= 1024) {
                    this.chipWidth = 60;
                    if (window.innerWidth >= 1920 || window.innerHeight >= 1920) {
                        this.chipWidth = 2 * 83;
                    }
                }
            }
            this.myPlayer = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var p = self.places().filter(function (item) {
                    return item.PlayerName() === self.currentLogin();
                });
                if (p.length > 0) {
                    return p[0];
                }
                return null;
            }, this).extend({ notify: "always" });
            this.myPlayerInGame = ko.computed(function () {
                var gid = self.gameId();
                if (gid === null || gid === 0) {
                    return false;
                }
                var myself = self.myPlayer();
                if (myself === null) {
                    return false;
                }
                return (myself.Cards() !== null) && (myself.Cards().length !== 0);
            }, this);
            this.currentLogin = ko.observable(authManager.login());
            authManager.login.subscribe(function (value) {
                self.currentLogin(value);
            });
            this.timeLeft = ko.computed(function () {
                if (self.frozen()) {
                    return -1;
                }
                var pass = self.timePass();
                pass = pass === null ? 0 : pass;
                return 30 - pass;
            });
            this.currentRaise = ko.computed({
                read: function () {
                    return self.actionBlock.tableSlider.current();
                },
                write: function (value) {
                    self.actionBlock.tableSlider.current(value);
                },
                owner: this
            });
            this.currentPlayer = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var p = self.places().filter(function (item) {
                    return item.IsCurrent();
                });
                if (p.length > 0) {
                    return p[0];
                }
                return null;
            }, this);
            this.activePlayersCount = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var activePlayersCount = self.places().reduce(function (prev, value) {
                    var isActive = value.WasInGame() && (value.Cards() !== null);
                    return prev + (isActive ? 1 : 0);
                }, 0);
                return activePlayersCount;
            }, this);
            this.myPlayerWasInGame = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var myself = self.myPlayer();
                if (myself === null) {
                    return false;
                }
                return (myself.WasInGame() === true);
            }, this);
            this.turnEnabled = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var cp = self.currentPlayer();
                if (cp === null) {
                    return false;
                }
                var activePlayersCount = this.places().reduce(function (prev, value) {
                    var isActive = value.WasInGame() && (value.Cards() !== null);
                    return prev + (isActive ? 1 : 0);
                }, 0);
                return (cp.PlayerName() === self.currentLogin()) && activePlayersCount > 1;
            }, this);
            this.isMyTurn = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var cp = self.myPlayer();
                if (cp === null || !cp.IsInGameStatus()) {
                    return false;
                }
                if (!self.gameStarted()) {
                    return false;
                }
                if (cp.IsCurrent() === null) {
                    return false;
                }
                return cp.IsCurrent();
            }, this);
            this.notMyTurn = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var cp = self.myPlayer();
                if (cp === null || !cp.IsInGameStatus()) {
                    return false;
                }
                if (cp.IsCurrent() === null) {
                    return true;
                }
                return !cp.IsCurrent();
            }, this);
            this.isInGame = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var cp = self.myPlayer();
                if (cp === null || !cp.IsInGameStatus()) {
                    return false;
                }
                return true;
            });
            this.maximumBet = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var result = 0;
                result = self.places().reduce(function (previousValue, currentValue) {
                    return Math.max(previousValue, currentValue.Bet());
                }, 0);
                return result;
            }, this);
            this.allBetsRounded = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var playersInGame = self.places().filter(function (value) { return value.WasInGame(); }).length;
                var activePlayers = self.places().filter(function (value) { return value.WasInGame() && (value.Cards() !== null); });
                var maxBet = self.maximumBet();
                var allRounded = activePlayers.filter(function (player) { return (player.Bet() === maxBet) || (player.Money() === 0); }).length
                    === activePlayers.length;
                if (allRounded && (self.actionsCount() >= playersInGame)) {
                    return true;
                }
                return false;
            }, this);
            this.currentBet = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var result = self.places().reduce(function (previousValue, currentValue) {
                    if (currentValue.IsCurrent()) {
                        return currentValue.Bet();
                    }
                    return previousValue;
                }, 0);
                return result;
            }, this);
            this.myBet = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var myPlayer = self.myPlayer();
                if (myPlayer === null) {
                    return 0;
                }
                return myPlayer.Bet();
            }, this);
            this.currentTotalBet = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var result = self.places().reduce(function (previousValue, currentValue) {
                    if (currentValue.IsCurrent()) {
                        return currentValue.TotalBet();
                    }
                    return previousValue;
                }, 0);
                return result;
            }, this);
            this.checkOrCallAmount = ko.computed(function () {
                return self.maximumBet() - self.currentBet();
            }, this);
            this.minimumRaiseAmount = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var currentPlayer = self.myPlayer();
                if (currentPlayer === null) {
                    return null;
                }
                var cb = self.currentBet();
                var mb = 2 * (self.lastRaise() - cb) - self.checkOrCallAmount();
                mb = Math.max(mb, self.bigBlind());
                mb = Math.min(mb, currentPlayer.Money());
                var addon = cb + self.checkOrCallAmount();
                var raiseAmount = mb + addon;
                var maxAmountOfMoneyForOtherActivePlayers = self.maxAmountOfMoneyForOtherActivePlayers();
                raiseAmount = Math.min(raiseAmount, maxAmountOfMoneyForOtherActivePlayers);
                return raiseAmount;
            }, this);
            this.isSitOut = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var currentPlayer = this.myPlayer();
                if (currentPlayer === null) {
                    return false;
                }
                return (currentPlayer.Status() & 1) !== 0;
            }, this);
            this.maxAmountOfMoneyForOtherActivePlayers = ko.computed(function () {
                var result = 0;
                result = self.places().reduce(function (previousValue, currentValue) {
                    if (currentValue.PlayerName() === self.currentLogin()) {
                        return previousValue;
                    }
                    if (!currentValue.IsInGameStatus()) {
                        return previousValue;
                    }
                    if (currentValue.Cards() === null) {
                        return previousValue;
                    }
                    return Math.max(previousValue, currentValue.Bet() + currentValue.Money());
                }, 0);
                return result;
            }).extend({ notify: "always" });
            this.maximumRaiseAmount = ko.computed(function () {
                var currentPlayer = self.myPlayer();
                if (currentPlayer === null) {
                    return null;
                }
                var max = self.maxAmountOfMoneyForOtherActivePlayers();
                var bet = currentPlayer.Bet();
                var money = currentPlayer.Money();
                return Math.min(money + bet, max) - bet;
            }, this).extend({ notify: "always" });
            this.totalPot = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var totalBetsOnTable = self.places().reduce(function (prev, item) {
                    return prev + item.Bet();
                }, 0);
                var potsArray = self.pots();
                if (potsArray === null) {
                    return totalBetsOnTable;
                }
                var pots = potsArray.reduce(function (prev, item) {
                    return prev + item;
                }, 0);
                return totalBetsOnTable + pots;
            }, this);
            this.totalPotCaption = ko.computed(function () {
                self.tablePlaces.placesRefreshTrigger();
                var totalPot = self.totalPot();
                if (totalPot === null || totalPot === 0) {
                    return null;
                }
                return languagemanager_17._("table.totalpot")
                    .replace("#amount", totalPot.toFixed());
            }, this);
            this.tableBetsCaption = ko.computed(function () {
                var hasAnte = self.ante() != null;
                if (hasAnte) {
                    return languagemanager_17._("table.betsWithAnte", { sb: self.smallBlind(), bb: self.bigBlind(), ante: self.ante() });
                }
                return languagemanager_17._("table.bets", { sb: self.smallBlind(), bb: self.bigBlind() });
            }, this);
            this.couldAddChips = ko.pureComputed(function () {
                var me = self.myPlayer();
                if (me == null) {
                    return false;
                }
                if (self.hasPendingMoney()) {
                    return false;
                }
                var totalBet = (me.TotalBet() == null ? 0 : me.TotalBet());
                var baseMinimalBuyIn = self.minimalBuyIn() * self.model.BigBlind;
                var tableTotal = totalBet + me.Money() + me.Bet();
                return (20 * baseMinimalBuyIn) > tableTotal;
            }, this);
            this.actionBlock.attach(this);
            this.initHandHistory();
        }
        /**
        * Gets current cards combination
        */
        TableView.prototype.getCurrentCombination = function () {
            var my = this.myPlayer();
            if (my == null) {
                return null;
            }
            var tableCards = this.tableCards.tableCardsData();
            return my.getCombination(tableCards);
        };
        TableView.prototype.startTimer = function (startTime) {
            if (startTime === void 0) { startTime = 1; }
            var self = this;
            if (this.frozen()) {
                return;
            }
            self.timePass(startTime);
            this.timerInterval = timeService.setInterval(function () {
                if (runtimeSettings.updateTimer) {
                    var time = self.timePass();
                    self.timePass(time + 1);
                    if (self.timeLeft() === 7) {
                        if (self.currentPlayer() === self.myPlayer()) {
                            if (self.soundEnabled) {
                                services_7.soundManager.playTurnReminder();
                            }
                        }
                    }
                }
            }, 1000);
        };
        TableView.prototype.clearTimer = function () {
            timeService.clearInterval(this.timerInterval);
            this.timePass(null);
        };
        TableView.prototype.clearInformation = function () {
            this.messages([]);
        };
        TableView.prototype.updateTableInformation = function () {
            /// <signature>
            ///     <summary>Updates the information about the table from the server</summary>
            /// </signature>
            var self = this;
            if (this.connectingRequest !== null && this.connectingRequest.state() === "pending") {
                // Re-schedule updating information.
                this.connectingRequest.fail(function () {
                    self.log("Rescheduling the updating information.");
                    self.updateTableInformation();
                });
                self.log("Cancelling the connection request process");
                self.cancelUpdateTableInformation();
                return;
            }
            this.connecting(true);
            var currentLoadingRequest = $.Deferred();
            this.clearTimer();
            var wrapper = services_7.connectionService.currentConnection;
            var hubId = wrapper.connection.id;
            var connectionInfo = "HID:" + hubId;
            this.log("Connecting to table " + this.tableId + " on connection " + connectionInfo);
            var startConnection = app.buildStartConnection();
            var api = new OnlinePoker.Commanding.API.Game(apiHost);
            api.SetOpenCardsParameters(this.tableId, !settings_5.settings.autoHideCards(), null);
            startConnection().then(function () {
                if (wrapper.terminated) {
                    return;
                }
                hubId = wrapper.connection.id;
                self.log("Attempting to connect to table and chat over connection " + hubId);
                var joinTableRequest = self.joinTable(wrapper);
                var joinChatRequest = self.joinChat(wrapper);
                var joinRequest = $.when(joinTableRequest, joinChatRequest);
                currentLoadingRequest.progress(function (command) {
                    self.log("Receiving request to cancel all joining operations");
                    joinTableRequest.notify(command);
                    joinChatRequest.notify(command);
                });
                joinRequest.then(function () {
                    if (wrapper.terminated) {
                        currentLoadingRequest.reject("Cancelled");
                        return;
                    }
                    self.log("Jointing to table finished");
                    currentLoadingRequest.resolve();
                }, function (result1, result2) {
                    if (wrapper.terminated) {
                        return;
                    }
                    var message;
                    if (result2 == null) {
                        var taskFailed = void 0;
                        if (result1[2] === joinTableRequest) {
                            taskFailed = "join table";
                        }
                        else {
                            taskFailed = "join chat";
                        }
                        message = "Rejecting request due to " + taskFailed + " failure in the connection."
                            + "Failed request: " + result1[0];
                    }
                    else {
                        message = "Rejecting request due to one of the error in the connection."
                            + "First request: " + result1[0]
                            + "Second request: " + result2[0];
                    }
                    self.log(message);
                    currentLoadingRequest.reject(message);
                });
            }, function (message) {
                self.log("Table connection failed. Error: " + message);
                currentLoadingRequest.reject("Table connection failed. Error: " + message);
            });
            this.connectingRequest = currentLoadingRequest;
        };
        TableView.prototype.cancelUpdateTableInformation = function () {
            if (this.connectingRequest !== null) {
                this.connectingRequest.notify("cancel");
                this.connectingRequest = null;
            }
        };
        TableView.prototype.joinTable = function (wrapper, maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var self = this;
            var result = $.Deferred();
            if (maxAttempts === 0 || wrapper.terminated) {
                this.log("Stop connecting to table");
                result.reject("Stop connecting to table", false);
                return result;
            }
            var hubId = services_7.connectionService.currentConnection.connection.id;
            var connectionInfo = "HID:" + hubId;
            this.log("Joining table on connection " + connectionInfo);
            var cancelled = false;
            var subsequentDeferred = null;
            var cancelOperation = function () {
                self.log("Cancelling join table request");
                result.reject("Cancelled", true);
            };
            wrapper.buildStartConnection()().pipe(function () {
                if (wrapper.terminated) {
                    cancelOperation();
                    return;
                }
                self.log("Executing Game.join on connection " + wrapper.connection.id + " in state " + wrapper.connection.state);
                var operation = wrapper.connection.Game.server.join(self.tableId)
                    .then(function () {
                    if (wrapper.terminated) {
                        cancelOperation();
                        return;
                    }
                    result.resolve();
                }, function (error) {
                    if (wrapper.terminated || cancelled || error === "Cancelled") {
                        cancelOperation();
                        return;
                    }
                    var message = "" + error;
                    self.log("Failed to join table " + self.tableId + ", " + connectionInfo + ". Reason: " + message);
                    if (message.indexOf("Connection was disconnected before invocation result was received.") >= 0) {
                        self.log("Stopped connecting to table since underlying connection is broken");
                        services_7.slowInternetService.showReconnectFailedPopup();
                        result.reject("Stopped connecting to table since underlying connection is broken", false);
                        return;
                    }
                    else {
                        subsequentDeferred = self.joinTable(wrapper, maxAttempts - 1);
                        return subsequentDeferred.then(function () {
                            result.resolve();
                        }, function (error, cancelled) {
                            result.reject(error, cancelled);
                        });
                    }
                });
                result.progress(function (command) {
                    this.cancelled = true;
                    result.reject("Cancelled");
                    if (subsequentDeferred !== null) {
                        subsequentDeferred.notify("cancel");
                        subsequentDeferred = null;
                    }
                });
            }, function () {
                cancelOperation();
            });
            return result;
        };
        TableView.prototype.joinChat = function (wrapper, maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var self = this;
            var result = $.Deferred();
            if (maxAttempts === 0 || wrapper.terminated) {
                this.log("Stop connecting to table chat");
                result.reject("Stop connecting to table chat", false);
                return result;
            }
            var cancelled = false;
            var subsequentDeferred = null;
            var cancelOperation = function () {
                self.log("Cancelling join table request");
                result.reject("Cancelled", true);
            };
            wrapper.buildStartConnection()().pipe(function () {
                if (wrapper.terminated) {
                    cancelOperation();
                    return;
                }
                self.log("Executing Game.join on connection " + wrapper.connection.id + " in state " + wrapper.connection.state);
                var operation = wrapper.connection.Chat.server.join(self.tableId)
                    .then(function () {
                    if (wrapper.terminated) {
                        cancelOperation();
                        return;
                    }
                    result.resolve();
                }, function (message) {
                    if (wrapper.terminated || cancelled) {
                        self.log("Cancelling join table chat request");
                        result.reject("Cancelled", true);
                        return;
                    }
                    self.log("Failed to join table " + self.tableId + " chat. Reason: " + message);
                    subsequentDeferred = self.joinChat(wrapper, maxAttempts - 1);
                    return subsequentDeferred.then(function () {
                        result.resolve();
                    }, function (error, cancelled) {
                        result.reject(error, cancelled);
                    });
                });
                result.progress(function (command) {
                    this.cancelled = true;
                    result.reject("Cancelled");
                    if (subsequentDeferred !== null) {
                        subsequentDeferred.notify("cancel");
                        subsequentDeferred = null;
                    }
                });
            }, function () {
                cancelOperation();
            });
            return result;
        };
        TableView.prototype.disconnect = function () {
            /// <signature>
            ///     <summary>Updates the information about the table from the server</summary>
            /// </signature>
            var self = this;
            // HACK: Game server should correctly leave from table.
            // Looks like when calling leave status stop receiving player specific 
            // notifications. Which is looks like SignalR connection is not correctly 
            // removed from the corresponding group.
            // connectionService.currentConnection.connection.Game.server.leave(this.tableId);
            // connectionService.currentConnection.connection.Chat.server.leave(this.tableId);
        };
        TableView.prototype.setDealer = function (dealerSeat) {
            var players = this.places();
            players.forEach(function (p) {
                if (p.Seat() === dealerSeat) {
                    p.IsDealer(true);
                }
                else {
                    p.IsDealer(null);
                }
            });
            // this.places(players);
        };
        TableView.prototype.setCurrent = function (currentPlayerId) {
            var players = this.places();
            players.forEach(function (p) {
                if (p.PlayerId() === currentPlayerId) {
                    p.IsCurrent(true);
                }
                else {
                    p.IsCurrent(null);
                }
            });
        };
        TableView.prototype.onSit = function (playerId, seat, playerName, amount, playerUrl, points, stars) {
            /// <signature>
            ///     <summary>Sits players on the table.</summary>
            ///     <param name="playerId" type="Number">Id of the player which join the table</param>
            ///     <param name="seat" type="Number">Number of seat where player sit.</param>
            ///     <param name="playerName" type="String">Name of the player which join the table</param>
            ///     <param name="amount" type="Number">Amount of money which player bring to the table</param>
            /// </signature>
            this.logGameEvent("Player " + playerId + " sit on the seat " + seat);
            var places = this.places();
            var playerModel = new tabpleplacemodel_2.TablePlaceModel({
                PlayerId: playerId,
                PlayerName: playerName,
                PlayerUrl: playerUrl,
                Seat: seat,
                Money: amount,
                Cards: null,
                Bet: null,
                Status: 0,
                Points: points,
                Stars: stars
            });
            this.tablePlaces.sit(seat, playerModel);
            this.refreshPlaces();
            this.actionBlock.updateNeedBB();
            this.actionBlock.updateBlocks();
            if (playerName === this.currentLogin()) {
                var api = new OnlinePoker.Commanding.API.Game(apiHost);
                api.SetOpenCardsParameters(this.tableId, !settings_5.settings.autoHideCards(), null);
            }
        };
        TableView.prototype.onStandup = function (playerId) {
            /// <signature>
            ///     <summary>Standup player from the table.</summary>
            ///     <param name="playerId" type="Number">Id of the player which join the table</param>
            /// </signature>
            var places = this.places();
            var lastAmount = 0;
            for (var p in places) {
                if (!places.hasOwnProperty(p)) {
                    continue;
                }
                var currentPlayer = places[p];
                if (currentPlayer.PlayerId() === playerId) {
                    if (currentPlayer.WasInGame()) {
                        this.actionsCount(Math.max(0, this.actionsCount() - 1));
                    }
                    lastAmount = currentPlayer.Money();
                    this.tablePlaces.standup(currentPlayer.Seat());
                    break;
                }
            }
            // If we no longer on the table then leave it.
            if (this.myPlayer() === null) {
                this.minimalPlayerBuyIn(lastAmount);
                this.updateCurrentCombination();
            }
            this.refreshPlaces();
            this.actionBlock.updateNeedBB();
            this.actionBlock.updateBlocks();
        };
        TableView.prototype.onFrozen = function () {
            this.frozen(true);
            this.clearTimer();
        };
        TableView.prototype.onUnfrozen = function () {
            this.frozen(false);
            this.startTimer();
        };
        TableView.prototype.onOpened = function () {
            var _this = this;
            this.pushCallback(function () {
                _this.opened(true);
            });
        };
        TableView.prototype.onClosed = function () {
            var _this = this;
            this.pushCallback(function () {
                _this.opened(false);
                _this.tablePlaces.clear();
                _this.pots([]);
                _this.tableCards.clear();
            });
        };
        TableView.prototype.onPaused = function () {
            this.paused(true);
            this.pauseDate(new Date().valueOf());
            this.updatePauseDescription();
        };
        TableView.prototype.onResumed = function () {
            this.clearPauseMessage();
        };
        /**
        * Informs about current state on the table.
        * @param players Array of PlayerStatusInfo objects which describe current status of the players
        * @param pots Array of pots on the table.
        * @param cards Cards open on the table
        * @param dealerSeat Current dealer seat. 0 if no game on the table.
        * @param buyIn Minimum amount of money which player should bring on the table.
        * @param baseBuyIn Base amount in BB for joining table under normal conditions.
        *                   20*baseBuyIn is maximum amount which player could bring-in.
        * @param leaveTime Time when player leave table last time.
        * @param timePass Time in second which is pass from last player turn.
        * @param currentPlayerId Id of the current player in the game.
        * @param lastRaise Amount of last raise in the game.
        * @param gameId Id of the game
        * @param authenticated Value indicating whether current user is authenticated or not.
        * @param actionsCount Count of actions which was performed by the player.
        * @param frozen A value indicating whether game on the table is frozen or not.
        * @param opened A value indicating whether the table is opened or not.
        * @param pauseData Unix time when game was paused. If game not paused, then null.
        * @param lastMessageId Last id of the message.
        */
        TableView.prototype.onTableStatusInfo = function (players, pots, cards, dealerSeat, buyIn, baseBuyIn, leaveTime, timePass, currentPlayerId, lastRaise, gameId, authenticated, actionsCount, frozen, opened, pauseDate, lastMessageId) {
            var _this = this;
            var tablePlayers = [];
            for (var i = 0; i < players.length; i++) {
                var p = players[i];
                tablePlayers.push(p);
            }
            this.lastMessageId = lastMessageId;
            this.frozen(frozen);
            this.opened(opened);
            this.paused(pauseDate != null);
            this.pauseDate(pauseDate);
            this.updatePauseDescription();
            this.queue.clear();
            if (opened) {
                this.gameId(gameId);
                this.actionsCount(actionsCount);
                var gameFinished = gameId === 0 || gameId === null;
                this.gameFinished(gameFinished);
                this.gameStarted(!gameFinished);
                this.prizesDistributed(this.gameFinished());
                this.tablePlaces.setPlayers(players);
                if (this.activePlayersCount() <= 1) {
                    this.gameFinished(true);
                }
                var cardsArr = decodeCardsArray(cards);
                this.setCards(cardsArr);
                this.setDealer(dealerSeat);
                this.actionBlock.buttonsEnabled(true);
                this.actionBlock.dealsAllowed(true);
                timeService.setTimeout(function () {
                    _this.actionBlock.updateBounds();
                }, 500);
                this.setCurrent(currentPlayerId);
                this.minimalPlayerBuyIn(buyIn);
                this.minimalBuyIn(baseBuyIn);
                this.pots(pots);
                this.refreshPlaces();
                this.clearTimer();
                this.startTimer(timePass);
                this.cardsReceived = true;
                if (cards != null) {
                    if (cards.length === 0) {
                        lastRaise = Math.max(lastRaise, this.bigBlind());
                    }
                }
                this.lastRaise(lastRaise);
                this.currentRaise(this.minimumRaiseAmount());
                this.actionBlock.isRaise(lastRaise > 0);
                this.actionBlock.isCheck(this.checkOrCallAmount() === 0);
                this.actionBlock.checkOrCallAmount(this.checkOrCallAmount());
                this.actionBlock.resetWaitBB();
                this.actionBlock.processing(false);
                this.actionBlock.updateBlocks();
                var myself = this.myPlayer();
                if (myself != null) {
                    this.actionBlock.updateAutomaticActionsText(myself.Money(), this.maximumBet() - this.myBet());
                }
                this.updateCurrentCombination();
            }
            else {
                this.tablePlaces.clear();
                this.pots([]);
                this.tableCards.clear();
            }
            this.cleanTableAfterGameFinish();
            this.connecting(false);
            this.log("Finishing connecting to table " + this.tableId);
        };
        TableView.prototype.onGameStartedCore = function (gameId, players, actions, dealerSeat) {
            /// <signature>
            ///     <summary>Start the game.</summary>
            ///     <param name="gameId" type="Number">Id of the player which join the table</param>
            ///     <param name="players" type="Array" 
            ///             value="[{PlayerId:1,PlayerName:'Login',PlayerUrl:'http://url/1.png',Money:100,Seat:9,Cards:[1,2],Status:1}]">
            ///     </param>
            ///     <param name="dealerSeat" type="Number">Seat where dealer sitting.</param>
            /// </signature>
            this.gameFinished(false);
            this.prizesDistributed(false);
            this.gameId(gameId);
            this.hasPendingMoney(false);
            this.actionBlock.isRaise(true);
            this.actionBlock.updateBounds();
            this.actionsCount(0);
            this.setDealer(dealerSeat);
            this.tableCards.clear();
            this.pots([]);
            this.logGameEvent("Game started");
            if (this.displayingRebuyAddonNotification) {
                this.clearNotification();
            }
            var combinations = [];
            this.cardsReceived = false;
            this.combinations(combinations);
            this.gamePlayers(players.map(function (item) { return item.PlayerId; }));
            this.places().forEach(function (value) {
                value.prepareForNewGame();
                players.forEach(function (pvalue) {
                    if (pvalue.PlayerId === value.PlayerId()) {
                        value.initializeForNewGame(pvalue.Money);
                    }
                });
                if (players.every(function (pvalue) { return pvalue.PlayerId !== value.PlayerId(); })) {
                    value.WasInGame(null);
                }
            });
            if (this.soundEnabled) {
                services_7.soundManager.playDealCards();
            }
            this.refreshPlaces();
            this.lastRaise(this.bigBlind());
            this.actionBlock.resetAutomaticAction();
            if (players.some(function (_) { return _.PlayerId === authManager.loginId(); })) {
                // Reset Wait BB status since player currently join the game.
                this.actionBlock.resetWaitBB();
            }
        };
        TableView.prototype.startDealCards = function () {
            var self = this;
            this.queue.pushCallback(function () {
                self.logGameEvent("Deal cards to players");
                self.places().forEach(function (value) {
                    if (value.WasInGame()) {
                        value.startDealCards();
                    }
                });
            });
            this.queue.wait(this.animationSettings.dealCardsTime);
            this.queue.pushCallback(function () {
                self.places().forEach(function (value) {
                    value.IsDealCards(false);
                });
                self.clearTimer();
                self.startTimer();
            });
        };
        TableView.prototype.onGameStarted = function (gameId, players, actions, dealerSeat) {
            var _this = this;
            this.queue.pushCallback(function () {
                _this.onGameStartedCore(gameId, players, actions, dealerSeat);
                _this.handHistory.onGameStarted(gameId, players, actions, dealerSeat);
            });
            var isInGame = players.some(function (_) { return _.PlayerId === authManager.loginId(); });
            if (!isInGame) {
                this.startDealCards();
            }
        };
        TableView.prototype.onGameFinished = function (gameId, winners, rake) {
            var _this = this;
            /// <signature>
            ///     <summary>Start the game.</summary>
            ///     <param name="gameId" type="Number">Id of the player which join the table</param>
            ///     <param name="winners" type="Array" 
            ///             value="[{PlayerId:1,Pot:1,Amount:100,CardsDescription:'Hands 1'}]">
            ///     </param>
            ///     <param name="rake" type="Number">Amount of rake paid.</param>
            /// </signature>
            var self = this;
            this.queue.wait(this.animationSettings.finishGamePrePause);
            this.queue.pushCallback(function () {
                _this.logGameEvent("Game finished");
                self.gameFinished(true);
                self.gameStarted(false);
                self.gamePlayers([]);
                var places = self.places();
                self.finishAnimation(places);
                var c = [];
                var needHightlightCards = true;
                var activePlayersCount = _this.activePlayersCount();
                _this.logGameEvent("Active players count", activePlayersCount);
                needHightlightCards = activePlayersCount > 1;
                if (needHightlightCards) {
                    self.tableCards.CardsHightlighted(true);
                }
                for (var w in winners) {
                    if (!winners.hasOwnProperty(w)) {
                        continue;
                    }
                    var currentWinner = winners[w];
                    if (c.indexOf(currentWinner.CardsDescription) === -1) {
                        c.push(currentWinner.CardsDescription);
                    }
                    var _loop_5 = function(p) {
                        if (!places.hasOwnProperty(p)) {
                            return "continue";
                        }
                        var currentPlayer = places[p];
                        if (needHightlightCards) {
                            currentPlayer.CardsHightlighted(true);
                        }
                        if (currentPlayer.PlayerId() === currentWinner.PlayerId) {
                            var winnerCards = [];
                            var tableCards = self.tableCards.tableCardsData();
                            if (tableCards != null) {
                                winnerCards = winnerCards.concat(tableCards);
                            }
                            winnerCards = winnerCards.concat(currentPlayer.RawCards());
                            var handRepresentation_1 = {
                                Cards: [],
                                Suits: []
                            };
                            winnerCards.forEach(function (card) {
                                handRepresentation_1.Cards.push((card % 13) + 2);
                                handRepresentation_1.Suits.push(1 << (card / 13));
                            });
                            if (winnerCards.length === 7 && needHightlightCards) {
                                var rank = HoldemHand.getCardRank(handRepresentation_1);
                                rank.WinnerCardsSet.forEach(function (cardIndex) {
                                    if (cardIndex === 5) {
                                        currentPlayer.Card1Hightlighted(true);
                                    }
                                    if (cardIndex === 6) {
                                        currentPlayer.Card2Hightlighted(true);
                                    }
                                    if (cardIndex === 0) {
                                        self.tableCards.Card1Hightlighted(true);
                                    }
                                    if (cardIndex === 1) {
                                        self.tableCards.Card2Hightlighted(true);
                                    }
                                    if (cardIndex === 2) {
                                        self.tableCards.Card3Hightlighted(true);
                                    }
                                    if (cardIndex === 3) {
                                        self.tableCards.Card4Hightlighted(true);
                                    }
                                    if (cardIndex === 4) {
                                        self.tableCards.Card5Hightlighted(true);
                                    }
                                });
                            }
                            currentPlayer.Money(currentPlayer.Money() + currentWinner.Amount);
                            if (currentPlayer.WinAmount() === null) {
                                currentPlayer.WinAmount(currentWinner.Amount);
                            }
                            else {
                                currentPlayer.WinAmount(currentPlayer.WinAmount() + currentWinner.Amount);
                            }
                        }
                    };
                    for (var p in places) {
                        _loop_5(p);
                    }
                }
                if (self.soundEnabled) {
                    services_7.soundManager.playWinChips();
                }
                self.combinations(c);
                self.setDealer(0);
                self.cardsReceived = false;
                self.actionBlock.buttonsEnabled(false);
                self.actionBlock.dealsAllowed(false);
                self.setCurrent(0);
                self.pots([]);
                self.refreshPlaces();
                self.clearTimer();
                self.gameId(null);
                self.handHistory.onGameFinished(gameId, winners, rake);
                self.saveHandHistory();
            });
            this.queue.wait(this.animationSettings.cleanupTableTimeout);
            this.queue.pushCallback(function () {
                self.cleanTableAfterGameFinish();
                self.proposeRebuyOrAddon();
                self.displayRebuyOrAddonTime();
            });
        };
        TableView.prototype.cleanTableAfterGameFinish = function () {
            if (this.gameFinished()) {
                this.prizesDistributed(true);
                this.tableCards.CardsHightlighted(false);
                this.setCards([]);
                this.pots([]);
                var places = this.places();
                places.forEach(function (item) {
                    item.prepareForNewGame();
                });
            }
        };
        TableView.prototype.onPlayerStatusCore = function (playerId, status) {
            this.logGameEvent("onPlayerStatus", playerId, status);
            var self = this;
            var places = this.places();
            places.forEach(function (p) {
                if (p.PlayerId() === playerId) {
                    var saveMask = 16;
                    p.Status(status | (p.Status() & saveMask));
                    if (playerId === authManager.loginId()) {
                        if (p.IsSitoutStatus()) {
                            self.actionBlock.processing(false);
                        }
                    }
                }
            });
            this.refreshPlaces();
            this.actionBlock.updateNeedBB();
            this.actionBlock.updateBlocks();
        };
        TableView.prototype.onPlayerStatus = function (playerId, status) {
            var _this = this;
            this.queue.pushCallback(function () {
                _this.onPlayerStatusCore(playerId, status);
            });
        };
        TableView.prototype.onPlayerCardsCore = function (playerId, cards) {
            var self = this;
            var oldCardsReceived = this.cardsReceived;
            this.cardsReceived = true;
            var myPlayer = this.myPlayer();
            var isHoleCards = false;
            if (myPlayer != null) {
                // If this is first cards received then this is our cards
                // Allow deals only when your cards is recived.
                if (!oldCardsReceived) {
                    this.actionBlock.dealsAllowed(true);
                    this.gameStarted(true);
                    isHoleCards = true;
                    this.handHistory.onPlayerHoleCards(playerId, cards);
                    this.logGameEvent("Received hole cards");
                    this.updateCurrentCombination();
                }
                if (myPlayer.IsCurrent()) {
                    this.currentRaise(this.actionBlock.tableSlider.minimum());
                    this.actionBlock.buttonsEnabled(true);
                    this.actionBlock.updateAdditionalButtons();
                    this.actionBlock.performAutomaticActions();
                    this.actionBlock.resetAutomaticAction();
                    this.logGameEvent("Player turn on the beginning of the game.");
                    this.onMyTurn.dispatch(this.tableId);
                }
            }
            this.logGameEvent("onPlayerCards", playerId, convertToCards(cards));
            var places = this.places();
            places.forEach(function (p) {
                if (p.PlayerId() === playerId) {
                    var myself = self.currentLogin() === p.PlayerName();
                    var couldDisplayOtherCards = myself || self.combinations().length > 0;
                    couldDisplayOtherCards = true;
                    if (couldDisplayOtherCards) {
                        p.setCards(cards);
                    }
                }
            });
            this.refreshPlaces();
            if (!isHoleCards) {
                this.handHistory.onPlayerCards(playerId, cards);
            }
        };
        TableView.prototype.onPlayerCards = function (playerId, cards) {
            var _this = this;
            var self = this;
            this.queue.pushCallback(function () {
                if (!self.cardsReceived) {
                    if (playerId === authManager.loginId()) {
                        self.startDealCards();
                        _this.queue.pushCallback(function () {
                            self.onPlayerCardsCore(playerId, cards);
                        });
                    }
                    else {
                        self.onPlayerCardsCore(playerId, cards);
                    }
                }
                else {
                    self.onPlayerCardsCore(playerId, cards);
                }
            });
        };
        TableView.prototype.onPlayerCardsMucked = function (playerId) {
            var _this = this;
            this.queue.pushCallback(function () {
                var currentPlayer = _this.tablePlaces.getPlaceByPlayerId(playerId);
                _this.foldCardsForPlayer(currentPlayer, true, _this.animationSettings.foldAnimationTimeout / 2);
            });
        };
        TableView.prototype.updateBetsAndMoney = function (currentPlayer, type, amount) {
            var playerId = currentPlayer.PlayerId();
            var myself = this.myPlayer();
            var totalBet = (currentPlayer.TotalBet() === null ? 0 : currentPlayer.TotalBet());
            var currentBet = currentPlayer.Bet();
            var maximumBet = this.maximumBet();
            var amountPlaced = amount - currentBet;
            if (type === 6) {
                currentPlayer.Money(currentPlayer.Money() + amount);
                this.handHistory.onReturnMoney(playerId, amount);
                return;
            }
            var newMoney = currentPlayer.Money() - amountPlaced;
            currentPlayer.Money(newMoney);
            // If player raise then update last raise amount.
            // We have to do that before current bet for the player is changed,
            // Since calculation depends on the amount of last bet.
            if (type === 3) {
                var raiseAmount = amount;
                // Since we have raise in this round, then we no longer could
                // do bets.
                this.actionBlock.isRaise(true);
                // The raise amount could be actually smaller then the last raise amount
                // because player goes all-in, other players should be forced to pay more
                // to do the raise.
                if (raiseAmount > this.lastRaise()) {
                    this.lastRaise(raiseAmount);
                }
            }
            if (type === 4) {
                this.foldCardsForPlayer(currentPlayer, false, this.animationSettings.foldAnimationTimeout);
                // Mark player as not in game.
                // During the new game this flag should be set automatically.
                currentPlayer.IsInGameStatus(false);
            }
            if (type !== 4) {
                // If player folds then current bet amount should not be changed.
                currentPlayer.Bet(amount);
                if (type === 0 || type === 1 || type === 5) {
                    currentPlayer.IsBetAnimationLocked(true);
                }
                else {
                    // Lock animation place after timeout.
                    this.queue.wait(this.animationSettings.betAnimationTimeout);
                    this.queue.pushCallback(function () {
                        currentPlayer.IsBetAnimationLocked(true);
                    });
                }
            }
            // Display action which current player executes.
            if (type === 4) {
                // Display fold action
                currentPlayer.startAction("table.actiontext.fold");
                this.handHistory.onFold(playerId);
                if (this.soundEnabled) {
                    services_7.soundManager.playFold();
                }
            }
            else if (type === 2) {
                // Display check/call action
                if (currentPlayer.Money() === 0) {
                    currentPlayer.startAction("table.actiontext.allin");
                    this.handHistory.onAllIn(playerId, amount);
                    if (this.soundEnabled) {
                        services_7.soundManager.playAllIn();
                    }
                }
                else {
                    if (amountPlaced === 0) {
                        currentPlayer.startAction("table.actiontext.check");
                        this.handHistory.onCheck(playerId, amount);
                        if (this.soundEnabled) {
                            services_7.soundManager.playCheck();
                        }
                    }
                    else {
                        currentPlayer.startAction("table.actiontext.call");
                        this.handHistory.onCall(playerId, amount);
                        if (this.soundEnabled) {
                            services_7.soundManager.playCall();
                        }
                    }
                }
            }
            else if (type === 3) {
                // Display bet/raise action
                if (currentPlayer.Money() === 0) {
                    currentPlayer.startAction("table.actiontext.allin");
                    this.handHistory.onAllIn(playerId, amount);
                    if (this.soundEnabled) {
                        services_7.soundManager.playAllIn();
                    }
                }
                else {
                    if (maximumBet === 0) {
                        currentPlayer.startAction("table.actiontext.bet");
                        this.handHistory.onBet2(playerId, amount);
                        if (this.soundEnabled) {
                            services_7.soundManager.playBet();
                        }
                    }
                    else {
                        currentPlayer.startAction("table.actiontext.raise");
                        this.handHistory.onRaise(playerId, amount);
                        if (this.soundEnabled) {
                            services_7.soundManager.playRaise();
                        }
                    }
                }
            }
        };
        TableView.prototype.foldCardsForPlayer = function (currentPlayer, forceAnimation, duration) {
            if (currentPlayer === null) {
                console.warn("Attempt to fold cards for the player");
                return;
            }
            var myself = this.myPlayer();
            // If player folds then cards should be hidden.
            var currentCards = currentPlayer.Cards();
            currentPlayer.startFoldAnimation();
            // Current player is not active, but it not yet folded cards.
            var activePlayersCount = this.activePlayersCount() - 1;
            var displayFoldedCards = myself !== null
                && currentPlayer.PlayerId() === myself.PlayerId()
                && activePlayersCount > 1
                && !forceAnimation;
            if (displayFoldedCards) {
                currentPlayer.FoldedCards(currentCards);
                currentPlayer.IsCardsFolded(true);
            }
            // Inject tasks in the reverse order.
            this.queue.injectCallback(function () {
                currentPlayer.finishFoldAnimation();
            });
            this.queue.injectWait(duration);
        };
        TableView.prototype.finishFoldAnimation = function () {
            // Do nothing.
        };
        TableView.prototype.onBetCore = function (playerId, type, amount, nextPlayerId) {
            this.logGameEvent("Doing bet of type " + type);
            var places = this.places();
            var myself = this.myPlayer();
            var currentPlayer = this.tablePlaces.getPlaceByPlayerId(playerId);
            if (currentPlayer === null) {
                this.logGameEvent("No player sitting on the table.");
                return;
            }
            this.updateBetsAndMoney(currentPlayer, type, amount);
            var isMyTurn = myself !== null && nextPlayerId === myself.PlayerId();
            // Don's enable buttons until cards would be recived for players.
            if (this.cardsReceived) {
                this.actionBlock.buttonsEnabled(true);
                if (isMyTurn) {
                    this.onMyTurn.dispatch(this.tableId);
                }
            }
            var forcedBets = type === 1 || type === 5;
            if (!(forcedBets && nextPlayerId === 0)) {
                this.setCurrent(nextPlayerId);
                if (isMyTurn) {
                    // This will trigger reevaluation of the minimumRaiseAmount
                    this.currentRaise(this.minimumRaiseAmount());
                }
                else {
                    this.currentRaise(null);
                }
            }
            // this.refreshPlaces();
            var userActions = type === 3 || type === 4 || type === 2;
            if (userActions) {
                this.actionsCount(this.actionsCount() + 1);
            }
            var allBetsRounded = this.allBetsRounded();
            this.logGameEvent("All bets rounded", allBetsRounded);
            if (allBetsRounded) {
                this.actionBlock.buttonsEnabled(false);
                this.actionBlock.dealsAllowed(false);
            }
            if (myself !== null) {
                var requiredBet = this.maximumBet() - this.myBet();
                this.actionBlock.updateSupportDirectAmountStatus(requiredBet);
                if (nextPlayerId === myself.PlayerId()) {
                    if (!allBetsRounded && userActions) {
                        this.actionBlock.performAutomaticActions();
                        this.actionBlock.resetAutomaticAction();
                    }
                }
                this.actionBlock.updateAutomaticActionsText(myself.Money(), requiredBet);
                this.actionBlock.updateAdditionalButtons();
            }
            if (nextPlayerId === authManager.loginId()) {
                this.logGameEvent("Player turn approached");
                this.tablePlaces.placesRefreshTrigger();
            }
            this.logGameEvent("Action block status: MMB:.", this.actionBlock.mainButtonsBlockVisible(), " BEn:", this.actionBlock.buttonsEnabled(), " TurnEn:", this.actionBlock.turnEnabled(), " TTurnEn:", this.turnEnabled(), " CP:", this.currentPlayer(), " DA:", this.actionBlock.dealsAllowed(), " InGame:", this.actionBlock.myPlayerInGame());
            this.clearTimer();
            this.startTimer();
        };
        TableView.prototype.onBet = function (playerId, type, amount, nextPlayerId) {
            var _this = this;
            /// <signature>
            ///     <summary>Do bet.</summary>
            ///     <param name="playerId" type="Number">Id of the player which join the table</param>
            ///     <param name="type" type="Number">
            ///     Type of the bet.<br/>
            ///     0 - Blind
            ///     1 - Ante
            ///     2 - Check/Call
            ///     3 - Bet/Raise
            ///     4 - Fold
            ///     5 - ForcedBet
            ///     6 - ReturnMoney
            ///     </param>
            ///     <param name="amount" type="Number">Amount of money which player put in the bet</param>
            ///     <param name="nextPlayerId" type="Number">Id of the player which join the table</param>
            /// </signature>
            this.queue.pushCallback(function () {
                _this.onBetCore(playerId, type, amount, nextPlayerId);
                _this.handHistory.onBet(playerId, type, amount, nextPlayerId);
            });
        };
        TableView.prototype.startMovingChipsToPotAnimation = function () {
            this.logGameEvent("Moving chips to center");
            var places = this.places();
            this.places().forEach(function (value) {
                value.IsBetAnimationLocked(false);
                value.IsMovingBetToPot(true);
            });
            // this.refreshPlaces();
        };
        TableView.prototype.onMoveMoneyToPot = function (amount) {
            // Trigger animation.
            var self = this;
            this.actionBlock.buttonsEnabled(false);
            this.queue.wait(this.animationSettings.movingMoneyToPotPrePause);
            this.queue.pushCallback(function () {
                self.startMovingChipsToPotAnimation();
            });
            this.queue.wait(this.animationSettings.movingMoneyToPotAnimationTimeout);
            this.queue.pushCallback(function () {
                self.logGameEvent("Updating pots");
                self.setPots(amount);
                self.logGameEvent("Clearing bets");
                var places = self.places();
                self.places().forEach(function (value) {
                    value.collectBet();
                });
                self.refreshPlaces();
                self.lastRaise(0);
                self.currentRaise(self.minimumRaiseAmount());
                self.actionBlock.isRaise(false);
                self.actionBlock.buttonsEnabled(true);
            });
        };
        TableView.prototype.onMoneyAdded = function (playerId, amount) {
            var self = this;
            var places = this.places();
            places.forEach(function (value) {
                if (value.PlayerId() === playerId) {
                    // Add money only if player not currently eligible to be in game.
                    // And when no game on the table.
                    if (!value.IsInGameStatus() || self.gameId() === null) {
                        value.Money(value.Money() + amount);
                    }
                }
            });
            this.refreshPlaces();
        };
        TableView.prototype.setPots = function (amount) {
            // Add first number to the last pot.
            var currentPots = this.pots();
            var skipCollectingPot = (currentPots.length === amount.length
                && currentPots[currentPots.length - 1] === amount[currentPots.length - 1]);
            if (!skipCollectingPot) {
                this.handHistory.onMoveMoneyToPot(amount);
            }
            var main = amount[0];
            if (this.pots().length === 0) {
                // First pot created.
                this.handHistory.onPotCreated(1, main);
            }
            else {
                // Update last pot
                main = amount[this.pots().length - 1];
                if (currentPots[this.pots().length - 1] !== main) {
                    this.handHistory.onPotUpdated(currentPots.length, main);
                }
            }
            // Append other amounts to the end of stack.
            // and generate split pots
            for (var i = Math.max(1, currentPots.length); i < amount.length; i++) {
                if (currentPots[i] !== amount[i]) {
                    this.handHistory.onPotCreated(i + 1, amount[i]);
                }
            }
            this.pots(amount);
        };
        TableView.prototype.setCards = function (cards) {
            /// <signature>
            ///     <summary>Set cards on the table.</summary>
            ///     <param name="cards" type="Array">Array of table cards to display</param>
            /// </signature>
            this.tableCards.setCards(cards);
        };
        TableView.prototype.showSitPrompt = function (seat) {
            if (this.sitting) {
                return;
            }
            this.sitting = true;
            var self = this;
            app.requireAuthentication().done(function (authenticated) {
                if (authenticated) {
                    var currentPlayer = self.myPlayer();
                    if (currentPlayer === null) {
                        app.joinTablePopup.tableView(self);
                        app.joinTablePopup.seatNumber(seat);
                        app.executeCommand("popup.joinTable");
                    }
                }
            }).always(function () {
                self.sitting = false;
            });
        };
        TableView.prototype.rebuy = function () {
            var self = this;
            var tournamentView = this.tournament();
            var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            tapi.Rebuy(tournamentView.tournamentId, false).then(function (data) {
                if (data.Status === "Ok") {
                    self.hasPendingMoney(true);
                    if (!self.hasPlayersWithoutMoney()) {
                    }
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.rebuy"), languagemanager_17._("tableMenu.rebuySuccess"));
                }
                else {
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.rebuy"), languagemanager_17._("errors." + data.Status));
                }
            }, function () {
                simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.rebuy"), languagemanager_17._("tableMenu.rebuyError"));
            });
        };
        TableView.prototype.doubleRebuy = function () {
            var self = this;
            var tournamentView = this.tournament();
            var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            tapi.Rebuy(tournamentView.tournamentId, true).then(function (data) {
                if (data.Status === "Ok") {
                    self.hasPendingMoney(true);
                    if (!self.hasPlayersWithoutMoney()) {
                    }
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.doublerebuy"), languagemanager_17._("tableMenu.doublerebuySuccess"));
                }
                else {
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.doublerebuy"), languagemanager_17._("errors." + data.Status));
                }
            }, function () {
                simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.doublerebuy"), languagemanager_17._("tableMenu.doublerebuyError"));
            });
        };
        TableView.prototype.addon = function () {
            var self = this;
            var tournamentView = this.tournament();
            var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            tapi.Addon(tournamentView.tournamentId).then(function (data) {
                if (data.Status === "Ok") {
                    self.hasPendingMoney(true);
                    if (!self.hasPlayersWithoutMoney()) {
                    }
                    // self.tournament().addonCount(self.tournament().addonCount() + 1);
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.addon"), languagemanager_17._("tableMenu.addonSuccess"));
                }
                else {
                    simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.addon"), languagemanager_17._("errors." + data.Status));
                }
            }, function () {
                simplepopup_10.SimplePopup.display(languagemanager_17._("tableMenu.addon"), languagemanager_17._("tableMenu.addonError"));
            });
        };
        TableView.prototype.showRebuyPrompt = function () {
            var self = this;
            var tdata = this.tournament().tournamentData();
            var prompt = [languagemanager_17._("table.rebuyPrompt", { price: tdata.RebuyFee + tdata.RebuyPrice })];
            app.prompt(languagemanager_17._("table.rebuyPromptCaption"), prompt).then(function () {
                self.rebuy();
            });
        };
        TableView.prototype.showDoubleRebuyPrompt = function () {
            var self = this;
            var tdata = this.tournament().tournamentData();
            var prompt = [languagemanager_17._("table.doubleRebuyPrompt", { price: 2 * (tdata.RebuyFee + tdata.RebuyPrice) })];
            app.prompt(languagemanager_17._("table.doubleRebuyPromptCaption"), prompt).then(function () {
                self.doubleRebuy();
            });
        };
        TableView.prototype.showAddonPrompt = function () {
            var self = this;
            var tdata = this.tournament().tournamentData();
            var prompt = [languagemanager_17._("table.addonPrompt", { price: tdata.AddonFee + tdata.AddonPrice })];
            app.prompt(languagemanager_17._("table.addonPromptCaption"), prompt).then(function () {
                self.addon();
            });
        };
        TableView.prototype.sit = function (seat, amount) {
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            var result = $.Deferred();
            gameApi.Sit(self.tableId, seat, amount, function (data, textStatus, jqXHR) {
                // report on successfull seating.
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    result.reject(data.Status, data.MinimalAmount);
                }
                else {
                    result.resolve();
                }
            }).fail(function () {
                result.reject("NotSufficiendFunds");
            });
            return result;
        };
        TableView.prototype.showStandupPrompt = function () {
            /// <signature>
            ///     <summary>Shows stand up prompt.</summary>
            ///     <param name="seat" type="Number">Seat where player willing to join</param>
            ///     <param name="tableView" type="TableView">Table view where to join</param>
            /// </signature>
            var self = this;
            var result = $.Deferred();
            if (self.myPlayer() != null) {
                var tournament = this.tournament();
                var messages = void 0;
                if (tournament == null) {
                    messages = [languagemanager_17._("table.standupPrompt")];
                }
                else {
                    if (tournament.finishedPlaying()) {
                        result.resolve();
                        return result;
                    }
                    messages = [languagemanager_17._("table.standupTournamentPrompt")];
                }
                app.prompt(languagemanager_17._("table.standupPromptCaption"), messages).then(function () {
                    if (self.tournament() == null) {
                        self.standup();
                    }
                    result.resolve();
                }, function () {
                    result.reject();
                });
            }
            else {
                result.resolve();
            }
            return result;
        };
        TableView.prototype.standup = function () {
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            gameApi.Standup(this.tableId, function (data, textStatus, jqXHR) {
                // report on successfull seating.
                if (data.Status === "AuthorizationError") {
                    self.reportApiError("Ошибка авторизации");
                }
            });
        };
        TableView.prototype.showAddBalancePrompt = function (seat) {
            /// <signature>
            ///     <summary>Shows add balance prompt.</summary>
            ///     <param name="seat" type="Number">Seat where player willing to join</param>
            ///     <param name="tableView" type="TableView">Table view where to join</param>
            /// </signature>
            var self = this;
            var targetPlayer = this.tablePlaces.getPlaceBySeat(seat);
            var playerId = targetPlayer.PlayerId();
            var amount = prompt("Укажите сумму для пополнения счета?", "1000");
            if (amount) {
                this.addBalance(Number(amount));
            }
        };
        TableView.prototype.addBalance = function (amount) {
            var self = this;
            var places = this.places();
            var targetPlayer = this.myPlayer();
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            var result = $.Deferred();
            gameApi.AddBalance(this.tableId, amount, function (data, textStatus, jqXHR) {
                // report on successfull seating.
                if (data.Status !== "Ok") {
                    result.reject(data.Status);
                }
                if (data.Status === "Ok") {
                    if (self.myPlayerInGame()) {
                        self.hasPendingMoney(true);
                    }
                }
                result.resolve(amount);
            }).fail(function () {
                result.reject("OperationNotAllowed");
            });
            return result;
        };
        TableView.prototype.addMessage = function (messageId, sender, message) {
            var messageExists = this.messages().filter(function (_) { return _.messageId === messageId; }).length > 0;
            if (!messageExists) {
                var m = new playerMessage_1.PlayerMessage(messageId, sender, message);
                this.messages.unshift(m);
                while (this.messages().length > 100) {
                    this.messages.pop();
                }
                var messages = this.messages();
                this.messages([]);
                this.messages(messages);
            }
            if (this.lastMessageId >= messageId) {
                return;
            }
            var player = this.getPlayerByLogin(sender);
            if (player != null) {
                player.displayChatMessage(message);
            }
        };
        TableView.prototype.addSystemMessage = function (messageId, message) {
            if (this.lastMessageId >= messageId && messageId !== 0) {
                return;
            }
            var m = new SystemMessage_1.SystemMessage(messageId, message);
            this.systemMessages.unshift(m);
            while (this.systemMessages().length > 100) {
                this.systemMessages.pop();
            }
        };
        TableView.prototype.updateMessage = function (messageId, sender, message) {
            // TODO: Implement updating messages.
        };
        TableView.prototype.updateSystemMessage = function (messageId, message) {
            // TODO: Implement updating messages.
        };
        TableView.prototype.sendMessage = function () {
            var _this = this;
            var chatApi = new OnlinePoker.Commanding.API.Chat(apiHost);
            chatApi.Send(this.tableId, this.chatMessage(), function (data, textStatus, jqXHR) {
                _this.chatMessage("");
            });
        };
        TableView.prototype.fold = function () {
            var self = this;
            this.actionBlock.buttonsEnabled(false);
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            gameApi.Fold(this.tableId, function (data, status, jqXHR) {
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    self.reportApiError(data.Status);
                }
            }).fail(function (jqXHR, textStatus, errorThrown) {
                self.turnRecovery();
            });
        };
        TableView.prototype.checkOrCall = function () {
            var self = this;
            this.actionBlock.buttonsEnabled(false);
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            gameApi.CheckOrCall(this.tableId, function (data, status, jqXHR) {
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    self.reportApiError(data.Status);
                }
            }).fail(function (jqXHR, textStatus, errorThrown) {
                self.turnRecovery();
            });
        };
        TableView.prototype.betOrRaise = function () {
            var self = this;
            this.actionBlock.buttonsEnabled(false);
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            var amount = this.currentRaise() - this.currentBet();
            gameApi.BetOrRaise(this.tableId, amount, function (data, status, jqXHR) {
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    self.reportApiError(data.Status);
                }
            }).fail(function (jqXHR, textStatus, errorThrown) {
                self.turnRecovery();
            });
        };
        TableView.prototype.onOpenCards = function (cards) {
            var _this = this;
            var self = this;
            this.queue.pushCallback(function () {
                self.actionsCount(0);
                var currentCardsOpened = self.tableCards.tableCards().length;
                var myPlayer = _this.myPlayer();
                if (myPlayer) {
                    myPlayer.cardsOverlayVisible(true);
                }
                self.tableCards.openCards(cards);
                self.handHistory.onOpenCards(cards);
                if (currentCardsOpened === 0 && cards.length === 3) {
                    self.handHistory.onFlop(cards[0], cards[1], cards[2]);
                    self.actionBlock.dealsAllowed(true);
                }
                if (currentCardsOpened === 3 && cards.length === 4) {
                    self.handHistory.onTurn(cards[3]);
                    self.actionBlock.dealsAllowed(true);
                }
                if (currentCardsOpened === 4 && cards.length === 5) {
                    self.handHistory.onRiver(cards[4]);
                    self.actionBlock.dealsAllowed(true);
                }
                if (currentCardsOpened === 3 && cards.length === 5) {
                    self.handHistory.onTurn(cards[3]);
                    self.handHistory.onRiver(cards[4]);
                }
                if (currentCardsOpened === 0 && cards.length === 5) {
                    self.handHistory.onFlop(cards[0], cards[1], cards[2]);
                    self.handHistory.onTurn(cards[3]);
                    self.handHistory.onRiver(cards[4]);
                }
                self.actionBlock.resetAutomaticAction();
                self.actionBlock.updateAdditionalButtons();
            });
            if (self.soundEnabled) {
                services_7.soundManager.playFlop();
            }
            this.queue.wait(this.animationSettings.showCardsTimeout);
            this.queue.pushCallback(function () {
                self.updateCurrentCombination();
                self.tableCards.clearAnimation();
            });
        };
        TableView.prototype.comeBack = function () {
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            return gameApi.ComeBack(this.tableId, function (data, status, jqXHR) {
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    self.reportApiError(data.Status);
                }
            });
        };
        TableView.prototype.sitOut = function () {
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            gameApi.SitOut(this.tableId, function (data, status, jqXHR) {
                if (data.Status === "OperationNotValidAtThisTime") {
                    return;
                }
                if (data.Status !== "Ok") {
                    self.reportApiError(data.Status);
                }
            });
        };
        TableView.prototype.showPlayerParameters = function () {
            // tablePlayerParameterSelector.showPlayerParameters(this);
        };
        /**
        * Displays last hand in the history
        */
        TableView.prototype.showPreviousHand = function () {
            app.handHistoryPopup.tableView(this);
            app.showPopup("handHistory");
        };
        TableView.prototype.rotate = function (offset) {
            var myself = this.myPlayer();
            if (myself == null) {
                return;
            }
            var currentOffset = this.tablePlaces.getRealOffset(myself.Seat());
            if (currentOffset === offset) {
                return;
            }
            var self = this;
            app.prompt(languagemanager_17._("table.changeplace"), [languagemanager_17._("table.doyouwantchangeplace")]).done(function () {
                self.tablePlaces.rotate(offset - currentOffset);
            });
        };
        TableView.prototype.hasPlayersWithoutMoney = function () {
            return this.places().some(function (_) { return _.Money() === 0; });
        };
        /*
        * Display counter which indicates how much time left to buy addon or rebuy
        * if player lose game.
        */
        TableView.prototype.displayRebuyOrAddonTime = function () {
            var self = this;
            var tournamentView = this.tournament();
            // Display notification about rebuy or addon prompt
            // only for tournament games.
            if (tournamentView === null || tournamentView === undefined) {
                return;
            }
            // Display notification about rebuy or addon prompt
            // only if rebuy or addon allowed.
            if (!tournamentView.rebuyAllowed() && !tournamentView.addonAllowed()) {
                return;
            }
            if (this.hasPlayersWithoutMoney()) {
                // This time should be 500ms bigger then the actual time specified 
                // on the server, to accomodate for the 500ms update period.
                this.displayingRebuyAddonNotification = true;
                var totalDuration_1 = 10.5;
                var i_1 = 0;
                var messageTemplate_1;
                var popupCaption_1;
                if (tournamentView.rebuyAllowed()) {
                    messageTemplate_1 = "table.waitingForOtherPlayersRebuy";
                    popupCaption_1 = "table.tournamentGameFinishedRebuyCaption";
                }
                else if (tournamentView.addonAllowed()) {
                    messageTemplate_1 = "table.waitingForOtherPlayersAddon";
                    popupCaption_1 = "table.tournamentGameFinishedAddonCaption";
                }
                else {
                    console.error("Unknown state during display rebuy of addon time");
                }
                this.notificationHandleInterval = timeService.setInterval(function () {
                    var secondsLeft = Math.floor(totalDuration_1 - (i_1 / 2));
                    self.showNotification(languagemanager_17._(messageTemplate_1, { time: secondsLeft }));
                    app.customPopup.title(languagemanager_17._(popupCaption_1, { time: secondsLeft }));
                    i_1++;
                    if (i_1 >= totalDuration_1 * 2) {
                        self.clearNotification();
                        if (app.currentPopup === "custom") {
                            app.closePopup();
                        }
                    }
                }, 500);
            }
        };
        TableView.prototype.proposeRebuyOrAddon = function () {
            var tournamentView = this.tournament();
            if (tournamentView !== null && this.myPlayer() !== null && this.myPlayer().Money() === 0 && !this.hasPendingMoney()) {
                if (tournamentView.rebuyAllowed()) {
                    this.proposeBuyRebuy();
                }
                if (tournamentView.addonAllowed() && tournamentView.addonCount() === 0) {
                    this.proposeBuyAddon();
                }
            }
        };
        /**
        * Propose buying rebuy or double rebuy
        */
        TableView.prototype.proposeBuyRebuy = function () {
            var self = this;
            var player = this.myPlayer();
            if (player === null) {
                return;
            }
            if (player.Money() === 0 && !this.hasPendingMoney()) {
                // Display popup with three buttons.
                var result_1 = $.Deferred();
                app.promptEx(languagemanager_17._("table.tournamentGameFinishedCaption"), [languagemanager_17._("table.tournamentGameFinishedRebuy")], [languagemanager_17._("table.rebuy"), languagemanager_17._("table.doubleRebuy"), languagemanager_17._("common.cancel")], [function () { return 1; }, function () { return 2; }, function () { }])
                    .then(function (value) {
                    if (value === 1) {
                        self.showRebuyPrompt();
                    }
                    if (value === 2) {
                        self.showDoubleRebuyPrompt();
                    }
                    result_1.resolve();
                }, function () {
                    result_1.reject();
                });
            }
        };
        /**
        * Propose buying addon
        */
        TableView.prototype.proposeBuyAddon = function () {
            var self = this;
            var player = this.myPlayer();
            if (player == null) {
                console.warn("Proposing but addon when no active player");
                return;
            }
            if (player.Money() === 0 && !this.hasPendingMoney() && this.tournament().addonCount() === 0) {
                // Display popup with two buttons.
                var result_2 = $.Deferred();
                app.promptEx(languagemanager_17._("table.tournamentGameFinishedCaption"), [languagemanager_17._("table.tournamentGameFinishedAddon")], [languagemanager_17._("table.addon"), languagemanager_17._("common.cancel")], [function () { return 1; }, function () { }])
                    .then(function (value) {
                    self.showAddonPrompt();
                    result_2.resolve();
                }, function () {
                    result_2.reject();
                });
            }
        };
        /**
        * Show table notification
        */
        TableView.prototype.showNotification = function (message) {
            this.notification(message);
            this.hasNotification(true);
        };
        /**
        * Show table notification for specific period of time
        */
        TableView.prototype.showNotificationWithDelay = function (message, duration) {
            var _this = this;
            this.clearNotification();
            this.showNotification(message);
            this.notificationHandleTimeout = timeService.setTimeout(function () {
                _this.clearNotification();
            }, duration);
        };
        TableView.prototype.pushCallback = function (callback) {
            this.queue.pushCallback(callback);
        };
        TableView.prototype.clearNotification = function () {
            this.hasNotification(false);
            this.displayingRebuyAddonNotification = false;
            if (this.notificationHandleTimeout != null) {
                timeService.clearTimeout(this.notificationHandleTimeout);
                this.notificationHandleTimeout = null;
            }
            if (this.notificationHandleInterval != null) {
                timeService.clearInterval(this.notificationHandleInterval);
                this.notificationHandleInterval = null;
            }
        };
        TableView.prototype.reportApiError = function (status) {
            if (debugsettings_13.debugSettings.tableView.reportApiError) {
                alert(status);
            }
        };
        TableView.prototype.updatePauseDescription = function () {
            var _this = this;
            if (this.pauseDescriptionHandle != null) {
                timeService.clearInterval(this.pauseDescriptionHandle);
                this.pauseDescriptionHandle = null;
            }
            if (this.pauseDate() != null) {
                var updateDescription = function () {
                    var pauseEnds = moment(_this.pauseDate()).add(_this.tournament().tournamentData().PauseTimeout, "minutes");
                    var currentMoment = moment().add(timeService.timeDiff, "ms");
                    var diff = pauseEnds.diff(currentMoment);
                    if (diff < 0) {
                        _this.clearPauseMessage();
                        return;
                    }
                    var duration = moment.duration(diff);
                    _this.pauseDescription(languagemanager_17._("table.gameContinue", { startTime: duration.humanize(true) }));
                };
                this.pauseDescriptionHandle = timeService.setInterval(updateDescription, 500);
            }
        };
        TableView.prototype.clearPauseMessage = function () {
            this.paused(false);
            this.pauseDate(null);
            this.updatePauseDescription();
        };
        /**
        * Finish animation on the table
        */
        TableView.prototype.finishAnimation = function (places) {
            for (var p in places) {
                if (!places.hasOwnProperty(p)) {
                    continue;
                }
                var currentPlayer = places[p];
                currentPlayer.Bet(null);
                if (currentPlayer.IsFoldCards()) {
                    currentPlayer.finishFoldAnimation();
                }
            }
        };
        /**
        * Updates current combination based on table status.
        */
        TableView.prototype.updateCurrentCombination = function () {
            this.currentCombination(this.getCurrentCombination());
        };
        /**
        * Saves last hand history
        */
        TableView.prototype.saveHandHistory = function () {
            this.lastHandHistory(null);
            if (this.handHistory != null && this.handHistory.valid) {
                this.lastHandHistory(this.handHistory);
                this.handHistory = new handhistory_1.HandHistory(this);
            }
        };
        /**
        * Initializes the hand history
        */
        TableView.prototype.initHandHistory = function () {
            var self = this;
            this.handHistory = new handhistory_1.HandHistory(this);
            this.lastHandHistory = ko.observable();
            this.hasPreviousHand = ko.computed(function () {
                var lastHand = self.lastHandHistory();
                return lastHand != null
                    && lastHand.id != null;
            }, this);
            this.currentHandCaption = ko.computed(function () {
                var currentGame = self.gameId();
                if (currentGame == null) {
                    return "";
                }
                return languagemanager_17._("table.currentHand", { id: currentGame });
            }, this);
            this.previousHandCaption = ko.computed(function () {
                var lastHand = self.lastHandHistory();
                if (lastHand == null) {
                    return "";
                }
                return languagemanager_17._("table.lastHand", { id: lastHand.id });
            }, this);
        };
        /*
        * Prompt to refresh places information.
        */
        TableView.prototype.refreshPlaces = function () {
            this.logGameEvent("Refreshing places");
            this.tablePlaces.refreshPlaces();
        };
        /**
        * Get player place by login
        */
        TableView.prototype.getPlayerByLogin = function (login) {
            var filteredPlaces = this.places().filter(function (_) { return _.PlayerName() === login; });
            if (filteredPlaces.length === 0) {
                return null;
            }
            return filteredPlaces[0];
        };
        /**
        * Performs recovery from the network error during making turns
        */
        TableView.prototype.turnRecovery = function () {
            if (app.currentPopup !== slowinternetservice_2.SlowInternetService.popupName) {
                simplepopup_10.SimplePopup.display(languagemanager_17._("table.turn"), languagemanager_17._("table.connectionError", { tableName: this.tableName() }));
            }
            this.actionBlock.buttonsEnabled(true);
        };
        /**
        * Record log message
        */
        TableView.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            var settings = debugsettings_13.debugSettings.tableView;
            if (!this.isLogEnabled()) {
                return;
            }
            console.log("Table " + this.tableId + ": " + message, params);
        };
        /**
        * Record log message
        */
        TableView.prototype.logGameEvent = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            var settings = debugsettings_13.debugSettings.tableView;
            if (!settings.traceGameEvents) {
                return;
            }
            this.log(message, params);
        };
        /**
        * Checks whether logging on this table is enabled.
        */
        TableView.prototype.isLogEnabled = function () {
            var settings = debugsettings_13.debugSettings.tableView;
            if (!settings.trace) {
                if (settings.traceTables == null) {
                    return false;
                }
                if (settings.traceTables.indexOf(this.tableId) < 0) {
                    return false;
                }
            }
            return true;
        };
        return TableView;
    }());
    exports.TableView = TableView;
});
/* tslint:disable:no-bitwise no-use-before-declare */
define("table/tablemanager", ["require", "exports", "knockout", "timeService", "table/tableview", "table/tournamentView", "services/index", "services/broadcastservice", "popups/simplepopup", "authmanager", "debugsettings", "settings", "commandManager", "languageManager"], function (require, exports, ko, timeService, tableview_1, tournamentview_1, services_8, broadcastService, simplepopup_11, authManager, debugsettings_14, settings_6, commandManager, languagemanager_18) {
    "use strict";
    var TableManager = (function () {
        function TableManager() {
            this.reserveTablesForTournaments = false;
            var self = this;
            this.tables = ko.observableArray([]);
            this.tournaments = ko.observableArray([]);
            this.currentIndex = ko.observable(0);
            services_8.connectionService.reconnected.add(function () { return self.onReconnected(); });
            this.duplicators = [];
            this.maxTablesReached = new signals.Signal();
            this.hasTurn = ko.computed(function () {
                var tablesWithTurn = self.tables().filter(function (_) { return _.isMyTurn(); });
                return tablesWithTurn.length > 0;
            }, this);
        }
        TableManager.prototype.initialize = function () {
            var self = this;
            commandManager.registerCommand("app.selectTable", function (parameters) {
                if (parameters.length < 1) {
                    console.log("Insufficient parameters to the 'app.selectTable' command");
                    return;
                }
                var table = parameters[0];
                var update = parameters.length <= 1 ? true : parameters[1];
                self.selectTable(table, update);
            });
            commandManager.registerCommand("app.leaveTable", function (parameters) {
                var result = $.Deferred();
                if (parameters.length < 1) {
                    console.log("Insufficient parameters to the 'app.leaveTable' command");
                    result.reject();
                    return result;
                }
                var tableId = parameters[0];
                var tableView = self.getTableById(tableId);
                console.log("Leaving table " + tableView.tableId.toString());
                if (tableView != null) {
                    tableView.showStandupPrompt().pipe(function () {
                        tableView.disconnect();
                        tableView = self.remove(tableView);
                        // Adjust currently selected table.
                        self.adjustTablePosition();
                        result.resolve();
                    }, function () {
                        result.reject();
                    });
                    return result;
                }
                result.reject();
                return result;
            });
            services_8.connectionService.newConnection.add(function () {
                if (authManager.authenticated()) {
                    self.initializeChatHub(services_8.connectionService.currentConnection);
                    self.initializeGameHub(services_8.connectionService.currentConnection);
                }
            });
            settings_6.settings.autoHideCards.subscribe(function (newValue) {
                var api = new OnlinePoker.Commanding.API.Game(apiHost);
                self.tables().forEach(function (tableView) {
                    api.SetOpenCardsParameters(tableView.tableId, !newValue, null);
                });
                settings_6.settings.saveSettings();
            });
        };
        TableManager.prototype.onReconnected = function () {
            this.connectTables();
            this.connectTournaments();
        };
        TableManager.prototype.getCurrentTables = function () {
            var self = this;
            var api = new OnlinePoker.Commanding.API.Game(apiHost);
            var result = $.Deferred();
            api.GetTables(null, false, 0, 0, 1, 0, null).done(function (data) {
                var tablesData = data.Data;
                api.GetSitingTables().done(function (data) {
                    var status = data.Status;
                    if (status === "Ok") {
                        var tables = data.Data;
                        if (tables != null) {
                            for (var i = 0; i < tables.length; i++) {
                                var tableId = tables[i];
                                var model = void 0;
                                for (var j = 0; j < tablesData.length; j++) {
                                    model = tablesData[j];
                                    if (model.TableId === tableId) {
                                        self.selectTable(model, false);
                                    }
                                }
                            }
                        }
                        result.resolve(tables);
                    }
                    else {
                        if (status === "AuthorizationError") {
                            result.resolve([]);
                        }
                        else {
                            result.reject();
                        }
                    }
                });
            }).fail(function () {
                result.reject();
            });
            return result;
        };
        TableManager.prototype.getCurrentTournaments = function () {
            var self = this;
            var gapi = new OnlinePoker.Commanding.API.Game(apiHost);
            var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            var result = $.Deferred();
            tapi.GetTournaments(0, 0, 0, 0, 0, null).done(function (data) {
                var tournamentsData = data.Data;
                tapi.GetRegisteredTournamentsStatus().then(function (data) {
                    var status = data.Status;
                    if (status === "Ok") {
                        var tournaments = data.Data;
                        if (tournaments !== null && tournaments.length !== 0) {
                            self.requestTournamentsInformation(tournaments).then(function () {
                                var args = [];
                                for (var _i = 0; _i < arguments.length; _i++) {
                                    args[_i - 0] = arguments[_i];
                                }
                                var tournaments = [];
                                for (var i = 0; i < arguments.length; i++) {
                                    tournaments.push(arguments[i]);
                                }
                                result.resolve(tournaments);
                            }, function () {
                                result.reject();
                            });
                        }
                        else {
                            result.resolve([]);
                        }
                    }
                    else {
                        if (status === "AuthorizationError") {
                            result.resolve([]);
                        }
                        else {
                            result.reject();
                        }
                    }
                });
            }).fail(function () {
                result.reject();
            });
            return result;
        };
        TableManager.prototype.getCurrentTablesAndTournaments = function () {
            if (!authManager.authenticated()) {
                var result = $.Deferred();
                result.resolve();
                return result;
            }
            var tablesRequest = this.getCurrentTables();
            var tournamentsRequest = this.getCurrentTournaments().then(function (value) {
                var startedTournaments = value.filter(function (_) {
                    return _.Status === TournamentStatus.LateRegistration
                        || _.Status === TournamentStatus.Started;
                });
                if (startedTournaments.length === 0) {
                    return;
                }
                var messages = [languagemanager_18._("tournament.areYouInFollowingTournaments")];
                startedTournaments.forEach(function (_) { return messages.push(_.TournamentName); });
                timeService.setTimeout(function () {
                    simplepopup_11.SimplePopup.display(languagemanager_18._("tournament.tournaments"), messages);
                }, 2000);
            });
            return $.when(tablesRequest, tournamentsRequest);
        };
        TableManager.prototype.selectTableCommandHandler = function (parameters) {
            // Do nothing.
        };
        TableManager.prototype.initializeChatHub = function (wrapper) {
            var self = this;
            var chatHub = wrapper.connection.Chat;
            chatHub.client.ChatConnected = function (tableId, lastMessageId) {
                if (wrapper.terminated) {
                    return;
                }
                self.logDataEvent("ChatConnected", tableId, lastMessageId);
                var tableView = tableManager.getTableById(tableId);
                if (tableView === null) {
                    return;
                }
                tableView.lastMessageId = lastMessageId;
            };
            chatHub.client.Message = function (messageId, tableId, type, sender, message) {
                if (wrapper.terminated) {
                    return;
                }
                self.logDataEvent("Message", messageId, tableId, type, sender, message);
                var tableView = tableManager.getTableById(tableId);
                if (tableView === null) {
                    return;
                }
                if (type === "C") {
                    tableView.addMessage(messageId, sender, message);
                }
                if (type === "S") {
                    tableView.addSystemMessage(messageId, message);
                }
                if (type === "B") {
                    broadcastService.displayMessage(message);
                }
            };
            chatHub.client.MessageChanged = function (messageId, tableId, type, sender, message) {
                if (wrapper.terminated) {
                    return;
                }
                self.logDataEvent("MessageChanged", messageId, tableId, type, sender, message);
                var tableView = tableManager.getTableById(tableId);
                if (tableView === null) {
                    return;
                }
                if (type === "C") {
                    tableView.updateMessage(messageId, sender, message);
                }
                if (type === "S") {
                    tableView.updateSystemMessage(messageId, message);
                }
            };
        };
        TableManager.prototype.initializeGameHub = function (wrapper) {
            var self = this;
            var gameHub = wrapper.connection.Game;
            gameHub.client.TableStatusInfo = function (tableId, players, pots, cards, dealerSeat, buyIn, baseBuyIn, leaveTime, timePass, currentPlayerId, lastRaise, gameId, authenticated, actionsCount, frozen, opened, pauseDate, lastMessageId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onTableStatusInfo(players, pots, cards, dealerSeat, buyIn, baseBuyIn, leaveTime, timePass, currentPlayerId, lastRaise, gameId, authenticated, actionsCount, frozen, opened, pauseDate, lastMessageId);
                var cardsArr = cards == null ? allNoneClasses : cardsArray(cards);
                self.logDataEvent("Table status info: TableId - ", tableId, " Players - ", players, players.length, " Pots - ", pots, " Cards - ", cardsArr.join(" "));
            };
            gameHub.client.GameStarted = function (tableId, gameId, players, actions, dealerSeat) {
                if (wrapper.terminated) {
                    return;
                }
                self.clearBetEvents(tableId);
                self.registerEvent(tableId, ["Game", "GameStarted", tableId, gameId, players, actions, dealerSeat]);
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onGameStarted(gameId, players, actions, dealerSeat);
                self.logDataEvent("Game started: TableId - ", tableId, " GameId - ", gameId, " Players - ", players);
            };
            gameHub.client.Bet = function (tableId, playerId, type, amount, nextPlayerId, actionId) {
                if (wrapper.terminated) {
                    return;
                }
                if (type !== 2 && type !== 3) {
                    self.registerEvent(tableId, ["Game", "Bet", tableId, playerId, type, amount, nextPlayerId, actionId]);
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView === null) {
                    return;
                }
                tableView.onBet(playerId, type, amount, nextPlayerId);
                var typeString = "Unknown";
                switch (type) {
                    case 0:
                        typeString = "Blind";
                        break;
                    case 1:
                        typeString = "Ante";
                        break;
                    case 2:
                        typeString = "Check/Call";
                        break;
                    case 3:
                        typeString = "Bet/Raise";
                        break;
                    case 4:
                        typeString = "Fold";
                        break;
                    case 5:
                        typeString = "ForcedBet";
                        break;
                    case 6:
                        typeString = "ReturnMoney";
                        break;
                }
                self.logDataEvent("Bet: TableId - ", tableId, " PlayerId - ", playerId, " Type - ", typeString, " Amount - ", amount, " Next Player Id - ", nextPlayerId);
            };
            gameHub.client.OpenCards = function (tableId, type, cards, pots) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onMoveMoneyToPot(pots);
                tableView.onOpenCards(decodeCardsArray(cards));
                switch (type) {
                    case 0:
                        type = "Flop";
                        break;
                    case 1:
                        type = "Turn";
                        break;
                    case 2:
                        type = "River";
                        break;
                }
                var cardsStirngs = cardsArray(cards);
                self.logDataEvent("Open cards: TableId - ", tableId, " Type - ", type, " Cards - ", cardsStirngs.join(" "));
            };
            gameHub.client.MoneyAdded = function (tableId, playerId, amount) {
                if (wrapper.terminated) {
                    return;
                }
                self.logDataEvent("Money added: TableId - ", tableId, " PlayerId - ", playerId, " Amount - ", amount);
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onMoneyAdded(playerId, amount);
            };
            gameHub.client.PlayerCards = function (tableId, playerId, cards) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onPlayerCards(playerId, decodeCardsArray(cards));
                var cardsString = cardsArray(cards);
                self.logDataEvent("TableId - ", tableId, " PlayerId - ", playerId, " Cards - ", cardsString.join(" "));
            };
            gameHub.client.PlayerCardsMucked = function (tableId, playerId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onPlayerCardsMucked(playerId);
                self.logDataEvent("Cards mucked. TableId - ", tableId, " PlayerId - ", playerId);
            };
            gameHub.client.MoveMoneyToPot = function (tableId, amount) {
                if (wrapper.terminated) {
                    return;
                }
                self.logDataEvent("TableId - ", tableId, " Amount - ", amount);
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                // tableView.onMoveMoneyToPot(amount);
            };
            gameHub.client.GameFinished = function (tableId, gameId, winners, rake) {
                if (wrapper.terminated) {
                    return;
                }
                self.registerEvent(tableId, ["Game", "GameFinished", tableId, gameId, winners, rake]);
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    return;
                }
                tableView.onGameFinished(gameId, winners, rake);
                self.logDataEvent("TableId - ", tableId, " GameId - ", gameId, " Winners - ", winners, " Rake - ", rake);
            };
            gameHub.client.PlayerStatus = function (tableId, playerId, status) {
                if (wrapper.terminated) {
                    return;
                }
                var statusString = [];
                if ((status & 1) !== 0) {
                    statusString.push("Sitout");
                }
                if ((status & 2) !== 0) {
                    statusString.push("Blocked");
                }
                if (statusString.length === 0) {
                    statusString.push("Regular");
                }
                if ((status & 8) !== 0) {
                    statusString.push("InGame");
                }
                self.logDataEvent("TableId - ", tableId, " PlayerId - ", playerId, " Status - ", statusString.join("+"));
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected PlayerStatus(" + tableId + "," + playerId + "," + status + ")");
                    return;
                }
                tableView.onPlayerStatus(playerId, status);
            };
            gameHub.client.Sit = function (tableId, playerId, playerName, seat, amount, playerUrl, points, stars) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected Sit(" + tableId + "," + playerId + "," + playerName + ")");
                    return;
                }
                self.logDataEvent("Sit: TableId - ", tableId, " PlayerId - ", playerId, " PlayerName- ", playerName);
                tableView.onSit(playerId, seat, playerName, amount, playerUrl, points, stars);
            };
            gameHub.client.Standup = function (tableId, playerId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected Standup(" + tableId + "," + playerId + ")");
                    return;
                }
                self.logDataEvent("Standup: TableId - ", tableId, " PlayerId - ", playerId);
                tableView.onStandup(playerId);
            };
            gameHub.client.TableFrozen = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TableFrozen(" + tableId + ")");
                    return;
                }
                tableView.onFrozen();
            };
            gameHub.client.TableUnfrozen = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TableUnfrozen(" + tableId + ")");
                    return;
                }
                tableView.onUnfrozen();
            };
            gameHub.client.TableOpened = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TableOpened(" + tableId + ")");
                    return;
                }
                tableView.onOpened();
            };
            gameHub.client.TableClosed = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TableClosed(" + tableId + ")");
                    return;
                }
                tableView.onClosed();
            };
            gameHub.client.TablePaused = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TablePaused(" + tableId + ")");
                    return;
                }
                tableView.onPaused();
            };
            gameHub.client.TableResumed = function (tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tableView = tableManager.getTableById(tableId);
                if (tableView == null) {
                    console.warn("Receive unexpected TableResumed(" + tableId + ")");
                    return;
                }
                tableView.onResumed();
            };
            gameHub.client.TournamentStatusChanged = function (tournamentId, status) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentStatusChanged(" + tournamentId + "," + status + ")");
                    return;
                }
                tournamentView.onTournamentStatusChanged(status);
            };
            gameHub.client.TournamentTableChanged = function (tournamentId, tableId) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentTableChanged(" + tournamentId + "," + tableId + ")");
                    return;
                }
                tournamentView.onTournamentTableChanged(tableId);
            };
            gameHub.client.TournamentPlayerGameCompleted = function (tournamentId, placeTaken) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentPlayerGameCompleted(" + tournamentId + "," + placeTaken + ")");
                    return;
                }
                tournamentView.onTournamentPlayerGameCompleted(placeTaken);
            };
            gameHub.client.TournamentBetLevelChanged = function (tournamentId, level) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentBetLevelChanged(" + tournamentId + "," + level + ")");
                    return;
                }
                tournamentView.onTournamentBetLevelChanged(level);
            };
            gameHub.client.TournamentRoundChanged = function (tournamentId, round) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentRoundChanged(" + tournamentId + "," + round + ")");
                    return;
                }
                tournamentView.onTournamentRoundChanged(round);
            };
            gameHub.client.TournamentRebuyStatusChanged = function (tournamentId, rebuyAllowed, addonAllowed) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentRebuyStatusChanged(" + tournamentId + ","
                        + rebuyAllowed + "," + addonAllowed + ")");
                    return;
                }
                tournamentView.onTournamentRebuyStatusChanged(rebuyAllowed, addonAllowed);
            };
            gameHub.client.TournamentRebuyCountChanged = function (tournamentId, rebuyCount, addonCount) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentRebuyCountChanged(" + tournamentId + "," + rebuyCount + "," + addonCount + ")");
                    return;
                }
                tournamentView.onTournamentRebuyCountChanged(rebuyCount, addonCount);
            };
            gameHub.client.TournamentFrozen = function (tournamentId) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentFrozen(" + tournamentId + ")");
                    return;
                }
                tournamentView.onTournamentFrozen();
            };
            gameHub.client.TournamentUnfrozen = function (tournamentId) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView == null) {
                    console.warn("Receive unexpected TournamentUnfrozen(" + tournamentId + ")");
                    return;
                }
                tournamentView.onTournamentUnfrozen();
            };
            gameHub.client.TournamentRegistration = function (tournamentId) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView != null) {
                    console.warn("Receive unexpected TournamentRegistration(" + tournamentId + ")");
                    return;
                }
                self.openTournamentById(tournamentId);
            };
            gameHub.client.TournamentRegistrationCancelled = function (tournamentId) {
                if (wrapper.terminated) {
                    return;
                }
                var tournamentView = tableManager.getTournamentById(tournamentId);
                if (tournamentView === null) {
                    console.warn("Receive unexpected TournamentRegistrationCancelled(" + tournamentId + ")");
                    return;
                }
                tournamentView.onTournamentRegistrationCancelled();
                gameHub.server.unsubscribeTournament(tournamentId);
                tableManager.removeTournamentById(tournamentId);
            };
        };
        TableManager.prototype.clear = function () {
            this.currentIndex(0);
            this.tables([]);
            this.tournaments([]);
        };
        TableManager.prototype.clearTables = function () {
            this.tables().forEach(function (table) {
                table.clearInformation();
            });
        };
        TableManager.prototype.connectTables = function () {
            this.tables().forEach(function (table) {
                table.updateTableInformation();
            });
        };
        TableManager.prototype.stopConnectingToTables = function () {
            this.tables().forEach(function (table) {
                table.cancelUpdateTableInformation();
            });
        };
        TableManager.prototype.clearTournaments = function () {
            this.tournaments().forEach(function (tournament) {
                tournament.clearInformation();
            });
        };
        TableManager.prototype.connectTournaments = function () {
            this.tournaments().forEach(function (tournament) {
                tournament.updateTournamentInformation();
            });
        };
        TableManager.prototype.openTournamentById = function (tournamentId, attempts) {
            var _this = this;
            if (attempts === void 0) { attempts = 3; }
            if (attempts === 0) {
                simplepopup_11.SimplePopup.display(languagemanager_18._("tournament.error"), languagemanager_18._("tournament.errorConnectonToTournament"));
                return;
            }
            var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            tournamentApi.GetTournament(tournamentId).then(function (_) {
                if (_.Status === "Ok") {
                    var tournamentData = _.Data;
                    tableManager.selectTournament(tournamentData, true);
                }
                else {
                    _this.openTournamentById(tournamentId, attempts - 1);
                }
            });
        };
        TableManager.prototype.getTablesReservation = function () {
            var tablesWithoutTournament = this.tables().filter(function (_) { return _.tournament() === null; });
            var notFinishedTournaments = this.tournaments().filter(function (_) {
                var tdata = _.tournamentData();
                var tplayer = tdata.TournamentPlayers.filter(function (_) {
                    return _.PlayerId === authManager.loginId();
                });
                if (tplayer.length === 0) {
                    return false;
                }
                if (tplayer[0].Status !== TournamentPlayerStatus.Playing) {
                    return false;
                }
                return tdata.Status !== TournamentStatus.Completed
                    && tdata.Status !== TournamentStatus.Cancelled
                    && tdata.Status !== TournamentStatus.RegistrationCancelled;
            });
            this.logDataEvent("Count of tables: " + tablesWithoutTournament.length + ", "
                + "count of tournaments: " + notFinishedTournaments.length);
            if (this.reserveTablesForTournaments) {
                return Math.max(tablesWithoutTournament.length + notFinishedTournaments.length, this.tables().length);
            }
            return this.tables().length;
        };
        TableManager.prototype.selectTournament = function (model, update) {
            var self = this;
            var tournamentId = model.TournamentId;
            var tournamentView = this.getTournamentById(tournamentId);
            var append = function () {
                if (tournamentView === null) {
                    tournamentView = self.addTournament(tournamentId, model);
                }
                if (update && services_8.connectionService.currentConnection !== null) {
                    tournamentView.updateTournamentInformation();
                }
            };
            if (this.getTablesReservation() >= 4 && tournamentView === null && this.reserveTablesForTournaments) {
                this.maxTablesReached.dispatch(append);
            }
            append();
        };
        TableManager.prototype.selectTable = function (model, update) {
            var self = this;
            var tableId = model.TableId;
            var tableView = this.getTableById(tableId);
            var append = function () {
                if (tableView === null) {
                    tableView = self.addTable(tableId, model);
                }
                self.selectById(tableId);
                if (update && services_8.connectionService.currentConnection !== null) {
                    tableView.updateTableInformation();
                }
            };
            if (tableView == null) {
                this.removeTournamentTables(model.TournamentId);
            }
            if (this.getTablesReservation() >= 4 && tableView === null) {
                this.removeClosedTables();
                if (this.getTablesReservation() >= 4 && tableView === null) {
                    this.maxTablesReached.dispatch(append);
                }
                append();
            }
            else {
                append();
            }
        };
        TableManager.prototype.addTable = function (tableId, model) {
            var table = new tableview_1.TableView(tableId, model);
            this.tables.push(table);
            table.onMyTurn.add(this.onMyTurn, this);
            return table;
        };
        TableManager.prototype.remove = function (table) {
            var tables = this.tables().filter(function (value) {
                return value !== table;
            });
            this.tables(tables);
            table.onMyTurn.remove(this.onMyTurn, this);
            return table;
        };
        TableManager.prototype.removeTableById = function (tableId) {
            var view = this.getTableById(tableId);
            if (view === null) {
                return null;
            }
            return this.remove(view);
        };
        TableManager.prototype.adjustTablePosition = function () {
            var maxIndex = this.tables().length - 1;
            maxIndex = maxIndex < 0 ? 0 : maxIndex;
            if (this.currentIndex() > maxIndex) {
                this.currentIndex(maxIndex);
            }
        };
        TableManager.prototype.prevTable = function () {
            var length = this.tables().length;
            if (length <= 1) {
                return;
            }
            var maxIndex = this.tables().length - 1;
            var index = this.currentIndex();
            var nextIndex = (index - 1 + this.tables().length) % this.tables().length;
            this.currentIndex(nextIndex);
        };
        TableManager.prototype.nextTable = function () {
            var length = this.tables().length;
            if (length <= 1) {
                return;
            }
            var maxIndex = this.tables().length - 1;
            var index = this.currentIndex();
            var nextIndex = (index + 1 + this.tables().length) % this.tables().length;
            this.currentIndex(nextIndex);
        };
        TableManager.prototype.addTournament = function (tournamentId, model) {
            var tournament = new tournamentview_1.TournamentView(tournamentId, model);
            this.tournaments.push(tournament);
            return tournament;
        };
        TableManager.prototype.removeTournament = function (tournament) {
            var tournaments = this.tournaments().filter(function (value) {
                return value !== tournament;
            });
            this.tournaments(tournaments);
            return tournament;
        };
        TableManager.prototype.removeTournamentById = function (tableId) {
            var view = this.getTournamentById(tableId);
            if (view === null) {
                return null;
            }
            return this.removeTournament(view);
        };
        /**
        * Handles notification about current turn of the player.
        * @param tableId Number Id of the table where table playing.
        */
        TableManager.prototype.onMyTurn = function (tableId) {
            if (settings_6.settings.autoSwitchTables()) {
                this.selectById(tableId);
            }
            if (settings_6.settings.soundEnabled()) {
            }
        };
        /**
         * Gets table view by id
         * @param tableId Id of the player which join the table
         * @returns Table view for the table with given id
         */
        TableManager.prototype.getTableById = function (tableId) {
            var result = this.tables().filter(function (value) {
                return value.tableId === tableId;
            });
            if (result.length === 0) {
                return null;
            }
            return result[0];
        };
        /**
        * Gets tournament by it's id.
        * tournamentId Number If of the tournament to retreive.
        */
        TableManager.prototype.getTournamentById = function (tournamentId) {
            var result = this.tournaments().filter(function (value) {
                return value.tournamentId === tournamentId;
            });
            if (result.length === 0) {
                return null;
            }
            return result[0];
        };
        TableManager.prototype.isOpened = function (tableId) {
            return this.getTableById(tableId) != null;
        };
        /**
         * Selects current table view by id.
         * @param tableId Id of the table which selected
         */
        TableManager.prototype.selectById = function (tableId) {
            var tables = this.tables();
            for (var i = 0; i < tables.length; i++) {
                var tableView = tables[i];
                if (tableView.tableId === tableId) {
                    this.currentIndex(i);
                    return;
                }
            }
            this.currentIndex(0);
        };
        /**
        * Remove all tables from the given tournament
        */
        TableManager.prototype.removeTournamentTables = function (tournamentId) {
            var tables = this.tables()
                .filter(function (_) { return _.tournament() === null || _.tournament().tournamentId !== tournamentId; });
            this.tables(tables);
        };
        TableManager.prototype.requestTournamentsInformation = function (tournaments) {
            var self = this;
            var result = $.Deferred();
            var deferreds = [];
            for (var i = 0; i < tournaments.length; i++) {
                var tournamentPlayerState = tournaments[i];
                var d = this.buildTournamentInformationRequest(tournamentPlayerState.TournamentId, tournamentPlayerState.TableId);
                deferreds.push(d);
            }
            return $.when.apply($, deferreds);
        };
        TableManager.prototype.buildTournamentInformationRequest = function (tournamentId, tableId) {
            var self = this;
            var gapi = new OnlinePoker.Commanding.API.Game(apiHost);
            var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            var d = tapi.GetTournament(tournamentId).then(function (data) {
                var tournamentData = data.Data;
                self.selectTournament(tournamentData, false);
                if (tableId != null) {
                    return gapi.GetTable(tableId).then(function (data) {
                        self.selectTable(data.Data, false);
                        var tournamentTableView = self.getTableById(tableId);
                        var tournamentView = self.getTournamentById(tournamentId);
                        tournamentTableView.tournament(tournamentView);
                        return data;
                    }).then(function (value) {
                        return tournamentData;
                    });
                }
                var temp = $.Deferred();
                temp.resolve(tournamentData);
                return temp.promise();
            });
            return d;
        };
        /**
        * Remove closed table
        */
        TableManager.prototype.removeClosedTables = function () {
            var tables = this.tables();
            var tableId = null;
            for (var t in tables) {
                if (!tables.hasOwnProperty(t)) {
                    continue;
                }
                var table = tables[t];
                if (!table.opened()) {
                    tableId = table.tableId;
                    break;
                }
            }
            if (tableId === null) {
                return;
            }
            this.removeTableById(tableId);
        };
        TableManager.prototype.registerEvent = function (tableId, data) {
            var finder = this.getDuplicator(tableId);
            finder.registerEvent(data);
            if (finder.validateDuplicateEvents()) {
                finder.printDebug();
                services_8.slowInternetService.showDuplicatedConnectionPopup();
            }
        };
        TableManager.prototype.clearBetEvents = function (tableId) {
            var finder = this.getDuplicator(tableId);
            finder.erase(function (value, index, c) {
                return value[1] !== "Bet";
            });
        };
        TableManager.prototype.getDuplicator = function (tableId) {
            var finder = this.duplicators[tableId];
            if (finder == null) {
                finder = new DuplicateFinder();
                this.duplicators[tableId] = finder;
            }
            return finder;
        };
        TableManager.prototype.logDataEvent = function (message) {
            var optionalParams = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                optionalParams[_i - 1] = arguments[_i];
            }
            if (debugsettings_14.debugSettings.connection.signalR && debugsettings_14.debugSettings.connection.dataEvents) {
                console.log(message, optionalParams);
            }
        };
        return TableManager;
    }());
    var tableManager = new TableManager();
    return tableManager;
});
/* tslint:disable:no-string-literal */
define("ui/pagebase", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_15) {
    "use strict";
    var PageBase = (function () {
        function PageBase() {
            this.visible = ko.observable(false);
        }
        /**
        * Method called upon page activation.
        * pageName String Name of the page which is activated.
        */
        PageBase.prototype.activate = function (pageName) {
            var theConstructor = this["constructor"];
            this.trace("Activate " + theConstructor.name);
            this.visible(true);
        };
        /**
        * Method called upon page deactivation.
        * pageName String Name of the page which is deactivated.
        */
        PageBase.prototype.deactivate = function (pageName) {
            var theConstructor = this["constructor"];
            this.trace("Deactivate " + theConstructor.name);
            this.visible(false);
        };
        /**
        * Performs check that page could be activated
        */
        PageBase.prototype.canActivate = function () {
            return true;
        };
        PageBase.prototype.trace = function (message) {
            if (debugsettings_15.debugSettings.ui.tracePages) {
                console.log(message);
            }
        };
        return PageBase;
    }());
    exports.PageBase = PageBase;
});
define("pages/homepage", ["require", "exports", "timeService", "metadatamanager", "authmanager", "ui/pagebase", "services/index", "debugsettings", "settings", "languageManager"], function (require, exports, timeService, metadataManager, authManager, pagebase_1, services_9, debugsettings_16, settings_7, languagemanager_19) {
    "use strict";
    var HomePage = (function (_super) {
        __extends(HomePage, _super);
        function HomePage() {
            var _this = this;
            _super.call(this);
            this.news = ko.observableArray([]);
            this.currentNews = ko.observable("");
            this.intervalHandle = null;
            this.banners = ko.observableArray([]);
            this.currentBanner = ko.observable({
                Id: 0,
                Title: "",
                Url: "",
                Link: ""
            });
            this.bannerIntervalHandle = null;
            this.online = metadataManager.online;
            this.registered = metadataManager.registered;
            this.captionLabel = ko.computed(function () {
                return languagemanager_19._("header.onlinePlayers")
                    .replace("#registered", this.registered())
                    .replace("#online", this.online());
            }, this);
            this.rememberMe = ko.observable(false);
            this.username = ko.observable("");
            this.password = ko.observable("");
            this.errorMessage = ko.observable();
            this.authenticated = ko.computed(function () {
                return authManager.authenticated();
            }, this);
            this.authenticatedUser = ko.computed(function () {
                return authManager.login();
            }, this);
            authManager.authenticated.subscribe(function (value) {
                _this.banners(metadataManager.smallBanners);
            });
        }
        HomePage.prototype.deactivate = function (pageName) {
            _super.prototype.deactivate.call(this, pageName);
            this.stopNews();
            this.stopBanner();
        };
        HomePage.prototype.activate = function (pageName) {
            _super.prototype.activate.call(this, pageName);
            this.update();
            this.username(settings_7.settings.login());
            this.password(settings_7.settings.password());
            if (this.username() != null) {
                this.rememberMe(true);
            }
            this.startNews();
            this.startBanner();
            app.processing(false);
        };
        HomePage.prototype.update = function () {
            var _this = this;
            this.logNews("Updating home page");
            var metadataApi = new OnlinePoker.Commanding.API.Metadata(apiHost);
            metadataManager.updateOnline();
            metadataApi.GetNews(function (data) {
                if (data.Status === "Ok") {
                    _this.news(data.Data);
                    var i = 0;
                    if (data.Data.length > 0 && i < data.Data.length) {
                        _this.currentNews(data.Data[i]);
                    }
                }
            });
        };
        HomePage.prototype.showGames = function () {
            app.showPageBlock("lobby");
            app.showSubPage("lobby");
        };
        HomePage.prototype.login = function () {
            var _this = this;
            var username = this.username();
            var password = this.password();
            if (username === null || username.trim() === "") {
                this.errorMessage(languagemanager_19._("homePage.userRequired"));
                return;
            }
            if (password == null || password.trim() === "") {
                this.errorMessage(languagemanager_19._("homePage.passwordRequired"));
                return;
            }
            if (app.processing()) {
                return;
            }
            services_9.keyboardActivationService.forceHideKeyboard();
            app.processing(true);
            authManager.authenticate(username, password, this.rememberMe())
                .pipe(function (status) {
                app.processing(false);
                if (status === "Ok") {
                    _this.errorMessage(null);
                    _this.username("");
                    _this.password("");
                    app.lobbyPageBlock.showLobby();
                }
                else {
                    if (status) {
                        _this.errorMessage(languagemanager_19._("errors." + status));
                    }
                    else {
                        _this.errorMessage(languagemanager_19._("auth.unspecifiedError"));
                    }
                }
            }, function () {
                app.processing(false);
            });
        };
        HomePage.prototype.logout = function () {
            // do nothing.
        };
        HomePage.prototype.forgetPassword = function () {
            app.showPopup("forgetPassword");
        };
        HomePage.prototype.startNews = function () {
            var _this = this;
            this.stopNews();
            var i = 0;
            this.intervalHandle = timeService.setInterval(function () {
                i++;
                if (_this.news().length <= i) {
                    _this.logNews("Reset iteration counter to 0 because " + i.toString() + " bigger then " + _this.news().length);
                    i = 0;
                }
                _this.logNews("Setting news item " + i.toString());
                _this.currentNews(_this.news()[i]);
            }, 20 * 1000);
        };
        HomePage.prototype.stopNews = function () {
            if (this.intervalHandle !== null) {
                timeService.clearInterval(this.intervalHandle);
                this.intervalHandle = null;
            }
        };
        HomePage.prototype.startBanner = function () {
            var _this = this;
            this.stopBanner();
            var j = 0;
            this.bannerIntervalHandle = timeService.setInterval(function () {
                j++;
                if (_this.banners().length <= j) {
                    j = 0;
                }
                _this.currentBanner(_this.banners()[j]);
            }, 4 * 1000);
        };
        HomePage.prototype.stopBanner = function () {
            if (this.bannerIntervalHandle !== null) {
                timeService.clearInterval(this.bannerIntervalHandle);
                this.bannerIntervalHandle = null;
            }
        };
        HomePage.prototype.showAuthPopup = function () {
            app.showPopup("auth");
        };
        /**
        * Performs one click authorization as a guest.
        */
        HomePage.prototype.loginAsGuest = function () {
            var _this = this;
            app.processing(true);
            authManager.loginAsGuest().then(function (status) {
                app.processing(false);
                if (!status) {
                    _this.errorMessage(languagemanager_19._("auth.unspecifiedError"));
                }
                else {
                    if (status !== "Ok") {
                        _this.errorMessage(languagemanager_19._("errors." + status));
                    }
                    else {
                        app.lobbyPageBlock.showLobby();
                    }
                }
            });
        };
        HomePage.prototype.openBanner = function () {
            window.open(this.currentBanner().Link, "_system", "location=yes");
        };
        HomePage.prototype.logNews = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_16.debugSettings.home.traceNews) {
                console.log(message, params);
            }
        };
        return HomePage;
    }(pagebase_1.PageBase));
    exports.HomePage = HomePage;
});
/* tslint:disable:no-bitwise */
define("pages/lobbypage", ["require", "exports", "knockout", "moment", "app", "metadatamanager", "timeService", "table/tablemanager", "ui/pagebase", "appconfig", "debugsettings", "services/index", "languageManager"], function (require, exports, ko, moment, app_1, metadataManager, timeService, tableManager, pagebase_2, appconfig_5, debugsettings_17, services_10, languagemanager_20) {
    "use strict";
    var CashOptions = (function () {
        function CashOptions() {
            // require("extenders");
            this.currency = ko.observable(2).extend({
                options: {
                    caption: "currency.caption",
                    items: [
                        { text: "currency.realmoney", value: 1 },
                        { text: "currency.gamemoney", value: 2 },
                    ]
                }
            });
            this.limits = ko.observable(0).extend({
                options: {
                    caption: "limits.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "limits.limit1", value: 1 },
                        { text: "limits.limit2", value: 2 },
                        { text: "limits.limit3", value: 3 },
                    ]
                }
            });
            this.bets = ko.observable(0).extend({
                options: {
                    caption: "bets.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "bets.level1", value: 1 << 0 | 1 << 1 | 1 << 2 },
                        { text: "bets.level2", value: 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 },
                        { text: "bets.level3", value: 1 << 7 | 1 << 8 | 1 << 9 },
                    ]
                }
            });
            this.maxPlayers = ko.observable(0).extend({
                options: {
                    caption: "players.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "players.players2", value: 2 },
                        { text: "players.players6", value: 6 },
                        { text: "players.players10", value: 10 },
                    ]
                }
            });
        }
        return CashOptions;
    }());
    exports.CashOptions = CashOptions;
    var TournamentOptions = (function () {
        function TournamentOptions() {
            // require("extenders");
            this.currency = ko.observable(2).extend({
                options: {
                    caption: "currency.caption",
                    items: [
                        { text: "currency.realmoney", value: 1 },
                        { text: "currency.gamemoney", value: 2 },
                    ]
                }
            });
            this.buyin = ko.observable(0).extend({
                options: {
                    caption: "buyin.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "buyin.freeroll", value: 1 },
                        { text: "buyin.level1", value: 1 << 1 | 1 << 2 | 1 << 3 },
                        { text: "buyin.level2", value: 1 << 4 | 1 << 5 | 1 << 6 },
                        { text: "buyin.level3", value: 1 << 7 | 1 << 8 | 1 << 9 },
                    ]
                }
            });
            this.speed = ko.observable(0).extend({
                options: {
                    caption: "speed.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "speed.normal", value: 1 },
                        { text: "speed.turbo", value: 2 },
                        { text: "speed.turbo2x", value: 3 },
                    ]
                }
            });
            this.maxPlayers = ko.observable(0).extend({
                options: {
                    caption: "players.caption",
                    items: [
                        { text: "common.any", value: 0 },
                        { text: "players.players2", value: 1 },
                        { text: "players.players6", value: 2 },
                        { text: "players.players10", value: 3 },
                    ]
                }
            });
        }
        return TournamentOptions;
    }());
    exports.TournamentOptions = TournamentOptions;
    var LobbyPage = (function (_super) {
        __extends(LobbyPage, _super);
        function LobbyPage() {
            _super.call(this);
            var self = this;
            app_1.App.addTabBarItemMapping("lobby", "tablesFilter");
            app_1.App.addTabBarItemMapping("lobby", "tournamentsFilter");
            app_1.App.addTabBarItemMapping("lobby", "sngFilter");
            this.currentTime = ko.computed(function () {
                return timeService.currentTime();
            }, this);
            this.online = metadataManager.online;
            this.registered = metadataManager.registered;
            this.captionLabel = ko.computed(function () {
                return languagemanager_20._("header.onlinePlayersShort")
                    .replace("#registered", this.registered())
                    .replace("#online", this.online());
            }, this);
            this.tournamentsCaption = ko.computed(function () {
                return languagemanager_20._("tournamentsList.headerCaption")
                    .replace("#count", "0".toString());
            }, this);
            this.cashOptions = new CashOptions();
            this.tournamentOptions = new TournamentOptions();
            this.sngOptions = new TournamentOptions();
            this.tournaments = ko.observableArray([]);
            this.sngs = ko.observableArray([]);
            this.tables = ko.observableArray([]);
            this.showFilterSlider = ko.observable(false);
            this.showItemsListSlider = ko.observable(true);
            this.filterLocked = ko.observable(false);
            this.slider = new Slider();
            this.slider.addOption(languagemanager_20._("lobby.cashGames"), "cash", null);
            this.slider.addOption(languagemanager_20._("lobby.tournaments"), "tournaments", null);
            this.slider.addOption(languagemanager_20._("lobby.sitAndGo"), "sng", null);
            if (!appconfig_5.appConfig.tournament.enabled) {
                this.slider.enabled(false);
            }
            this.loading = ko.observable(false);
            this.selectionCaption = ko.computed(function () {
                if (self.slider.currentIndex() === 0) {
                    return languagemanager_20._("tablesList.headerCaption")
                        .replace("#count", self.tables().length.toString());
                }
                if (self.slider.currentIndex() === 1) {
                    return languagemanager_20._("tournamentsList.headerCaption")
                        .replace("#count", self.tournaments().length.toString());
                }
                return languagemanager_20._("tournamentsList.sngCaption")
                    .replace("#count", self.sngs().length.toString());
            }, this);
            tableManager.tables.subscribe(function () {
                self.updateOpenedTables();
            });
        }
        LobbyPage.prototype.deactivate = function (pageName) {
            _super.prototype.deactivate.call(this, pageName);
            // Clear tables and tournaments since this is 
            // show page faster and it could quicker respond
            // to the user input.
            this.tables([]);
            this.tournaments([]);
            this.sngs([]);
        };
        LobbyPage.prototype.activate = function (pageName) {
            if (this.visible()) {
                return;
            }
            var self = this;
            this.showFilterSlider(PageBlock.useDoubleView);
            this.showItemsListSlider(!PageBlock.useDoubleView);
            if (pageName === "lobby") {
                this.update(false);
                if (!PageBlock.useDoubleView) {
                    if (ko.contextFor($(".sub-page.filter .swipe")[0]).$swiper) {
                        ko.contextFor($(".sub-page.filter .swipe")[0]).$swiper.enable(false);
                    }
                }
                services_10.reloadManager.setReloadCallback(function () { return self.update(true); });
            }
            _super.prototype.activate.call(this, pageName);
        };
        LobbyPage.prototype.updateOpenedTables = function () {
            var tables = this.tables();
            tables.forEach(function (item) {
                item.IsOpened = tableManager.isOpened(item.TableId);
            });
            this.tables([]);
            this.tables(tables);
        };
        LobbyPage.prototype.update = function (force) {
            if (this.loading() && !force) {
                return;
            }
            var self = this;
            metadataManager.updateOnline();
            // Added reloading of the all information.
            var resetLoading = function () {
                self.loading(false);
            };
            this.loading(true);
            $.when(this.refreshTables(), this.refreshTournaments(2), this.refreshTournaments(3)).pipe(resetLoading, resetLoading);
        };
        LobbyPage.prototype.showGames = function () {
            if (this.slider.currentIndex() === 0) {
                app.lobbyPageBlock.showSecondary("tablesList");
            }
            else {
                if (this.slider.currentIndex() === 1) {
                    app.lobbyPageBlock.tournamentsListPage.setOptions(2, this.tournamentOptions);
                    app.lobbyPageBlock.showSecondary("tournamentsList");
                }
                else {
                    app.lobbyPageBlock.sngListPage.setOptions(3, this.sngOptions);
                    app.lobbyPageBlock.showSecondary("sngList");
                }
            }
        };
        LobbyPage.prototype.refreshTables = function () {
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            var privateTables = false;
            var fullTables = null;
            var options = this.cashOptions;
            var maxPlayers = options.maxPlayers() === 0 ? 0 : 1 << options.maxPlayers();
            var betLevels = options.bets();
            var moneyType = options.currency();
            var limitType = options.limits();
            return gameApi.GetTables(fullTables, privateTables, maxPlayers, betLevels, moneyType, limitType, function (data) {
                if (!self.visible()) {
                    return;
                }
                if (data.Status === "Ok") {
                    self.log("Informaton about tables received: ", data.Data);
                    var tables = data.Data;
                    tables.forEach(function (item) {
                        item.IsOpened = tableManager.isOpened(item.TableId);
                    });
                    self.tables(tables);
                }
            });
        };
        LobbyPage.prototype.refreshTournaments = function (tournamentType) {
            var self = this;
            var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            var options = tournamentType === 2 ? this.tournamentOptions : this.sngOptions;
            var prizeCurrency = options.currency();
            var tournamentTypeMask = 1 << tournamentType;
            var speed = options.speed() === 0 ? 0 : 1 << options.speed();
            var buyin = options.buyin();
            var maxPlayers = options.maxPlayers() === 0 ? 0 : 1 << (options.maxPlayers() - 1);
            return tournamentApi.GetTournaments(prizeCurrency, tournamentTypeMask, speed, buyin, maxPlayers, function (data) {
                if (!self.visible()) {
                    return;
                }
                if (data.Status === "Ok") {
                    self.log("Informaton about tournaments received: ", data.Data);
                    var enchance = function (item) {
                        var result = item;
                        var startDate = moment(item.StartDate);
                        var currentMoment = moment().add(timeService.timeDiff, "ms");
                        var duration = moment.duration(currentMoment.diff(startDate));
                        var m = duration.minutes();
                        result.duration = duration.hours() + languagemanager_20._("common.hours")
                            + languagemanager_20._("common.timeseparator")
                            + (m < 10 ? "0" + m : "" + m) + languagemanager_20._("common.minutes");
                        return result;
                    };
                    if (tournamentType === 2) {
                        self.tournaments(data.Data.map(enchance));
                    }
                    else {
                        self.sngs(data.Data.map(enchance));
                    }
                }
            });
        };
        LobbyPage.prototype.selectTable = function (table) {
            app.processing(true);
            app.requireGuestAuthentication().done(function (newValue, wasAuthenticated) {
                if (newValue) {
                    app.executeCommand("app.selectTable", [table, wasAuthenticated]);
                    if (appconfig_5.appConfig.game.seatMode) {
                        app.executeCommand("page.seats");
                    }
                    else {
                        app.executeCommand("page.tables");
                    }
                    app.processing(false);
                }
                else {
                    app.processing(false);
                }
            });
        };
        LobbyPage.prototype.selectTournament = function (tournament) {
            app.lobbyPageBlock.selectTournament(tournament);
        };
        LobbyPage.prototype.showFilterOptions = function () {
            if (this.slider.currentIndex() === 0) {
                app.lobbyPageBlock.showSecondary("filter");
            }
            if (this.slider.currentIndex() === 1) {
                app.lobbyPageBlock.showSecondary("filter");
            }
            if (this.slider.currentIndex() === 2) {
                app.lobbyPageBlock.showSecondary("filter");
            }
        };
        LobbyPage.prototype.showLobby = function () {
            if (this.slider.currentIndex() === 0) {
                app.lobbyPageBlock.showSecondary("lobby");
            }
            if (this.slider.currentIndex() === 1) {
                app.lobbyPageBlock.showSecondary("lobby");
            }
            if (this.slider.currentIndex() === 2) {
                app.lobbyPageBlock.showSecondary("lobby");
            }
        };
        LobbyPage.prototype.selectFilterParameter = function (parameter, value) {
            parameter(value);
            if (PageBlock.useDoubleView) {
                app.lobbyPageBlock.showSecondary("lobby");
            }
        };
        LobbyPage.prototype.refresh = function () {
            var self = this;
            var resetLoading = function () {
                self.loading(false);
            };
            metadataManager.updateOnline();
            if (this.slider.currentIndex() === 0) {
                this.loading(true);
                this.refreshTables().pipe(resetLoading, resetLoading);
            }
            if (this.slider.currentIndex() === 1) {
                this.loading(true);
                this.refreshTournaments(2).pipe(resetLoading, resetLoading);
            }
            if (this.slider.currentIndex() === 2) {
                this.loading(true);
                this.refreshTournaments(3).pipe(resetLoading, resetLoading);
            }
        };
        LobbyPage.prototype.touchLock = function (e) {
            if (this.filterLocked()) {
                e.preventDefault();
            }
        };
        LobbyPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_17.debugSettings.lobby.trace) {
                console.log(message, params);
            }
        };
        return LobbyPage;
    }(pagebase_2.PageBase));
    exports.LobbyPage = LobbyPage;
});
/// <reference path="../poker.commanding.api.ts" />
define("pages/tableslistpage", ["require", "exports", "ui/pagebase", "table/tablemanager", "debugsettings", "services/index", "languageManager"], function (require, exports, pagebase_3, tableManager, debugsettings_18, services_11, languagemanager_21) {
    "use strict";
    var TablesListPage = (function (_super) {
        __extends(TablesListPage, _super);
        function TablesListPage() {
            var _this = this;
            _super.call(this);
            this.tables = ko.observableArray([]);
            this.tablesCaption = ko.computed(function () {
                return languagemanager_21._("tablesList.headerCaption")
                    .replace("#count", this.tables().length.toString());
            }, this);
            this.loading = ko.observable(false);
            tableManager.tables.subscribe(function () {
                _this.updateOpenedTables();
            });
        }
        TablesListPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        TablesListPage.prototype.activate = function () {
            var _this = this;
            _super.prototype.activate.call(this);
            this.refreshTables(false);
            services_11.reloadManager.setReloadCallback(function () { return _this.refreshTables(true); });
        };
        TablesListPage.prototype.refreshTables = function (force) {
            if (this.loading() && !force) {
                return;
            }
            this.loading(true);
            var self = this;
            var gameApi = new OnlinePoker.Commanding.API.Game(apiHost);
            var privateTables = false;
            var fullTables = null;
            var lobbyPage = app.lobbyPageBlock.lobbyPage;
            /* tslint:disable:no-bitwise */
            var maxPlayers = lobbyPage.cashOptions.maxPlayers() === 0 ? 0 : 1 << lobbyPage.cashOptions.maxPlayers();
            var betLevels = lobbyPage.cashOptions.bets() === 0 ? 0 : 1 << (lobbyPage.cashOptions.bets() - 1);
            /* tslint:enable:no-bitwise */
            var moneyType = lobbyPage.cashOptions.currency();
            var limitType = lobbyPage.cashOptions.limits();
            gameApi.GetTables(fullTables, privateTables, maxPlayers, betLevels, moneyType, limitType, function (data) {
                self.loading(false);
                if (!self.visible()) {
                    return;
                }
                if (data.Status === "Ok") {
                    self.log("Informaton about tables received: ", data.Data);
                    var tables = data.Data;
                    tables.forEach(function (item) {
                        item.IsOpened = tableManager.isOpened(item.TableId);
                    });
                    self.tables(tables);
                }
            });
        };
        TablesListPage.prototype.updateOpenedTables = function () {
            var tables = this.tables();
            tables.forEach(function (item) {
                item.IsOpened = tableManager.isOpened(item.TableId);
            });
            this.tables([]);
            this.tables(tables);
        };
        TablesListPage.prototype.back = function () {
            app.lobbyPageBlock.showLobby();
        };
        TablesListPage.prototype.selectTable = function (table) {
            app.executeCommand("app.selectTable", [table]);
            app.executeCommand("page.tables");
        };
        TablesListPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_18.debugSettings.lobby.trace) {
                console.log(message, params);
            }
        };
        return TablesListPage;
    }(pagebase_3.PageBase));
    exports.TablesListPage = TablesListPage;
});
/// <reference path="../poker.commanding.api.ts" />
/* tslint:disable:no-bitwise */
define("pages/tournamentslistpage", ["require", "exports", "knockout", "ui/pagebase", "debugsettings", "services/index", "languageManager"], function (require, exports, ko, pagebase_4, debugsettings_19, services_12, languagemanager_22) {
    "use strict";
    var TournamentsListPage = (function (_super) {
        __extends(TournamentsListPage, _super);
        function TournamentsListPage() {
            _super.call(this);
            this.tournaments = ko.observableArray([]);
            this.tournamentsCaption = ko.computed(function () {
                return languagemanager_22._("tournamentsList.headerCaption")
                    .replace("#count", this.tournaments().length.toString());
            }, this);
            this.loading = ko.observable(false);
            this.slider = new Slider();
            this.slider.addOption(languagemanager_22._("lobby.cashGames"), "cash", null);
            this.slider.addOption(languagemanager_22._("lobby.tournaments"), "tournaments", null);
            this.slider.addOption(languagemanager_22._("lobby.sitAndGo"), "sng", null);
            this.slider.selectPrev();
        }
        TournamentsListPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
        };
        TournamentsListPage.prototype.activate = function () {
            var _this = this;
            _super.prototype.activate.call(this);
            this.refreshTournaments(false);
            services_12.reloadManager.setReloadCallback(function () { return _this.refreshTournaments(true); });
        };
        TournamentsListPage.prototype.setOptions = function (tournamentType, options) {
            this.tournamentType = tournamentType;
            this.options = options;
        };
        TournamentsListPage.prototype.refreshTournaments = function (force) {
            if (this.loading() && !force) {
                return;
            }
            this.loading(true);
            var self = this;
            var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            var options = this.options;
            var prizeCurrency = options.currency();
            var tournamentTypeMask = 1 << this.tournamentType;
            var speed = options.speed() === 0 ? 0 : 1 << options.speed();
            var buyin = options.buyin() === 0 ? 0 : 1 << options.buyin();
            self.tournaments([]);
            tournamentApi.GetTournaments(prizeCurrency, tournamentTypeMask, speed, buyin, null, function (data) {
                self.loading(false);
                if (!self.visible()) {
                    return;
                }
                if (data.Status === "Ok") {
                    self.log("Informaton about tournaments received: ", data.Data);
                    self.tournaments(data.Data);
                }
            });
        };
        TournamentsListPage.prototype.back = function () {
            app.lobbyPageBlock.showLobby();
        };
        TournamentsListPage.prototype.selectTournament = function (tournament) {
            app.lobbyPageBlock.selectTournament(tournament);
        };
        TournamentsListPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_19.debugSettings.lobby.trace) {
                console.log(message, params);
            }
        };
        return TournamentsListPage;
    }(pagebase_4.PageBase));
    exports.TournamentsListPage = TournamentsListPage;
});
/// <reference path="../poker.commanding.api.ts" />
define("pages/tournamentlobbypage", ["require", "exports", "knockout", "moment", "metadatamanager", "popups/simplepopup", "table/tablemanager", "timeService", "authmanager", "ui/pagebase", "debugsettings", "services/index", "languageManager"], function (require, exports, ko, moment, metadataManager, simplepopup_12, tableManager, timeService, authManager, pagebase_5, debugsettings_20, services_13, languagemanager_23) {
    "use strict";
    var TournamentLobbyPage = (function (_super) {
        __extends(TournamentLobbyPage, _super);
        function TournamentLobbyPage() {
            _super.call(this);
            this.tournamentId = 0;
            this.playersSortOrder = ko.observable("asc");
            this.playersColumnOrder = ko.observable("Login");
            var self = this;
            this.tournamentData = ko.observable(null);
            this.tablesData = ko.observableArray([]);
            this.tournamentData.subscribe(function (data) {
                if (data == null) {
                    self.tablesData([]);
                    return;
                }
                var result = [];
                var _loop_6 = function(i) {
                    var table = data.TournamentTables[i];
                    var tableModel = {
                        id: table.TableId,
                        name: table.TableName,
                        isClosed: table.IsClosed,
                        selected: ko.observable(false),
                        players: []
                    };
                    if (!table.IsClosed) {
                        table.Players.forEach(function (item) {
                            tableModel.players.push({ id: item.PlayerId, login: item.PlayerName });
                        });
                    }
                    result.push(tableModel);
                };
                for (var i = 0; i < data.TournamentTables.length; i++) {
                    _loop_6(i);
                }
                if (result.length > 0) {
                    result[0].selected(true);
                }
                self.tablesData(result);
            });
            this.loading = ko.observable(true);
            this.currentView = ko.observable(1);
            authManager.authenticated.subscribe(function (newValue) {
                self.refreshTournament();
            });
            this.lateRegistrationAllowed = ko.computed(function () {
                var data = self.tournamentData();
                if (data === null) {
                    return false;
                }
                return data.RegistrationEndDate > data.StartDate;
            }, this);
            this.totalPrize = ko.computed(function () {
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return null;
                }
                return tdata.PrizeAmount + (tdata.CollectedPrizeAmount || 0);
            }, this);
            this.stackInformation = ko.computed(function () {
                var data = self.tournamentData();
                if (data === null) {
                    return null;
                }
                var activePlayers = data.TournamentPlayers.filter(function (item) {
                    return item.Status === 2;
                });
                var maxStack = 0;
                var minStack = 10000000000;
                var stackSum = 0;
                activePlayers.forEach(function (item) {
                    var stack = item.Stack === null ? 0 : item.Stack;
                    if (stack > maxStack) {
                        maxStack = stack;
                    }
                    if (stack < minStack) {
                        minStack = stack;
                    }
                    stackSum += stackSum;
                });
                var avgStack = stackSum / activePlayers.length;
                var result = languagemanager_23._("tournamentLobby.stackInformation")
                    .replace("#max", maxStack.toString())
                    .replace("#min", minStack.toString())
                    .replace("#avg", avgStack.toFixed(0));
                return result;
            }, this);
            this.participantsInformation = ko.computed(function () {
                var data = self.tournamentData();
                if (data == null) {
                    return null;
                }
                var activePlayers = data.TournamentPlayers.filter(function (item) {
                    return item.Status === 2;
                });
                return languagemanager_23._("tournamentLobby.participantsInformation")
                    .replace("#joined", data.JoinedPlayers.toString())
                    .replace("#tablesCount", data.TournamentTables.length.toString())
                    .replace("#playersCount", activePlayers.length.toString());
            }, this);
            this.betLevelInformation = ko.computed(function () {
                var data = self.tournamentData();
                if (data == null) {
                    return null;
                }
                if (data.BetLevel == null) {
                    return null;
                }
                var betStructure = metadataManager.bets[data.WellKnownBetStructure]
                    .sort(function (a, b) { return a.Level - b.Level; });
                // Values in the betStructure is starting from 1, be caution.
                // Different methods could produce different indexation.
                var betLevel = betStructure.length < data.BetLevel ? betStructure.length : data.BetLevel;
                var currentBets = betStructure.filter(function (_) { return _.Level === betLevel; })[0];
                if (currentBets.Ante !== null && currentBets.Ante !== 0) {
                    return languagemanager_23._("tournamentLobby.betStructureWithAnte")
                        .replace("#sb", currentBets.SmallBlind.toString())
                        .replace("#bb", currentBets.BigBlind.toString())
                        .replace("#ante", currentBets.Ante.toString());
                }
                return languagemanager_23._("tournamentLobby.betStructureWithoutAnte")
                    .replace("#sb", currentBets.SmallBlind.toString())
                    .replace("#bb", currentBets.BigBlind.toString());
            }, this);
            this.getBetStructure = ko.computed(function () {
                var data = self.tournamentData();
                if (data == null) {
                    return [];
                }
                var currentPlayers = data.JoinedPlayers;
                var prizeStructure = metadataManager.bets[data.WellKnownBetStructure];
                var sortedPrizes = prizeStructure.sort(function (a, b) {
                    return a.Level > b.Level
                        ? 1
                        : (a.Level < b.Level ? -1 : 0);
                });
                return sortedPrizes;
            }, this);
            this.getTournamentPlayers = ko.computed(function () {
                var tdata = self.tournamentData();
                if (tdata === null || tdata === undefined) {
                    return null;
                }
                var players = tdata.TournamentPlayers, modifier = self.playersSortOrder() === "asc" ? 1 : -1;
                var sortByName = function (a, b) {
                    return a.PlayerName > b.PlayerName ? modifier : a.PlayerName < b.PlayerName ? -modifier : 0;
                };
                var sortByPlace = function (a, b) {
                    var aPrize = a.Prize === null ? 0 : a.Prize;
                    var bPrize = b.Prize === null ? 0 : b.Prize;
                    return aPrize > bPrize ? modifier : aPrize < bPrize ? -modifier : sortByName(a, b);
                };
                var sortByStack = function (a, b) {
                    if (a.Prize === null && b.Prize === null) {
                        var aStack = a.Stack === null ? 0 : a.Stack;
                        var bStack = b.Stack === null ? 0 : b.Stack;
                        return aStack > bStack ? modifier : aStack < bStack ? -modifier : -sortByPlace(a, b);
                    }
                    else {
                        if (a.Prize === null) {
                            return modifier;
                        }
                        if (b.Prize === null) {
                            return -modifier;
                        }
                        return -sortByPlace(a, b);
                    }
                };
                if (self.playersColumnOrder() === "Login") {
                    players = players.sort(sortByName);
                }
                if (self.playersColumnOrder() === "Stack") {
                    players = players.sort(sortByStack);
                }
                if (self.playersColumnOrder() === "Prize") {
                    players = players.sort(sortByPlace);
                }
                return players;
            }, this);
            this.authenticated = ko.computed(function () {
                var value = authManager.authenticated();
                return value;
            }, this);
            this.tablesAvailable = ko.computed(function () {
                var data = self.tournamentData();
                return data != null
                    && data.TournamentTables != null
                    && data.TournamentTables.length > 0;
            }, this);
            this.tournamentCaption = ko.computed(function () {
                var data = self.tournamentData();
                if (data === null) {
                    return;
                }
                if (debugsettings_20.debugSettings.lobby.useTournamentNameForTournamentCaption) {
                    return data.TournamentName;
                }
                return languagemanager_23._("tournamentLobby.caption")
                    .replace("#number", data.TournamentId.toString());
            }, this);
            this.getSelectedTablePlayers = ko.computed(function () {
                var selectedTables = self.tablesData().filter(function (item) {
                    return item.selected();
                });
                if (selectedTables.length < 1) {
                    return [];
                }
                return selectedTables[0].players;
            }, this);
            this.couldRegister = ko.computed(function () {
                if (!self.authenticated()) {
                    return false;
                }
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return false;
                }
                return !tdata.IsRegistered
                    && (tdata.Status === TournamentStatus.RegistrationStarted
                        || tdata.Status === TournamentStatus.LateRegistration);
            }, this);
            this.couldUnregister = ko.computed(function () {
                if (!self.authenticated()) {
                    return false;
                }
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return false;
                }
                return tdata.IsRegistered
                    && tdata.Status === TournamentStatus.RegistrationStarted;
            }, this);
            this.couldContinueGame = ko.computed(function () {
                if (!self.authenticated()) {
                    return false;
                }
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return false;
                }
                var tplayer = tdata.TournamentPlayers.filter(function (item) {
                    return item.PlayerId === authManager.loginId();
                });
                if (tplayer.length === 0 || tplayer[0].Status !== TournamentPlayerStatus.Playing) {
                    return false;
                }
                return tdata.Status === TournamentStatus.LateRegistration
                    || tdata.Status === TournamentStatus.Started;
            }, this);
            this.couldView = ko.computed(function () {
                if (!self.authenticated()) {
                    return false;
                }
                var tdata = self.tournamentData();
                if (tdata === null) {
                    return false;
                }
                var tplayer = tdata.TournamentPlayers.filter(function (item) {
                    return item.PlayerId === authManager.loginId();
                });
                if (tplayer.length === 0) {
                    return tdata.Status === TournamentStatus.Started;
                }
                if (tplayer[0].Status !== TournamentPlayerStatus.Playing) {
                    return tdata.Status === TournamentStatus.LateRegistration
                        || tdata.Status === TournamentStatus.Started;
                }
                return false;
            }, this);
            this.currentTime = ko.pureComputed(function () {
                return timeService.currentTime();
            }, this);
            this.duration = ko.pureComputed(function () {
                var tdata = self.tournamentData();
                if (tdata == null) {
                    return "";
                }
                var startDate = moment(tdata.StartDate);
                var currentMoment = moment().add(timeService.timeDiff, "ms");
                var duration = moment.duration(currentMoment.diff(startDate));
                var m = duration.minutes();
                return duration.hours() + languagemanager_23._("common.hours") + languagemanager_23._("common.timeseparator") + (m < 10 ? "0" + m : "" + m) + languagemanager_23._("common.minutes");
            }, this);
            this.lateRegistrationLeft = ko.pureComputed(function () {
                var tdata = self.tournamentData();
                if (tdata == null) {
                    return "";
                }
                var startDate = moment(tdata.StartDate);
                var registrationEndDate = moment(tdata.RegistrationEndDate);
                var currentMoment = moment().add(timeService.timeDiff, "ms");
                var duration = moment.duration(currentMoment.diff(registrationEndDate));
                var m = duration.minutes();
                return duration.hours() + languagemanager_23._("common.hours") + languagemanager_23._("common.timeseparator") + (m < 10 ? "0" + m : "" + m) + languagemanager_23._("common.minutes");
            }, this);
            this.lateRegistrationRunning = ko.pureComputed(function () {
                var tdata = self.tournamentData();
                if (tdata == null) {
                    return false;
                }
                var registrationEndDate = moment(tdata.RegistrationEndDate);
                var currentMoment = moment().add(timeService.timeDiff, "ms");
                if (currentMoment.diff(registrationEndDate)) {
                    return false;
                }
                return true;
            }, this);
            this.prizesCount = ko.computed(function () {
                var data = self.tournamentData();
                if (data === null) {
                    return null;
                }
                var currentPlayers = data.JoinedPlayers;
                var prizeStructure = metadataManager.prizes[data.WellKnownPrizeStructure];
                var sortedPrizes = prizeStructure.sort(function (a, b) {
                    return a.MaxPlayer > b.MaxPlayer
                        ? 1
                        : (a.MaxPlayer < b.MaxPlayer ? -1 : 0);
                });
                var filteredPrizes = sortedPrizes.filter(function (a) {
                    return a.MaxPlayer >= currentPlayers;
                });
                var currentPrize;
                if (filteredPrizes.length === 0) {
                    currentPrize = sortedPrizes[0];
                }
                else {
                    currentPrize = filteredPrizes[0];
                }
                return currentPrize.PrizeLevel.length;
            }, this);
            var scrollTriggerCounter = 0;
            this.scrollTrigger = ko.computed(function () {
                self.loading();
                self.currentView();
                return scrollTriggerCounter++;
            }, this);
        }
        TournamentLobbyPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (PageBlock.useDoubleView) {
                app.lobbyPageBlock.lobbyPage.filterLocked(false);
            }
        };
        TournamentLobbyPage.prototype.activate = function () {
            var _this = this;
            _super.prototype.activate.call(this);
            this.refreshTournament();
            if (PageBlock.useDoubleView) {
                app.lobbyPageBlock.lobbyPage.filterLocked(true);
            }
            services_13.reloadManager.setReloadCallback(function () { return _this.refreshTournament(); });
        };
        TournamentLobbyPage.prototype.selectTable = function (table) {
            var tables = this.tablesData();
            tables.forEach(function (item) {
                item.selected(false);
            });
            table.selected(true);
            this.tablesData(tables);
        };
        TournamentLobbyPage.prototype.setTournament = function (tournamentId, parentView) {
            this.tournamentId = tournamentId || 0;
            this.parentView = parentView;
        };
        TournamentLobbyPage.prototype.refreshTournament = function () {
            if (this.tournamentId === 0) {
                return $.Deferred().resolve({ Status: "Ok", Data: null });
            }
            var self = this;
            var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
            this.loading(true);
            return tournamentApi.GetTournament(this.tournamentId, function (data) {
                if (data.Status === "Ok") {
                    var tournamentData = data.Data;
                    self.log("Informaton about tournament ", self.tournamentId, " received: ", data.Data);
                    self.log(tournamentData.TournamentName);
                    self.tournamentData(tournamentData);
                    var tplayer = tournamentData.TournamentPlayers.filter(function (_) { return _.PlayerId === authManager.loginId(); });
                    if (tournamentData.Status === TournamentStatus.Started
                        || tournamentData.Status === TournamentStatus.LateRegistration) {
                        // Connect to the tournament so player could monitor game on it
                        tableManager.selectTournament(tournamentData, true);
                    }
                    if (tournamentData.Status === TournamentStatus.Started
                        || tournamentData.Status === TournamentStatus.LateRegistration) {
                        self.setDefaultSortOrder("Stack");
                    }
                    else if (tournamentData.Status === TournamentStatus.Completed) {
                        self.setDefaultSortOrder("Prize");
                    }
                    else {
                        self.setDefaultSortOrder("Login");
                    }
                }
                self.loading(false);
            }).fail(function () {
                self.log("Failed to get information about tournament " + self.tournamentId);
                self.loading(false);
            });
        };
        TournamentLobbyPage.prototype.back = function () {
            app.lobbyPageBlock.showSecondary(this.parentView);
        };
        TournamentLobbyPage.prototype.register = function () {
            var self = this;
            var tournament = self.tournamentData();
            var name = tournament.TournamentName;
            var joinAmount = tournament.JoinFee + tournament.BuyIn;
            /* tslint:disable:no-string-literal */
            var numericTextBinding = ko.bindingHandlers["numericText"];
            /* tslint:enable:no-string-literal */
            var joinAmountString = numericTextBinding.withCommas(joinAmount.toFixed(0));
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            self.loading(true);
            api.GetPersonalAccount(function (data) {
                self.loading(false);
                if (data.Status === "Ok") {
                    var personalAccountData = data.Data;
                    var balance = 0;
                    var tournament_1 = self.tournamentData();
                    if (tournament_1.CurrencyId === 1) {
                        balance = personalAccountData.RealMoney;
                    }
                    else {
                        balance = personalAccountData.GameMoney;
                    }
                    self.promptRegister(balance);
                }
                else {
                    simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("errors." + data.Status));
                }
            }).fail(function () {
                self.loading(false);
                simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("tournamentLobby.registrationError"));
            });
        };
        TournamentLobbyPage.prototype.unregister = function () {
            var self = this;
            var tournament = self.tournamentData();
            var name = tournament.TournamentName;
            var operationTitle = languagemanager_23._("tournamentLobby.registrationCancelled");
            app.prompt(languagemanager_23._("tournamentLobby.tournamentRegistrationCancelPromptCaption"), [languagemanager_23._("tournamentLobby.tournamentRegistrationCancelPrompt").replace("#name", name)])
                .done(function () {
                self.loading(true);
                var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
                tournamentApi.CancelRegistration(self.tournamentId, function (data) {
                    self.loading(false);
                    if (data.Status === "Ok") {
                        self.log("Registration cancelled");
                        tableManager.removeTournamentById(self.tournamentId);
                        simplepopup_12.SimplePopup.display(operationTitle, languagemanager_23._("tournamentLobby.registrationCancelledCompleteSuccess"));
                        self.refreshTournament()
                            .fail(function () {
                            simplepopup_12.SimplePopup.display(operationTitle, languagemanager_23._("tournamentLobby.registrationCancelError"));
                        });
                    }
                    else {
                        simplepopup_12.SimplePopup.display(operationTitle, languagemanager_23._("errors." + data.Status));
                    }
                }).fail(function () {
                    simplepopup_12.SimplePopup.display(operationTitle, languagemanager_23._("tournamentLobby.registrationCancelError"));
                });
            });
        };
        TournamentLobbyPage.prototype.selectView = function (view) {
            this.currentView(view);
        };
        TournamentLobbyPage.prototype.viewGame = function () {
            var tablesView = 2;
            if (this.currentView() !== tablesView) {
                this.currentView(tablesView);
            }
            else {
                var selectedTable = this.tablesData().filter(function (_) {
                    return _.selected();
                });
                if (selectedTable.length !== 0) {
                    var table = selectedTable[0];
                    if (!table.isClosed) {
                        this.showTournamentTable(selectedTable[0].id);
                    }
                }
            }
        };
        TournamentLobbyPage.prototype.continueGame = function () {
            var tournamentView = tableManager.getTournamentById(this.tournamentId);
            this.showTournamentTable(tournamentView.currentTableId);
        };
        TournamentLobbyPage.prototype.showTable = function (table) {
            if (!this.couldView() && this.couldContinueGame()) {
                return;
            }
            this.showTournamentTable(table.TableId);
        };
        TournamentLobbyPage.prototype.applyPlayersSort = function (sortColumn) {
            if (this.playersColumnOrder() === sortColumn) {
                this.playersSortOrder(this.playersSortOrder() === "asc" ? "desc" : "asc");
                return;
            }
            this.playersColumnOrder(sortColumn);
            switch (sortColumn) {
                case "Login":
                    this.playersSortOrder("asc");
                    break;
                case "Stack":
                    this.playersSortOrder("desc");
                    break;
                case "Prize":
                    this.playersSortOrder("asc");
                    break;
                default:
                    this.playersSortOrder("asc");
                    break;
            }
        };
        TournamentLobbyPage.prototype.setDefaultSortOrder = function (sortColumn) {
            this.playersColumnOrder(sortColumn);
            switch (sortColumn) {
                case "Login":
                    this.playersSortOrder("asc");
                    break;
                case "Stack":
                    this.playersSortOrder("desc");
                    break;
                case "Prize":
                    this.playersSortOrder("asc");
                    break;
                default:
                    this.playersSortOrder("asc");
                    break;
            }
        };
        /**
        * Shows already opened tournament table, or load new one if needed.
        * @param tableId Number Id of the tournament table to show.
        */
        TournamentLobbyPage.prototype.showTournamentTable = function (tableId) {
            var self = this;
            var tableView = tableManager.getTableById(tableId);
            if (tableView === null) {
                this.loading(true);
                var tdata_2 = this.tournamentData();
                var api = new OnlinePoker.Commanding.API.Game(apiHost);
                var currentTournamentTitle_1 = languagemanager_23._("tournamentLobby.caption", { "number": tdata_2.TournamentId });
                api.GetTable(tableId).then(function (data) {
                    if (data.Status === "Ok") {
                        var tapi = new OnlinePoker.Commanding.API.Tournament(apiHost);
                        tapi.GetTournament(tdata_2.TournamentId).then(function (tournamentData) {
                            if (data.Status === "Ok") {
                                self.loading(false);
                                tableManager.selectTournament(tournamentData.Data, false);
                                tableManager.selectTable(data.Data, true);
                                var currentTable = tableManager.getTableById(tableId);
                                if (currentTable != null) {
                                    currentTable.tournament(tableManager.getTournamentById(tdata_2.TournamentId));
                                    app.showSubPage("tables");
                                }
                            }
                            else {
                                self.loading(false);
                                simplepopup_12.SimplePopup.display(currentTournamentTitle_1, languagemanager_23._("errors." + data.Status));
                            }
                        }, function () {
                            self.loading(false);
                            self.log("Could not get tournament information for tournament #" + tdata_2.TournamentId);
                            simplepopup_12.SimplePopup.display(currentTournamentTitle_1, languagemanager_23._("tournamentLobby.showTableError"));
                        });
                    }
                    else {
                        self.loading(false);
                        simplepopup_12.SimplePopup.display(currentTournamentTitle_1, languagemanager_23._("errors." + data.Status));
                    }
                }, function () {
                    self.loading(false);
                    self.log("Could not get table information for table #" + tableId);
                    simplepopup_12.SimplePopup.display(currentTournamentTitle_1, languagemanager_23._("tournamentLobby.showTableError"));
                });
            }
            else {
                tableManager.selectById(tableId);
                app.showSubPage("tables");
            }
        };
        TournamentLobbyPage.prototype.promptRegister = function (currentBalance) {
            var self = this;
            var tournament = self.tournamentData();
            var name = tournament.TournamentName;
            var joinAmount = tournament.JoinFee + tournament.BuyIn;
            /* tslint:disable:no-string-literal */
            var numericTextBinding = ko.bindingHandlers["numericText"];
            /* tslint:enable:no-string-literal */
            var joinAmountString = numericTextBinding.withCommas(joinAmount.toFixed(0));
            if (currentBalance < joinAmount) {
                simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.tournamentRegistrationPromptCaption"), languagemanager_23._("tournamentLobby.insufficientFunds"));
                return;
            }
            var balanceString = numericTextBinding.withCommas(currentBalance.toFixed(0));
            app.prompt(languagemanager_23._("tournamentLobby.tournamentRegistrationPromptCaption"), [
                languagemanager_23._("tournamentLobby.tournamentRegistrationPrompt", { name: name }),
                languagemanager_23._("tournamentLobby.tournamentRegistrationAmount").replace("#amount", joinAmountString),
                languagemanager_23._("tournamentLobby.tournamentRegistrationPromptBalance").replace("#amount", balanceString),
            ])
                .done(function () {
                self.loading(true);
                var tournamentApi = new OnlinePoker.Commanding.API.Tournament(apiHost);
                tournamentApi.Register(self.tournamentId, function (data) {
                    self.loading(false);
                    if (data.Status === "Ok") {
                        tableManager.openTournamentById(self.tournamentId);
                        self.log("Registration success");
                        simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("tournamentLobby.registrationCompleteSuccess"));
                        self.refreshTournament()
                            .fail(function () {
                            simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("tournamentLobby.registrationError"));
                        });
                    }
                    else {
                        simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("errors." + data.Status));
                    }
                }).fail(function () {
                    self.loading(false);
                    simplepopup_12.SimplePopup.display(languagemanager_23._("tournamentLobby.registrationSuccess"), languagemanager_23._("tournamentLobby.registrationError"));
                });
            });
            app.okcancelPopup.customStyle("popup-container-left");
        };
        TournamentLobbyPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_20.debugSettings.lobby.trace) {
                console.log(message, params);
            }
        };
        return TournamentLobbyPage;
    }(pagebase_5.PageBase));
    exports.TournamentLobbyPage = TournamentLobbyPage;
});
/// <reference path="../pageblock.ts" />
/// <reference path="lobbypage.ts" />
/// <reference path="tableslistpage.ts" />
/// <reference path="tournamentslistpage.ts" />
/// <reference path="tournamentlobbypage.ts" />
define("pages/lobbypageblock", ["require", "exports", "app", "pages/tournamentlobbypage", "pages/tableslistpage", "pages/lobbypage", "pages/tournamentslistpage"], function (require, exports, app_2, tournamentlobbypage_1, tableslistpage_1, lobbypage_1, tournamentslistpage_1) {
    "use strict";
    var LobbyPageBlock = (function (_super) {
        __extends(LobbyPageBlock, _super);
        function LobbyPageBlock() {
            var lobbyPage = new lobbypage_1.LobbyPage();
            _super.call(this, "lobby", "filter", lobbyPage);
            this.lobbyPage = lobbyPage;
            this.tablesListPage = new tableslistpage_1.TablesListPage();
            this.addSecondary("tablesList", this.tablesListPage);
            this.tournamentLobbyPage = new tournamentlobbypage_1.TournamentLobbyPage();
            app_2.App.addTabBarItemMapping("lobby", "tournamentLobby");
            this.addSecondary("tournamentLobby", this.tournamentLobbyPage);
            this.tournamentsListPage = new tournamentslistpage_1.TournamentsListPage();
            app_2.App.addTabBarItemMapping("lobby", "tournamentsList");
            this.addSecondary("tournamentsList", this.tournamentsListPage);
            this.sngListPage = new tournamentslistpage_1.TournamentsListPage();
            app_2.App.addTabBarItemMapping("lobby", "sngList");
            this.addSecondary("sngList", this.sngListPage);
            this.addSecondary("lobby", this.lobbyPage, true);
            this.currentPage = "lobby";
        }
        LobbyPageBlock.prototype.showLobby = function () {
            app.executeCommand("pageblock.lobby");
            app.lobbyPageBlock.showSecondary("lobby");
        };
        LobbyPageBlock.prototype.selectTournament = function (tournament) {
            this.tournamentLobbyPage.setTournament(tournament.TournamentId, this.currentPage);
            this.showSecondary("tournamentLobby");
        };
        return LobbyPageBlock;
    }(PageBlock));
    exports.LobbyPageBlock = LobbyPageBlock;
});
define("pages/cashierpage", ["require", "exports", "knockout", "authmanager", "ui/pagebase", "services/index"], function (require, exports, ko, authManager, pagebase_6, services_14) {
    "use strict";
    var CashierPage = (function (_super) {
        __extends(CashierPage, _super);
        function CashierPage() {
            _super.call(this);
            this.requireAuthentication = true;
            var self = this;
            this.loading = ko.observable(false);
            this.cashierCaption = ko.observable();
            authManager.authenticated.subscribe(function (newValue) {
                if (newValue) {
                    self.updateInformation();
                }
                else {
                    self.player({
                        login: "",
                        accounts: [],
                        lastTransaction: null
                    });
                }
            });
            this.player = ko.observable({
                login: "",
                accounts: [],
                lastTransaction: null
            });
        }
        CashierPage.prototype.activate = function () {
            _super.prototype.activate.call(this);
            var self = this;
            if (!authManager.authenticated()) {
                app.showPopup("auth");
            }
            else {
                this.updateInformation();
            }
            services_14.reloadManager.setReloadCallback(function () { return self.updateInformation(); });
        };
        CashierPage.prototype.updateInformation = function () {
            var self = this;
            var realMoneySupported = true;
            var gameMoneySupported = false;
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            self.loading(true);
            services_14.accountService.getAccount().done(function (result) {
                self.loading(false);
                self.player(result);
            }).fail(function () {
                app.closePopup();
            });
        };
        CashierPage.prototype.back = function () {
            app.lobbyPageBlock.showLobby();
        };
        CashierPage.prototype.withdrawal = function () {
            services_14.websiteService.withdrawal();
        };
        CashierPage.prototype.profile = function () {
            services_14.websiteService.profile();
        };
        CashierPage.prototype.showHistory = function () {
            app.cashierPageBlock.showSecondary("operationsHistory");
        };
        CashierPage.prototype.deposit = function () {
            services_14.websiteService.madeDeposit();
        };
        return CashierPage;
    }(pagebase_6.PageBase));
});
define("pages/withdrawalpage", ["require", "exports", "app", "authmanager", "ui/pagebase"], function (require, exports, app_3, authManager, pagebase_7) {
    "use strict";
    var WithdrawalPage = (function (_super) {
        __extends(WithdrawalPage, _super);
        function WithdrawalPage() {
            _super.call(this);
            app_3.App.addTabBarItemMapping("cashier", "withdrawal");
            var self = this;
            this.withdrawalAmount = ko.observable(null);
            this.withdrawalMethod = ko.observable(null);
            this.withdrawalMethods = ko.observableArray([
                { id: 1, text: "Unitbag" },
                { id: 2, text: "Visa" },
                { id: 3, text: "MasterCard" },
            ]);
            this.accountNumber = ko.observable(null);
            authManager.authenticated.subscribe(function (newValue) {
                if (newValue) {
                    var api = new OnlinePoker.Commanding.API.Account(apiHost);
                    api.GetPersonalAccount(function (data) {
                        var personalAccountData = data.Data;
                        self.player({
                            login: authManager.login(),
                            amount: personalAccountData.RealMoney
                        });
                    });
                }
                else {
                    self.player({
                        login: "",
                        amount: null
                    });
                }
            });
            this.player = ko.observable({
                login: "",
                amount: null
            });
        }
        WithdrawalPage.prototype.activate = function () {
            _super.prototype.activate.call(this);
            if (!authManager.authenticated()) {
                app.showPopup("auth");
            }
        };
        WithdrawalPage.prototype.back = function () {
            if (!PageBlock.useDoubleView) {
                app.cashierPageBlock.showPrimary();
            }
            else {
                app.lobbyPageBlock.showLobby();
            }
        };
        return WithdrawalPage;
    }(pagebase_7.PageBase));
    exports.WithdrawalPage = WithdrawalPage;
});
/// <reference path="../poker.commanding.api.ts" />
define("pages/operationshistorypage", ["require", "exports", "knockout", "app", "ui/pagebase", "services/index"], function (require, exports, ko, app_4, pagebase_8, services_15) {
    "use strict";
    var OperationsHistoryPage = (function (_super) {
        __extends(OperationsHistoryPage, _super);
        function OperationsHistoryPage() {
            _super.call(this);
            app_4.App.addTabBarItemMapping("cashier", "operationsHistory");
            this.loading = ko.observable(false);
            this.operations = ko.observableArray([]);
            this.from = ko.observable();
            this.to = ko.observable();
            this.errors = ko.validation.group(this);
        }
        OperationsHistoryPage.prototype.activate = function () {
            var _this = this;
            _super.prototype.activate.call(this);
            this.applyFilter(false);
            if (!PageBlock.useDoubleView) {
                app.tabBar.visible(false);
                services_15.orientationService.setOrientation("landscape");
                /* tslint:disable:no-string-literal no-unused-expression */
                window["StatusBar"] && StatusBar.hide();
            }
            services_15.reloadManager.setReloadCallback(function () { return _this.applyFilter(true); });
        };
        OperationsHistoryPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (!PageBlock.useDoubleView) {
                app.tabBar.visible(true);
                services_15.orientationService.setOrientation("portrait");
                /* tslint:disable:no-string-literal no-unused-expression */
                window["StatusBar"] && StatusBar.show();
            }
        };
        OperationsHistoryPage.prototype.back = function () {
            if (!PageBlock.useDoubleView) {
                app.cashierPageBlock.showPrimary();
            }
            else {
                app.lobbyPageBlock.showLobby();
            }
        };
        OperationsHistoryPage.prototype.applyFilter = function (force) {
            var _this = this;
            if (this.loading() && !force) {
                return;
            }
            this.loading(true);
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            api.GetPlayerAccountHistory(this.from(), this.to(), null, null, null, function (data) {
                _this.loading(false);
                if (!_this.visible()) {
                    return;
                }
                if (data.Status === "Ok") {
                    _this.operations(data.Data);
                }
            });
        };
        return OperationsHistoryPage;
    }(pagebase_8.PageBase));
    exports.OperationsHistoryPage = OperationsHistoryPage;
});
define("pages/accountpage", ["require", "exports", "knockout", "authmanager", "ui/pagebase", "settings", "app"], function (require, exports, ko, authManager, pagebase_9, settings_8, app_5) {
    "use strict";
    var AccountPage = (function (_super) {
        __extends(AccountPage, _super);
        function AccountPage() {
            _super.call(this);
            app_5.App.addTabBarItemMapping("cashier", "account");
            var self = this;
            this.cashierCaption = ko.observable();
            this.loading = ko.observable(false);
            var emptyElement = {
                login: "",
                email: "",
                firstName: "",
                lastName: "",
                monthPoints: 0,
                yearPoints: 0,
                status: "status1",
                accounts: [],
                stars: 0
            };
            authManager.authenticated.subscribe(function (newValue) {
                if (newValue) {
                    self.updateInformation(true);
                }
                else {
                    self.player(emptyElement);
                }
            });
            this.player = ko.observable(emptyElement);
        }
        AccountPage.prototype.activate = function () {
            _super.prototype.activate.call(this);
            if (!authManager.authenticated()) {
                app.showPopup("auth");
            }
            else {
                this.updateInformation(false);
            }
        };
        AccountPage.prototype.updateInformation = function (force) {
            var _this = this;
            if (this.loading() && !force) {
                return;
            }
            var self = this;
            var realMoneySupported = !settings_8.settings.isGuest();
            var gameMoneySupported = settings_8.settings.isGuest();
            var pointsSupported = true;
            this.loading(true);
            this.requestData(realMoneySupported, gameMoneySupported, pointsSupported).then(function () { return self.loading(false); }, function () { return _this.requestData(realMoneySupported, gameMoneySupported, pointsSupported); });
        };
        AccountPage.prototype.back = function () {
            app.lobbyPageBlock.showLobby();
        };
        AccountPage.prototype.changePassword = function () {
            app.showPopup("changePassword");
        };
        AccountPage.prototype.requestData = function (realMoneySupported, gameMoneySupported, pointsSupported) {
            var self = this;
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            return api.GetPlayerDefinition(function (data) {
                self.loading(false);
                var personalAccountData = data.Data;
                console.log(personalAccountData);
                if (!self.visible()) {
                    return;
                }
                var accountsData = [];
                if (realMoneySupported) {
                    accountsData.push({
                        currencyName: "currency.realmoney",
                        available: personalAccountData.RealMoney,
                        ingame: 0,
                        total: personalAccountData.RealMoney
                    });
                }
                if (gameMoneySupported) {
                    accountsData.push({
                        currencyName: "currency.gamemoney",
                        available: personalAccountData.GameMoney,
                        ingame: 0,
                        total: personalAccountData.GameMoney
                    });
                }
                if (pointsSupported) {
                    accountsData.push({
                        currencyName: "currency.rewardpoints",
                        available: personalAccountData.Points,
                        ingame: 0,
                        total: personalAccountData.Points
                    });
                }
                var status = "status0";
                if (parseInt(personalAccountData.Properties.Points, 10) >= 100000) {
                    status = "status1";
                }
                if (parseInt(personalAccountData.Properties.Points, 10) >= 200000) {
                    status = "status2";
                }
                if (parseInt(personalAccountData.Properties.Points, 10) >= 500000) {
                    status = "status3";
                }
                self.player({
                    login: authManager.login(),
                    email: personalAccountData.Email,
                    firstName: personalAccountData.FirstName,
                    lastName: personalAccountData.LastName,
                    monthPoints: parseInt(personalAccountData.Properties.Points, 10),
                    yearPoints: parseInt(personalAccountData.Properties.Points, 10),
                    status: status,
                    accounts: accountsData,
                    stars: personalAccountData.Properties.Stars
                });
            });
        };
        return AccountPage;
    }(pagebase_9.PageBase));
    exports.AccountPage = AccountPage;
});
define("pages/ratingpage", ["require", "exports", "app"], function (require, exports, app_6) {
    "use strict";
    var RatingPage = (function () {
        function RatingPage() {
            this.ratings = ko.observableArray();
            this.loading = ko.observable(false);
            app_6.App.addTabBarItemMapping("more", "rating");
        }
        RatingPage.prototype.deactivate = function () {
            // Do nothing.
        };
        RatingPage.prototype.activate = function () {
            var self = this;
            this.loading(true);
            var api = new OnlinePoker.Commanding.API.Account(apiHost);
            api.GetBestPlayers().done(function (data, status) {
                self.loading(false);
                if (data.Status === "Ok") {
                    var ratings = data.Data;
                    ratings.forEach(function (_) {
                        var points = parseInt(_.Points, 10);
                        _.IsGold = points >= 500000;
                        _.IsSilver = points >= 200000 && points < 500000;
                        _.IsBronse = points >= 100000 && points < 200000;
                    });
                    self.ratings(ratings);
                }
            }).fail(function () {
                self.loading(false);
            });
        };
        RatingPage.prototype.back = function () {
            app.lobbyPageBlock.showLobby();
        };
        return RatingPage;
    }());
    exports.RatingPage = RatingPage;
});
/// <reference path="../pageblock.ts" />
/// <reference path="cashierpage.ts" />
/// <reference path="withdrawalpage.ts" />
/// <reference path="operationshistorypage.ts" />
define("pages/cashierpageblock", ["require", "exports", "pages/accountpage", "pages/ratingpage", "pages/operationshistorypage", "pages/withdrawalpage"], function (require, exports, accountpage_1, ratingpage_1, operationshistorypage_1, withdrawalpage_1) {
    "use strict";
    var CashierPageBlock = (function (_super) {
        __extends(CashierPageBlock, _super);
        function CashierPageBlock() {
            var accountPage = new accountpage_1.AccountPage();
            _super.call(this, "cashier", "account", accountPage);
            this.accountPage = accountPage;
            this.ratingPage = new ratingpage_1.RatingPage();
            this.requireAuthentication = true;
            // this.addSecondary("account", this.accountPage);
            this.addSecondary("rating", this.ratingPage, true);
            this.currentPage = "rating";
            this.addSecondary("operationsHistory", new operationshistorypage_1.OperationsHistoryPage());
            this.addSecondary("withdrawal", new withdrawalpage_1.WithdrawalPage());
        }
        return CashierPageBlock;
    }(PageBlock));
    exports.CashierPageBlock = CashierPageBlock;
});
define("pages/infopage", ["require", "exports", "services/index", "ui/pagebase"], function (require, exports, services_16, pagebase_10) {
    "use strict";
    var InfoPage = (function (_super) {
        __extends(InfoPage, _super);
        function InfoPage() {
            _super.call(this);
        }
        InfoPage.prototype.backToLobby = function () {
            services_16.keyboardActivationService.forceHideKeyboard();
            app.lobbyPageBlock.showLobby();
        };
        InfoPage.prototype.back = function () {
            services_16.keyboardActivationService.forceHideKeyboard();
            app.infoPageBlock.showPrimary();
        };
        InfoPage.prototype.showLicenseAgreement = function () {
            app.infoPageBlock.showLicenseAgreement();
        };
        InfoPage.prototype.showMobileSystemSupport = function () {
            app.infoPageBlock.showMobileSystemSupport();
        };
        InfoPage.prototype.showAccountBlock = function () {
            app.infoPageBlock.showAccountBlock();
        };
        InfoPage.prototype.showPrivacyPolicy = function () {
            app.infoPageBlock.showPrivacyPolicy();
        };
        InfoPage.prototype.showSupport = function () {
            app.infoPageBlock.showSupport();
        };
        InfoPage.prototype.showContactUs = function () {
            app.infoPageBlock.showContactUs();
        };
        InfoPage.prototype.showHowToPlay = function () {
            app.infoPageBlock.showHowToPlay();
        };
        InfoPage.prototype.showChatRules = function () {
            app.infoPageBlock.showChatRules();
        };
        InfoPage.prototype.showTerminology = function () {
            app.infoPageBlock.showTerminology();
        };
        InfoPage.prototype.showFaq = function () {
            app.infoPageBlock.showFaq();
        };
        return InfoPage;
    }(pagebase_10.PageBase));
    exports.InfoPage = InfoPage;
});
define("pages/supportpage", ["require", "exports", "knockout", "popups/simplepopup", "ui/pagebase", "services/index", "languageManager"], function (require, exports, ko, simplepopup_13, pagebase_11, services_17, languagemanager_24) {
    "use strict";
    var SupportPage = (function (_super) {
        __extends(SupportPage, _super);
        function SupportPage() {
            _super.call(this);
            this.displayFullName = false;
            this.displaySubject = false;
            this.loading = ko.observable(false);
            this.fullName = ko.observable("").extend({ required: this.displayFullName, maxLength: 30 });
            this.email = ko.observable("").extend({ required: true, email: true });
            this.subject = ko.observable("").extend({ required: this.displaySubject, maxLength: 30 });
            this.body = ko.observable("").extend({ required: true, maxLength: 1000 });
            this.errorMessage = ko.observable();
            this.errors = ko.validation.group(this);
        }
        SupportPage.prototype.backToLobby = function () {
            services_17.keyboardActivationService.forceHideKeyboard();
            app.lobbyPageBlock.showLobby();
        };
        SupportPage.prototype.back = function () {
            services_17.keyboardActivationService.forceHideKeyboard();
            app.infoPageBlock.showPrimary();
        };
        /**
        * Method called upon page activation.
        * pageName String Name of the page which is activated.
        */
        SupportPage.prototype.activate = function (pageName) {
            this.fullName("");
            this.email("");
            this.subject("");
            this.body("");
            _super.prototype.activate.call(this, pageName);
        };
        /**
        * Method called upon page deactivation.
        * pageName String Name of the page which is deactivated.
        */
        SupportPage.prototype.deactivate = function (pageName) {
            _super.prototype.deactivate.call(this, pageName);
        };
        SupportPage.prototype.send = function () {
            var self = this;
            var isValid = this.isValid();
            if (!isValid) {
                this.errors.showAllMessages(true);
                return;
            }
            if (this.loading()) {
                return;
            }
            this.loading(true);
            this.errorMessage(null);
            this.errors.showAllMessages(false);
            var api = new OnlinePoker.Commanding.API.Support(apiHost);
            api.ContactUs(this.fullName(), this.email(), this.subject(), this.body()).then(function (data) {
                self.loading(false);
                if (data.Status === "Ok") {
                    self.fullName("");
                    self.email("");
                    self.subject("");
                    self.body("");
                    simplepopup_13.SimplePopup.display(languagemanager_24._("contactUs.contactUsTitle"), languagemanager_24._("contactUs.requestSent"));
                }
                else {
                    self.errorMessage(languagemanager_24._("errors." + data.Status));
                }
            }, function (error) {
                self.loading(false);
                self.errorMessage(languagemanager_24._("common.unspecifiedError"));
            });
        };
        return SupportPage;
    }(pagebase_11.PageBase));
    exports.SupportPage = SupportPage;
});
/// <reference path="../pageblock.ts" />
/// <reference path="infopage.ts" />
/// <reference path="supportpage.ts" />
define("pages/infopageblock", ["require", "exports", "pages/supportpage", "pages/infopage"], function (require, exports, supportpage_1, infopage_1) {
    "use strict";
    var InfoPageBlock = (function (_super) {
        __extends(InfoPageBlock, _super);
        function InfoPageBlock() {
            _super.call(this, "info", "pagesList", new infopage_1.InfoPage());
            this.requireAuthentication = false;
            this.addSecondary("licenseAgreement", new infopage_1.InfoPage());
            this.addSecondary("mobileSystemSupport", new infopage_1.InfoPage());
            this.addSecondary("accountBlock", new infopage_1.InfoPage());
            this.addSecondary("privacyPolicy", new infopage_1.InfoPage());
            this.addSecondary("support", new infopage_1.InfoPage());
            this.addSecondary("contactUs", new supportpage_1.SupportPage());
            this.addSecondary("howToPlay", new infopage_1.InfoPage());
            this.addSecondary("chatRules", new infopage_1.InfoPage());
            this.addSecondary("terminology", new infopage_1.InfoPage());
            this.addSecondary("faq", new infopage_1.InfoPage());
        }
        InfoPageBlock.prototype.showLicenseAgreement = function () {
            this.showSecondary("licenseAgreement");
        };
        InfoPageBlock.prototype.showMobileSystemSupport = function () {
            this.showSecondary("mobileSystemSupport");
        };
        InfoPageBlock.prototype.showAccountBlock = function () {
            this.showSecondary("accountBlock");
        };
        InfoPageBlock.prototype.showPrivacyPolicy = function () {
            this.showSecondary("privacyPolicy");
        };
        InfoPageBlock.prototype.showSupport = function () {
            this.showSecondary("support");
        };
        InfoPageBlock.prototype.showContactUs = function () {
            this.showSecondary("contactUs");
        };
        InfoPageBlock.prototype.showHowToPlay = function () {
            this.showSecondary("howToPlay");
        };
        InfoPageBlock.prototype.showChatRules = function () {
            this.showSecondary("chatRules");
        };
        InfoPageBlock.prototype.showTerminology = function () {
            this.showSecondary("terminology");
        };
        InfoPageBlock.prototype.showFaq = function () {
            this.showSecondary("faq");
        };
        return InfoPageBlock;
    }(PageBlock));
    exports.InfoPageBlock = InfoPageBlock;
});
define("pages/settingspage", ["require", "exports", "app", "metadatamanager", "languageManager"], function (require, exports, app_7, metadataManager, languagemanager_25) {
    "use strict";
    var SettingsPage = (function () {
        function SettingsPage() {
            app_7.App.addTabBarItemMapping("more", "settings");
            this.online = metadataManager.online;
            this.registered = metadataManager.registered;
            this.captionLabel = ko.computed(function () {
                return languagemanager_25._("header.onlinePlayers")
                    .replace("#registered", this.registered())
                    .replace("#online", this.online());
            }, this);
        }
        SettingsPage.prototype.deactivate = function () {
            // Do nothing.
        };
        SettingsPage.prototype.activate = function () {
            this.update();
        };
        SettingsPage.prototype.update = function () {
            // Do nothing.
        };
        return SettingsPage;
    }());
    exports.SettingsPage = SettingsPage;
});
define("pages/chatpage", ["require", "exports", "knockout", "table/playerMessage", "timeService", "services/index", "popups/simplepopup", "languageManager", "app"], function (require, exports, ko, playerMessage_2, timeService, services_18, simplepopup_14, languagemanager_26, app_8) {
    "use strict";
    var ChatPage = (function () {
        function ChatPage() {
            this.timeoutHandler = 0;
            app_8.App.addTabBarItemMapping("more", "chat");
            this.currentMessage = ko.observable();
            this.messages = ko.observableArray([]);
            this.loading = ko.observable(false);
            var self = this;
            services_18.connectionService.newConnection.add(function () {
                var chatHub = services_18.connectionService.currentConnection.connection.createHubProxy("chat");
                var handler = function () {
                    var msg = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        msg[_i - 0] = arguments[_i];
                    }
                    var messageId = msg[0];
                    var tableId = msg[1];
                    var type = msg[2];
                    var sender = msg[3];
                    var message = msg[4];
                    if (tableId !== 0) {
                        return;
                    }
                    self.loading(false);
                    self.addMessage(messageId, sender, message);
                };
                chatHub.on("Message", handler);
                services_18.connectionService.terminatedConnection.addOnce(function () {
                    chatHub.off("Message", handler);
                }, self, 0);
            });
        }
        ChatPage.prototype.deactivate = function () {
            if (this.timeoutHandler !== 0) {
                timeService.clearTimeout(this.timeoutHandler);
            }
        };
        ChatPage.prototype.activate = function () {
            var self = this;
            this.loading(true);
            this.timeoutHandler = timeService.setTimeout(function () {
                self.loading(false);
                self.timeoutHandler = 0;
            }, 2000);
        };
        ChatPage.prototype.back = function () {
            services_18.keyboardActivationService.forceHideKeyboard();
            app.lobbyPageBlock.showLobby();
        };
        ChatPage.prototype.send = function () {
            var self = this;
            var message = this.currentMessage();
            if (message === "" || message === null) {
                return;
            }
            var api = new OnlinePoker.Commanding.API.Chat(apiHost);
            api.Send(0, message, function (data, textStatus, jqXHR) {
                if (data.Status !== "Ok") {
                    simplepopup_14.SimplePopup.display(languagemanager_26._("chat.sendingMessage"), languagemanager_26._("errors." + data.Status));
                }
            }).fail(function () {
                simplepopup_14.SimplePopup.display(languagemanager_26._("chat.sendingMessage"), languagemanager_26._("chat.sendingMessageGenericError"));
            });
            this.currentMessage("");
        };
        ChatPage.prototype.addMessage = function (messageId, sender, message) {
            var m = new playerMessage_2.PlayerMessage(messageId, sender, message);
            if (this.messages().some(function (item) { return item.messageId === messageId; })) {
                // Skip adding message if it is already in the list.
                return;
            }
            this.messages.unshift(m);
            while (this.messages().length > 100) {
                this.messages.pop();
            }
            var messages = this.messages();
            this.messages([]);
            this.messages(messages);
        };
        return ChatPage;
    }());
    exports.ChatPage = ChatPage;
});
define("pages/changepasswordpage", ["require", "exports", "knockout", "ui/pagebase", "app"], function (require, exports, ko, pagebase_12, app_9) {
    "use strict";
    var ChangePasswordPage = (function (_super) {
        __extends(ChangePasswordPage, _super);
        function ChangePasswordPage() {
            _super.call(this);
            app_9.App.addTabBarItemMapping("more", "changePassword");
            this.oldpassword = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.confirmpassword = ko.observable().extend({ required: true });
            this.errors = ko.validation.group(this);
        }
        return ChangePasswordPage;
    }(pagebase_12.PageBase));
    exports.ChangePasswordPage = ChangePasswordPage;
});
/// <reference path="../pageblock.ts" />
define("pages/otherpageblock", ["require", "exports", "app", "pages/accountpage", "pages/ratingpage", "pages/settingspage", "pages/chatpage", "pages/changepasswordpage"], function (require, exports, app_10, accountpage_2, ratingpage_2, settingspage_1, chatpage_1, changepasswordpage_1) {
    "use strict";
    var OtherPageBlock = (function (_super) {
        __extends(OtherPageBlock, _super);
        function OtherPageBlock() {
            var accountPage = new accountpage_2.AccountPage();
            _super.call(this, "other", "account", accountPage);
            this.accountPage = accountPage;
            this.morePage = new settingspage_1.SettingsPage();
            app_10.App.addTabBarItemMapping("more", "other");
            this.ratingPage = new ratingpage_2.RatingPage();
            this.changePasswordPage = new changepasswordpage_1.ChangePasswordPage();
            this.chatPage = new chatpage_1.ChatPage();
            this.requireAuthentication = true;
            this.currentPage = "account";
            this.addSecondary("more", this.morePage);
            this.addSecondary("rating", this.ratingPage);
            this.addSecondary("chat", this.chatPage, true);
            this.addSecondary("changePassword", this.changePasswordPage);
        }
        return OtherPageBlock;
    }(PageBlock));
    exports.OtherPageBlock = OtherPageBlock;
});
define("pages/tablespage", ["require", "exports", "knockout", "table/tablemanager", "services/index", "table/tableview", "timeService", "services/uimanager", "ui/pagebase", "debugsettings", "settings", "commandManager"], function (require, exports, ko, tableManager, services_19, tableview_2, timeService, uimanager_2, pagebase_13, debugsettings_21, settings_9, commandManager) {
    "use strict";
    var TablesPage = (function (_super) {
        __extends(TablesPage, _super);
        function TablesPage() {
            _super.call(this);
            this.splashShown = ko.observable(false);
            this.tablesShown = ko.observable(true);
            var self = this;
            this.slideWidth = ko.observable(0);
            this.isConnectionSlow = ko.observable(false);
            this.calculateLandscapeWidth();
            this.currentIndex = ko.computed({
                read: function () {
                    return tableManager.currentIndex();
                },
                write: function (value) {
                    tableManager.currentIndex(value);
                    self.log("Switched to table with index " + value);
                }
            });
            this.currentIndex1 = ko.computed({
                read: function () {
                    return self.currentIndex() + 1;
                },
                write: function (value) {
                    self.currentIndex(value - 1);
                },
                owner: this
            });
            this.currentTable = ko.computed(function () {
                var tables = tableManager.tables();
                if (tables.length === 0) {
                    return new tableview_2.TableView(0, null);
                }
                var index = this.currentIndex();
                if (!tables.hasOwnProperty(index)) {
                    return new tableview_2.TableView(0, null);
                }
                return tables[index];
            }, this);
            this.selectedTables = ko.computed(function () {
                var tables = tableManager.tables();
                return tables;
            }, this);
            this.loading = ko.computed(function () {
                var ct = self.currentTable();
                if (ct == null) {
                    return false;
                }
                return ct.connecting();
            }, this);
            this.frozen = ko.computed(function () {
                var ct = self.currentTable();
                if (ct === null) {
                    return false;
                }
                return ct.frozen();
            }, this);
            this.opened = ko.computed(function () {
                var ct = self.currentTable();
                if (ct === null) {
                    return false;
                }
                return ct.opened();
            }, this);
            this.currentTable.subscribe(function (value) {
                tableManager.tables().forEach(function (_) {
                    if (_ !== value) {
                        _.soundEnabled = false;
                        _.animationSuppressed(true);
                    }
                });
                if (value != null) {
                    value.soundEnabled = true;
                    value.animationSuppressed(false);
                }
            });
        }
        TablesPage.prototype.calculateLandscapeWidth = function () {
            // When running not within browser, skip calculations.
            if (typeof window === "undefined") {
                return;
            }
            var viewportLandscapeWidth = 640;
            var currentWidth = $("body").width();
            if (currentWidth >= 1024 || (currentWidth === 768 && $("body").height() === 0)) {
                viewportLandscapeWidth = 1024;
                if (currentWidth >= 1920) {
                    viewportLandscapeWidth = 1920;
                }
            }
            if (currentWidth < 360) {
                if (window.innerHeight > 500) {
                    viewportLandscapeWidth = 568;
                }
                else {
                    viewportLandscapeWidth = 480;
                }
            }
            this.slideWidth(viewportLandscapeWidth);
        };
        TablesPage.prototype.recordConnection = function () {
            this.lastConnecton = navigator.connection.type;
        };
        TablesPage.prototype.setConnecting = function () {
            if (this.lastConnecton !== navigator.connection.type) {
                this.lastConnecton = navigator.connection.type;
                tableManager.tables().forEach(function (table) { return table.connecting(true); });
            }
        };
        TablesPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (this.activeHandler != null) {
                this.activeHandler.detach();
            }
            if (this.resignHandler != null) {
                this.resignHandler.detach();
            }
            if (this.slowConnectionHandler != null) {
                this.slowConnectionHandler.detach();
            }
            if (this.reconnectedHandler != null) {
                this.reconnectedHandler.detach();
            }
            if (this.disconnectedHandler != null) {
                this.disconnectedHandler.detach();
            }
            uimanager_2.uiManager.showPage("main");
            app.tabBar.visible(true);
            app.tabBar.select("tables", false);
            app.processing(true);
            var oldColor = $(".progress-background").css("background-color");
            $(".progress-background").css("background-color", "black");
            timeService.setTimeout(function () {
                app.processing(false);
                $(".progress-background").css("background-color", oldColor);
            }, 500);
            /* tslint:disable:no-string-literal no-unused-expression */
            window["StatusBar"] && StatusBar.show();
            /* tslint:enable:no-string-literal no-unused-expression */
            services_19.soundManager.tableSoundsEnabled(false);
            if (!PageBlock.useDoubleView) {
                services_19.orientationService.setOrientation("portrait");
            }
        };
        TablesPage.prototype.activate = function () {
            _super.prototype.activate.call(this);
            this.activeHandler = services_19.deviceEvents.active.add(this.setConnecting, this);
            this.resignHandler = services_19.deviceEvents.resignActive.add(this.recordConnection, this);
            this.slowConnectionHandler = services_19.connectionService.connectionSlow.add(this.onConnectionSlow, this);
            this.reconnectedHandler = services_19.connectionService.reconnected.add(this.onResetConnectionSlow, this);
            this.disconnectedHandler = services_19.connectionService.disconnected.add(this.onResetConnectionSlow, this);
            uimanager_2.uiManager.showPage("table");
            app.tabBar.visible(false);
            services_19.orientationService.setOrientation("landscape");
            timeService.setTimeout(function () {
                services_19.orientationService.lock();
            }, 200);
            var currentTable = this.currentTable();
            if (currentTable != null) {
                timeService.setTimeout(function () {
                    currentTable.actionBlock.updateBounds();
                }, 300);
            }
            /* tslint:disable:no-string-literal no-unused-expression */
            window["StatusBar"] && StatusBar.hide();
            /* tslint:enable:no-string-literal no-unused-expression */
            services_19.soundManager.enabled(settings_9.settings.soundEnabled());
            services_19.soundManager.tableSoundsEnabled(true);
            services_19.reloadManager.setReloadCallback(function () {
                if (debugsettings_21.debugSettings.application.reloadTablesDataOnResume) {
                    tableManager.clearTables();
                    tableManager.connectTables();
                    tableManager.clearTournaments();
                    tableManager.connectTournaments();
                }
            });
        };
        TablesPage.prototype.canActivate = function () {
            return tableManager.tables().length !== 0;
        };
        TablesPage.prototype.switchTable = function (index) {
            this.currentIndex(index());
        };
        TablesPage.prototype.prevTable = function () {
            tableManager.prevTable();
        };
        TablesPage.prototype.nextTable = function () {
            tableManager.nextTable();
        };
        TablesPage.prototype.addTable = function () {
            app.lobbyPageBlock.showLobby();
            this.deactivate();
        };
        TablesPage.prototype.toLobby = function () {
            var tableView = this.currentTable();
            if (tableView.myPlayer() != null) {
                app.lobbyPageBlock.showLobby();
                this.deactivate();
            }
            else {
                this.leave();
            }
        };
        TablesPage.prototype.leave = function () {
            var self = this;
            // Unsubscribe from table notifications.
            var tableView = this.currentTable();
            var removeCurrentTable = function () {
                // Navigate back to the lobby.
                if (tableManager.tables().length === 0) {
                    app.lobbyPageBlock.showLobby();
                    self.deactivate();
                }
            };
            var leaved = commandManager.executeCommand("app.leaveTable", [tableView.tableId]);
            leaved.done(removeCurrentTable);
        };
        TablesPage.prototype.showMenu = function () {
            app.executeCommand("popup.tableMenu");
        };
        /**
        * Removes tournament tables which are finished.
        */
        TablesPage.prototype.removeFinishedTournamentTable = function () {
            var finishedTournamentTables = tableManager.tables().filter(function (_) {
                var tournament = _.tournament();
                if (tournament == null) {
                    return false;
                }
                return tournament.finishedPlaying();
            });
            finishedTournamentTables.forEach(function (_) { return tableManager.remove(_); });
        };
        TablesPage.prototype.onConnectionSlow = function () {
            var self = this;
            this.isConnectionSlow(true);
            // Clear message after some time passed by.
            timeService.setTimeout(function () {
                self.isConnectionSlow(false);
            }, 3000);
        };
        TablesPage.prototype.onResetConnectionSlow = function () {
            this.isConnectionSlow(false);
        };
        TablesPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_21.debugSettings.tableView.trace) {
                console.log(message, params);
            }
        };
        return TablesPage;
    }(pagebase_13.PageBase));
    exports.TablesPage = TablesPage;
});
define("pages/SeatPage", ["require", "exports", "knockout", "table/tablemanager", "services/index", "table/tableview", "timeService", "services/uimanager", "ui/pagebase", "debugsettings", "settings", "commandManager"], function (require, exports, ko, tableManager, services_20, tableview_3, timeService, uimanager_3, pagebase_14, debugsettings_22, settings_10, commandManager) {
    "use strict";
    var SeatPage = (function (_super) {
        __extends(SeatPage, _super);
        function SeatPage() {
            _super.call(this);
            this.splashShown = ko.observable(false);
            this.tablesShown = ko.observable(true);
            var self = this;
            this.slideWidth = ko.observable(0);
            this.isConnectionSlow = ko.observable(false);
            this.calculateLandscapeWidth();
            this.currentIndex = ko.computed({
                read: function () {
                    return tableManager.currentIndex();
                },
                write: function (value) {
                    tableManager.currentIndex(value);
                    self.log("Switched to table with index " + value);
                }
            });
            this.currentIndex1 = ko.computed({
                read: function () {
                    return self.currentIndex() + 1;
                },
                write: function (value) {
                    self.currentIndex(value - 1);
                },
                owner: this
            });
            this.currentTable = ko.computed(function () {
                var tables = tableManager.tables();
                if (tables.length === 0) {
                    return new tableview_3.TableView(0, null);
                }
                var index = this.currentIndex();
                if (!tables.hasOwnProperty(index)) {
                    return new tableview_3.TableView(0, null);
                }
                return tables[index];
            }, this);
            this.selectedTables = ko.computed(function () {
                var tables = tableManager.tables();
                return tables;
            }, this);
            this.loading = ko.computed(function () {
                var ct = self.currentTable();
                if (ct == null) {
                    return false;
                }
                return ct.connecting();
            }, this);
            this.frozen = ko.computed(function () {
                var ct = self.currentTable();
                if (ct === null) {
                    return false;
                }
                return ct.frozen();
            }, this);
            this.opened = ko.computed(function () {
                var ct = self.currentTable();
                if (ct === null) {
                    return false;
                }
                return ct.opened();
            }, this);
            this.currentTable.subscribe(function (value) {
                tableManager.tables().forEach(function (_) {
                    if (_ !== value) {
                        _.soundEnabled = false;
                        _.animationSuppressed(true);
                    }
                });
                if (value != null) {
                    value.soundEnabled = true;
                    value.animationSuppressed(false);
                }
            });
        }
        SeatPage.prototype.calculateLandscapeWidth = function () {
            // When running not within browser, skip calculations.
            if (typeof window === "undefined") {
                return;
            }
            var viewportLandscapeWidth = 640;
            var currentWidth = $("body").width();
            if (currentWidth >= 1024 || (currentWidth === 768 && $("body").height() === 0)) {
                viewportLandscapeWidth = 1024;
                if (currentWidth >= 1920) {
                    viewportLandscapeWidth = 1920;
                }
            }
            if (currentWidth < 360) {
                if (window.innerHeight > 500) {
                    viewportLandscapeWidth = 568;
                }
                else {
                    viewportLandscapeWidth = 480;
                }
            }
            this.slideWidth(viewportLandscapeWidth);
        };
        SeatPage.prototype.recordConnection = function () {
            this.lastConnecton = navigator.connection.type;
        };
        SeatPage.prototype.setConnecting = function () {
            if (this.lastConnecton !== navigator.connection.type) {
                this.lastConnecton = navigator.connection.type;
                tableManager.tables().forEach(function (table) { return table.connecting(true); });
            }
        };
        SeatPage.prototype.deactivate = function () {
            _super.prototype.deactivate.call(this);
            if (this.activeHandler != null) {
                this.activeHandler.detach();
            }
            if (this.resignHandler != null) {
                this.resignHandler.detach();
            }
            if (this.slowConnectionHandler != null) {
                this.slowConnectionHandler.detach();
            }
            if (this.reconnectedHandler != null) {
                this.reconnectedHandler.detach();
            }
            if (this.disconnectedHandler != null) {
                this.disconnectedHandler.detach();
            }
            uimanager_3.uiManager.showPage("main");
            app.tabBar.visible(true);
            app.tabBar.select("tables", false);
            app.processing(true);
            var oldColor = $(".progress-background").css("background-color");
            $(".progress-background").css("background-color", "black");
            timeService.setTimeout(function () {
                app.processing(false);
                $(".progress-background").css("background-color", oldColor);
            }, 500);
            /* tslint:disable:no-string-literal no-unused-expression */
            window["StatusBar"] && StatusBar.show();
            /* tslint:enable:no-string-literal no-unused-expression */
            services_20.soundManager.tableSoundsEnabled(false);
            if (!PageBlock.useDoubleView) {
                services_20.orientationService.setOrientation("portrait");
            }
        };
        SeatPage.prototype.activate = function () {
            _super.prototype.activate.call(this);
            this.activeHandler = services_20.deviceEvents.active.add(this.setConnecting, this);
            this.resignHandler = services_20.deviceEvents.resignActive.add(this.recordConnection, this);
            this.slowConnectionHandler = services_20.connectionService.connectionSlow.add(this.onConnectionSlow, this);
            this.reconnectedHandler = services_20.connectionService.reconnected.add(this.onResetConnectionSlow, this);
            this.disconnectedHandler = services_20.connectionService.disconnected.add(this.onResetConnectionSlow, this);
            uimanager_3.uiManager.showPage("seat");
            app.tabBar.visible(false);
            services_20.orientationService.setOrientation("landscape");
            timeService.setTimeout(function () {
                services_20.orientationService.lock();
            }, 200);
            var currentTable = this.currentTable();
            if (currentTable != null) {
                timeService.setTimeout(function () {
                    currentTable.actionBlock.updateBounds();
                }, 300);
            }
            /* tslint:disable:no-string-literal no-unused-expression */
            window["StatusBar"] && StatusBar.hide();
            /* tslint:enable:no-string-literal no-unused-expression */
            services_20.soundManager.enabled(settings_10.settings.soundEnabled());
            services_20.soundManager.tableSoundsEnabled(true);
            services_20.reloadManager.setReloadCallback(function () {
                if (debugsettings_22.debugSettings.application.reloadTablesDataOnResume) {
                    tableManager.clearTables();
                    tableManager.connectTables();
                    tableManager.clearTournaments();
                    tableManager.connectTournaments();
                }
            });
        };
        SeatPage.prototype.canActivate = function () {
            return tableManager.tables().length !== 0;
        };
        SeatPage.prototype.switchTable = function (index) {
            this.currentIndex(index());
        };
        SeatPage.prototype.prevTable = function () {
            tableManager.prevTable();
        };
        SeatPage.prototype.nextTable = function () {
            tableManager.nextTable();
        };
        SeatPage.prototype.addTable = function () {
            app.lobbyPageBlock.showLobby();
            this.deactivate();
        };
        SeatPage.prototype.toLobby = function () {
            var tableView = this.currentTable();
            if (tableView.myPlayer() != null) {
                app.lobbyPageBlock.showLobby();
                this.deactivate();
            }
            else {
                this.leave();
            }
        };
        SeatPage.prototype.leave = function () {
            var self = this;
            // Unsubscribe from table notifications.
            var tableView = this.currentTable();
            var removeCurrentTable = function () {
                // Navigate back to the lobby.
                if (tableManager.tables().length === 0) {
                    app.lobbyPageBlock.showLobby();
                    self.deactivate();
                }
            };
            var leaved = commandManager.executeCommand("app.leaveTable", [tableView.tableId]);
            leaved.done(removeCurrentTable);
        };
        SeatPage.prototype.showMenu = function () {
            app.executeCommand("popup.tableMenu");
        };
        /**
        * Removes tournament tables which are finished.
        */
        SeatPage.prototype.removeFinishedTournamentTable = function () {
            var finishedTournamentTables = tableManager.tables().filter(function (_) {
                var tournament = _.tournament();
                if (tournament == null) {
                    return false;
                }
                return tournament.finishedPlaying();
            });
            finishedTournamentTables.forEach(function (_) { return tableManager.remove(_); });
        };
        SeatPage.prototype.onConnectionSlow = function () {
            var self = this;
            this.isConnectionSlow(true);
            // Clear message after some time passed by.
            timeService.setTimeout(function () {
                self.isConnectionSlow(false);
            }, 3000);
        };
        SeatPage.prototype.onResetConnectionSlow = function () {
            this.isConnectionSlow(false);
        };
        SeatPage.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_22.debugSettings.tableView.trace) {
                console.log(message, params);
            }
        };
        return SeatPage;
    }(pagebase_14.PageBase));
    exports.SeatPage = SeatPage;
});
define("pages/index", ["require", "exports", "pages/homepage", "pages/lobbypageblock", "pages/lobbypage", "pages/cashierpageblock", "pages/infopageblock", "pages/otherpageblock", "pages/tablespage", "pages/SeatPage"], function (require, exports, homepage_1, lobbypageblock_1, lobbypage_2, cashierpageblock_1, infopageblock_1, otherpageblock_1, tablespage_1, SeatPage_1) {
    "use strict";
    exports.HomePage = homepage_1.HomePage;
    exports.LobbyPageBlock = lobbypageblock_1.LobbyPageBlock;
    exports.LobbyPage = lobbypage_2.LobbyPage;
    exports.CashierPageBlock = cashierpageblock_1.CashierPageBlock;
    exports.InfoPageBlock = infopageblock_1.InfoPageBlock;
    exports.OtherPageBlock = otherpageblock_1.OtherPageBlock;
    exports.TablesPage = tablespage_1.TablesPage;
    exports.SeatPage = SeatPage_1.SeatPage;
});
/// <reference types="knockout" />
/// <reference path="./debugsettings.ts" />
define("tabbar", ["require", "exports", "debugsettings"], function (require, exports, debugsettings_23) {
    "use strict";
    var TabBarItem = (function () {
        function TabBarItem() {
        }
        TabBarItem.prototype.handler = function () {
            if (this.action != null) {
                this.action.call(this);
            }
        };
        return TabBarItem;
    }());
    var TabBar = (function () {
        function TabBar() {
            this.items = ko.observableArray([]);
            this.visible = ko.observable(true);
        }
        TabBar.prototype.addItem = function (name, caption, cssClass, action) {
            var item = new TabBarItem();
            item.name = name;
            item.caption = caption;
            item.cssClass = cssClass;
            item.action = action;
            item.selected = ko.observable(false);
            item.disabled = ko.observable(false);
            item.notice = ko.observable(false);
            this.items.push(item);
        };
        TabBar.prototype.isSelected = function (name) {
            var result = false;
            this.items().forEach(function (item) {
                if (item.name === name) {
                    result = item.selected();
                }
            });
            return result;
        };
        TabBar.prototype.select = function (name, enable) {
            this.log("Tabbar item '" + name + "' " + (enable ? "selected" : "deselected"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.selected(enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.enable = function (name, enable) {
            this.log("Tabbar item '" + name + "' " + (enable ? "enabled" : "disabled"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.disabled(!enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.notice = function (name, enable) {
            this.log("Notice for tabbar item '" + name + "' " + (enable ? "set" : "unset"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.notice(enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_23.debugSettings.application.debugTabbar) {
                var traceFunction = console.trace;
                traceFunction.bind(console, message, params)();
            }
        };
        return TabBar;
    }());
    exports.TabBar = TabBar;
});
define("app", ["require", "exports", "timeService", "metadatamanager", "popups/simplepopup", "authmanager", "settings", "services/broadcastservice", "commandManager", "languageManager", "services/index", "table/tablemanager", "pages/index", "popups/index", "services/uimanager", "tabbar", "debugsettings", "table/runtimesettings"], function (require, exports, timeService, metadataManager, simplepopup_15, authManager, settings_11, broadcastService, commandManager, languagemanager_27, services_21, tableManager, pages_1, popups_2, uimanager_4, tabbar_1, debugsettings_24, runtimeSettings) {
    "use strict";
    var App = (function () {
        function App() {
            this.currentPopup = null;
            this.newsPopup = new popups_2.NewsPopup();
            this.selectAvatarPopup = new popups_2.SelectAvatarPopup();
            this.stopped = false;
            this.savedPopup = null;
            var self = this;
            this.loadPromises = [];
            // register pages.
            this.homePage = new pages_1.HomePage();
            this.otherPageBlock = new pages_1.OtherPageBlock();
            this.infoPageBlock = new pages_1.InfoPageBlock();
            this.lobbyPageBlock = new pages_1.LobbyPageBlock();
            this.cashierPageBlock = new pages_1.CashierPageBlock();
            this.tablesPage = new pages_1.TablesPage();
            this.seatsPage = new pages_1.SeatPage();
            this.authPopup = new popups_2.AuthPopup();
            this.forgetPasswordPopup = new popups_2.ForgetPasswordPopup();
            this.continueForgetPasswordPopup = new popups_2.ContinueForgetPasswordPopup();
            this.changePasswordPopup = new popups_2.ChangePasswordPopup();
            this.registrationPopup = new popups_2.RegistrationPopup();
            this.simplePopup = new simplepopup_15.SimplePopup();
            this.okcancelPopup = new popups_2.OkCancelPopup();
            this.customPopup = new popups_2.CustomPopup();
            this.joinTablePopup = new popups_2.JoinTablePopup();
            this.tableMenuPopup = new popups_2.TableMenuPopup();
            this.addMoneyPopup = new popups_2.AddMoneyPopup();
            this.slowConnectionPopup = new popups_2.SlowConnectionPopup();
            this.tableChatPopup = new popups_2.ChatPopup();
            this.handHistoryPopup = new popups_2.HandHistoryPopup();
            this.accountStatusPopup = new popups_2.AccountStatusPopup();
            this.morePopup = new popups_2.MorePopup();
            this.processing = ko.observable(false);
            this.bindSubPage("home", self.homePage);
            this.bindSubPage("tables", self.tablesPage);
            this.bindSubPage("seats", self.seatsPage);
            this.bindPopup("auth", self.authPopup);
            this.bindPopup("forgetPassword", self.forgetPasswordPopup);
            this.bindPopup("continueForgetPassword", self.continueForgetPasswordPopup);
            this.bindPopup("changePassword", self.changePasswordPopup);
            this.bindPopup("registration", self.registrationPopup);
            this.bindPopup("simple", self.simplePopup);
            this.bindPopup("okcancel", self.okcancelPopup);
            this.bindPopup("custom", self.customPopup);
            this.bindPopup("joinTable", self.joinTablePopup);
            this.bindPopup("tableMenu", self.tableMenuPopup);
            this.bindPopup("addMoney", self.addMoneyPopup);
            this.bindPopup("slowConnection", self.slowConnectionPopup);
            this.bindPopup("tableChat", self.tableChatPopup);
            this.bindPopup("handHistory", self.handHistoryPopup);
            this.bindPopup("accountStatus", self.accountStatusPopup);
            this.bindPopup("news", self.newsPopup);
            this.bindPopup("selectAvatar", self.selectAvatarPopup);
            this.bindPageBlock("lobby", this.lobbyPageBlock);
            this.bindPageBlock("other", this.otherPageBlock);
            this.bindPageBlock("cashier", this.cashierPageBlock);
            this.bindPageBlock("info", this.infoPageBlock);
            this.bindUIElement(".more-block", this.morePopup);
            this.mainSelector = new Selector();
            if (typeof window !== "undefined") {
                var mainSelectorElement = $(".page.main .sub-page.selector")[0];
                if (mainSelectorElement) {
                    ko.applyBindings(this.mainSelector, mainSelectorElement);
                }
            }
            this.initializeTabbar();
            this.initializeConnection();
            // var progressBackgroundElement = $(".progress-background")[0];
            // ko.applyBindings(this, progressBackgroundElement);
            if (typeof window !== "undefined") {
                this.processing.subscribe(function (newValue) {
                    if (newValue) {
                        $(".progress-background").show();
                    }
                    else {
                        $(".progress-background").hide();
                    }
                });
            }
            // show startup page
            uimanager_4.uiManager.showPage("initialization");
            if (typeof window !== "undefined") {
                var opts = {
                    lines: 13,
                    length: 4,
                    width: 2,
                    radius: 6,
                    corners: 1,
                    rotate: 0,
                    direction: 1,
                    color: "#fff",
                    speed: 1,
                    trail: 60,
                    shadow: false,
                    hwaccel: false,
                    className: "spinner",
                    zIndex: 2e9,
                    top: "auto",
                    left: "auto" // Left position relative to parent in px
                };
                this.spinner = new Spinner(opts);
                var target = document.getElementById("spinner");
                this.spinner.spin(target);
                var progressTarget = document.getElementById("progress-spinner");
                this.progressSpinner = new Spinner(opts).spin(progressTarget);
            }
            this.popupClosed = new signals.Signal();
        }
        App.prototype.initializeTabbar = function () {
            this.tabBar = new tabbar_1.TabBar();
            this.tabBar.addItem("home", languagemanager_27._("tabbar.home"), "home", function () {
                app.executeCommand("page.home");
                app.showPageBlock("home");
            });
            this.tabBar.addItem("lobby", languagemanager_27._("tabbar.lobby"), "lobby", function () {
                app.lobbyPageBlock.showLobby();
            });
            this.tabBar.addItem("tables", languagemanager_27._("tabbar.tables"), "tables", function () {
                var currentTable = app.tablesPage.currentTable();
                if (currentTable === null || currentTable.model === null) {
                    console.warn("No tables opened. Could not open tables page");
                    simplepopup_15.SimplePopup.display(languagemanager_27._("menu.tables"), languagemanager_27._("tablesList.noTablesSelected"));
                    return;
                }
                app.executeCommand("app.selectTable", [currentTable.model]);
                app.showSubPage("tables");
            });
            this.tabBar.addItem("cashier", languagemanager_27._("tabbar.cashier"), "cashier", function () {
                app.executeCommand("pageblock.cashier");
                // app.executeCommand("pageblock.other");
            });
            this.tabBar.addItem("more", languagemanager_27._("tabbar.more"), "more", function () {
                var currentTabBarItem = uimanager_4.UIManager.getTabBarItemForPage(uimanager_4.uiManager.currentPageBlock);
                var isMoreSelected = app.tabBar.isSelected("more");
                var isMoreOpened = $("body").hasClass("more-opened");
                if (isMoreOpened) {
                    app.tabBar.select("more", false);
                    app.tabBar.select(currentTabBarItem, true);
                    $(".more-block").hide();
                    $("body").removeClass("more-opened");
                }
                else {
                    app.tabBar.select(currentTabBarItem, false);
                    app.tabBar.select("more", true);
                    app.morePopup.update();
                    $(".more-block").show();
                    $("body").addClass("more-opened");
                }
                app.morePopup.visible(!app.morePopup.visible());
            });
            this.tabBar.enable("tables", false);
            if (typeof window !== "undefined") {
                var tabBarElement = $(".toolbar-container")[0];
                if (tabBarElement) {
                    ko.applyBindings(this.tabBar, tabBarElement);
                }
            }
            uimanager_4.uiManager.subPageHiding.add(function (blockHiding) {
                var name = uimanager_4.UIManager.getTabBarItemForPage(blockHiding);
                app.tabBar.select(name, false);
            });
            uimanager_4.uiManager.subPageShowing.add(function (blockShowing) {
                var name = uimanager_4.UIManager.getTabBarItemForPage(blockShowing);
                app.tabBar.select(name, true);
            });
            uimanager_4.uiManager.pageBlockHiding.add(function (blockHiding) {
                var name = uimanager_4.UIManager.getTabBarItemForPage(blockHiding);
                app.tabBar.select(name, false);
            });
            uimanager_4.uiManager.pageBlockShowing.add(function (blockShowing) {
                var name = uimanager_4.UIManager.getTabBarItemForPage(blockShowing);
                app.tabBar.select(name, true);
            });
            uimanager_4.uiManager.pageBlockHidden.add(function (pageBlock) {
                if (app.tabBar.isSelected("more")) {
                    if (pageBlock !== "more" && !app.morePopup.visible()) {
                        app.tabBar.select("more", false);
                    }
                    app.hideMoreBlock();
                }
            });
        };
        // Bind Event Listeners
        //
        // Bind any events that are required on startup. Common events are:
        // "load", "deviceready", "offline", and "online".
        App.prototype.bindEvents = function () {
            document.addEventListener("deviceready", this.onDeviceReady, false);
            if (debugsettings_24.debugSettings.ios.hasMultitasking) {
                document.addEventListener("resign", this.onResign, false);
                document.addEventListener("active", this.onActive, false);
            }
            document.addEventListener("pause", this.onPause, false);
            document.addEventListener("resume", this.onResume, false);
            services_21.deviceEvents.initialize();
        };
        App.prototype.onResign = function () {
            var self = app;
            self.logEvent("Application resign active");
            // Wrap code in the timeout to prevent application from freezing.
            self.terminateConnection();
        };
        App.prototype.onActive = function () {
            var self = app;
            self.logEvent("Application active");
            // Wrap code in the timeout to prevent application from freezing.
            self.establishConnection();
        };
        App.prototype.onPause = function () {
            var self = app;
            var device = window.device;
            if (platformInfo.isTablet) {
                services_21.orientationService.enableRotation();
            }
            if (platformInfo.reloadOnResume) {
                // Don't do anything if reloading on resume.
                if (uimanager_4.uiManager.currentPage === "tables") {
                    settings_11.settings.lastPage("tables");
                }
                else {
                    settings_11.settings.lastPage("main");
                }
                settings_11.settings.authToken(authToken);
                settings_11.settings.lastTime(new Date().valueOf());
                settings_11.settings.saveSettings();
                return;
            }
            if (self.suppressResume) {
                return;
            }
            if (debugsettings_24.debugSettings.application.deactivateOnPause) {
                uimanager_4.uiManager.deactivateSubPage(uimanager_4.uiManager.currentPage);
            }
            self.logEvent("Pause application");
            // Wrap code in the timeout to prevent application from freezing.
            timeService.setTimeout(function () {
                self.logEvent("Pause application#2");
                self.showSplash();
                settings_11.settings.saveSettings();
                if (self.currentPopup === "registration") {
                    self.savedPopup = self.currentPopup;
                }
                else {
                    self.savedPopup = null;
                }
                self.closePopup();
                timeService.printDebug();
            }, 0);
            timeService.stop();
            self.processing(false);
            self.spinner.stop();
            self.terminateConnection();
        };
        App.prototype.onResume = function () {
            var self = app;
            var device = window.device;
            if (platformInfo.reloadOnResume) {
                // Reload to the main page with special configuration parameter
                self.showSplash();
                location.replace(location.protocol + "//" + location.host + location.pathname + "?restore=true");
                return;
            }
            if (self.suppressResume) {
                self.suppressResume = false;
                if (self.savedPopup) {
                    self.showPopup(self.savedPopup, true);
                    self.savedPopup = null;
                }
                return;
            }
            self.logEvent("Resume application");
            timeService.start();
            self.showSplash();
            var target = document.getElementById("spinner");
            self.spinner.spin(target);
            var pageBeforeClosing = uimanager_4.uiManager.effectivePageContainer;
            var pageBlockBeforeClosing = uimanager_4.uiManager.currentPageBlock;
            var subPageBeforeClosing = uimanager_4.uiManager.currentPage;
            if (pageBeforeClosing === "initialization" || pageBeforeClosing === null) {
                pageBeforeClosing = "main";
                pageBlockBeforeClosing = "home";
                subPageBeforeClosing = "home";
            }
            uimanager_4.uiManager.showPage("initialization");
            settings_11.settings.loadSettings();
            self.setInitializationState();
            services_21.orientationService.setLastOrientation();
            if (platformInfo.isTablet) {
                services_21.orientationService.suppressRotation();
            }
            if (services_21.slowInternetService.fatalError) {
                self.hideSplash();
                services_21.slowInternetService.showDuplicatedConnectionPopup();
                return;
            }
            if (services_21.slowInternetService.hasInternet()) {
                services_21.slowInternetService.closePopup();
                self.updateMetadataOnResume(pageBeforeClosing, pageBlockBeforeClosing, subPageBeforeClosing);
                services_21.slowInternetService.setRetryHandler(null);
                self.hideSplash();
            }
            else {
                services_21.slowInternetService.onOffline();
                self.hideSplash();
                services_21.slowInternetService.setRetryHandler(function () {
                    self.updateMetadataOnResume(pageBeforeClosing, pageBlockBeforeClosing, subPageBeforeClosing);
                });
            }
        };
        App.prototype.updateMetadataOnResume = function (lastPage, pageBlockBeforeClosing, subPageBeforeClosing) {
            var self = this;
            if (debugsettings_24.debugSettings.initialization.stopOnResume) {
                return;
            }
            console.log("Launch intialization of metadata on resume");
            if (!this.fullyInitialized) {
                console.log("Application was never initialized, performing full initialization.");
                this.updateMetadataOnLaunch();
                return;
            }
            var failHandler = function () {
                console.log("Failed updating metadata on resume, rescheduling attempt.");
                services_21.slowInternetService.showReconnectFailedPopup();
                services_21.slowInternetService.setRetryHandler(function () {
                    self.updateMetadataOnResume(lastPage, pageBlockBeforeClosing, subPageBeforeClosing);
                });
            };
            var successPath = function () {
                self.preloadTableImages();
                metadataManager.update().done(function () {
                    tableManager.initialize();
                    tableManager.getCurrentTablesAndTournaments().done(function () {
                        self.spinner.stop();
                        self.establishConnection().then(function (wrapper) {
                            if (wrapper.terminated) {
                                return;
                            }
                            self.logEvent("Showing last page");
                            if (lastPage != null) {
                                if (!debugsettings_24.debugSettings.application.goToLobbyAfterPause) {
                                    uimanager_4.uiManager.showPage(lastPage);
                                }
                                else {
                                    uimanager_4.uiManager.showPage("main");
                                    if (lastPage === "table") {
                                        uimanager_4.uiManager.showPageBlock("lobby");
                                        uimanager_4.uiManager.showSubPage("lobby");
                                    }
                                }
                            }
                            else {
                                uimanager_4.uiManager.showPage("main");
                            }
                            if (debugsettings_24.debugSettings.application.deactivateOnPause) {
                                self.logEvent("Activating subpage");
                                uimanager_4.uiManager.activateSubPage(uimanager_4.uiManager.currentPage);
                            }
                            if (self.savedPopup) {
                                self.showPopup(self.savedPopup, true);
                                self.savedPopup = null;
                            }
                            services_21.orientationService.setLastOrientation();
                            services_21.reloadManager.execute();
                        });
                    }).fail(failHandler);
                }).fail(failHandler);
            };
            this.versionCheck(successPath);
        };
        App.prototype.updateMetadataOnLaunch = function () {
            var self = this;
            if (debugsettings_24.debugSettings.initialization.stopOnLaunch) {
                return;
            }
            console.log("Launch intialization of metadata first time");
            var failHandler = function () {
                console.log("Failed updating metadata for first time, rescheduling attempt.");
                services_21.slowInternetService.setRetryHandler(function () {
                    self.updateMetadataOnLaunch();
                });
            };
            var successPath = function () {
                self.preloadTableImages();
                metadataManager.update().done(function () {
                    self.spinner.stop();
                    tableManager.initialize();
                    tableManager.getCurrentTablesAndTournaments().done(function () {
                        self.establishConnection();
                        self.fullyInitialized = true;
                        metadataManager.setReady(null);
                    }).fail(failHandler);
                }).fail(failHandler);
            };
            this.versionCheck(successPath);
        };
        App.prototype.versionCheck = function (successPath) {
            metadataManager.versionCheck().done(function () {
                successPath();
            }).fail(function (couldWork) {
                // Display dialog which prompts for the update.
                app.promptEx(languagemanager_27._("updater.title"), [languagemanager_27._("updater.line1")], [languagemanager_27._("updater.button")], [function () {
                        services_21.websiteService.navigateUpdateApk();
                    }]);
            });
        };
        // deviceready Event Handler
        //
        // The scope of 'this' is the event. In order to call the 'receivedEvent'
        // function, we must explicity call 'app.receivedEvent(...);'
        App.prototype.onDeviceReady = function () {
            var device = window.device;
            if (device != null && device.available) {
                if (device.platform.toLowerCase() === "ios") {
                    if (device.version.toString().indexOf("7.0") === 0) {
                        StatusBar.overlaysWebView(false);
                        /* tslint:disable:no-string-literal */
                        ko.bindingHandlers["image"].options.enabled = false;
                        /* tslint:enable:no-string-literal */
                        runtimeSettings.setShowNewsAfterLogin(false);
                    }
                }
            }
            if (platformInfo.hasMenu()) {
                app.setupMenu();
            }
            // Recalculate width of the landscape mode.
            var currentWidth = $("body").width();
            if (currentWidth === 1024) {
                platformInfo.isTablet = true;
            }
            app.tablesPage.calculateLandscapeWidth();
            services_21.pushService.register();
            tableManager.maxTablesReached.add(function (continuation) {
                simplepopup_15.SimplePopup.display(languagemanager_27._("maxtables.caption"), languagemanager_27._("maxtables.maxtablesreached"));
            });
            tableManager.hasTurn.subscribe(function (value) {
                app.tabBar.notice("tables", value);
            });
            app.popupClosed.add(function (popupName) {
                services_21.keyboardActivationService.forceHideKeyboard();
                console.log("Popup " + popupName + " closed");
            });
            app.receivedEvent("deviceready");
        };
        App.prototype.setupTouchActivation = function () {
            // This is a fix which updates binding for the knockout value binding.
            // This is nescessary since touchstart event start handling.
            $("body").on("keyup", "input", function (event) {
                $(this).trigger("change");
            });
            $("body").on("touchstart", ".button, .actionable", function (event) {
                $(this).addClass("pressed");
            }).on("touchend", ".button, .actionable", function (event) {
                $(this).removeClass("pressed");
            }).on("touchcancel", ".button, .actionable", function (event) {
                $(this).removeClass("pressed");
            });
        };
        App.prototype.setupMenu = function () {
            var self = this;
            menu.initialize([
                { id: 1, order: 1, name: languagemanager_27._("menu.home") },
                { id: 2, order: 1, name: languagemanager_27._("menu.lobby") },
                { id: 3, order: 1, name: languagemanager_27._("menu.tables") },
                { id: 4, order: 1, name: languagemanager_27._("menu.cashier") },
                { id: 5, order: 1, name: languagemanager_27._("menu.account") },
                { id: 6, order: 1, name: languagemanager_27._("menu.chat") },
                { id: 7, order: 1, name: languagemanager_27._("menu.rating") },
                { id: 8, order: 1, name: languagemanager_27._("menu.messages") },
                { id: 9, order: 1, name: languagemanager_27._("menu.information") },
            ]);
            menu.optionItemClick = function (itemId) {
                if (itemId === 1) {
                    app.executeCommand("page.home");
                    app.showPageBlock("home");
                }
                if (itemId === 2) {
                    app.lobbyPageBlock.showLobby();
                }
                if (itemId === 3) {
                    app.executeCommand("page.tables");
                }
                if (itemId === 4) {
                    app.executeCommand("pageblock.cashier");
                }
                if (itemId === 5) {
                    app.executeCommand("page.account");
                }
                if (itemId === 6) {
                    app.executeCommand("page.chat");
                }
                if (itemId === 7) {
                    app.executeCommand("page.rating");
                }
                if (itemId === 8) {
                    app.executeCommand("page.messages");
                }
                if (itemId === 9) {
                    app.executeCommand("page.information");
                }
                console.log(itemId);
            };
        };
        // Update DOM on a Received Event
        App.prototype.receivedEvent = function (id) {
            var self = this;
            timeService.start();
            settings_11.settings.soundEnabled.subscribe(function (value) {
                services_21.soundManager.enabled(value);
            });
            settings_11.settings.loadSettings();
            settings_11.settings.isGuest.subscribe(function (value) {
                if (authManager.authenticated() && !value) {
                    app.lobbyPageBlock.lobbyPage.cashOptions.currency(1);
                    app.lobbyPageBlock.lobbyPage.sngOptions.currency(1);
                    app.lobbyPageBlock.lobbyPage.tournamentOptions.currency(1);
                }
                else {
                    app.lobbyPageBlock.lobbyPage.cashOptions.currency(2);
                    app.lobbyPageBlock.lobbyPage.sngOptions.currency(2);
                    app.lobbyPageBlock.lobbyPage.tournamentOptions.currency(2);
                }
            });
            authManager.authenticated.subscribe(function (value) {
                if (value && !settings_11.settings.isGuest()) {
                    app.lobbyPageBlock.lobbyPage.cashOptions.currency(1);
                    app.lobbyPageBlock.lobbyPage.sngOptions.currency(1);
                    app.lobbyPageBlock.lobbyPage.tournamentOptions.currency(1);
                }
                else {
                    app.lobbyPageBlock.lobbyPage.cashOptions.currency(2);
                    app.lobbyPageBlock.lobbyPage.sngOptions.currency(2);
                    app.lobbyPageBlock.lobbyPage.tournamentOptions.currency(2);
                }
            });
            $.when(this.loadPromises).done(function () {
                services_21.keyboardActivationService.setup();
                self.setupTouchActivation();
            });
            this.setInitializationState();
            var startPage = "main";
            if (this.getParameterByName("restore") === "true") {
                var currentTime = new Date();
                var timeDiff = currentTime.valueOf() - settings_11.settings.lastTime();
                if (timeDiff > 1000 * 60 * 20) {
                    authToken = null;
                    startPage = "main";
                }
                else {
                    authToken = settings_11.settings.authToken();
                    startPage = settings_11.settings.lastPage();
                }
            }
            this.logEvent("Received Event: " + id);
            if (window.innerWidth > 640) {
                PageBlock.useDoubleView = true;
                AnimationSettings.platform = "tablet";
            }
            this.setDesiredOrientation();
            if (platformInfo.isTablet) {
                services_21.orientationService.suppressRotation();
            }
            this.setupClosePopupOnClick();
            metadataManager.setFailed(function () {
                services_21.slowInternetService.onConnectionSlow();
                services_21.slowInternetService.onDisconnected();
                self.metadataUpdateFailed();
            });
            metadataManager.setReady(function () {
                // Adjust height
                var toolpadHeight = platformInfo.hasTabBar() ? 49 : 15;
                var logoHeight = 102;
                var noLogoHeight = 57;
                var pageHeaderHeight = 58;
                if (platformInfo.hasTabBar()) {
                    $(".page.main").addClass("has-toolbar");
                }
                noLogoHeight = window.innerHeight - toolpadHeight - noLogoHeight - platformInfo.statusBarHeight();
                $(".popup-container.scroll-container").css("max-height", noLogoHeight);
                var device = window.device;
                var tournamentLobbyAdjustment = 57;
                var lobbyAdjustment = 0;
                if (device != null && device.available) {
                    if (device.platform.toLowerCase() === "ios") {
                        tournamentLobbyAdjustment = 57;
                    }
                    else {
                        tournamentLobbyAdjustment = 81;
                        lobbyAdjustment = -30;
                        app.setupMenu();
                    }
                }
                if (startPage === "main") {
                    uimanager_4.uiManager.showPage("main");
                    self.showPageBlock("home");
                    self.showSubPage("home");
                }
                else {
                    app.showSubPage("tables");
                    self.loadTablesAndTournaments(true);
                }
                tableManager.tables.subscribe(function (newValue) {
                    self.updateTabbar(authManager.authenticated(), newValue);
                });
                authManager.authenticated.subscribe(function (newValue) {
                    self.updateTabbar(newValue, tableManager.tables());
                    if (newValue && metadataManager.banners != null) {
                        var filteredBanners = metadataManager.banners.filter(function (_) { return _.Id > settings_11.settings.lastBannerId(); }).sort(function (a, b) { return a.Id - b.Id; });
                        if (filteredBanners.length > 0) {
                            var currentBanner = filteredBanners[0];
                            settings_11.settings.lastBannerId(currentBanner.Id);
                            settings_11.settings.saveSettings();
                            if (runtimeSettings.showNewsAfterLogin) {
                                app.newsPopup.setData(currentBanner);
                                app.showPopup("news");
                            }
                        }
                    }
                    console.log("Authentication changed.");
                    self.terminateConnection();
                    self.loadTablesAndTournaments(newValue);
                });
            });
            services_21.slowInternetService.initialize();
            var hasInternet = services_21.slowInternetService.hasInternet();
            console.log("Detecting internet status..." + hasInternet ? "connected" : "not connected");
            if (hasInternet) {
                services_21.slowInternetService.setRetryHandler(null);
                self.updateMetadataOnLaunch();
                self.hideSplash();
            }
            else {
                services_21.slowInternetService.onOffline();
                self.hideSplash();
                services_21.slowInternetService.setRetryHandler(function () { return self.updateMetadataOnLaunch(); });
            }
        };
        App.prototype.loadTablesAndTournaments = function (authenticated) {
            var self = this;
            if (authenticated) {
                tableManager.getCurrentTablesAndTournaments().done(function () {
                    self.establishConnection();
                }).fail(function () {
                    console.log("Could not get current tables!");
                    services_21.slowInternetService.showReconnectFailedPopup();
                });
            }
            else {
                tableManager.clear();
                self.establishConnection();
            }
        };
        App.prototype.setInitializationState = function () {
            var parentElement = document.getElementById("deviceready");
            if (parentElement) {
                var listeningElement = parentElement.querySelector(".listening");
                var receivedElement = parentElement.querySelector(".received");
                listeningElement.setAttribute("style", "display:none;");
                receivedElement.setAttribute("style", "display:block;");
            }
        };
        App.prototype.setFailedState = function () {
            var parentElement = document.getElementById("deviceready");
            if (parentElement) {
                var listeningElement = parentElement.querySelector(".listening");
                var receivedElement = parentElement.querySelector(".received");
                receivedElement.setAttribute("style", "display:none;");
                var failedElement = parentElement.querySelector(".failed");
                failedElement.setAttribute("style", "display:block;");
            }
        };
        App.prototype.metadataUpdateFailed = function () {
            this.setFailedState();
            console.log("Metadata retreiving failure");
        };
        App.prototype.setDesiredOrientation = function () {
            if (!PageBlock.useDoubleView) {
                services_21.orientationService.setOrientation("portrait");
            }
            else {
                services_21.orientationService.setOrientation("landscape");
            }
        };
        App.prototype.initializeConnection = function () {
            var self = this;
            // connectionService.initializeConnection();
            services_21.connectionService.recoverableError.add(function () {
                self.establishConnection();
            });
        };
        App.prototype.terminateConnection = function (forceDisconnect) {
            if (forceDisconnect === void 0) { forceDisconnect = false; }
            tableManager.stopConnectingToTables();
            services_21.connectionService.terminateConnection(forceDisconnect);
        };
        App.prototype.establishConnection = function (maxAttempts) {
            if (maxAttempts === void 0) { maxAttempts = 3; }
            var self = this;
            // This part should be moved up to the stack to remove dependency on other services 
            // in the connection management.
            services_21.connectionService.initializeConnection();
            return services_21.connectionService.establishConnection(maxAttempts).then(function (wrapper) {
                if (wrapper.terminated) {
                    return wrapper;
                }
                self.logEvent("Setting up connection dependent services.");
                services_21.slowInternetService.manualDisconnect = false;
                tableManager.connectTables();
                tableManager.connectTournaments();
                var connection = wrapper.connection;
                try {
                    self.logEvent("Joining lobby chat.");
                    connection.Chat.server.join(0);
                }
                catch (error) {
                    console.log(error);
                    throw new Error("Could not join chat after establishingConnection");
                }
                self.logEvent("Listening lobby chat messages.");
                var chatHub = connection.createHubProxy("chat");
                chatHub.on("Message", function () {
                    var msg = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        msg[_i - 0] = arguments[_i];
                    }
                    var messageId = msg[0];
                    var tableId = msg[1];
                    var type = msg[2];
                    var sender = msg[3];
                    var message = msg[4];
                    if (tableId !== 0) {
                        return;
                    }
                    if (type === "B") {
                        broadcastService.displayMessage(message);
                    }
                });
                return wrapper;
            });
        };
        App.prototype.buildStartConnection = function () {
            return services_21.connectionService.buildStartConnection();
        };
        App.prototype.updateTabbar = function (authenticated, tables) {
            var self = this;
            if (authenticated) {
                var tablesEnabled = tables.length > 0;
                self.tabBar.enable("tables", tablesEnabled);
            }
            else {
                self.tabBar.enable("tables", false);
            }
        };
        App.prototype.showSubPage = function (pageName) {
            this.hideMoreBlock();
            uimanager_4.uiManager.showSubPage(pageName);
        };
        App.prototype.showPageBlock = function (pageBlockName) {
            uimanager_4.uiManager.showPageBlock(pageBlockName);
        };
        App.addTabBarItemMapping = function (tabBarItem, pageName) {
            uimanager_4.UIManager.addTabBarItemMapping(tabBarItem, pageName);
        };
        App.prototype.showSelector = function (selectorCaption, options, success) {
            var successCallback = function (item) {
                $(".page .page-block." + uimanager_4.uiManager.currentPageBlock).css("display", "block");
                $(".page .sub-page." + uimanager_4.uiManager.currentPage).css("display", "block");
                $(".page .sub-page.selector").css("display", "none");
                success(item);
            };
            var cancelCallback = function (item) {
                $(".page .page-block." + uimanager_4.uiManager.currentPageBlock).css("display", "block");
                $(".page .sub-page." + uimanager_4.uiManager.currentPage).css("display", "block");
                $(".page .sub-page.selector").css("display", "none");
            };
            $(".page .page-block." + uimanager_4.uiManager.currentPageBlock).css("display", "none");
            $(".page .sub-page." + uimanager_4.uiManager.currentPage).css("display", "none");
            $(".page .sub-page.selector").css("display", "block");
            this.mainSelector.setParams(selectorCaption, options, successCallback, cancelCallback);
        };
        App.prototype.bindPageBlock = function (pageBlockName, viewModel) {
            var self = this;
            commandManager.registerCommand("pageblock." + pageBlockName, function () {
                var requireAuthentication = viewModel.requireAuthentication;
                if (!requireAuthentication) {
                    if (!viewModel.requireGuestAuthentication) {
                        self.showPageBlock(pageBlockName);
                    }
                    else {
                        app.requireGuestAuthentication().done(function (value) {
                            if (value) {
                                self.showPageBlock(pageBlockName);
                            }
                        });
                    }
                }
                else {
                    app.requireAuthentication().done(function (value) {
                        if (value) {
                            self.showPageBlock(pageBlockName);
                        }
                    });
                }
            });
            for (var i = 0; i < viewModel.loadPromises.length; i++) {
                var item = viewModel.loadPromises[i];
                this.loadPromises.push(item);
            }
        };
        App.prototype.bindSubPage = function (pageName, viewModel) {
            var self = this;
            commandManager.registerCommand("page." + pageName, function () {
                var requireAuthentication = viewModel.requireAuthentication || false;
                if (!requireAuthentication) {
                    self.showSubPage(pageName);
                }
                else {
                    app.requireAuthentication().done(function (value) {
                        if (value) {
                            self.showSubPage(pageName);
                        }
                    });
                }
            });
            if (typeof window === "undefined") {
                return;
            }
            var pagejElement = $(".page .sub-page." + pageName);
            if (pagejElement.length === 0) {
                console.error("Could not bind sub page " + pageName + " since DOM element not found.");
                return;
            }
            if (pagejElement.length > 1) {
                console.warn("Page " + pageName + " has more then one element to bind.");
                return;
            }
            var pageElement = pagejElement[0];
            if (!pageElement.hasChildNodes()) {
                var templateSource = pagejElement.data("template");
                var pageLoadPromise = $.get(templateSource, "text/html").then(function (data) {
                    pagejElement.html(data);
                    try {
                        ko.applyBindings(viewModel, pageElement);
                    }
                    catch (e) {
                        console.log("Bind page " + pageName + " failed");
                    }
                });
                this.loadPromises.push(pageLoadPromise);
                return;
            }
            ko.applyBindings(viewModel, pageElement);
        };
        App.prototype.bindPopup = function (popup, viewModel) {
            var self = this;
            commandManager.registerCommand("popup." + popup, function () {
                self.showPopup(popup);
            });
            if (typeof window === "undefined") {
                return;
            }
            var popupjElement = $(".popup." + popup);
            if (popupjElement.length === 0) {
                console.error("Could not bind popup " + popup + " since DOM element not found.");
                return;
            }
            if (popupjElement.length > 1) {
                console.warn("Popup " + popup + " has more then one element to bind.");
                return;
            }
            var popupElement = popupjElement[0];
            if (!popupElement.hasChildNodes()) {
                var templateSource = popupjElement.data("template");
                $.get(templateSource, "text/html").then(function (data) {
                    popupjElement.html(data);
                    try {
                        ko.applyBindings(viewModel, popupElement);
                    }
                    catch (e) {
                        console.log("Bind popup " + popup + " failed");
                    }
                });
                return;
            }
            ko.applyBindings(viewModel, popupElement);
        };
        App.prototype.bindUIElement = function (className, viewModel) {
            var self = this;
            if (typeof window === "undefined") {
                return;
            }
            var uiElement = $(className);
            if (uiElement.length === 0) {
                console.error("Could not bind UI element with class " + className + " since DOM element not found.");
                return;
            }
            if (uiElement.length > 1) {
                console.warn("UI element with class " + className + " has more then one element.");
                return;
            }
            var domElement = uiElement[0];
            if (!domElement.hasChildNodes()) {
                var templateSource = uiElement.data("template");
                $.get(templateSource, "text/html").then(function (data) {
                    uiElement.html(data);
                    try {
                        ko.applyBindings(viewModel, domElement);
                    }
                    catch (e) {
                        console.log("Bind UI element " + className + " failed");
                    }
                });
                return;
            }
            ko.applyBindings(viewModel, domElement);
        };
        App.prototype.showPopup = function (popupName) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (!popupName) {
                console.error("The empty popup passed");
                throw new Error("The empty popup passed");
            }
            if (this.currentPopup != null) {
                this.hideMoreBlock();
                this.closePopup();
            }
            this.currentPopup = popupName;
            console.log("Show popup " + popupName);
            var result = $.Deferred();
            this.popupClosed.addOnce(function (name, dialogResults) {
                if (popupName !== name) {
                    console.warn("Responding to popup " + name + " instead of " + popupName);
                }
                var signalData = {
                    name: name,
                    result: dialogResults
                };
                result.resolve(signalData);
            }, this, 1);
            var popupObject = this[this.currentPopup + "Popup"];
            if (popupObject != null) {
                popupObject.shown(args);
            }
            if (typeof window !== "undefined") {
                var popupContainer = $(".popup." + popupName + " .popup-container");
                if (popupContainer.length > 0) {
                    popupContainer[0].scrollTop = 0;
                }
                $(".popup." + popupName).css("display", "block");
                $(".popup-background").css("display", "block");
            }
            return result;
        };
        App.prototype.closePopup = function (result) {
            if (this.currentPopup) {
                console.log("Close popup " + this.currentPopup);
                if (typeof window !== "undefined") {
                    $(".popup." + this.currentPopup + " .popup-container")[0].scrollTop = 0;
                    $(".popup." + this.currentPopup).css("display", "none");
                    $(".popup-background").css("display", "none");
                }
                var popupName = this.currentPopup;
                var popupObject = this[this.currentPopup + "Popup"];
                /* tslint:disable:no-string-literal */
                if (popupObject !== undefined && popupObject["visible"] !== undefined) {
                    popupObject.visible(false);
                }
                /* tslint:enable:no-string-literal */
                this.currentPopup = null;
                this.popupClosed.dispatch(popupName, result);
            }
        };
        App.prototype.executeCommand = function (commandName, parameters) {
            if (parameters === void 0) { parameters = []; }
            if (commandName === "popup.auth.show") {
                this.showPopup("auth");
                return;
            }
            if (commandName === "popup.auth.forgetPassword") {
                this.showPopup("forgetPassword");
                return;
            }
            if (commandName === "popup.auth.registration") {
                this.showPopup("registration");
                return;
            }
            if (commandName === "popup.auth.continueForgetPassword") {
                this.showPopup("continueForgetPassword");
                return;
            }
            if (commandName === "popup.close") {
                this.closePopup();
                return;
            }
            if (commandName === "legal.eula") {
                app.closePopup();
                app.executeCommand("pageblock.info");
                app.infoPageBlock.showLicenseAgreement();
                return;
            }
            if (commandName === "popup.cancel") {
                this.closePopup("cancel");
                return;
            }
            if (commandName === "more.close") {
                this.hideMoreBlock();
                var currentTabBarItem = uimanager_4.UIManager.getTabBarItemForPage(uimanager_4.uiManager.currentPageBlock);
                app.tabBar.select(currentTabBarItem, true);
                return;
            }
            if (commandName === "logout") {
                authManager.logout();
                return;
            }
            if (commandName === "app.exit") {
                authManager.logout();
                this.reloadApplication();
                return;
            }
            commandManager.executeCommand(commandName, parameters);
        };
        App.prototype.reloadApplication = function () {
            /* tslint:disable:no-unused-expression no-string-literal */
            window["StatusBar"] && StatusBar.show();
            /* tslint:enable:no-unused-expression no-string-literal */
            window.location.reload();
        };
        App.prototype.shouldRotateToOrientation = function (interfaceOrientation) {
            /// Checks that given orientation currently supported
            /// For now this is works in iOS.
            return ScreenOrientation.shouldRotateToOrientation(interfaceOrientation);
        };
        App.prototype.hideMoreBlock = function () {
            app.morePopup.visible(false);
            $(".more-block").hide();
            $("body").removeClass("more-opened");
            app.tabBar.select("more", false);
        };
        App.prototype.requireAuthentication = function () {
            var self = this;
            var result = $.Deferred();
            if (!authManager.authenticated()) {
                // We don't authenticated, so display authentication popup.
                this.popupClosed.addOnce(function () {
                    // Resolve with current authentication status, so 
                    // caller would know operation was successful or not.
                    result.resolve(authManager.authenticated(), false);
                }, this, null);
                this.showPopup("auth");
            }
            else {
                // We already authenticated, no need to show authentication popup.
                result.resolve(true, true);
            }
            return result;
        };
        App.prototype.requireGuestAuthentication = function () {
            var self = this;
            var result = $.Deferred();
            if (!authManager.authenticated()) {
                // We don't authenticated, so display authentication popup.
                authManager.loginAsGuest().then(function (status) {
                    result.resolve(authManager.authenticated(), false);
                });
            }
            else {
                // We already authenticated, no need to show authentication popup.
                result.resolve(true, true);
            }
            return result;
        };
        App.prototype.prompt = function (title, messages, buttons) {
            if (buttons === void 0) { buttons = null; }
            if (buttons === null) {
                buttons = [languagemanager_27._("common.ok"), languagemanager_27._("common.cancel")];
            }
            this.showPopup("okcancel");
            var popupObject = this.okcancelPopup;
            var deferred = popupObject.deferred;
            popupObject.title(title);
            popupObject.messages(messages);
            popupObject.buttons(buttons);
            popupObject.customStyle("");
            return deferred;
        };
        App.prototype.promptEx = function (title, messages, buttons, actions) {
            this.showPopup("custom");
            var popupObject = this.customPopup;
            var deferred = popupObject.deferred;
            popupObject.title(title);
            popupObject.messages(messages);
            popupObject.buttons(buttons);
            popupObject.actions(actions);
            return deferred;
        };
        App.prototype.setupClosePopupOnClick = function () {
            $(".popup-background").on("tap", function () {
                app.closePopup();
            });
        };
        App.prototype.preloadTableImages = function () {
            services_21.imagePreloadService.preloadResource("img/lobby/table2-empty");
            services_21.imagePreloadService.preloadResource("img/lobby/table2-s1");
            services_21.imagePreloadService.preloadResource("img/lobby/table2-s2");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-empty");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s1");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s2");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s3");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s4");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s5");
            services_21.imagePreloadService.preloadResource("img/lobby/table6-s6");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-empty");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s1");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s2");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s3");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s4");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s5");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s6");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s7");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s8");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s9");
            services_21.imagePreloadService.preloadResource("img/lobby/table10-s10");
            services_21.imagePreloadService.preloadDeviceSpecificResource("img/images/tableImages/poker_table");
        };
        App.prototype.logEvent = function (message) {
            if (debugsettings_24.debugSettings.device.events) {
                console.log(message);
            }
        };
        App.prototype.getParameterByName = function (name) {
            name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
            var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"), results = regex.exec(location.search);
            return results == null
                ? ""
                : decodeURIComponent(results[1].replace(/\+/g, " "));
        };
        App.prototype.showSplash = function () {
            /* tslint:disable:no-unused-expression no-string-literal */
            navigator["splashscreen"] && navigator.splashscreen.show();
            /* tslint:enable:no-unused-expression no-string-literal */
        };
        App.prototype.hideSplash = function () {
            /* tslint:disable:no-unused-expression no-string-literal */
            navigator["splashscreen"] && navigator.splashscreen.hide();
            /* tslint:enable:no-unused-expression no-string-literal */
        };
        return App;
    }());
    exports.App = App;
});
/* tslint:disable:no-string-literal */
define("bindings", ["require", "exports", "knockout", "moment", "timeService", "debugsettings", "languageManager"], function (require, exports, ko, moment, timeService, debugsettings_25, languagemanager_28) {
    "use strict";
    function registerBindings() {
        // Binding set loading variable for short amount of time.
        ko.bindingHandlers["loading"] = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var observable;
                var duration;
                if (ko.isObservable(value)) {
                    observable = value;
                    duration = 500;
                }
                else {
                    observable = value.option;
                    duration = (value.duration || 500);
                }
                observable(true);
                setTimeout(function () { return observable(false); }, duration);
            }
        };
        var imageBindingHandler = {
            options: {
                enabled: true
            },
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                if (!imageBindingHandler.options.enabled) {
                    return;
                }
                var canvas = document.createElement("canvas");
                canvas.width = window.devicePixelRatio * $(element).width();
                canvas.height = window.devicePixelRatio * $(element).height();
                canvas.style.width = "100%";
                canvas.style.height = "100%";
                element.appendChild(canvas);
                if (bindingContext["$canvas"] !== undefined) {
                    console.warn("Canvas already defined for the element." + (element.id || ""));
                }
                bindingContext["$canvas"] = canvas;
                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    delete bindingContext["$canvas"];
                });
                var innerBindingContext = bindingContext.extend({ $canvas: canvas });
                ko.applyBindingsToDescendants(innerBindingContext, element);
                return { controlsDescendantBindings: true };
            },
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                if (!imageBindingHandler.options.enabled) {
                    return;
                }
                var value = valueAccessor();
                var imageUrl = ko.unwrap(value);
                if (imageUrl === undefined || imageUrl === null) {
                    return;
                }
                var canvas = bindingContext["$canvas"];
                canvas.width = window.devicePixelRatio * $(element).width();
                canvas.height = window.devicePixelRatio * $(element).height();
                var ctx = canvas.getContext("2d");
                var imageLoader = new Image();
                imageLoader.onload = function () {
                    ctx.drawImage(imageLoader, 0, 0, imageLoader.width, imageLoader.height);
                };
                imageLoader.src = imageUrl;
            }
        };
        ko.bindingHandlers["image"] = imageBindingHandler;
        ko.bindingHandlers["ltext"] = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var parameters = ko.utils.unwrapObservable(valueAccessor());
                if (parameters === null || parameters === undefined) {
                    return "";
                }
                var value;
                if (typeof parameters === "string") {
                    value = languagemanager_28._(parameters);
                }
                else if (typeof parameters === "Function") {
                    value = languagemanager_28._(parameters);
                }
                else {
                    value = languagemanager_28._(parameters.key, parameters.params);
                }
                ko.bindingHandlers.text.update(element, function () { return value; }, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["lattr"] = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var propsObject = ko.utils.unwrapObservable(valueAccessor());
                var val = {};
                for (var propName in propsObject) {
                    if (propsObject.hasOwnProperty(propName)) {
                        var value = languagemanager_28._(propsObject[propName]);
                        val[propName] = value;
                    }
                }
                ko.bindingHandlers.attr.update(element, function () { return val; }, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["lhtml"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                ko.bindingHandlers.html.init(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
            },
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var lockey = ko.utils.unwrapObservable(valueAccessor());
                var value = languagemanager_28._(lockey);
                ko.bindingHandlers.html.update(element, function () { return value; }, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["spinner"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var opts = {
                    lines: 13,
                    length: 4,
                    width: 2,
                    radius: 6,
                    corners: 1,
                    rotate: 0,
                    direction: 1,
                    color: "#fff",
                    speed: 1,
                    trail: 60,
                    shadow: false,
                    hwaccel: false,
                    className: "spinner",
                    zIndex: 2e9,
                    top: "auto",
                    left: "auto" // Left position relative to parent in px
                };
                var spinner = new Spinner(opts).spin(element);
            },
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                ko.bindingHandlers.visible.update(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        var dateBindingHander = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var lockey = ko.utils.unwrapObservable(valueAccessor());
                var value = languagemanager_28._(lockey);
                ko.bindingHandlers.text.update(element, function () {
                    var dateValue = moment(value);
                    if (debugsettings_25.debugSettings.application.useUtcDates) {
                        dateValue = dateValue.utc();
                    }
                    if (timeService.timeDiff !== null) {
                    }
                    return dateValue.format(dateBindingHander.format);
                }, allBindingsAccessor, viewModel, bindingContext);
            },
            format: "D MMM, H:mm"
        };
        ko.bindingHandlers["date"] = dateBindingHander;
        ko.bindingHandlers["selector"] = {
            init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
                var value = valueAccessor();
                ko.bindingHandlers.click.init(element, function () {
                    return function () {
                        var val = value();
                        value.options.forEach(function (item) {
                            item.selected = val === item.value;
                        });
                        app.showSelector(value.caption, value.options, function (item) {
                            value(item.value);
                        });
                    };
                }, allBindings, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["command"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                value = ko.unwrap(value);
                var isFast = false;
                var commandTag = value;
                if (typeof value === "object") {
                    isFast = value.fast || false;
                    commandTag = value.command;
                }
                var wrapperValueAccessor;
                var handler = function (viewMovel, event) {
                    if (typeof commandTag === "string") {
                        app.executeCommand(commandTag);
                    }
                    else if (typeof commandTag === "function") {
                        viewModel = bindingContext["$data"];
                        var functionValue = commandTag;
                        functionValue.apply(viewModel, [event]);
                    }
                    else {
                        console.error("Invalid argument " + commandTag + " passed to the binding");
                    }
                };
                if (platformInfo.hasTouch()) {
                    wrapperValueAccessor = function () {
                        if (isFast) {
                            return {
                                touchstart: handler
                            };
                        }
                        return {
                            tap: handler
                        };
                    };
                }
                else {
                    wrapperValueAccessor = function () {
                        return {
                            click: handler
                        };
                    };
                }
                ko.bindingHandlers.event.init(element, wrapperValueAccessor, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        var numericTextHandler = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = ko.utils.unwrapObservable(valueAccessor());
                var positions = ko.utils.unwrapObservable(allBindingsAccessor().positions) || numericTextHandler.defaultPositions;
                var finalFormatted;
                if (value != null) {
                    var numberValue = parseFloat(value);
                    if (!isNaN(numberValue)) {
                        var formattedValue = numberValue.toFixed(positions);
                        finalFormatted = numericTextHandler.withCommas(formattedValue);
                    }
                    else {
                        finalFormatted = "-";
                    }
                }
                else {
                    finalFormatted = "";
                }
                ko.bindingHandlers.text.update(element, function () { return finalFormatted; }, allBindingsAccessor, viewModel, bindingContext);
            },
            defaultPositions: 2,
            separator: ",",
            withCommas: function (original) {
                original += "";
                var x = original.split(".");
                var x1 = x[0];
                var x2 = x.length > 1 ? "." + x[1] : "";
                var rgx = /(\d+)(\d{3})/;
                while (rgx.test(x1)) {
                    x1 = x1.replace(rgx, "$1" + numericTextHandler.separator + "$2");
                }
                return x1 + x2;
            }
        };
        ko.bindingHandlers["numericText"] = numericTextHandler;
        var currencySymbolBindingHandler = {
            moneySymbol: "$",
            chipsSymbol: "",
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = ko.utils.unwrapObservable(valueAccessor());
                var positions = ko.utils.unwrapObservable(allBindingsAccessor());
                var text;
                if (value === 1) {
                    text = currencySymbolBindingHandler.moneySymbol;
                }
                else {
                    text = currencySymbolBindingHandler.chipsSymbol;
                }
                ko.bindingHandlers.text.update(element, function () { return text; }, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["currencySymbol"] = currencySymbolBindingHandler;
        var swipeHandler = {
            insideUpdate: false,
            insideCallback: false,
            firstTime: true,
            logging: false,
            defaultDuration: 400,
            log: function (message) {
                var params = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    params[_i - 1] = arguments[_i];
                }
                if (swipeHandler.logging) {
                    console.log(message, params);
                }
            },
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var valObservable = valueAccessor();
                var autoFix = valObservable.autoFix === undefined ? true : !!valObservable.autoFix;
                var breakDistance = valObservable.breakDistance === undefined ? null : valObservable.breakDistance;
                if (valObservable.disabled) {
                    element.style.visibility = "visible";
                    return;
                }
                bindingContext.firstTime = true;
                bindingContext.insideUpdate = false;
                bindingContext.insideCallback = false;
                element.onscroll = function () {
                    element.scrollLeft = 0;
                };
                var updateOnSlideChange = valObservable.updateOnSlideChange === undefined
                    ? true
                    : valObservable.updateOnSlideChange;
                function endHandler(index, elem) {
                    swipeHandler.log("Callback: index - ", index, ", value - ", valObservable.index());
                    if (bindingContext.insideUpdate) {
                        swipeHandler.log("swipeHandler.insideUpdate = false");
                        bindingContext.insideUpdate = false;
                        return;
                    }
                    bindingContext.insideCallback = true;
                    swipeHandler.log("swipeHandler.insideCallback = true");
                    valObservable.index(index + 1);
                    setTimeout(function () {
                        bindingContext.insideCallback = false;
                    }, updateOnSlideChange ? 100 : 0);
                }
                var options = {
                    startSlide: valObservable.index() - 1,
                    speed: valObservable.duration || swipeHandler.defaultDuration,
                    slideWidth: ko.unwrap(valObservable.slideWidth || 0),
                    continuous: true,
                    disableScroll: false,
                    stopPropagation: false,
                    breakDistance: breakDistance
                };
                if (updateOnSlideChange) {
                    options.callback = endHandler;
                }
                else {
                    // When using callback instead of transitionEnd
                    // The scrolling animation feels luggish when swiping very fast.
                    // Luggish appears because next translation which already started and continued, was 
                    // stopped during updating valObservable, and thus probably causing the 
                    // animation restart.
                    options.transitionEnd = endHandler;
                }
                bindingContext.$swiper = new Swipe(element, options);
            },
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var valObservable = valueAccessor();
                if (valObservable.disabled) {
                    return;
                }
                swipeHandler.log("Update: value - ", valObservable.index());
                bindingContext.$swiperIndex = valObservable.index();
                if (bindingContext.firstTime) {
                    bindingContext.firstTime = false;
                    return;
                }
                if (bindingContext.insideCallback) {
                    swipeHandler.log("swipeHandler.insideCallback = false");
                    swipeHandler.insideCallback = false;
                    return;
                }
                var swiper = bindingContext.$swiper;
                swipeHandler.log("swipeHandler.insideUpdate = true");
                bindingContext.insideUpdate = true;
                swiper.slide(valObservable.index() - 1, valObservable.duration || swipeHandler.defaultDuration);
            }
        };
        ko.bindingHandlers["swipe"] = swipeHandler;
        ko.bindingHandlers["swipeForeach"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                swipeHandler.log("swipeForeach init ", valueAccessor()(), viewModel, bindingContext.$swiper);
                ko.bindingHandlers.foreach.init(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                return { controlsDescendantBindings: true };
            },
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                swipeHandler.log("swipeForeach update ", valueAccessor()(), viewModel, bindingContext.$swiper);
                ko.bindingHandlers.foreach.update(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                swipeHandler.log("Preious slider position ", bindingContext.$swiperIndex);
                if (bindingContext.$swiper) {
                    bindingContext.$swiper.kill();
                    bindingContext.$swiper.setup();
                    bindingContext.$swiper.attachEvents();
                    bindingContext.$swiper.slide(bindingContext.$swiperIndex - 1, 0);
                }
            }
        };
        ko.virtualElements.allowedBindings["swipeForeach"] = true;
        var scrollHandler = {
            logging: false,
            defaultDuration: 400,
            log: function (message) {
                var params = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    params[_i - 1] = arguments[_i];
                }
                if (scrollHandler.logging) {
                    console.log(message, params);
                }
            },
            setup: function (element, options) {
                var scroller;
                var scrollerOptions = {
                    hScrollbar: false,
                    vScrollbar: false,
                    lockDirection: true,
                    deceleration: 0.0001,
                    bounceEasing: "quadratic"
                };
                setTimeout(function () {
                    scrollHandler.log("Initializing scroller for the first time");
                    scroller = new IScroll(element, scrollerOptions);
                    if ($(element).css("box-sizing") === "border-box") {
                        var top_1 = parseInt($(".registration .popup-container").css("padding-top"), 10) || 0;
                        var bottom = parseInt($(".registration .popup-container").css("padding-bottom"), 10) || 0;
                        var adjustment = top_1 + bottom;
                        console.log(scroller.maxScrollY, top_1, bottom);
                        if (scroller.maxScrollY > 0) {
                            scroller.maxScrollY = scroller.maxScrollY + adjustment;
                        }
                        else {
                            scroller.maxScrollY = scroller.maxScrollY - adjustment;
                        }
                    }
                    ko.utils.domData.set(element, "iscroll", scroller);
                }, 500);
                /* We have non zero timeout here above to prevent scroll area in the lobby
                from freezing for the first time when it is opened.*/
                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
                    // This will be called when the element is removed by Knockout or
                    // if some other part of your code calls ko.removeNode(element)
                    scrollHandler.log("Disposing scroller");
                    var iscroller = ko.utils.domData.get(element, "iscroll");
                    if (iscroller != null) {
                        iscroller.destroy();
                    }
                });
                if (options.refreshTrigger != null) {
                    var left = ko.computed(function () {
                        return options.refreshTrigger();
                    }).extend({ rateLimit: 10 }).subscribe(function (value) {
                        setTimeout(function () {
                            scrollHandler.log("Refresh trigger");
                            var iscroller = ko.utils.domData.get(element, "iscroll");
                            if (iscroller != null) {
                                iscroller.refresh();
                            }
                        }, 0);
                    });
                }
            },
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var valObservable = valueAccessor();
                var suppressed = valObservable.suppress || false;
                if (!suppressed) {
                    scrollHandler.setup(element, valObservable);
                }
            }
        };
        ko.bindingHandlers["scroll"] = scrollHandler;
        ko.bindingHandlers["bet"] = {
            update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = ko.unwrap(valueAccessor());
                if (value === null || value === undefined || value === 0) {
                    element.innerHTML = "";
                }
                else {
                    while (element.firstChild) {
                        element.removeChild(element.firstChild);
                    }
                    var chipItem = new ChipItem(1);
                    var stackInfo = chipItem.getData(value);
                    var container = document.createElement("div");
                    if (stackInfo.length === 1) {
                        container.setAttribute("class", "chip-container stack1");
                    }
                    else {
                        container.setAttribute("class", "chip-container stack2");
                    }
                    for (var i = 0; i < stackInfo.length; i++) {
                        var chipsInfo = stackInfo[i];
                        var stack = document.createElement("div");
                        stack.setAttribute("class", "chip chip-stack");
                        for (var j = 0; j < chipsInfo.length; j++) {
                            var chip = document.createElement("div");
                            chip.setAttribute("class", "chip chip" + chipsInfo[j].toString());
                            stack.appendChild(chip);
                        }
                        container.appendChild(stack);
                    }
                    element.appendChild(container);
                    var label = document.createElement("div");
                    label.setAttribute("class", "label");
                    label.innerText = value.toString();
                    element.appendChild(label);
                }
            }
        };
        ko.bindingHandlers["forceFocus"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var handler = function (data, event) {
                    setTimeout(function () {
                        var element = event.currentTarget;
                        element.focus();
                    }, 0);
                };
                ko.bindingHandlers.event.init(element, function () { return { touchstart: handler }; }, allBindingsAccessor, viewModel, bindingContext);
            }
        };
        ko.bindingHandlers["slideup"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var functionValue = value;
                var hammer = Hammer(element, { dragMinDistance: 3 });
                hammer.on("dragup", function (event) {
                    functionValue.apply(viewModel);
                });
            }
        };
        ko.bindingHandlers["slidedown"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var functionValue = value;
                var hammer = Hammer(element, { dragMinDistance: 3 });
                hammer.on("dragdown", function (event) {
                    functionValue.apply(viewModel);
                });
            }
        };
        ko.bindingHandlers["slideleft"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var functionValue = value;
                var hammer = Hammer(element, { threshold: 10 });
                hammer.on("swipeleft", function (event) {
                    functionValue.apply(viewModel);
                });
            }
        };
        ko.bindingHandlers["slideright"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var functionValue = value;
                var hammer = Hammer(element, { threshold: 10 });
                hammer.on("swiperight", function (event) {
                    functionValue.apply(viewModel);
                });
            }
        };
        ko.bindingHandlers["handle"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var functionValue = value;
                var hammer = Hammer(element, { drag_block_horizontal: true, drag_lock_to_axis: true });
                var parent = element.parentElement;
                hammer.on("drag", function (event) {
                    var touch = event.gesture.touches[0];
                    var offset = 50;
                    if (PageBlock.useDoubleView) {
                        offset = $(element).width() / 4;
                    }
                    var relativePosition = touch.pageX - $(parent).offset().left - offset;
                    var currentRelativePosition = functionValue.apply(viewModel);
                    if (Math.abs(currentRelativePosition - relativePosition) < $(".slider-handle").width()) {
                        viewModel.setPosition(relativePosition);
                    }
                });
            }
        };
        ko.bindingHandlers["fadeTable"] = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
                var value = valueAccessor();
                var duration = value.duration || 1000;
                var subscription = value.item.subscribe(function (val) {
                    if (bindingContext.$timeout != null) {
                        return;
                    }
                    var continuation = function () {
                        value.target(false);
                        bindingContext.$timeout = null;
                    };
                    value.target(true);
                    bindingContext.$timeout = setTimeout(continuation, duration);
                });
                ko.utils.domNodeDisposal.addDisposeCallback(element, function () { return subscription.dispose(); });
            }
        };
    }
    exports.registerBindings = registerBindings;
});
/* tslint:disable:no-string-literal */
define("extenders", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    function registerExtenders() {
        ko.extenders["options"] = function (target, option) {
            target.options = option.items;
            target.caption = option.caption;
            target.currentValue = ko.computed(function () {
                var value = target();
                var selectedItem = option.items.filter(function (item) {
                    return item.value === value;
                });
                if (selectedItem.length === 0) {
                    return "";
                }
                return selectedItem[0].text;
            });
            return target;
        };
    }
    exports.registerExtenders = registerExtenders;
});
define("validationConfiguration", ["require", "exports", "knockout.validation", "knockout"], function (require, exports, kov, ko) {
    "use strict";
    function updateDefaultMessages() {
        kov;
        ko.validation.rules.required.message = "Это поле обязательно";
        ko.validation.rules.email.message = "Укажите правильный email";
        ko.validation.rules.equal.message = "Значения должны совпадать";
        ko.validation.rules.minLength.message = "Логин должен быть не менее {0} символов";
        ko.validation.rules.maxLength.message = "Логин должен быть не более {0} символов";
    }
    exports.updateDefaultMessages = updateDefaultMessages;
});
define("components/registration", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var trivialViewModelFactory = function (params, componentInfo) {
        return params.data;
    };
    var tableIconViewModelFactory = function (params, componentInfo) {
        var tableData = params.data;
        return {
            displayTakenSeats: true,
            hasLoadingIndicator: false,
            MaxPlayers: tableData.MaxPlayers,
            SeatMask: tableData.SeatMask,
            loading: ko.observable(false)
        };
    };
    function getTemplateDefinition(name) {
        var useRequire = window["PokerComponents"] === undefined;
        if (useRequire) {
            return { require: "text!app/components/" + name };
        }
        return window["PokerComponents"][name];
    }
    function registerComponents() {
        /**
        * Tournament lobby sub-components
        */
        ko.components.register("tournament-information", {
            template: getTemplateDefinition("lobby/tournament/information.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("tournament-tables", {
            template: getTemplateDefinition("lobby/tournament/tables.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("tournament-players", {
            template: getTemplateDefinition("lobby/tournament/players.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("tournament-prizes", {
            template: getTemplateDefinition("lobby/tournament/prizes.html"),
            viewModel: { require: "app/components/lobby/tournament/prizes" }
        });
        ko.components.register("checkbox", {
            template: getTemplateDefinition("shared/checkbox/checkbox.html"),
            viewModel: { require: "app/components/shared/checkbox/checkbox" }
        });
        ko.components.register("tournament-blinds", {
            template: getTemplateDefinition("lobby/tournament/blinds.html")
        });
        ko.components.register("rotate-phone-block", {
            template: getTemplateDefinition("rotate-phone-block/rotate-phone-block.html")
        });
        /**
        * Lobby components
        */
        ko.components.register("tables-list", {
            template: getTemplateDefinition("lobby/tables/list.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("table-list-item", {
            template: getTemplateDefinition("lobby/tables/list-item.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("table-icon", {
            template: getTemplateDefinition("lobby/tables/table-icon.html"),
            viewModel: { createViewModel: tableIconViewModelFactory }
        });
        ko.components.register("tournaments-list", {
            template: getTemplateDefinition("lobby/tournaments/list.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("tournament-list-item", {
            template: getTemplateDefinition("lobby/tournaments/list-item.html"),
            viewModel: { createViewModel: trivialViewModelFactory }
        });
        ko.components.register("cash-options", {
            template: getTemplateDefinition("lobby/filter/cash-options.html"),
            viewModel: { require: "app/components/lobby/filter/options" }
        });
        ko.components.register("tournament-options", {
            template: getTemplateDefinition("lobby/filter/tournament-options.html"),
            viewModel: { require: "app/components/lobby/filter/options" }
        });
        ko.components.register("sng-options", {
            template: getTemplateDefinition("lobby/filter/sng-options.html"),
            viewModel: { require: "app/components/lobby/filter/options" }
        });
        /**
        * Tabbar
        */
        ko.components.register("tabbar", {
            template: getTemplateDefinition("tabbar/tabbar.html"),
            viewModel: { createViewModel: tableIconViewModelFactory }
        });
        /**
        * Time block
        */
        ko.components.register("timeblock", {
            template: getTemplateDefinition("timeblock/timeblock.html"),
            viewModel: { require: "app/components/timeblock/timeblock" }
        });
        /**
        * Game type selector
        */
        ko.components.register("game-type-selector", {
            template: getTemplateDefinition("game-type-selector/game-type-selector.html"),
            viewModel: { require: "app/components/game-type-selector/gametypeselector" }
        });
        /**
        * Game table action block
        */
        ko.components.register("table-action-block", {
            template: getTemplateDefinition("table/actionBlock/actionBlock.html"),
            viewModel: { require: "app/components/table/actionBlock/actionBlock" }
        });
        ko.components.register("table-menu", {
            template: getTemplateDefinition("table/menu/menu.html"),
            viewModel: { require: "app/components/table/menu/menu" }
        });
    }
    exports.registerComponents = registerComponents;
});
define("appInit", ["require", "exports", "knockout", "app", "bindings", "extenders", "validationConfiguration", "components/registration", "languageManager", "table/tableview"], function (require, exports, ko, app_11, bindings_1, extenders_1, validationConfiguration_1, registration_1, languagemanager_29, tableView_1) {
    "use strict";
    function isStandaloneSupported() {
        return ("standalone" in window.navigator);
    }
    function isRunningStandalone() {
        return (window.matchMedia("(display-mode: standalone)").matches
            || ("standalone" in window.navigator && window.navigator["standalone"] === true));
    }
    function bootstrap() {
        if (typeof host === "undefined") {
            console.error("File environment.js is missing");
            return;
        }
        var baseUrl = host, apiHost = baseUrl + "/api/rest/v1", app = new app_11.App();
        $.connection.hub.url = baseUrl + "/signalr";
        $.connection.hub.logging = false;
        OnlinePoker.Commanding.API.logging = false;
        // GameActionsQueue.waitDisabled = true;
        var numericTextHandler = ko.bindingHandlers["numericText"];
        numericTextHandler.defaultPositions = 0;
        numericTextHandler.separator = " ";
        window["moment"].locale("ru");
        // This function prevents the rotation from 
        function shouldRotateToOrientation(interfaceOrientation) {
            return app.shouldRotateToOrientation(interfaceOrientation);
        }
        exposeCardsConstants();
        window["shouldRotateToOrientation"] = shouldRotateToOrientation;
        window["app"] = app;
        window["authToken"] = null;
        window["apiHost"] = apiHost;
        window["baseUrl"] = baseUrl;
        window["_"] = languagemanager_29._;
        window.onerror = function (message, url, lineNumber, colno, error) {
            console.log("Error: " + message + " in " + url + " at line " + lineNumber);
            if (error != null) {
                console.log(error);
            }
            window["appInsights"].trackException(error, "window.onerror");
        };
        app.bindEvents();
        if (window["cordova"] === undefined) {
            app.onDeviceReady();
            if (isStandaloneSupported() && !isRunningStandalone()) {
                // Show splash screen.
                $("#install_prompt").show();
            }
        }
    }
    window["ko"] = ko;
    window["TableView"] = tableView_1.TableView;
    bindings_1.registerBindings();
    extenders_1.registerExtenders();
    registration_1.registerComponents();
    validationConfiguration_1.updateDefaultMessages();
    bootstrap();
});
/// <reference types="knockout" />
var SliderItem = (function () {
    function SliderItem() {
    }
    return SliderItem;
}());
var Slider = (function () {
    function Slider() {
        this.enabled = ko.observable(true);
        var self = this;
        this.items = [];
        this.currentIndex = ko.observable(0);
        this.currentIndex1 = ko.computed({
            read: function () {
                return self.currentIndex() + 1;
            },
            write: function (value) {
                if (this.enabled()) {
                    self.currentIndex(value - 1);
                }
            },
            owner: this
        });
        this.current = ko.computed(function () {
            var index = this.currentIndex();
            if (index < 0 || index >= this.items.length) {
                return null;
            }
            return self.items[index];
        }, this);
        this.next = ko.computed(function () {
            var index = this.currentIndex();
            if (index < 0 || index >= this.items.length) {
                return null;
            }
            index = index + 1;
            if (index >= this.items.length) {
                index = 0;
            }
            return self.items[index];
        }, this);
        this.prev = ko.computed(function () {
            var index = this.currentIndex();
            if (index < 0 || index >= this.items.length) {
                return null;
            }
            index = index - 1;
            if (index < 0) {
                index = this.items.length - 1;
            }
            return self.items[index];
        }, this);
    }
    Slider.prototype.addOption = function (caption, childTemplate, callback) {
        var item = new SliderItem();
        item.caption = caption;
        item.childTemplate = childTemplate;
        item.action = callback;
        this.items.push(item);
        this.currentIndex.valueHasMutated();
        return item;
    };
    Slider.prototype.selectNext = function () {
        if (!this.enabled()) {
            return;
        }
        var index = this.currentIndex();
        index = index + 1;
        if (index >= this.items.length) {
            index = 0;
        }
        this.currentIndex(index);
    };
    Slider.prototype.selectPrev = function () {
        if (!this.enabled()) {
            return;
        }
        var index = this.currentIndex();
        index = index - 1;
        if (index < 0) {
            index = this.items.length - 1;
        }
        this.currentIndex(index);
    };
    return Slider;
}());
/// <reference types="knockout" />
/// <reference path="../../slider.ts" />
define("components/game-type-selector/gametypeselector", ["require", "exports", "knockout", "languageManager"], function (require, exports, ko, languagemanager_30) {
    "use strict";
    var TimeBlockComponent = (function () {
        function TimeBlockComponent(params) {
            this.slider = new Slider();
            this.slider.addOption(languagemanager_30._("lobby.cashGames"), "cash", null);
            this.slider.addOption(languagemanager_30._("lobby.tournaments"), "tournaments", null);
            this.slider.addOption(languagemanager_30._("lobby.sitAndGo"), "sng", null);
            if (params && params.initial) {
                this.slider.currentIndex(ko.unwrap(params.initial));
            }
            if (params && params.selected) {
                this.slider.currentIndex.subscribe(function (newValue) {
                    params.selected(newValue);
                });
            }
        }
        return TimeBlockComponent;
    }());
    return TimeBlockComponent;
});
/// <reference types="knockout" />
define("components/lobby/filter/options", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var FilterOptionsComponent = (function () {
        function FilterOptionsComponent(params) {
            this.displayCurrency = true;
            this.options = params.options;
            if (params && params.displayCurrency !== undefined && params.displayCurrency !== null) {
                this.displayCurrency = ko.unwrap(params.displayCurrency);
            }
        }
        return FilterOptionsComponent;
    }());
    return FilterOptionsComponent;
});
define("components/lobby/tournament/prizes", ["require", "exports", "knockout", "metadatamanager"], function (require, exports, ko, metadataManager) {
    "use strict";
    ;
    var TournamentPrizeInformationComponent = (function () {
        function TournamentPrizeInformationComponent(params) {
            var self = this;
            this.data = params.data;
            this.totalPrize = ko.computed(function () {
                var tdata = self.data();
                if (tdata == null) {
                    return null;
                }
                return tdata.PrizeAmount + (tdata.CollectedPrizeAmount || 0);
            }, this);
            this.structure = ko.computed(function () {
                var data = self.data();
                if (data == null) {
                    return [];
                }
                var currentPlayers = data.JoinedPlayers;
                var prizeStructure = metadataManager.prizes[data.WellKnownPrizeStructure];
                var sortedPrizes = prizeStructure.sort(function (a, b) {
                    return a.MaxPlayer > b.MaxPlayer
                        ? 1
                        : (a.MaxPlayer < b.MaxPlayer ? -1 : 0);
                });
                var filteredPrizes = sortedPrizes.filter(function (a) {
                    return a.MaxPlayer >= currentPlayers;
                });
                var currentPrize;
                if (filteredPrizes.length === 0) {
                    currentPrize = sortedPrizes[0];
                }
                else {
                    currentPrize = filteredPrizes[0];
                }
                var i = 1;
                var result = [];
                var totalPrize = self.totalPrize();
                currentPrize.PrizeLevel.forEach(function (item, index) {
                    result.push({
                        place: index + 1,
                        amount: totalPrize * item / 100,
                        percent: item
                    });
                });
                return result;
            }, this);
        }
        return TournamentPrizeInformationComponent;
    }());
    return TournamentPrizeInformationComponent;
});
/// <reference types="knockout" />
define("components/shared/checkbox/checkbox", ["require", "exports", "knockout"], function (require, exports, ko) {
    "use strict";
    var Checkbox = (function () {
        function Checkbox(params, componentInfo) {
            var checked = params.checked;
            if (typeof checked === "boolean") {
                this.checked = ko.observable(checked);
            }
            else {
                this.checked = checked;
            }
            var label = params.label;
            if (typeof label === "string") {
                this.label = ko.observable(label);
            }
            else {
                this.label = label;
            }
            var right = params.right;
            if (typeof right === "boolean") {
                this.right = ko.observable(right);
            }
            else {
                if (right) {
                    this.right = right;
                }
                else {
                    this.right = ko.observable(false);
                }
            }
            var color = params.color;
            if (typeof color === "string") {
                this.color = ko.observable(color);
            }
            else {
                if (color) {
                    this.color = color;
                }
                else {
                    this.color = ko.observable("unknown");
                }
            }
        }
        Checkbox.prototype.doClick = function (e) {
            this.checked(!this.checked());
            e.preventDefault();
            e.stopPropagation();
            return false;
        };
        return Checkbox;
    }());
    return Checkbox;
});
/// <reference types="knockout" />
define("components/tabbar/tabbar", ["require", "exports", "knockout", "debugsettings"], function (require, exports, ko, debugsettings_26) {
    "use strict";
    var TabBarItem = (function () {
        function TabBarItem() {
        }
        TabBarItem.prototype.handler = function () {
            if (this.action != null) {
                this.action.call(this);
            }
        };
        return TabBarItem;
    }());
    var TabBar = (function () {
        function TabBar() {
            this.items = ko.observableArray([]);
            this.visible = ko.observable(true);
        }
        TabBar.prototype.addItem = function (name, caption, cssClass, action) {
            var item = new TabBarItem();
            item.name = name;
            item.caption = caption;
            item.cssClass = cssClass;
            item.action = action;
            item.selected = ko.observable(false);
            item.disabled = ko.observable(false);
            item.notice = ko.observable(false);
            this.items.push(item);
        };
        TabBar.prototype.isSelected = function (name) {
            var result = false;
            this.items().forEach(function (item) {
                if (item.name === name) {
                    result = item.selected();
                }
            });
            return result;
        };
        TabBar.prototype.select = function (name, enable) {
            this.log("Tabbar item '" + name + "' " + (enable ? "selected" : "deselected"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.selected(enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.enable = function (name, enable) {
            this.log("Tabbar item '" + name + "' " + (enable ? "enabled" : "disabled"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.disabled(!enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.notice = function (name, enable) {
            this.log("Notice for tabbar item '" + name + "' " + (enable ? "set" : "unset"));
            this.items.valueWillMutate();
            this.items().forEach(function (item) {
                if (item.name === name) {
                    item.notice(enable);
                }
            });
            this.items.valueHasMutated();
        };
        TabBar.prototype.log = function (message) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            if (debugsettings_26.debugSettings.application.debugTabbar) {
                var traceFunction = console.trace;
                traceFunction.bind(console, message, params)();
            }
        };
        return TabBar;
    }());
    return TabBar;
});
/// <reference types="knockout" />
define("components/table/actionBlock/actionBlock", ["require", "exports"], function (require, exports) {
    "use strict";
    var ActionBlockComponent = (function () {
        function ActionBlockComponent(params) {
            this.actionBlock = params.data;
        }
        return ActionBlockComponent;
    }());
    return ActionBlockComponent;
});
/// <reference types="knockout" />
define("components/table/menu/menu", ["require", "exports"], function (require, exports) {
    "use strict";
    var TableMenuComponent = (function () {
        function TableMenuComponent(params) {
            this.page = params.data;
        }
        return TableMenuComponent;
    }());
    return TableMenuComponent;
});
/// <reference types="knockout" />
define("components/timeblock/timeblock", ["require", "exports", "knockout", "timeService"], function (require, exports, ko, timeService) {
    "use strict";
    var TimeBlockComponent = (function () {
        function TimeBlockComponent(params) {
            this.currentTime = ko.computed(function () {
                return timeService.currentTime();
            }, this);
        }
        return TimeBlockComponent;
    }());
    return TimeBlockComponent;
});
require(["appInit"], function () {
    if (window["testInit"] !== undefined) {
        testInit();
    }
});
/// <reference types="knockout" />
var Selector = (function () {
    function Selector() {
        this.caption = ko.observable(null);
        this.items = ko.observableArray([]);
    }
    Selector.prototype.setParams = function (caption, items, success, cancel) {
        this.caption(caption);
        this.items.valueWillMutate();
        this.items([]);
        this.items(items);
        this.items.valueHasMutated();
        this.selected = success;
        this.cancelled = cancel;
    };
    Selector.prototype.select = function (item) {
        this.selected(item);
    };
    Selector.prototype.back = function () {
        this.cancelled();
    };
    return Selector;
}());
var DuplicateFinder = (function () {
    function DuplicateFinder() {
        this.dataEvents = [];
    }
    DuplicateFinder.prototype.registerEvent = function (data) {
        this.dataEvents.push(data);
        while (this.dataEvents.length > 8) {
            this.dataEvents.shift();
        }
    };
    DuplicateFinder.prototype.clear = function () {
        this.dataEvents = [];
    };
    DuplicateFinder.prototype.erase = function (callbackfn) {
        this.dataEvents = this.dataEvents.filter(callbackfn);
    };
    DuplicateFinder.prototype.validateDuplicateEvents = function () {
        var target = this.dataEvents.slice(0);
        var duplicatesCount = 0;
        for (var i = 0; i < target.length; i++) {
            var current = target[i];
            for (var j = i + 1; j < target.length; j++) {
                var test = target[j];
                if (test.length !== current.length) {
                    continue;
                }
                var notFound = false;
                for (var k = 0; k < current.length; k++) {
                    if (current[k] !== test[k]) {
                        notFound = true;
                        break;
                    }
                }
                if (notFound) {
                    continue;
                }
                duplicatesCount++;
                target = target.splice(j, 1);
                j--;
            }
        }
        if (duplicatesCount > 0) {
            return true;
        }
        return false;
    };
    DuplicateFinder.prototype.printDebug = function () {
        console.log("Duplicate events detected.");
        this.dataEvents.forEach(function (_) {
            console.log(JSON.stringify(_));
        });
    };
    return DuplicateFinder;
}());
var AnimationSettings = (function () {
    function AnimationSettings() {
        this.dealCardsTime = 300;
        this.finishGamePrePause = 100;
        this.cleanupTableTimeout = 2000;
        this.foldAnimationTimeout = 150;
        this.betAnimationTimeout = 100;
        this.movingMoneyToPotPrePause = 100;
        this.movingMoneyToPotAnimationTimeout = 100;
        this.showCardsTimeout = 100;
    }
    AnimationSettings.getSettings = function () {
        if (AnimationSettings.platform === "android") {
            return AnimationSettings.androidSettings();
        }
        if (AnimationSettings.platform === "tablet") {
            return AnimationSettings.tabletSettings();
        }
        return AnimationSettings.defaultSettings();
    };
    AnimationSettings.defaultSettings = function () {
        return new AnimationSettings();
    };
    AnimationSettings.androidSettings = function () {
        var settings = new AnimationSettings();
        return settings;
    };
    AnimationSettings.tabletSettings = function () {
        var settings = new AnimationSettings();
        settings.dealCardsTime = 500;
        return settings;
    };
    AnimationSettings.platform = "default";
    return AnimationSettings;
}());
var allBacks = "/v4=";
var allNone = "//8=";
var allBacksClasses = ["cards back", "cards back"];
var allNoneClasses = ["cards back", "cards back"];
function exposeCardsConstants() {
    window["allBacks"] = allBacks;
    window["allNone"] = allNone;
    window["allBacksClasses"] = allBacksClasses;
    window["allNoneClasses"] = allNoneClasses;
}
function cardValue(card) {
    /// <signature>
    ///   <summary>Get card string</summary>
    ///   <param name="card" type="Number">A byte value of the card.</param>
    /// </signature>
    if (card === 254) {
        return "cards back";
    }
    if (card === 255) {
        return "cards none";
    }
    var suitId = Math.floor(card / 13);
    var valueId = card % 13;
    var value;
    switch (valueId) {
        case 0:
            value = "2";
            break;
        case 1:
            value = "3";
            break;
        case 2:
            value = "4";
            break;
        case 3:
            value = "5";
            break;
        case 4:
            value = "6";
            break;
        case 5:
            value = "7";
            break;
        case 6:
            value = "8";
            break;
        case 7:
            value = "9";
            break;
        case 8:
            value = "10";
            break;
        case 9:
            value = "J";
            break;
        case 10:
            value = "Q";
            break;
        case 11:
            value = "K";
            break;
        case 12:
            value = "A";
            break;
        default:
            break;
    }
    var suit;
    var color;
    switch (suitId) {
        case 0:
            color = "blue";
            suit = "\u2663";
            suit = "clubs";
            break;
        case 1:
            color = "green";
            suit = "\u2666";
            suit = "diamonds";
            break;
        case 2:
            color = "red";
            suit = "\u2665";
            suit = "hearts";
            break;
        case 3:
            color = "black";
            suit = "\u2660";
            suit = "spades";
            break;
        default:
            color = "grey";
            suit = "unk suit";
            break;
    }
    return "cards " + suit + " c" + value;
    // return "<span class="cards " + value + suit + ""></span>";
}
/**
* Decode cards array from the base64 representation.
*/
function decodeCardsArray(cardsData) {
    /// <signature>
    ///   <summary>Get card string</summary>
    ///   <param name="cards" type="Array">Array of the cards.</param>
    /// </signature>
    if (cardsData === null || cardsData === null) {
        return null;
    }
    if (typeof cardsData === "object") {
        console.log(cardsData[0]);
        console.log(cardsData[1]);
        cardsData = "//8=";
    }
    if (cardsData === allBacks) {
        return [254, 254];
    }
    if (cardsData === "//8=") {
        return [255, 255];
    }
    var cardsString = window.atob(cardsData);
    var cards = [];
    for (var i = 0; i < cardsString.length; i++) {
        cards.push(cardsString.charCodeAt(i));
    }
    return cards;
}
function convertToCards(data) {
    if (data == null) {
        return null;
    }
    return data.map(function (n) {
        return cardValue(n);
    });
}
function cardsArray(cardsData) {
    /// <signature>
    ///   <summary>Get card string</summary>
    ///   <param name="cards" type="Array">Array of the cards.</param>
    /// </signature>
    if (cardsData === null) {
        return ["cards none", "cards none"];
    }
    if (typeof cardsData === "object") {
        console.log(cardsData[0]);
        console.log(cardsData[1]);
        cardsData = "//8=";
    }
    if (cardsData === allBacks) {
        return ["cards back", "cards back"];
    }
    if (cardsData === allNone) {
        return ["cards none", "cards none"];
    }
    var cardsString = window.atob(cardsData);
    var cards = new Array();
    for (var i = 0; i < cardsString.length; i++) {
        cards.push(cardValue(cardsString.charCodeAt(i)));
    }
    return cards;
}
var ChipItem = (function () {
    function ChipItem(base) {
        this.maxStackCount = 5;
        this.baseAmount = ko.observable(base);
    }
    ChipItem.prototype.getData = function (amount) {
        var self = this;
        var list = [1, 5, 10, 50, 100, 500].map(function (item) { return (item * self.baseAmount()); });
        var stack = this.calculateStackSimple(amount, list);
        return this.transform(stack);
    };
    ChipItem.prototype.transform = function (stack) {
        var result = [];
        for (var i = 0; i < stack.length; i++) {
            var item = [];
            for (var j = 0; j < stack[i].amount; j++) {
                item.push(stack[i].type);
            }
            result.push(item);
        }
        return result;
    };
    ChipItem.prototype.calculateStack = function (amount, chipAmounts) {
        var stack = this.calculateStackInternal(amount, chipAmounts);
        if (stack.length > this.maxStackCount) {
            stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
            if (stack.length > this.maxStackCount) {
                stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
                if (stack.length > this.maxStackCount) {
                    stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
                    if (stack.length > this.maxStackCount) {
                        stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
                        if (stack.length > this.maxStackCount) {
                            stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
                            if (stack.length > this.maxStackCount) {
                                stack = this.calculateStackInternal(amount, chipAmounts.slice(0, stack.length - 1));
                                if (stack.length > this.maxStackCount) {
                                    throw new Error("Could not divide stack for amount " + amount);
                                }
                            }
                        }
                    }
                }
            }
        }
        return stack;
    };
    ChipItem.prototype.calculateStackSimple = function (amount, chipAmounts) {
        if (amount === 0) {
            return [];
        }
        var item1 = this.calculateStackSimpleInternal(amount, chipAmounts, false);
        if (item1.difference > 0) {
            var item2 = this.calculateStackSimpleInternal(item1.difference, chipAmounts, true);
            if (item2.count === 0) {
                return [{
                        amount: Math.max(item1.count, 1),
                        type: item1.index
                    }];
            }
            if (item1.count === 0) {
                return [{
                        amount: Math.max(item2.count, 1),
                        type: item2.index
                    }];
            }
            return [{
                    amount: Math.max(item1.count, 1),
                    type: item1.index
                }, {
                    amount: Math.max(item2.count, 1),
                    type: item2.index
                }];
        }
        return [{
                amount: Math.max(item1.count, 1),
                type: item1.index
            }];
    };
    ChipItem.prototype.calculateStackSimpleInternal = function (amount, chipAmounts, allowOveradd) {
        var self = this;
        var minimumAmount = chipAmounts[0];
        var intermediate = chipAmounts.filter(function (ca) { return amount >= ca; });
        var upperBound;
        if (intermediate.length === 0) {
            upperBound = minimumAmount;
        }
        else {
            upperBound = intermediate.reduce(function (prev, current) { return Math.max(prev, current); }, Number.MIN_VALUE);
        }
        if (allowOveradd) {
            intermediate = chipAmounts.filter(function (ca) { return amount <= ca; });
            if (intermediate.length === 0) {
                upperBound = minimumAmount;
            }
            else {
                upperBound = intermediate.reduce(function (prev, current) { return Math.min(prev, current); }, Number.MAX_VALUE);
            }
        }
        var effectiveChipsAmount = chipAmounts.filter(function (ca) { return ca <= upperBound; });
        var target;
        if (allowOveradd) {
            target = effectiveChipsAmount.map(function (item) {
                return {
                    amount: item,
                    chipsAmount: Math.min(Math.floor((amount + item - 1) / item), self.maxStackCount),
                    difference: 0
                };
            });
            target = target.concat(effectiveChipsAmount.map(function (item) {
                return {
                    amount: item,
                    chipsAmount: Math.min(Math.floor(amount / item), self.maxStackCount),
                    difference: 0
                };
            }));
        }
        else {
            target = effectiveChipsAmount.map(function (item) {
                return {
                    amount: item,
                    chipsAmount: Math.min(Math.floor(amount / item), self.maxStackCount),
                    difference: 0
                };
            });
        }
        target = target.filter(function (item) { return item.chipsAmount <= self.maxStackCount; })
            .map(function (item) {
            return {
                amount: item.amount,
                chipsAmount: item.chipsAmount,
                difference: amount - (item.amount * item.chipsAmount)
            };
        });
        var difference = target.map(function (item) { return Math.abs(item.difference); }).reduce(function (prev, curr) { return Math.min(prev, curr); }, Number.MAX_VALUE);
        var minimalSlice = target.filter(function (item) { return Math.abs(item.difference) === difference; })[0];
        return {
            difference: minimalSlice.difference,
            count: minimalSlice.chipsAmount,
            index: chipAmounts.filter(function (item) { return item <= minimalSlice.amount; }).length
        };
    };
    ChipItem.prototype.calculateStackInternal = function (amount, chipAmounts) {
        var result = [];
        var amountLeft = amount;
        for (var i = 0; i < chipAmounts.length; i++) {
            var chipAmount = chipAmounts[i];
            var chipsQty = Math.floor(amountLeft / chipAmount);
            if (chipsQty === 0) {
                continue;
            }
            result.push({ type: i + 1, amount: chipsQty });
            amountLeft -= chipsQty * chipAmount;
        }
        if (amountLeft > 0 && result.length === 0) {
            result.push({ type: 1, amount: 1 });
        }
        return result;
    };
    return ChipItem;
}());
// Poker Hand Evaluator by Andrey Kurdyumov ©2013
// v1.0.0
/* tslint:disable:no-bitwise */
var HoldemHand;
(function (HoldemHand) {
    (function (HandParseResultStatus) {
        HandParseResultStatus[HandParseResultStatus["Ok"] = 0] = "Ok";
        HandParseResultStatus[HandParseResultStatus["InvalidHand"] = 1] = "InvalidHand";
        HandParseResultStatus[HandParseResultStatus["CardsMissing"] = 2] = "CardsMissing";
        HandParseResultStatus[HandParseResultStatus["SuitsMissing"] = 3] = "SuitsMissing";
        HandParseResultStatus[HandParseResultStatus["AllCardsShouldHaveOneSuit"] = 4] = "AllCardsShouldHaveOneSuit";
        HandParseResultStatus[HandParseResultStatus["InsufficientCardsAndDuplicates"] = 5] = "InsufficientCardsAndDuplicates";
        HandParseResultStatus[HandParseResultStatus["InsufficientCards"] = 6] = "InsufficientCards";
        HandParseResultStatus[HandParseResultStatus["DuplicatesDetected"] = 7] = "DuplicatesDetected";
    })(HoldemHand.HandParseResultStatus || (HoldemHand.HandParseResultStatus = {}));
    var HandParseResultStatus = HoldemHand.HandParseResultStatus;
    /// <summary>Human readable names of the hand types.</summary>
    HoldemHand.handTypeNames = ["4 of a Kind", "Straight Flush", "Straight", "Flush", "High Card",
        "1 Pair", "2 Pair", "Royal Flush", "3 of a Kind", "Full House", "-Invalid-"];
    /// <summary>Strength of each hand type.</summary>
    /// <remarks>
    /// Due to nature of the algorithm, the hand types not 
    /// have order corresponding to the strength or cards combinations.
    /// </remarks>
    HoldemHand.handTypeRanks = [8, 9, 5, 6, 1, 2, 3, 10, 4, 7, 0];
    function getHandType(hand) {
        /// <summary>Gets type of hand for given hand representation.</summary>
        /// <param name="hand" type="HandRepresentation">5 cards hand for which current combination should be calculated.</param>
        /// <returns>Hand type.</returns>
        /// <remarks>To obtain real strength of hand type, use hand 
        /// type returned from this function as index to the 
        /// handTypeRanks array.
        /// < / remarks>
        var rankCountBitMask = 0, // index of the card.
        o = 0, rankBitMask;
        for (var i = -1; i < 5; i++, o = Math.pow(2, hand.Cards[i] * 4)) {
            rankCountBitMask += o * ((rankCountBitMask / o & 15) + 1);
        }
        if ((rankCountBitMask %= 15) !== 5) {
            return rankCountBitMask - 1;
        }
        rankBitMask = 1 << hand.Cards[0]
            | 1 << hand.Cards[1]
            | 1 << hand.Cards[2]
            | 1 << hand.Cards[3]
            | 1 << hand.Cards[4];
        rankCountBitMask -= ((rankBitMask / (rankBitMask & -rankBitMask) === 31) || (rankBitMask === 0x403c) ? 3 : 1);
        var temp = (hand.Suits[0] === (hand.Suits[0] | hand.Suits[1] | hand.Suits[2] | hand.Suits[3] | hand.Suits[4]));
        return rankCountBitMask - temp * ((rankBitMask === 0x7c00) ? -5 : 1);
    }
    HoldemHand.getHandType = getHandType;
    function cardValue(v) {
        if (v === 14) {
            return "A";
        }
        if (v === 13) {
            return "K";
        }
        if (v === 12) {
            return "Q";
        }
        if (v === 11) {
            return "J";
        }
        return v.toString();
    }
    HoldemHand.cardValue = cardValue;
    function getHandTypeEx(hand) {
        var isSameSuit = hand.Suits[0] === (hand.Suits[0] | hand.Suits[1] | hand.Suits[2] | hand.Suits[3] | hand.Suits[4]);
        var sortedCards = hand.Cards.slice(0);
        sortedCards = sortedCards.sort(function (left, right) {
            if (left < right) {
                return 1;
            }
            if (left > right) {
                return -1;
            }
            return 0;
        });
        var isStraight = sortedCards[0] === sortedCards[1] + 1
            && sortedCards[1] === sortedCards[2] + 1
            && sortedCards[2] === sortedCards[3] + 1
            && sortedCards[3] === sortedCards[4] + 1;
        if (!isStraight) {
            isStraight = sortedCards[3] === sortedCards[4] + 1
                && sortedCards[1] === sortedCards[2] + 1
                && sortedCards[2] === sortedCards[3] + 1
                && sortedCards[4] === 2
                && sortedCards[0] === 14;
        }
        if (isSameSuit && isStraight) {
            return {
                Type: 1,
                Cards: [sortedCards[0]]
            };
        }
        var isCare = (sortedCards[0] === sortedCards[1]
            && sortedCards[1] === sortedCards[2]
            && sortedCards[2] === sortedCards[3])
            || (sortedCards[1] === sortedCards[2]
                && sortedCards[2] === sortedCards[3]
                && sortedCards[3] === sortedCards[4]);
        if (isCare) {
            if (sortedCards[0] === sortedCards[1]) {
                return {
                    Type: 0,
                    Cards: [sortedCards[0], sortedCards[4]]
                };
            }
            return {
                Type: 0,
                Cards: [sortedCards[1], sortedCards[0]]
            };
        }
        var isFullHouse = (sortedCards[0] === sortedCards[1]
            && sortedCards[1] === sortedCards[2]
            && sortedCards[3] === sortedCards[4])
            || (sortedCards[0] === sortedCards[1]
                && sortedCards[2] === sortedCards[3]
                && sortedCards[3] === sortedCards[4]);
        if (isFullHouse) {
            return {
                Type: 9,
                Cards: [sortedCards[1], sortedCards[3]]
            };
        }
        if (isSameSuit) {
            return {
                Type: 3,
                Cards: sortedCards
            };
        }
        if (isStraight) {
            return {
                Type: 2,
                Cards: [Math.max(sortedCards[0], sortedCards[4])]
            };
        }
        // is triple
        if (sortedCards[0] === sortedCards[1]
            && sortedCards[1] === sortedCards[2]) {
            return {
                Type: 8,
                Cards: [sortedCards[0], sortedCards[3], sortedCards[4]]
            };
        }
        if (sortedCards[1] === sortedCards[2]
            && sortedCards[2] === sortedCards[3]) {
            return {
                Type: 8,
                Cards: [sortedCards[1], sortedCards[0], sortedCards[4]]
            };
        }
        if (sortedCards[2] === sortedCards[3]
            && sortedCards[3] === sortedCards[4]) {
            return {
                Type: 8,
                Cards: [sortedCards[2], sortedCards[0], sortedCards[1]]
            };
        }
        // is 2 pair
        if (sortedCards[0] === sortedCards[1]
            && sortedCards[2] === sortedCards[3]) {
            return {
                Type: 6,
                Cards: [sortedCards[0], sortedCards[2], sortedCards[4]]
            };
        }
        if (sortedCards[0] === sortedCards[1]
            && sortedCards[3] === sortedCards[4]) {
            return {
                Type: 6,
                Cards: [sortedCards[0], sortedCards[3], sortedCards[2]]
            };
        }
        if (sortedCards[1] === sortedCards[2]
            && sortedCards[3] === sortedCards[4]) {
            return {
                Type: 6,
                Cards: [sortedCards[1], sortedCards[3], sortedCards[0]]
            };
        }
        // is pair
        if (sortedCards[0] === sortedCards[1]) {
            return {
                Type: 5,
                Cards: [sortedCards[0], sortedCards[2], sortedCards[3], sortedCards[4]]
            };
        }
        if (sortedCards[1] === sortedCards[2]) {
            return {
                Type: 5,
                Cards: [sortedCards[1], sortedCards[0], sortedCards[3], sortedCards[4]]
            };
        }
        if (sortedCards[2] === sortedCards[3]) {
            return {
                Type: 5,
                Cards: [sortedCards[2], sortedCards[0], sortedCards[1], sortedCards[4]]
            };
        }
        if (sortedCards[3] === sortedCards[4]) {
            return {
                Type: 5,
                Cards: [sortedCards[3], sortedCards[0], sortedCards[1], sortedCards[2]]
            };
        }
        return {
            Type: 4,
            Cards: sortedCards
        };
    }
    HoldemHand.getHandTypeEx = getHandTypeEx;
    function getCombinations(k, n) {
        /// <signature>
        ///    <summary>
        ///    Generates all possible unordered permutations 
        ///    of k element in the set with n elements.
        ///    </summary>
        /// </signature>
        var result = [], comb = [];
        function next_comb(comb, k, n) {
            var i;
            if (comb.length === 0) {
                for (i = 0; i < k; ++i) {
                    comb[i] = i;
                }
                return true;
            }
            i = k - 1;
            ++comb[i];
            while ((i > 0) && (comb[i] >= n - k + 1 + i)) {
                --i;
                ++comb[i];
            }
            // No more combinations can be generated
            if (comb[0] > n - k) {
                return false;
            }
            for (i = i + 1; i < k; ++i) {
                comb[i] = comb[i - 1] + 1;
            }
            return true;
        }
        while (next_comb(comb, k, n)) {
            var nextPermutation = comb.slice(null);
            result.push(nextPermutation);
        }
        return result;
    }
    HoldemHand.getCombinations = getCombinations;
    function decodeScore(score) {
        var result = [];
        result.push((score & 15));
        score = score >> 4;
        result.push((score & 15));
        score = score >> 4;
        result.push((score & 15));
        score = score >> 4;
        result.push((score & 15));
        return result;
    }
    HoldemHand.decodeScore = decodeScore;
    function getPokerScore(cards) {
        var tempCards = cards.slice(0), cardsCount = {};
        for (var i = 0; i < 5; i++) {
            var cardValue_1 = tempCards[i];
            cardsCount[cardValue_1] = (cardsCount[cardValue_1] >= 1) ? cardsCount[cardValue_1] + 1 : 1;
        }
        tempCards.sort(function (left, right) {
            if (cardsCount[left] < cardsCount[right]) {
                return 1;
            }
            if (cardsCount[left] > cardsCount[right]) {
                return -1;
            }
            return right - left;
        });
        return tempCards[0] << 16
            | tempCards[1] << 12
            | tempCards[2] << 8
            | tempCards[3] << 4
            | tempCards[4];
    }
    HoldemHand.getPokerScore = getPokerScore;
    function parseHand(str) {
        /// <summary>Convert string to internal representation.</summary>
        /// <param name="str" type="String">String representation of the hand which should be converted.</param>
        /// <returns>Parsing result of internal conversion.</returns>
        if (str.match(/((?:\s*)(10|[2-9]|[J|Q|K|A])[♠|♣|♥|♦](?:\s*)){1,7}/g) === null) {
            return { Status: HandParseResultStatus.InvalidHand };
        }
        var cardStr = str.replace(/A/g, "14").replace(/K/g, "13").replace(/Q/g, "12")
            .replace(/J/g, "11").replace(/♠|♣|♥|♦/g, ",");
        var cards = cardStr.replace(/\s/g, "").slice(0, -1).split(",");
        var suits = str.match(/♠|♣|♥|♦/g);
        if (cards === null) {
            return { Status: HandParseResultStatus.CardsMissing };
        }
        if (suits === null) {
            return { Status: HandParseResultStatus.SuitsMissing };
        }
        if (cards.length !== suits.length) {
            return { Status: HandParseResultStatus.AllCardsShouldHaveOneSuit };
        }
        var o = {}, keyCount = 0, j, i;
        for (var i_2 = 0; i_2 < cards.length; i_2++) {
            var e = cards[i_2] + suits[i_2];
            o[e] = 1;
        }
        for (var j_1 in o) {
            if (o.hasOwnProperty(j_1)) {
                keyCount++;
            }
        }
        var insufficientCards = cards.length < 5;
        var duplicateCards = cards.length !== keyCount;
        var status;
        if (insufficientCards && duplicateCards) {
            status = HandParseResultStatus.InsufficientCardsAndDuplicates;
        }
        else if (insufficientCards) {
            status = HandParseResultStatus.InsufficientCards;
        }
        else if (duplicateCards) {
            status = HandParseResultStatus.DuplicatesDetected;
        }
        else {
            status = HandParseResultStatus.Ok;
        }
        for (var i_3 = 0; i_3 < cards.length; i_3++) {
            cards[i_3] -= 0; // Conversion to the numbers.
        }
        for (var i_4 = 0; i_4 < suits.length; i_4++) {
            // Convert the suit characters to the numbers.
            // Since unicode symbols starts with 0x2660 (9824) Unicode symbols 
            suits[i_4] = Math.pow(2, (suits[i_4].charCodeAt(0) - 9824));
        }
        return {
            Status: status, Hand: { Cards: cards, Suits: suits }
        };
    }
    HoldemHand.parseHand = parseHand;
    function getCardRank(hand) {
        /// <summary>Get universal hand strength.</summary>
        /// <param name="hand" type="HandRepresentation">Hand representation for which strength should be converted.</param>
        /// <returns>Rank of the card across all possible hands.</returns>
        /// <remarks>This function could accept hands from 5 to 7 cards.</remarks>
        var totalCardsCount = hand.Cards.length;
        var permutations = getCombinations(5, totalCardsCount);
        var maxRank = 0, winIndex = 10, winningScore = -1, wci;
        // Generate permuted version of the original array.
        var applyPermutation5 = function (source, permutation) {
            return [
                source[permutation[0]],
                source[permutation[1]],
                source[permutation[2]],
                source[permutation[3]],
                source[permutation[4]]
            ];
        };
        for (var i = 0; i < permutations.length; i++) {
            var currentPermutation = permutations[i];
            var cs = applyPermutation5(hand.Cards, currentPermutation);
            var ss = applyPermutation5(hand.Suits, currentPermutation);
            var index = getHandType({ Cards: cs, Suits: ss });
            if (HoldemHand.handTypeRanks[index] > maxRank) {
                maxRank = HoldemHand.handTypeRanks[index];
                winIndex = index;
                wci = currentPermutation.slice(0);
                winningScore = getPokerScore(cs);
            }
            else if (HoldemHand.handTypeRanks[index] === maxRank) {
                // If by chance we have a tie, find the best one
                var score1 = getPokerScore(cs);
                if (score1 > winningScore) {
                    wci = currentPermutation.slice(0);
                }
            }
        }
        return {
            HandType: winIndex,
            Score: winningScore,
            WinnerCardsSet: wci
        };
    }
    HoldemHand.getCardRank = getCardRank;
})(HoldemHand || (HoldemHand = {}));
define("table/tournamentManager", ["require", "exports"], function (require, exports) {
    "use strict";
    var TournamentManager = (function () {
        function TournamentManager() {
            /**
            * Notifies that tournament started
            */
            this.tournamentStarted = new signals.Signal();
            /**
            * Notifies that tournament cancelled
            */
            this.tournamentCancelled = new signals.Signal();
        }
        /**
        * Gets tournament by it's id
        * @param id Number Id of the tournament to retreive
        */
        TournamentManager.prototype.getTournament = function (id) {
            throw new Error("Not implemented");
        };
        /**
        * Initialize manager
        */
        TournamentManager.prototype.initialize = function () {
            // $.connection.hub.Game.client.
        };
        return TournamentManager;
    }());
});
define("tablesfilterpage", ["require", "exports", "knockout", "ui/pagebase"], function (require, exports, ko, pagebase_15) {
    "use strict";
    var TablesFilterPage = (function (_super) {
        __extends(TablesFilterPage, _super);
        function TablesFilterPage() {
            _super.call(this);
            this.oldpassword = ko.observable().extend({ required: true });
            this.password = ko.observable().extend({ required: true });
            this.confirmpassword = ko.observable().extend({ required: true });
            this.errors = ko.validation.group(this);
        }
        return TablesFilterPage;
    }(pagebase_15.PageBase));
    exports.TablesFilterPage = TablesFilterPage;
});
//# sourceMappingURL=application.js.map